/********************************************************************************
**
**  Xilinx, Inc. 2004                 www.xilinx.com
**
**  Synthesizable DDR SDRAM Controller
**
*********************************************************************************
**
**  File name :   backend_fifos.v
**
**  Description     : This module instantiates the backend fifos required for the 
**                       ddr interface. This includes the write address and data fifos,
**                       and the read address and data fifos. The FIFOs are implemented
**                       using the Virtex-4 block rams which can also be used as FIFOs
**                        
**
**  Date - revision : 
**                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
**
**  Author :      Lakshmi Gopalakrishnan
**
**  Contact :     e-mail  hotline@xilinx.com
**                phone   + 1 800 255 7778 
**
**  Disclaimer:   LIMITED WARRANTY AND DISCLAMER. These designs are 
**                provided to you "as is". Xilinx and its licensors make and you 
**                receive no warranties or conditions, express, implied, 
**                statutory or otherwise, and Xilinx specifically disclaims any 
**                implied warranties of merchantability, non-infringement, or 
**                fitness for a particular purpose. Xilinx does not warrant that 
**                the functions contained in these designs will meet your 
**                requirements, or that the operation of these designs will be 
**                uninterrupted or error free, or that defects in the Designs 
**                will be corrected. Furthermore, Xilinx does not warrant or 
**                make any representations regarding use or the results of the 
**                use of the designs in terms of correctness, accuracy, 
**                reliability, or otherwise. 
**
**                LIMITATION OF LIABILITY. In no event will Xilinx or its 
**                licensors be liable for any loss of data, lost profits, cost 
**                or procurement of substitute goods or services, or for any 
**                special, incidental, consequential, or indirect damages 
**                arising from the use or operation of the designs or 
**                accompanying documentation, however caused and on any theory 
**                of liability. This limitation will apply even if Xilinx 
**                has been advised of the possibility of such damage. This 
**                limitation shall apply not-withstanding the failure of the 
**                essential purpose of any limited remedies herein. 
**
**  Copyright  2002 - 2005 Xilinx, Inc.
**  All rights reserved 
** 
******************************************************************************/
`timescale 1ps / 1ps
module backend_fifos (
                      USER_CLK,
                      DDR_CLK,                  
                      rst,
                      //Write address fifo signals
                      app_af_addr, 
                      app_af_WrEn,
                      ctrl_af_RdEn,
                      af_addr,
                      af_Almost_Empty,
                      af_empty,         
                      //Write data fifo signals
                      app_Wdf_data,
                      app_Wdf_WrEn,
                      ctrl_Wdf_RdEn,
                      Wdf_data,
                      Wdf_Empty,
                      Wdf_Almost_Full,
                      //read address fifo signals
                      app_cmd_addr,
                      app_cmd_WrEn,
                      ctrl_cmd_RdEn,
                      cmd_addr,
                      cmd_Almost_Empty,
                      cmd_empty,
                      cmd_Almost_Full                     
                      );
                      
input         USER_CLK;
input         DDR_CLK;
input         rst;
input[31:0]   app_af_addr;  
input         app_af_WrEn;
input         ctrl_af_RdEn;
output[26:0]  af_addr;
output        af_Almost_Empty;
output         af_empty;

//wdf signals
input[127:0]  app_Wdf_data;
input         app_Wdf_WrEn;
input         ctrl_Wdf_RdEn;
output[127:0] Wdf_data;
output        Wdf_Empty;
output        Wdf_Almost_Full;  

//cmd signals
input [31:0]  app_cmd_addr;
input         app_cmd_WrEn;
input         ctrl_cmd_RdEn;
output [23:0] cmd_addr;
output        cmd_Almost_Empty;
output        cmd_Almost_Full;
output        cmd_empty;

wire [31:0]  fifo_input_write_addr;
wire [31:0]  fifo_input_read_addr;
reg [31:0]  addr_fifo_output;
reg [31:0]  cmd_fifo_output;
wire         wr_df_almost_full_1;
wire         wr_df_almost_full_2;
wire         wr_df_almost_full_3;
wire         wr_df_almost_full_4;
wire         wr_df_almost_full_5;
wire         wr_df_almost_full_6;
wire         wr_df_almost_full_7;
wire         wr_df_almost_full_8;
wire         wr_df_almost_full;
wire         wr_af_almost_empty;
wire         wr_af_rderr;
wire         rd_af_almost_empty;
wire         rd_af_rderr;
wire         rd_af_almost_full;
wire         rd_af_wrerr;
wire         wr_af_almost_full;

wire         wr_df_wrerr_1;
wire         wr_df_wrerr_2;
wire         wr_df_wrerr_3;
wire         wr_df_wrerr_4;
wire         wr_df_wrerr_5;
wire         wr_df_wrerr_6;
wire         wr_df_wrerr_7;
wire         wr_df_wrerr_8;
wire         wr_df_wrerr;
reg [11:0]   test_wr_pointer;
reg [11:0]   test_rd_pointer;
wire [11:0]  wrcount_1;
wire [11:0]  rdcount_1;



//assign fifo_input_write_addr[31:0] = app_af_addr[31:0];
//assign fifo_input_read_addr[31:0] = app_cmd_addr[31:0]; 
assign af_addr[26:0] = addr_fifo_output[26:0];
assign cmd_addr[23:0] = cmd_fifo_output[23:0];
assign af_Almost_Empty = wr_af_almost_empty || wr_af_rderr;
assign cmd_Almost_Empty = rd_af_almost_empty || rd_af_rderr;
assign cmd_Almost_Full  = rd_af_almost_full || rd_af_wrerr;                              
assign Wdf_Almost_Full      = wr_df_almost_full || wr_df_wrerr;
assign wr_df_almost_full = (wr_df_almost_full_1 && wr_df_almost_full_2 && 
                            wr_df_almost_full_3 && wr_df_almost_full_4);                            
assign wr_df_wrerr = (wr_df_wrerr_1 && wr_df_wrerr_2 && 
                      wr_df_wrerr_3 && wr_df_wrerr_4 );


// Address and Command fifos replaced with registers
// necessary for 133 PCI-X in order to meet 16 clock latency spec for reads

always@(posedge USER_CLK)begin
      if(rst)begin
          addr_fifo_output <= 0;
      end else if(app_af_WrEn)begin 
          addr_fifo_output <=  app_af_addr;
      end else begin
          addr_fifo_output <=  addr_fifo_output;
      end
end
always@(posedge USER_CLK)begin
      if(rst)begin
          cmd_fifo_output <= 0;
      end else if(app_cmd_WrEn)begin 
          cmd_fifo_output <=  app_cmd_addr;
      end else begin
          cmd_fifo_output <=  cmd_fifo_output;
      end
end

coregen_wr_fifo Wdf_1 (
	.din(app_Wdf_data[31:0]),
	.rd_clk(DDR_CLK),
	.rd_en(ctrl_Wdf_RdEn),
	.rst(rst),
	.wr_clk(USER_CLK),
	.wr_en(app_Wdf_WrEn),
	.dout(Wdf_data[31:0]),
	.empty(Wdf_Empty),
	.full());

coregen_wr_fifo Wdf_2 (
	.din(app_Wdf_data[63:32]),
	.rd_clk(DDR_CLK),
	.rd_en(ctrl_Wdf_RdEn),
	.rst(rst),
	.wr_clk(USER_CLK),
	.wr_en(app_Wdf_WrEn),
	.dout(Wdf_data[63:32]),
	.empty(),
	.full());

coregen_wr_fifo Wdf_3 (
	.din(app_Wdf_data[95:64]),
	.rd_clk(DDR_CLK),
	.rd_en(ctrl_Wdf_RdEn),
	.rst(rst),
	.wr_clk(USER_CLK),
	.wr_en(app_Wdf_WrEn),
	.dout(Wdf_data[95:64]),
	.empty(),
	.full());

coregen_wr_fifo Wdf_4 (
	.din(app_Wdf_data[127:96]),
	.rd_clk(DDR_CLK),
	.rd_en(ctrl_Wdf_RdEn),
	.rst(rst),
	.wr_clk(USER_CLK),
	.wr_en(app_Wdf_WrEn),
	.dout(Wdf_data[127:96]),
	.empty(),
	.full());

/*
// Write Data FIFO (rising edge 16 and falling edge 16 = 32 bit data)) 
//synthesis translate_off
defparam Wdf_1.ALMOST_FULL_OFFSET = 12'h00F;
defparam Wdf_1.ALMOST_EMPTY_OFFSET = 12'h006;
defparam Wdf_1.DATA_WIDTH = 36;
//synthesis translate_on
//synthesis attribute ALMOST_FULL_OFFSET of Wdf_1 is 12'h00F
//synthesis attribute ALMOST_EMPTY_OFFSET of Wdf_1 is 12'h006
//synthesis attribute DATA_WIDTH of Wdf_1 is 36

FIFO16  Wdf_1(
           .ALMOSTEMPTY(), 
           .ALMOSTFULL(wr_df_almost_full_1), 
           .DO(Wdf_data[31:0]), 
           .DOP(), 
           .EMPTY(Wdf_Empty), 
           .FULL(), 
           .RDCOUNT(), 
           .RDERR(), 
           .WRCOUNT(), 
           .WRERR(wr_df_wrerr_1), 
           .DI(app_Wdf_data[31:0]), 
           .DIP(4'b0000), 
           .RDCLK(DDR_CLK), 
           .RDEN(ctrl_Wdf_RdEn), 
           .RST(rst), 
           .WRCLK(USER_CLK), 
           .WREN(app_Wdf_WrEn)
          )*//* synthesis xc_props="ALMOST_FULL_OFFSET = 12'h00F,ALMOST_EMPTY_OFFSET = 12'h006,DATA_WIDTH = 36"*//*;*/
/*
//synthesis translate_off
defparam Wdf_2.ALMOST_FULL_OFFSET = 12'h00F;
defparam Wdf_2.ALMOST_EMPTY_OFFSET = 12'h006;
defparam Wdf_2.DATA_WIDTH = 36;
//synthesis translate_on

//synthesis attribute ALMOST_FULL_OFFSET of Wdf_2 is 12'h00F
//synthesis attribute ALMOST_EMPTY_OFFSET of Wdf_2 is 12'h006
//synthesis attribute DATA_WIDTH of Wdf_2 is 36

FIFO16  Wdf_2(
           .ALMOSTEMPTY(), 
           .ALMOSTFULL(wr_df_almost_full_2), 
           .DO(Wdf_data[63:32]), 
           .DOP(), 
           .EMPTY(), 
           .FULL(), 
           .RDCOUNT(), 
           .RDERR(), 
           .WRCOUNT(), 
           .WRERR(wr_df_wrerr_2), 
           .DI(app_Wdf_data[63:32]), 
           .DIP(4'b0000), 
           .RDCLK(DDR_CLK), 
           .RDEN(ctrl_Wdf_RdEn), 
           .RST(rst), 
           .WRCLK(USER_CLK), 
           .WREN(app_Wdf_WrEn)
          )*//* synthesis xc_props="ALMOST_FULL_OFFSET = 12'h00F,ALMOST_EMPTY_OFFSET = 12'h006,DATA_WIDTH = 36"*//*;*/
/*          
//synthesis translate_off
defparam Wdf_3.ALMOST_FULL_OFFSET = 12'h00F;
defparam Wdf_3.ALMOST_EMPTY_OFFSET = 12'h006;
defparam Wdf_3.DATA_WIDTH = 36;
//synthesis translate_on

//synthesis attribute ALMOST_FULL_OFFSET of Wdf_3 is 12'h00F
//synthesis attribute ALMOST_EMPTY_OFFSET of Wdf_3 is 12'h006
//synthesis attribute DATA_WIDTH of Wdf_3 is 36

FIFO16  Wdf_3(
           .ALMOSTEMPTY(), 
           .ALMOSTFULL(wr_df_almost_full_3), 
           .DO(Wdf_data[95:64]), 
           .DOP(), 
           .EMPTY(), 
           .FULL(), 
           .RDCOUNT(), 
           .RDERR(), 
           .WRCOUNT(), 
           .WRERR(wr_df_wrerr_3), 
           .DI(app_Wdf_data[95:64]), 
           .DIP(4'b0000), 
           .RDCLK(DDR_CLK), 
           .RDEN(ctrl_Wdf_RdEn), 
           .RST(rst), 
           .WRCLK(USER_CLK), 
           .WREN(app_Wdf_WrEn)
          )*//* synthesis xc_props="ALMOST_FULL_OFFSET = 12'h00F,ALMOST_EMPTY_OFFSET = 12'h006,DATA_WIDTH = 36"*//*; */
/*          
//synthesis translate_off
defparam Wdf_4.ALMOST_FULL_OFFSET = 12'h00F;
defparam Wdf_4.ALMOST_EMPTY_OFFSET = 12'h006;
defparam Wdf_4.DATA_WIDTH = 36;
//synthesis translate_on

//synthesis attribute ALMOST_FULL_OFFSET of Wdf_4 is 12'h00F
//synthesis attribute ALMOST_EMPTY_OFFSET of Wdf_4 is 12'h006
//synthesis attribute DATA_WIDTH of Wdf_4 is 36

FIFO16  Wdf_4(
           .ALMOSTEMPTY(), 
           .ALMOSTFULL(wr_df_almost_full_4), 
           .DO(Wdf_data[127:96]), 
           .DOP(), 
           .EMPTY(), 
           .FULL(), 
           .RDCOUNT(), 
           .RDERR(), 
           .WRCOUNT(), 
           .WRERR(wr_df_wrerr_4), 
           .DI(app_Wdf_data[127:96]), 
           .DIP(4'b0000), 
           .RDCLK(DDR_CLK), 
           .RDEN(ctrl_Wdf_RdEn), 
           .RST(rst), 
           .WRCLK(USER_CLK), 
           .WREN(app_Wdf_WrEn)
          )*//* synthesis xc_props="ALMOST_FULL_OFFSET = 12'h00F,ALMOST_EMPTY_OFFSET = 12'h006,DATA_WIDTH = 36"*//*;*/      
          


endmodule

