//*******************************************************************************
//
//  File name :       clk_module.v
//
//  Description :     This module instantiates DCM_BASE primitive for the memory
//                    interface and the 200 MHz clock for the IDELAYCTRL module. 
// 
//                    
//  Date - revision : 3/01/2004
//                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
//
//  Author :          Maria George
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2002 - 2005 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************

`timescale 1ns/100ps

module clk_module (                                  
                   SYS_CLK_IN, 
                   DDR_CLK90,
                   DDR_CLKDIV,                                             
                   REF_CLK200_IN,                    
                   CLK,                              
                   CLK90,                            
                   CLK200,                           
                   REFRESH_CLK,                      
                   DDR_DIMM_CLK0_P,                 
                   DDR_DIMM_CLK0_N,                 
                   DDR_DIMM_CLK1_P,                 
                   DDR_DIMM_CLK1_N,  
                   DDR_DIMM_CLK2_P,                 
                   DDR_DIMM_CLK2_N                                                           
                   );
                   
             
input      SYS_CLK_IN;
input      DDR_CLK90;
input      DDR_CLKDIV;
input      REF_CLK200_IN;
output     CLK;
output     CLK90;
output     CLK200;
output     REFRESH_CLK;
output     DDR_DIMM_CLK0_P;
output     DDR_DIMM_CLK0_N;
output     DDR_DIMM_CLK1_P;
output     DDR_DIMM_CLK1_N;
output     DDR_DIMM_CLK2_P;
output     DDR_DIMM_CLK2_N;

wire       clk0_bufg1_out;
wire       vcc;
wire       gnd;

assign CLK         = SYS_CLK_IN;
assign CLK90       = DDR_CLK90;
assign CLK200      = clk0_bufg1_out;
assign REFRESH_CLK = DDR_CLKDIV;
assign vcc         = 1'b1;
assign gnd         = 1'b0;
               
BUFG dcm1_clk0 (
                .O(clk0_bufg1_out),
                .I(REF_CLK200_IN)
                );                                  

//synthesis translate_off
defparam oddr_clk0.SRTYPE = "SYNC";
defparam oddr_clk0.DDR_CLK_EDGE = "OPPOSITE_EDGE";
//synthesis translate_on

//synthesis attribute SRTYPE of oddr_clk0 is "SYNC"
//synthesis attribute DDR_CLK_EDGE of oddr_clk0 is "OPPOSITE_EDGE"   

ODDR oddr_clk0 (
                .Q(DDR_DIMM_CLK0_N),
                .C(SYS_CLK_IN),
                .CE(vcc),
                .D1(gnd),
                .D2(vcc),
                .R(gnd),
                .S(gnd)
                )/* synthesis xc_props="SRTYPE = SYNC,DDR_CLK_EDGE = OPPOSITE_EDGE"*/;
               
//synthesis translate_off
defparam oddr_clk0n.SRTYPE = "SYNC";
defparam oddr_clk0n.DDR_CLK_EDGE = "OPPOSITE_EDGE";
//synthesis translate_on

//synthesis attribute SRTYPE of oddr_clk0n is "SYNC"
//synthesis attribute DDR_CLK_EDGE of oddr_clk0n is "OPPOSITE_EDGE"    

ODDR oddr_clk0n (
                 .Q(DDR_DIMM_CLK0_P),
                 .C(SYS_CLK_IN),
                 .CE(vcc),
                 .D1(vcc),
                 .D2(gnd),
                 .R(gnd),
                 .S(gnd)
                 )/* synthesis xc_props="SRTYPE = SYNC,DDR_CLK_EDGE = OPPOSITE_EDGE"*/;
                 
//synthesis translate_off
defparam oddr_clk1.SRTYPE = "SYNC";
defparam oddr_clk1.DDR_CLK_EDGE = "OPPOSITE_EDGE";
//synthesis translate_on

//synthesis attribute SRTYPE of oddr_clk1 is "SYNC"
//synthesis attribute DDR_CLK_EDGE of oddr_clk1 is "OPPOSITE_EDGE"    

ODDR oddr_clk1 (
                .Q(DDR_DIMM_CLK1_N),
                .C(SYS_CLK_IN),
                .CE(vcc),
                .D1(gnd),
                .D2(vcc),
                .R(gnd),
                .S(gnd)
                )/* synthesis xc_props="SRTYPE = SYNC,DDR_CLK_EDGE = OPPOSITE_EDGE"*/;
               
//synthesis translate_off
defparam oddr_clk1n.SRTYPE = "SYNC";
defparam oddr_clk1n.DDR_CLK_EDGE = "OPPOSITE_EDGE";
//synthesis translate_on

//synthesis attribute SRTYPE of oddr_clk1n is "SYNC"
//synthesis attribute DDR_CLK_EDGE of oddr_clk1n is "OPPOSITE_EDGE"   

ODDR oddr_clk1n (
                 .Q(DDR_DIMM_CLK1_P),
                 .C(SYS_CLK_IN),
                 .CE(vcc),
                 .D1(vcc),
                 .D2(gnd),
                 .R(gnd),
                 .S(gnd)
                 )/* synthesis xc_props="SRTYPE = SYNC,DDR_CLK_EDGE = OPPOSITE_EDGE"*/;
  
 //synthesis translate_off
defparam oddr_clk2.SRTYPE = "SYNC";
defparam oddr_clk2.DDR_CLK_EDGE = "OPPOSITE_EDGE";
//synthesis translate_on

//synthesis attribute SRTYPE of oddr_clk2 is "SYNC"
//synthesis attribute DDR_CLK_EDGE of oddr_clk2 is "OPPOSITE_EDGE"    

ODDR oddr_clk2 (
                .Q(DDR_DIMM_CLK2_N),
                .C(SYS_CLK_IN),
                .CE(vcc),
                .D1(gnd),
                .D2(vcc),
                .R(gnd),
                .S(gnd)
                )/* synthesis xc_props="SRTYPE = SYNC,DDR_CLK_EDGE = OPPOSITE_EDGE"*/;
               
//synthesis translate_off
defparam oddr_clk2n.SRTYPE = "SYNC";
defparam oddr_clk2n.DDR_CLK_EDGE = "OPPOSITE_EDGE";
//synthesis translate_on

//synthesis attribute SRTYPE of oddr_clk2n is "SYNC"
//synthesis attribute DDR_CLK_EDGE of oddr_clk2n is "OPPOSITE_EDGE"   

ODDR oddr_clk2n (
                 .Q(DDR_DIMM_CLK2_P),
                 .C(SYS_CLK_IN),
                 .CE(vcc),
                 .D1(vcc),
                 .D2(gnd),
                 .R(gnd),
                 .S(gnd)
                 )/* synthesis xc_props="SRTYPE = SYNC,DDR_CLK_EDGE = OPPOSITE_EDGE"*/;
                                 
endmodule                                                        