//*******************************************************************************
//
//  File name :       data_path.v
//
//  Description :     This module is the top level for the physical layer and  
//                    instantiates the following modules: tap_ctrl, data_tap_inc,
//                    idelay_ctrl, idelay_rd_en, v4_dqs_iob, v4_dq_iob, and
//                    rd_data_fifo.
// 
//                    
//  Date - revision : 1/27/2004
//                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
//
//  Author :          Maria George
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2002 - 2005 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************

`timescale 1ns/100ps


module data_path (
                     CLK,
                     CLK90,
                     CLK200,
                     RESET0,
                     RESET90,
                     CTRL_DUMMYREAD_START,
                     WDF_DATA,
                     DM_DATA,
                     CTRL_WREN,
                     CTRL_DQS_RST,
                     CTRL_DQS_EN,
                     DDR_DQ,
                     DDR_DQS,
                     DDR_DM,
                     SEL_DONE,
                     READ_DATA_RISE,
                     READ_DATA_FALL,
                     FIRST_RISING,
                     test_tap
                     )/* synthesis syn_noprune=1 */;
                     
   input          CLK;
   input          CLK90;
   input          CLK200;
   input          RESET0;
   input          RESET90;

   input          CTRL_DUMMYREAD_START;         // dummy read enable does not need to be normalized
   input [127:0]  WDF_DATA;
   input [15:0]   DM_DATA;
   input          CTRL_WREN;
   input          CTRL_DQS_RST;
   input          CTRL_DQS_EN;
   
   inout [63:0]   DDR_DQ;
   inout [7:0]    DDR_DQS;   
   output [7:0]   DDR_DM;
   output          SEL_DONE;
   output [63:0]  READ_DATA_RISE;
   output [63:0]  READ_DATA_FALL;
   output [3:0]    FIRST_RISING;
   output [5:0]    test_tap;
   
   wire [7:0]     data_idelay_inc; 
   wire [7:0]     data_idelay_ce;    
   wire [7:0]     data_idelay_rst;    
   wire [63:0]     rd_data_rise;
   wire [63:0]     rd_data_fall;
   wire [7:0]     dqs_idelay_inc;    
   wire [7:0]     dqs_idelay_ce;    
   wire [7:0]     dqs_idelay_rst;    
   wire [7:0]     dqs_delayed;    
   wire           idelay_ctrl_rdy;
   wire [7:0]     data_tap_select;  
   wire [7:0]     dqs_tap_sel_done; 
   wire [7:0]     rising_first;      
   wire [7:0]     valid_tap_count;  
   wire [5:0]     data_tap_count0;
   wire [5:0]     data_tap_count1;
   wire [5:0]     data_tap_count2;
   wire [5:0]     data_tap_count3;
   wire [5:0]     data_tap_count4;
   wire [5:0]     data_tap_count5;
   wire [5:0]     data_tap_count6;
   wire [5:0]     data_tap_count7;

      
   wire[3:0]      first_rising_detected;
   wire[3:0]      rd_en_delayed_rise;
   wire[3:0]      rd_en_delayed_fall;
   reg [127:0]     wr_data;
   reg [127:0]     wr_data_r1;
   reg [63:0]     wr_data_rise;
   reg [63:0]     wr_data_fall;
   reg [15:0]      mask_data;
   reg [15:0]      mask_data_r1;
   reg [7:0]      mask_data_rise;
   reg [7:0]      mask_data_fall;
   reg            dqs_rst_r1;
   reg            dqs_rst_r2;
   reg            dqs_rst_r3;
   reg            dqs_en_r1;
   reg            dqs_en_r2;
   reg            dqs_en_r3;
   reg            dqs_en_r4;
   reg            wr_en_clk270_r1;
   reg            wr_en_clk180_r2;
   reg            wr_en_clk90_r3;
   
   
   assign test_tap = data_tap_count0;
// For controller to stop dummy reads
   assign SEL_DONE = &data_tap_select[7:0];
   assign READ_DATA_RISE[63:0]  = rd_data_rise[63:0];
   assign READ_DATA_FALL[63:0]  = rd_data_fall[63:0];
   assign rd_en_delayed_fall[3:2] = {rd_en_delayed_fall[0], rd_en_delayed_fall[0]};
   assign FIRST_RISING[3:0]          = first_rising_detected[3:0];
   assign first_rising_detected[0] = (rising_first[0] && rising_first[1]);  
   assign first_rising_detected[1] = (rising_first[2] && rising_first[3]);  
   assign first_rising_detected[2] = (rising_first[4] && rising_first[5]); 
   assign first_rising_detected[3] = (rising_first[6] && rising_first[7]);                                                                                                       

   always @ (negedge CLK90)
   begin
     if (RESET90 == 1'b1)
       begin
         wr_data[127:0]   <= 128'h00000000000000000000000000000000;   
         mask_data[15:0]  <= 16'h0000;
         wr_en_clk270_r1  <= 1'b0;
         dqs_rst_r1       <= 1'b0;
         dqs_en_r1        <= 1'b0;         
       end                        
     else                         
       begin                      
         wr_data[63:0]   <= WDF_DATA[63:0];     
         wr_data[127:64] <= WDF_DATA[127:64];  
         mask_data[7:0]  <= DM_DATA[7:0];       
         mask_data[15:8] <= DM_DATA[15:8];    
         wr_en_clk270_r1 <= CTRL_WREN;
         dqs_rst_r1      <= CTRL_DQS_RST;
         dqs_en_r1       <= ~CTRL_DQS_EN;                             
       end
   end
   
   always @ (negedge CLK)   
   begin
     if (RESET0 == 1'b1)
       begin
         wr_en_clk180_r2   <= 1'b0;
         wr_data_r1[127:0]  <= 128'h00000000000000000000000000000000;   
         mask_data_r1[15:0] <= 16'h0000;
         dqs_rst_r2        <= 1'b0;
         dqs_rst_r3        <= 1'b0;
         dqs_en_r2         <= 1'b0;
         dqs_en_r3         <= 1'b0;
         dqs_en_r4         <= 1'b0;                           
                                           
       end
     else
       begin
         wr_en_clk180_r2   <= wr_en_clk270_r1;
         wr_data_r1[127:0]  <= wr_data[127:0];
         mask_data_r1[15:0] <= mask_data[15:0];
         dqs_rst_r2        <= dqs_rst_r1;
         dqs_rst_r3        <= dqs_rst_r2;
         dqs_en_r2         <= dqs_en_r1;
         dqs_en_r3         <= dqs_en_r2;
         dqs_en_r4         <= dqs_en_r3;                                    
                                                 
       end
   end   
   
   always @ (posedge CLK90)
   begin
     if (RESET90 == 1'b1)
       begin
         wr_en_clk90_r3      <= 1'b0;
         wr_data_rise[63:0]  <= 64'h0000000000000000;
         wr_data_fall[63:0]  <= 64'h0000000000000000;
         mask_data_rise[7:0] <= 8'h00;
         mask_data_fall[7:0] <= 8'h00;                                    
       end                   
     else                    
       begin
         wr_en_clk90_r3      <= wr_en_clk180_r2;
         wr_data_rise[63:0]  <= wr_data_r1[127:64];       
         wr_data_fall[63:0]  <= wr_data_r1[63:0];             
         mask_data_rise[7:0] <= mask_data_r1[15:8];
         mask_data_fall[7:0] <= mask_data_r1[7:0];                                   
       end
   end         
                               
   
///////////////////////////////////////////////////////////////////////////////
// IDELAYCTRL instantiation
///////////////////////////////////////////////////////////////////////////////

idelay_ctrl idelay_ctrl0(
                         .CLK200(CLK200),
                    .RESET(RESET0),
                    .RDY_STATUS(idelay_ctrl_rdy) 
                        );
                        
///////////////////////////////////////////////////////////////////////////////
// 36 DQS instances
///////////////////////////////////////////////////////////////////////////////

v4_dqs_iob  dqs0 (
                  .CLK(CLK),
                  .DLYINC(dqs_idelay_inc[0]), 
                  .DLYCE(dqs_idelay_ce[0]),  
                  .DLYRST(dqs_idelay_rst[0]),
                  .CTRL_DQS_RST(dqs_rst_r3),
                  .CTRL_DQS_EN(dqs_en_r4),
                  .DDR_DQS(DDR_DQS[0]),
                  .DQS(dqs_delayed[0]) 
                 );
                 
v4_dqs_iob  dqs1 (
                  .CLK(CLK),
                  .DLYINC(dqs_idelay_inc[1]), 
                  .DLYCE(dqs_idelay_ce[1]),  
                  .DLYRST(dqs_idelay_rst[1]),
                  .CTRL_DQS_RST(dqs_rst_r3),
                  .CTRL_DQS_EN(dqs_en_r4),
                  .DDR_DQS(DDR_DQS[1]),
                  .DQS(dqs_delayed[1]) 
                 );
                 
v4_dqs_iob  dqs2 (
                  .CLK(CLK),
                  .DLYINC(dqs_idelay_inc[2]), 
                  .DLYCE(dqs_idelay_ce[2]),  
                  .DLYRST(dqs_idelay_rst[2]),
                  .CTRL_DQS_RST(dqs_rst_r3),
                  .CTRL_DQS_EN(dqs_en_r4),
                  .DDR_DQS(DDR_DQS[2]),
                  .DQS(dqs_delayed[2]) 
                 );
                 
v4_dqs_iob  dqs3 (
                  .CLK(CLK),
                  .DLYINC(dqs_idelay_inc[3]), 
                  .DLYCE(dqs_idelay_ce[3]),  
                  .DLYRST(dqs_idelay_rst[3]),
                  .CTRL_DQS_RST(dqs_rst_r3),
                  .CTRL_DQS_EN(dqs_en_r4),
                  .DDR_DQS(DDR_DQS[3]),
                  .DQS(dqs_delayed[3]) 
                 );
                 
v4_dqs_iob  dqs4 (
                  .CLK(CLK),
                  .DLYINC(dqs_idelay_inc[4]), 
                  .DLYCE(dqs_idelay_ce[4]),  
                  .DLYRST(dqs_idelay_rst[4]),
                  .CTRL_DQS_RST(dqs_rst_r3),
                  .CTRL_DQS_EN(dqs_en_r4),
                  .DDR_DQS(DDR_DQS[4]),
                  .DQS(dqs_delayed[4]) 
                 );
                 
v4_dqs_iob  dqs5 (
                  .CLK(CLK),
             .DLYINC(dqs_idelay_inc[5]), 
             .DLYCE(dqs_idelay_ce[5]),  
             .DLYRST(dqs_idelay_rst[5]),
             .CTRL_DQS_RST(dqs_rst_r3),
             .CTRL_DQS_EN(dqs_en_r4),
             .DDR_DQS(DDR_DQS[5]),
             .DQS(dqs_delayed[5]) 
                 );
                 
v4_dqs_iob  dqs6 (
                  .CLK(CLK),
             .DLYINC(dqs_idelay_inc[6]), 
             .DLYCE(dqs_idelay_ce[6]),  
             .DLYRST(dqs_idelay_rst[6]),
             .CTRL_DQS_RST(dqs_rst_r3),
             .CTRL_DQS_EN(dqs_en_r4),
             .DDR_DQS(DDR_DQS[6]),
             .DQS(dqs_delayed[6]) 
                 );                  
                 
v4_dqs_iob  dqs7 (
                  .CLK(CLK),
             .DLYINC(dqs_idelay_inc[7]), 
             .DLYCE(dqs_idelay_ce[7]),  
             .DLYRST(dqs_idelay_rst[7]),
             .CTRL_DQS_RST(dqs_rst_r3),
             .CTRL_DQS_EN(dqs_en_r4),
             .DDR_DQS(DDR_DQS[7]),
             .DQS(dqs_delayed[7]) 
                 );
                                       
///////////////////////////////////////////////////////////////////////////////
// 36 tap_ctrl instances for 36 DDR_DQS strobes
///////////////////////////////////////////////////////////////////////////////

tap_ctrl  tap_ctrl_dqs0 (
                         .CLK(CLK),
                    .RESET(RESET0),
                    .RDY_STATUS(idelay_ctrl_rdy),
                    .DQS(dqs_delayed[0]), 
                    .CTRL_DUMMYREAD_START(CTRL_DUMMYREAD_START), 
                    .DLYINC(dqs_idelay_inc[0]), 
                    .DLYCE(dqs_idelay_ce[0]),  
                    .DLYRST(dqs_idelay_rst[0]),
                    .SEL_DONE(dqs_tap_sel_done[0]),
                    .FIRST_RISING(rising_first[0]),
                    .VALID_DATA_TAP_COUNT(valid_tap_count[0]),
                    .DATA_TAP_COUNT(data_tap_count0[5:0])
                         );
                         
tap_ctrl  tap_ctrl_dqs1 (
                         .CLK(CLK),
                    .RESET(RESET0),
                    .RDY_STATUS(idelay_ctrl_rdy),
                    .DQS(dqs_delayed[1]), 
                    .CTRL_DUMMYREAD_START(CTRL_DUMMYREAD_START),
                    .DLYINC(dqs_idelay_inc[1]), 
                    .DLYCE(dqs_idelay_ce[1]),  
                    .DLYRST(dqs_idelay_rst[1]),
                    .SEL_DONE(dqs_tap_sel_done[1]),
                    .FIRST_RISING(rising_first[1]),
                    .VALID_DATA_TAP_COUNT(valid_tap_count[1]),
                    .DATA_TAP_COUNT(data_tap_count1[5:0])
                         );
                         
tap_ctrl  tap_ctrl_dqs2 (
                         .CLK(CLK),
                    .RESET(RESET0),
                    .RDY_STATUS(idelay_ctrl_rdy),
                    .DQS(dqs_delayed[2]), 
                    .CTRL_DUMMYREAD_START(CTRL_DUMMYREAD_START),
                    .DLYINC(dqs_idelay_inc[2]), 
                    .DLYCE(dqs_idelay_ce[2]),  
                    .DLYRST(dqs_idelay_rst[2]),
                    .SEL_DONE(dqs_tap_sel_done[2]),
                    .FIRST_RISING(rising_first[2]),
                    .VALID_DATA_TAP_COUNT(valid_tap_count[2]),
                    .DATA_TAP_COUNT(data_tap_count2[5:0])
                         );
                         
tap_ctrl  tap_ctrl_dqs3 (
                         .CLK(CLK),
                    .RESET(RESET0),
                    .RDY_STATUS(idelay_ctrl_rdy),
                    .DQS(dqs_delayed[3]), 
                    .CTRL_DUMMYREAD_START(CTRL_DUMMYREAD_START),
                    .DLYINC(dqs_idelay_inc[3]), 
                    .DLYCE(dqs_idelay_ce[3]),  
                    .DLYRST(dqs_idelay_rst[3]),
                    .SEL_DONE(dqs_tap_sel_done[3]),
                    .FIRST_RISING(rising_first[3]),
                    .VALID_DATA_TAP_COUNT(valid_tap_count[3]),
                    .DATA_TAP_COUNT(data_tap_count3[5:0])
                         );
                         
tap_ctrl  tap_ctrl_dqs4 (
                         .CLK(CLK),
                    .RESET(RESET0),
                    .RDY_STATUS(idelay_ctrl_rdy),
                    .DQS(dqs_delayed[4]), 
                    .CTRL_DUMMYREAD_START(CTRL_DUMMYREAD_START),
                    .DLYINC(dqs_idelay_inc[4]), 
                    .DLYCE(dqs_idelay_ce[4]),  
                    .DLYRST(dqs_idelay_rst[4]),
                    .SEL_DONE(dqs_tap_sel_done[4]),
                    .FIRST_RISING(rising_first[4]),
                    .VALID_DATA_TAP_COUNT(valid_tap_count[4]),
                    .DATA_TAP_COUNT(data_tap_count4[5:0])
                         );
                         
tap_ctrl  tap_ctrl_dqs5(
                         .CLK(CLK),
                    .RESET(RESET0),
                    .RDY_STATUS(idelay_ctrl_rdy),
                    .DQS(dqs_delayed[5]), 
                    .CTRL_DUMMYREAD_START(CTRL_DUMMYREAD_START), 
                    .DLYINC(dqs_idelay_inc[5]), 
                    .DLYCE(dqs_idelay_ce[5]),  
                    .DLYRST(dqs_idelay_rst[5]),
                    .SEL_DONE(dqs_tap_sel_done[5]),
                    .FIRST_RISING(rising_first[5]),
                    .VALID_DATA_TAP_COUNT(valid_tap_count[5]),
                    .DATA_TAP_COUNT(data_tap_count5[5:0])
                         );
                         
tap_ctrl  tap_ctrl_dqs6 (
                         .CLK(CLK),
                    .RESET(RESET0),
                    .RDY_STATUS(idelay_ctrl_rdy),
                    .DQS(dqs_delayed[6]), 
                    .CTRL_DUMMYREAD_START(CTRL_DUMMYREAD_START),  
                    .DLYINC(dqs_idelay_inc[6]), 
                    .DLYCE(dqs_idelay_ce[6]),  
                    .DLYRST(dqs_idelay_rst[6]),
                    .SEL_DONE(dqs_tap_sel_done[6]),
                    .FIRST_RISING(rising_first[6]),
                    .VALID_DATA_TAP_COUNT(valid_tap_count[6]),
                    .DATA_TAP_COUNT(data_tap_count6[5:0])
                         );
                         
tap_ctrl  tap_ctrl_dqs7 (
                         .CLK(CLK),
                    .RESET(RESET0),
                    .RDY_STATUS(idelay_ctrl_rdy),
                    .DQS(dqs_delayed[7]), 
                    .CTRL_DUMMYREAD_START(CTRL_DUMMYREAD_START), 
                    .DLYINC(dqs_idelay_inc[7]), 
                    .DLYCE(dqs_idelay_ce[7]),  
                    .DLYRST(dqs_idelay_rst[7]),
                    .SEL_DONE(dqs_tap_sel_done[7]),
                    .FIRST_RISING(rising_first[7]),
                    .VALID_DATA_TAP_COUNT(valid_tap_count[7]),
                    .DATA_TAP_COUNT(data_tap_count7[5:0])
                         );
                                                                                                 
///////////////////////////////////////////////////////////////////////////////
// 36 instances of data_tap_inc for each dqs and associated tap_ctrl
///////////////////////////////////////////////////////////////////////////////

data_tap_inc  data_tap_inc_dq7_0 (
                                  .CLK(CLK),
                             .RESET(RESET0),
                             .DATA_DLYINC(data_idelay_inc[0]), 
                             .DATA_DLYCE(data_idelay_ce[0]),  
                             .DATA_DLYRST(data_idelay_rst[0]),
                             .DATA_TAP_SEL_DONE(data_tap_select[0]),
                             .VALID_DATA_TAP_COUNT(valid_tap_count[0]),
                             .DATA_TAP_COUNT(data_tap_count0[5:0]) 
                                 );
                                 
data_tap_inc  data_tap_inc_dq15_8 (
                                   .CLK(CLK),
                              .RESET(RESET0),
                              .DATA_DLYINC(data_idelay_inc[1]), 
                              .DATA_DLYCE(data_idelay_ce[1]),  
                              .DATA_DLYRST(data_idelay_rst[1]),
                              .DATA_TAP_SEL_DONE(data_tap_select[1]),
                              .VALID_DATA_TAP_COUNT(valid_tap_count[1]),
                              .DATA_TAP_COUNT(data_tap_count1[5:0]) 
                                  );
                                  
data_tap_inc  data_tap_inc_dq23_16 (
                                  .CLK(CLK),
                             .RESET(RESET0),
                             .DATA_DLYINC(data_idelay_inc[2]), 
                             .DATA_DLYCE(data_idelay_ce[2]),  
                             .DATA_DLYRST(data_idelay_rst[2]),
                             .DATA_TAP_SEL_DONE(data_tap_select[2]),
                             .VALID_DATA_TAP_COUNT(valid_tap_count[2]),
                             .DATA_TAP_COUNT(data_tap_count2[5:0]) 
                                 );
                                 
data_tap_inc  data_tap_inc_dq31_24 (
                                   .CLK(CLK),
                              .RESET(RESET0),
                              .DATA_DLYINC(data_idelay_inc[3]), 
                              .DATA_DLYCE(data_idelay_ce[3]),  
                              .DATA_DLYRST(data_idelay_rst[3]),
                              .DATA_TAP_SEL_DONE(data_tap_select[3]),
                              .VALID_DATA_TAP_COUNT(valid_tap_count[3]),
                              .DATA_TAP_COUNT(data_tap_count3[5:0]) 
                                  );
                                  
data_tap_inc  data_tap_inc_dq39_32 (
                                  .CLK(CLK),
                             .RESET(RESET0),
                             .DATA_DLYINC(data_idelay_inc[4]), 
                             .DATA_DLYCE(data_idelay_ce[4]),  
                             .DATA_DLYRST(data_idelay_rst[4]),
                             .DATA_TAP_SEL_DONE(data_tap_select[4]),
                             .VALID_DATA_TAP_COUNT(valid_tap_count[4]),
                             .DATA_TAP_COUNT(data_tap_count4[5:0]) 
                                 );
                                 
data_tap_inc  data_tap_inc_dq47_40 (
                                   .CLK(CLK),
                              .RESET(RESET0),
                              .DATA_DLYINC(data_idelay_inc[5]), 
                              .DATA_DLYCE(data_idelay_ce[5]),  
                              .DATA_DLYRST(data_idelay_rst[5]),
                              .DATA_TAP_SEL_DONE(data_tap_select[5]),
                              .VALID_DATA_TAP_COUNT(valid_tap_count[5]),
                              .DATA_TAP_COUNT(data_tap_count5[5:0]) 
                                  );
                                  
data_tap_inc  data_tap_inc_dq55_48 (
                                  .CLK(CLK),
                             .RESET(RESET0),
                             .DATA_DLYINC(data_idelay_inc[6]), 
                             .DATA_DLYCE(data_idelay_ce[6]),  
                             .DATA_DLYRST(data_idelay_rst[6]),
                             .DATA_TAP_SEL_DONE(data_tap_select[6]),
                             .VALID_DATA_TAP_COUNT(valid_tap_count[6]),
                             .DATA_TAP_COUNT(data_tap_count6[5:0]) 
                                 );
                                 
data_tap_inc  data_tap_inc_dq63_56 (
                                   .CLK(CLK),
                              .RESET(RESET0),
                              .DATA_DLYINC(data_idelay_inc[7]), 
                              .DATA_DLYCE(data_idelay_ce[7]),  
                              .DATA_DLYRST(data_idelay_rst[7]),
                              .DATA_TAP_SEL_DONE(data_tap_select[7]),
                              .VALID_DATA_TAP_COUNT(valid_tap_count[7]),
                              .DATA_TAP_COUNT(data_tap_count7[5:0]) 
                                  );
                                                           
///////////////////////////////////////////////////////////////////////////////
// DM instances
///////////////////////////////////////////////////////////////////////////////
v4_dm_iob dm_bit0 
                   (
                     .CLK90(CLK90),
                     .DM_RISE(mask_data_rise[0]),
                     .DM_FALL(mask_data_fall[0]),
                     .DDR_DM(DDR_DM[0])
                    );
v4_dm_iob dm_bit1 
                   (
                     .CLK90(CLK90),
                     .DM_RISE(mask_data_rise[1]),
                     .DM_FALL(mask_data_fall[1]),
                     .DDR_DM(DDR_DM[1])
                    );
v4_dm_iob dm_bit2 
                   (
                     .CLK90(CLK90),
                     .DM_RISE(mask_data_rise[2]),
                     .DM_FALL(mask_data_fall[2]),
                     .DDR_DM(DDR_DM[2])
                    );
v4_dm_iob dm_bit3 
                   (
                     .CLK90(CLK90),
                     .DM_RISE(mask_data_rise[3]),
                     .DM_FALL(mask_data_fall[3]),
                     .DDR_DM(DDR_DM[3])
                    );
v4_dm_iob dm_bit4 
                   (
                     .CLK90(CLK90),
                     .DM_RISE(mask_data_rise[4]),
                     .DM_FALL(mask_data_fall[4]),
                     .DDR_DM(DDR_DM[4])
                    );
v4_dm_iob dm_bit5 
                   (
                     .CLK90(CLK90),
                     .DM_RISE(mask_data_rise[5]),
                     .DM_FALL(mask_data_fall[5]),
                     .DDR_DM(DDR_DM[5])
                    );
v4_dm_iob dm_bit6 
                   (
                     .CLK90(CLK90),
                     .DM_RISE(mask_data_rise[6]),
                     .DM_FALL(mask_data_fall[6]),
                     .DDR_DM(DDR_DM[6])
                    );
v4_dm_iob dm_bit7 
                   (
                     .CLK90(CLK90),
                     .DM_RISE(mask_data_rise[7]),
                     .DM_FALL(mask_data_fall[7]),
                     .DDR_DM(DDR_DM[7])
                    );

                   
  
///////////////////////////////////////////////////////////////////////////////
// 36 instances of DQ_IOB4 instances
///////////////////////////////////////////////////////////////////////////////

v4_dq_iob dq_bit0 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[0]),
                     .DATA_DLYCE(data_idelay_ce[0]),
                     .DATA_DLYRST(data_idelay_rst[0]),
                     .WRITE_DATA_RISE(wr_data_rise[0]),
                     .WRITE_DATA_FALL(wr_data_fall[0]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[0]),
                     .READ_DATA_RISE(rd_data_rise[0]),
                     .READ_DATA_FALL(rd_data_fall[0])
                    );


v4_dq_iob dq_bit1 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[0]),
                     .DATA_DLYCE(data_idelay_ce[0]),
                     .DATA_DLYRST(data_idelay_rst[0]),
                     .WRITE_DATA_RISE(wr_data_rise[1]),
                     .WRITE_DATA_FALL(wr_data_fall[1]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[1]),
                     .READ_DATA_RISE(rd_data_rise[1]),
                     .READ_DATA_FALL(rd_data_fall[1])
                    );
                    
v4_dq_iob dq_bit2 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[0]),
                     .DATA_DLYCE(data_idelay_ce[0]),
                     .DATA_DLYRST(data_idelay_rst[0]),
                     .WRITE_DATA_RISE(wr_data_rise[2]),
                     .WRITE_DATA_FALL(wr_data_fall[2]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[2]),
                     .READ_DATA_RISE(rd_data_rise[2]),
                     .READ_DATA_FALL(rd_data_fall[2])
                    );

v4_dq_iob dq_bit3 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[0]),
                     .DATA_DLYCE(data_idelay_ce[0]),
                     .DATA_DLYRST(data_idelay_rst[0]),
                     .WRITE_DATA_RISE(wr_data_rise[3]),
                     .WRITE_DATA_FALL(wr_data_fall[3]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[3]),
                     .READ_DATA_RISE(rd_data_rise[3]),
                     .READ_DATA_FALL(rd_data_fall[3])
                    );

v4_dq_iob dq_bit4 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[0]),   
                     .DATA_DLYCE(data_idelay_ce[0]),
                     .DATA_DLYRST(data_idelay_rst[0]),
                     .WRITE_DATA_RISE(wr_data_rise[4]),
                     .WRITE_DATA_FALL(wr_data_fall[4]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[4]),
                     .READ_DATA_RISE(rd_data_rise[4]),
                     .READ_DATA_FALL(rd_data_fall[4])
                    );

v4_dq_iob dq_bit5 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[0]),    
                     .DATA_DLYCE(data_idelay_ce[0]),
                     .DATA_DLYRST(data_idelay_rst[0]),
                     .WRITE_DATA_RISE(wr_data_rise[5]),
                     .WRITE_DATA_FALL(wr_data_fall[5]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[5]),
                     .READ_DATA_RISE(rd_data_rise[5]),
                     .READ_DATA_FALL(rd_data_fall[5])
                    );

v4_dq_iob dq_bit6 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[0]),   
                     .DATA_DLYCE(data_idelay_ce[0]),
                     .DATA_DLYRST(data_idelay_rst[0]),
                     .WRITE_DATA_RISE(wr_data_rise[6]),
                     .WRITE_DATA_FALL(wr_data_fall[6]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[6]),
                     .READ_DATA_RISE(rd_data_rise[6]),
                     .READ_DATA_FALL(rd_data_fall[6])
                    );

v4_dq_iob dq_bit7 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[0]),    
                     .DATA_DLYCE(data_idelay_ce[0]),
                     .DATA_DLYRST(data_idelay_rst[0]),
                     .WRITE_DATA_RISE(wr_data_rise[7]),
                     .WRITE_DATA_FALL(wr_data_fall[7]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[7]),
                     .READ_DATA_RISE(rd_data_rise[7]),
                     .READ_DATA_FALL(rd_data_fall[7])
                    );

v4_dq_iob dq_bit8 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[1]),
                     .DATA_DLYCE(data_idelay_ce[1]),
                     .DATA_DLYRST(data_idelay_rst[1]),
                     .WRITE_DATA_RISE(wr_data_rise[8]),
                     .WRITE_DATA_FALL(wr_data_fall[8]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[8]),
                     .READ_DATA_RISE(rd_data_rise[8]),
                     .READ_DATA_FALL(rd_data_fall[8])
                    );

v4_dq_iob dq_bit9 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[1]),    
                     .DATA_DLYCE(data_idelay_ce[1]),
                     .DATA_DLYRST(data_idelay_rst[1]),
                     .WRITE_DATA_RISE(wr_data_rise[9]),
                     .WRITE_DATA_FALL(wr_data_fall[9]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[9]),
                     .READ_DATA_RISE(rd_data_rise[9]),
                     .READ_DATA_FALL(rd_data_fall[9])
                    );

v4_dq_iob dq_bit10 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[1]),
                     .DATA_DLYCE(data_idelay_ce[1]),
                     .DATA_DLYRST(data_idelay_rst[1]),
                     .WRITE_DATA_RISE(wr_data_rise[10]),
                     .WRITE_DATA_FALL(wr_data_fall[10]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[10]),
                     .READ_DATA_RISE(rd_data_rise[10]),
                     .READ_DATA_FALL(rd_data_fall[10])
                    );

v4_dq_iob dq_bit11 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[1]),   
                     .DATA_DLYCE(data_idelay_ce[1]),
                     .DATA_DLYRST(data_idelay_rst[1]),
                     .WRITE_DATA_RISE(wr_data_rise[11]),
                     .WRITE_DATA_FALL(wr_data_fall[11]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[11]),
                     .READ_DATA_RISE(rd_data_rise[11]),
                     .READ_DATA_FALL(rd_data_fall[11])
                    );

v4_dq_iob dq_bit12 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[1]),    
                     .DATA_DLYCE(data_idelay_ce[1]),
                     .DATA_DLYRST(data_idelay_rst[1]),
                     .WRITE_DATA_RISE(wr_data_rise[12]),
                     .WRITE_DATA_FALL(wr_data_fall[12]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[12]),
                     .READ_DATA_RISE(rd_data_rise[12]),
                     .READ_DATA_FALL(rd_data_fall[12])
                    );

v4_dq_iob dq_bit13 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[1]),   
                     .DATA_DLYCE(data_idelay_ce[1]),
                     .DATA_DLYRST(data_idelay_rst[1]),
                     .WRITE_DATA_RISE(wr_data_rise[13]),
                     .WRITE_DATA_FALL(wr_data_fall[13]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[13]),
                     .READ_DATA_RISE(rd_data_rise[13]),
                     .READ_DATA_FALL(rd_data_fall[13])
                    );

v4_dq_iob dq_bit14 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[1]),     
                     .DATA_DLYCE(data_idelay_ce[1]),
                     .DATA_DLYRST(data_idelay_rst[1]),
                     .WRITE_DATA_RISE(wr_data_rise[14]),
                     .WRITE_DATA_FALL(wr_data_fall[14]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[14]),
                     .READ_DATA_RISE(rd_data_rise[14]),
                     .READ_DATA_FALL(rd_data_fall[14])
                    );

v4_dq_iob dq_bit15 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[1]),    
                     .DATA_DLYCE(data_idelay_ce[1]),
                     .DATA_DLYRST(data_idelay_rst[1]),
                     .WRITE_DATA_RISE(wr_data_rise[15]),
                     .WRITE_DATA_FALL(wr_data_fall[15]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[15]),
                     .READ_DATA_RISE(rd_data_rise[15]),
                     .READ_DATA_FALL(rd_data_fall[15])
                    );                    
   
v4_dq_iob dq_bit16 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[2]),     
                     .DATA_DLYCE(data_idelay_ce[2]),
                     .DATA_DLYRST(data_idelay_rst[2]),
                     .WRITE_DATA_RISE(wr_data_rise[16]),
                     .WRITE_DATA_FALL(wr_data_fall[16]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[16]),
                     .READ_DATA_RISE(rd_data_rise[16]),
                     .READ_DATA_FALL(rd_data_fall[16])
                    );    
                    
v4_dq_iob dq_bit17 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[2]),   
                     .DATA_DLYCE(data_idelay_ce[2]),
                     .DATA_DLYRST(data_idelay_rst[2]),
                     .WRITE_DATA_RISE(wr_data_rise[17]),
                     .WRITE_DATA_FALL(wr_data_fall[17]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[17]),
                     .READ_DATA_RISE(rd_data_rise[17]),
                     .READ_DATA_FALL(rd_data_fall[17])
                    );    
                    
v4_dq_iob dq_bit18 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[2]),    
                     .DATA_DLYCE(data_idelay_ce[2]),
                     .DATA_DLYRST(data_idelay_rst[2]),
                     .WRITE_DATA_RISE(wr_data_rise[18]),
                     .WRITE_DATA_FALL(wr_data_fall[18]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[18]),
                     .READ_DATA_RISE(rd_data_rise[18]),
                     .READ_DATA_FALL(rd_data_fall[18])
                    );    
                    
v4_dq_iob dq_bit19 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[2]),    
                     .DATA_DLYCE(data_idelay_ce[2]),
                     .DATA_DLYRST(data_idelay_rst[2]),
                     .WRITE_DATA_RISE(wr_data_rise[19]),
                     .WRITE_DATA_FALL(wr_data_fall[19]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[19]),
                     .READ_DATA_RISE(rd_data_rise[19]),
                     .READ_DATA_FALL(rd_data_fall[19])
                    );                                                                
                    

v4_dq_iob dq_bit20 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[2]),   
                     .DATA_DLYCE(data_idelay_ce[2]),
                     .DATA_DLYRST(data_idelay_rst[2]),
                     .WRITE_DATA_RISE(wr_data_rise[20]),
                     .WRITE_DATA_FALL(wr_data_fall[20]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[20]),
                     .READ_DATA_RISE(rd_data_rise[20]),
                     .READ_DATA_FALL(rd_data_fall[20])
                    );


v4_dq_iob dq_bit21 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[2]),    
                     .DATA_DLYCE(data_idelay_ce[2]),
                     .DATA_DLYRST(data_idelay_rst[2]),
                     .WRITE_DATA_RISE(wr_data_rise[21]),
                     .WRITE_DATA_FALL(wr_data_fall[21]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[21]),
                     .READ_DATA_RISE(rd_data_rise[21]),
                     .READ_DATA_FALL(rd_data_fall[21])
                    );
                    
v4_dq_iob dq_bit22 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[2]),    
                     .DATA_DLYCE(data_idelay_ce[2]),
                     .DATA_DLYRST(data_idelay_rst[2]),
                     .WRITE_DATA_RISE(wr_data_rise[22]),
                     .WRITE_DATA_FALL(wr_data_fall[22]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[22]),
                     .READ_DATA_RISE(rd_data_rise[22]),
                     .READ_DATA_FALL(rd_data_fall[22])
                    );

v4_dq_iob dq_bit23 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[2]),
                     .DATA_DLYCE(data_idelay_ce[2]),
                     .DATA_DLYRST(data_idelay_rst[2]),
                     .WRITE_DATA_RISE(wr_data_rise[23]),
                     .WRITE_DATA_FALL(wr_data_fall[23]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[23]),
                     .READ_DATA_RISE(rd_data_rise[23]),
                     .READ_DATA_FALL(rd_data_fall[23])
                    );


v4_dq_iob dq_bit24 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[3]),   
                     .DATA_DLYCE(data_idelay_ce[3]),
                     .DATA_DLYRST(data_idelay_rst[3]),
                     .WRITE_DATA_RISE(wr_data_rise[24]),
                     .WRITE_DATA_FALL(wr_data_fall[24]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[24]),
                     .READ_DATA_RISE(rd_data_rise[24]),
                     .READ_DATA_FALL(rd_data_fall[24])
                    );

v4_dq_iob dq_bit25 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[3]),    
                     .DATA_DLYCE(data_idelay_ce[3]),
                     .DATA_DLYRST(data_idelay_rst[3]),
                     .WRITE_DATA_RISE(wr_data_rise[25]),
                     .WRITE_DATA_FALL(wr_data_fall[25]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[25]),
                     .READ_DATA_RISE(rd_data_rise[25]),
                     .READ_DATA_FALL(rd_data_fall[25])
                    );

v4_dq_iob dq_bit26 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[3]),    
                     .DATA_DLYCE(data_idelay_ce[3]),
                     .DATA_DLYRST(data_idelay_rst[3]),
                     .WRITE_DATA_RISE(wr_data_rise[26]),
                     .WRITE_DATA_FALL(wr_data_fall[26]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[26]),
                     .READ_DATA_RISE(rd_data_rise[26]),
                     .READ_DATA_FALL(rd_data_fall[26])
                    );

v4_dq_iob dq_bit27 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[3]),    
                     .DATA_DLYCE(data_idelay_ce[3]),
                     .DATA_DLYRST(data_idelay_rst[3]),
                     .WRITE_DATA_RISE(wr_data_rise[27]),
                     .WRITE_DATA_FALL(wr_data_fall[27]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[27]),
                     .READ_DATA_RISE(rd_data_rise[27]),
                     .READ_DATA_FALL(rd_data_fall[27])
                    );

v4_dq_iob dq_bit28 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[3]),    
                     .DATA_DLYCE(data_idelay_ce[3]),
                     .DATA_DLYRST(data_idelay_rst[3]),
                     .WRITE_DATA_RISE(wr_data_rise[28]),
                     .WRITE_DATA_FALL(wr_data_fall[28]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[28]),
                     .READ_DATA_RISE(rd_data_rise[28]),
                     .READ_DATA_FALL(rd_data_fall[28])
                    );

v4_dq_iob dq_bit29 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[3]),   
                     .DATA_DLYCE(data_idelay_ce[3]),
                     .DATA_DLYRST(data_idelay_rst[3]),
                     .WRITE_DATA_RISE(wr_data_rise[29]),
                     .WRITE_DATA_FALL(wr_data_fall[29]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[29]),
                     .READ_DATA_RISE(rd_data_rise[29]),
                     .READ_DATA_FALL(rd_data_fall[29])
                    );
                    
v4_dq_iob dq_bit30 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[3]),     
                     .DATA_DLYCE(data_idelay_ce[3]),
                     .DATA_DLYRST(data_idelay_rst[3]),
                     .WRITE_DATA_RISE(wr_data_rise[30]),
                     .WRITE_DATA_FALL(wr_data_fall[30]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[30]),
                     .READ_DATA_RISE(rd_data_rise[30]),
                     .READ_DATA_FALL(rd_data_fall[30])
                    );


v4_dq_iob dq_bit31 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[3]),   
                     .DATA_DLYCE(data_idelay_ce[3]),
                     .DATA_DLYRST(data_idelay_rst[3]),
                     .WRITE_DATA_RISE(wr_data_rise[31]),
                     .WRITE_DATA_FALL(wr_data_fall[31]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[31]),
                     .READ_DATA_RISE(rd_data_rise[31]),
                     .READ_DATA_FALL(rd_data_fall[31])
                    );

 
v4_dq_iob dq_bit32 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[4]),    
                     .DATA_DLYCE(data_idelay_ce[4]),
                     .DATA_DLYRST(data_idelay_rst[4]),
                     .WRITE_DATA_RISE(wr_data_rise[32]),
                     .WRITE_DATA_FALL(wr_data_fall[32]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[32]),
                     .READ_DATA_RISE(rd_data_rise[32]),
                     .READ_DATA_FALL(rd_data_fall[32])
                    );

v4_dq_iob dq_bit33 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[4]),   
                     .DATA_DLYCE(data_idelay_ce[4]),
                     .DATA_DLYRST(data_idelay_rst[4]),
                     .WRITE_DATA_RISE(wr_data_rise[33]),
                     .WRITE_DATA_FALL(wr_data_fall[33]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[33]),
                     .READ_DATA_RISE(rd_data_rise[33]),
                     .READ_DATA_FALL(rd_data_fall[33])
                    );

v4_dq_iob dq_bit34 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[4]),   
                     .DATA_DLYCE(data_idelay_ce[4]),
                     .DATA_DLYRST(data_idelay_rst[4]),
                     .WRITE_DATA_RISE(wr_data_rise[34]),
                     .WRITE_DATA_FALL(wr_data_fall[34]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[34]),
                     .READ_DATA_RISE(rd_data_rise[34]),
                     .READ_DATA_FALL(rd_data_fall[34])
                    );

v4_dq_iob dq_bit35 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[4]),   
                     .DATA_DLYCE(data_idelay_ce[4]),
                     .DATA_DLYRST(data_idelay_rst[4]),
                     .WRITE_DATA_RISE(wr_data_rise[35]),
                     .WRITE_DATA_FALL(wr_data_fall[35]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[35]),
                     .READ_DATA_RISE(rd_data_rise[35]),
                     .READ_DATA_FALL(rd_data_fall[35])
                    );

v4_dq_iob dq_bit36 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[4]),     
                     .DATA_DLYCE(data_idelay_ce[4]),
                     .DATA_DLYRST(data_idelay_rst[4]),
                     .WRITE_DATA_RISE(wr_data_rise[36]),
                     .WRITE_DATA_FALL(wr_data_fall[36]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[36]),
                     .READ_DATA_RISE(rd_data_rise[36]),
                     .READ_DATA_FALL(rd_data_fall[36])
                    );

v4_dq_iob dq_bit37 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[4]),   
                     .DATA_DLYCE(data_idelay_ce[4]),
                     .DATA_DLYRST(data_idelay_rst[4]),
                     .WRITE_DATA_RISE(wr_data_rise[37]),
                     .WRITE_DATA_FALL(wr_data_fall[37]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[37]),
                     .READ_DATA_RISE(rd_data_rise[37]),
                     .READ_DATA_FALL(rd_data_fall[37])
                    );

v4_dq_iob dq_bit38 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[4]),      
                     .DATA_DLYCE(data_idelay_ce[4]),
                     .DATA_DLYRST(data_idelay_rst[4]),
                     .WRITE_DATA_RISE(wr_data_rise[38]),
                     .WRITE_DATA_FALL(wr_data_fall[38]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[38]),
                     .READ_DATA_RISE(rd_data_rise[38]),
                     .READ_DATA_FALL(rd_data_fall[38])
                    );

v4_dq_iob dq_bit39 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[4]),   
                     .DATA_DLYCE(data_idelay_ce[4]),
                     .DATA_DLYRST(data_idelay_rst[4]),
                     .WRITE_DATA_RISE(wr_data_rise[39]),
                     .WRITE_DATA_FALL(wr_data_fall[39]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[39]),
                     .READ_DATA_RISE(rd_data_rise[39]),
                     .READ_DATA_FALL(rd_data_fall[39])
                    );   
 

v4_dq_iob dq_bit40 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[5]),  
                     .DATA_DLYCE(data_idelay_ce[5]),
                     .DATA_DLYRST(data_idelay_rst[5]),
                     .WRITE_DATA_RISE(wr_data_rise[40]),
                     .WRITE_DATA_FALL(wr_data_fall[40]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[40]),
                     .READ_DATA_RISE(rd_data_rise[40]),
                     .READ_DATA_FALL(rd_data_fall[40])
                    );


v4_dq_iob dq_bit41 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[5]),   
                     .DATA_DLYCE(data_idelay_ce[5]),
                     .DATA_DLYRST(data_idelay_rst[5]),
                     .WRITE_DATA_RISE(wr_data_rise[41]),
                     .WRITE_DATA_FALL(wr_data_fall[41]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[41]),
                     .READ_DATA_RISE(rd_data_rise[41]),
                     .READ_DATA_FALL(rd_data_fall[41])
                    );
                    
v4_dq_iob dq_bit42 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[5]),   
                     .DATA_DLYCE(data_idelay_ce[5]),
                     .DATA_DLYRST(data_idelay_rst[5]),
                     .WRITE_DATA_RISE(wr_data_rise[42]),
                     .WRITE_DATA_FALL(wr_data_fall[42]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[42]),
                     .READ_DATA_RISE(rd_data_rise[42]),
                     .READ_DATA_FALL(rd_data_fall[42])
                    );

v4_dq_iob dq_bit43 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[5]),   
                     .DATA_DLYCE(data_idelay_ce[5]),
                     .DATA_DLYRST(data_idelay_rst[5]),
                     .WRITE_DATA_RISE(wr_data_rise[43]),
                     .WRITE_DATA_FALL(wr_data_fall[43]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[43]),
                     .READ_DATA_RISE(rd_data_rise[43]),
                     .READ_DATA_FALL(rd_data_fall[43])
                    );

v4_dq_iob dq_bit44 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[5]),  
                     .DATA_DLYCE(data_idelay_ce[5]),
                     .DATA_DLYRST(data_idelay_rst[5]),
                     .WRITE_DATA_RISE(wr_data_rise[44]),
                     .WRITE_DATA_FALL(wr_data_fall[44]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[44]),
                     .READ_DATA_RISE(rd_data_rise[44]),
                     .READ_DATA_FALL(rd_data_fall[44])
                    );

v4_dq_iob dq_bit45 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[5]),    
                     .DATA_DLYCE(data_idelay_ce[5]),
                     .DATA_DLYRST(data_idelay_rst[5]),
                     .WRITE_DATA_RISE(wr_data_rise[45]),
                     .WRITE_DATA_FALL(wr_data_fall[45]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[45]),
                     .READ_DATA_RISE(rd_data_rise[45]),
                     .READ_DATA_FALL(rd_data_fall[45])
                    );

v4_dq_iob dq_bit46 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[5]),
                     .DATA_DLYCE(data_idelay_ce[5]),
                     .DATA_DLYRST(data_idelay_rst[5]),
                     .WRITE_DATA_RISE(wr_data_rise[46]),
                     .WRITE_DATA_FALL(wr_data_fall[46]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[46]),
                     .READ_DATA_RISE(rd_data_rise[46]),
                     .READ_DATA_FALL(rd_data_fall[46])
                    );

v4_dq_iob dq_bit47 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[5]),   
                     .DATA_DLYCE(data_idelay_ce[5]),
                     .DATA_DLYRST(data_idelay_rst[5]),
                     .WRITE_DATA_RISE(wr_data_rise[47]),
                     .WRITE_DATA_FALL(wr_data_fall[47]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[47]),
                     .READ_DATA_RISE(rd_data_rise[47]),
                     .READ_DATA_FALL(rd_data_fall[47])
                    );

v4_dq_iob dq_bit48 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[6]), 
                     .DATA_DLYCE(data_idelay_ce[6]),
                     .DATA_DLYRST(data_idelay_rst[6]),
                     .WRITE_DATA_RISE(wr_data_rise[48]),
                     .WRITE_DATA_FALL(wr_data_fall[48]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[48]),
                     .READ_DATA_RISE(rd_data_rise[48]),
                     .READ_DATA_FALL(rd_data_fall[48])
                    );

v4_dq_iob dq_bit49 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[6]),  
                     .DATA_DLYCE(data_idelay_ce[6]),
                     .DATA_DLYRST(data_idelay_rst[6]),
                     .WRITE_DATA_RISE(wr_data_rise[49]),
                     .WRITE_DATA_FALL(wr_data_fall[49]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[49]),
                     .READ_DATA_RISE(rd_data_rise[49]),
                     .READ_DATA_FALL(rd_data_fall[49])
                    );  
                    
v4_dq_iob dq_bit50 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[6]),    
                     .DATA_DLYCE(data_idelay_ce[6]),
                     .DATA_DLYRST(data_idelay_rst[6]),
                     .WRITE_DATA_RISE(wr_data_rise[50]),
                     .WRITE_DATA_FALL(wr_data_fall[50]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[50]),
                     .READ_DATA_RISE(rd_data_rise[50]),
                     .READ_DATA_FALL(rd_data_fall[50])
                    );


v4_dq_iob dq_bit51 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[6]),
                     .DATA_DLYCE(data_idelay_ce[6]),
                     .DATA_DLYRST(data_idelay_rst[6]),
                     .WRITE_DATA_RISE(wr_data_rise[51]),
                     .WRITE_DATA_FALL(wr_data_fall[51]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[51]),
                     .READ_DATA_RISE(rd_data_rise[51]),
                     .READ_DATA_FALL(rd_data_fall[51])
                    );
                    
v4_dq_iob dq_bit52 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[6]),   
                     .DATA_DLYCE(data_idelay_ce[6]),
                     .DATA_DLYRST(data_idelay_rst[6]),
                     .WRITE_DATA_RISE(wr_data_rise[52]),
                     .WRITE_DATA_FALL(wr_data_fall[52]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[52]),
                     .READ_DATA_RISE(rd_data_rise[52]),
                     .READ_DATA_FALL(rd_data_fall[52])
                    );

v4_dq_iob dq_bit53 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[6]), 
                     .DATA_DLYCE(data_idelay_ce[6]),
                     .DATA_DLYRST(data_idelay_rst[6]),
                     .WRITE_DATA_RISE(wr_data_rise[53]),
                     .WRITE_DATA_FALL(wr_data_fall[53]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[53]),
                     .READ_DATA_RISE(rd_data_rise[53]),
                     .READ_DATA_FALL(rd_data_fall[53])
                    );

v4_dq_iob dq_bit54 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[6]),
                     .DATA_DLYCE(data_idelay_ce[6]),
                     .DATA_DLYRST(data_idelay_rst[6]),
                     .WRITE_DATA_RISE(wr_data_rise[54]),
                     .WRITE_DATA_FALL(wr_data_fall[54]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[54]),
                     .READ_DATA_RISE(rd_data_rise[54]),
                     .READ_DATA_FALL(rd_data_fall[54])
                    );

v4_dq_iob dq_bit55 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[6]), 
                     .DATA_DLYCE(data_idelay_ce[6]),
                     .DATA_DLYRST(data_idelay_rst[6]),
                     .WRITE_DATA_RISE(wr_data_rise[55]),
                     .WRITE_DATA_FALL(wr_data_fall[55]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[55]),
                     .READ_DATA_RISE(rd_data_rise[55]),
                     .READ_DATA_FALL(rd_data_fall[55])
                    );

v4_dq_iob dq_bit56 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[7]),  
                     .DATA_DLYCE(data_idelay_ce[7]),
                     .DATA_DLYRST(data_idelay_rst[7]),
                     .WRITE_DATA_RISE(wr_data_rise[56]),
                     .WRITE_DATA_FALL(wr_data_fall[56]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[56]),
                     .READ_DATA_RISE(rd_data_rise[56]),
                     .READ_DATA_FALL(rd_data_fall[56])
                    );

v4_dq_iob dq_bit57 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[7]), 
                     .DATA_DLYCE(data_idelay_ce[7]),
                     .DATA_DLYRST(data_idelay_rst[7]),
                     .WRITE_DATA_RISE(wr_data_rise[57]),
                     .WRITE_DATA_FALL(wr_data_fall[57]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[57]),
                     .READ_DATA_RISE(rd_data_rise[57]),
                     .READ_DATA_FALL(rd_data_fall[57])
                    );

v4_dq_iob dq_bit58 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[7]),   
                     .DATA_DLYCE(data_idelay_ce[7]),
                     .DATA_DLYRST(data_idelay_rst[7]),
                     .WRITE_DATA_RISE(wr_data_rise[58]),
                     .WRITE_DATA_FALL(wr_data_fall[58]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[58]),
                     .READ_DATA_RISE(rd_data_rise[58]),
                     .READ_DATA_FALL(rd_data_fall[58])
                    );

v4_dq_iob dq_bit59 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[7]),  
                     .DATA_DLYCE(data_idelay_ce[7]),
                     .DATA_DLYRST(data_idelay_rst[7]),
                     .WRITE_DATA_RISE(wr_data_rise[59]),
                     .WRITE_DATA_FALL(wr_data_fall[59]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[59]),
                     .READ_DATA_RISE(rd_data_rise[59]),
                     .READ_DATA_FALL(rd_data_fall[59])
                    );                                   
 
v4_dq_iob dq_bit60 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[7]),  
                     .DATA_DLYCE(data_idelay_ce[7]),
                     .DATA_DLYRST(data_idelay_rst[7]),
                     .WRITE_DATA_RISE(wr_data_rise[60]),
                     .WRITE_DATA_FALL(wr_data_fall[60]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[60]),
                     .READ_DATA_RISE(rd_data_rise[60]),
                     .READ_DATA_FALL(rd_data_fall[60])
                    );


v4_dq_iob dq_bit61 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[7]), 
                     .DATA_DLYCE(data_idelay_ce[7]),
                     .DATA_DLYRST(data_idelay_rst[7]),
                     .WRITE_DATA_RISE(wr_data_rise[61]),
                     .WRITE_DATA_FALL(wr_data_fall[61]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[61]),
                     .READ_DATA_RISE(rd_data_rise[61]),
                     .READ_DATA_FALL(rd_data_fall[61])
                    );
                    
v4_dq_iob dq_bit62 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[7]), 
                     .DATA_DLYCE(data_idelay_ce[7]),
                     .DATA_DLYRST(data_idelay_rst[7]),
                     .WRITE_DATA_RISE(wr_data_rise[62]),
                     .WRITE_DATA_FALL(wr_data_fall[62]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[62]),
                     .READ_DATA_RISE(rd_data_rise[62]),
                     .READ_DATA_FALL(rd_data_fall[62])
                    );

v4_dq_iob dq_bit63 
                   (
                     .CLK(CLK),
                     .CLK90(CLK90),
                     .RESET(RESET90),
                     .DATA_DLYINC(data_idelay_inc[7]),  
                     .DATA_DLYCE(data_idelay_ce[7]),
                     .DATA_DLYRST(data_idelay_rst[7]),
                     .WRITE_DATA_RISE(wr_data_rise[63]),
                     .WRITE_DATA_FALL(wr_data_fall[63]),
                     .CTRL_WREN(wr_en_clk90_r3),
                     .DDR_DQ(DDR_DQ[63]),
                     .READ_DATA_RISE(rd_data_rise[63]),
                     .READ_DATA_FALL(rd_data_fall[63])
                    );

                                                          
endmodule 

 
 
