//******************************************************************************
//
//  Xilinx, Inc. 2005                 www.xilinx.com
//
//
//*******************************************************************************
//
//  File name :       data_tap_inc.v
//
//  Description :     This module implements the tap selection controller 
//                    for data bits associated with a strobe.
// 
//                    
//  Date - revision : 1/16/2004
//                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
//
//  Author :          Maria George
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2002 - 2005 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************

`timescale 1ns/100ps

module data_tap_inc (
                     CLK,
                     RESET,
                     DATA_DLYINC, 
                     DATA_DLYCE,  
                     DATA_DLYRST,
                     DATA_TAP_SEL_DONE,
                     VALID_DATA_TAP_COUNT,
                     DATA_TAP_COUNT 
                    );

   input        CLK;               
   input        RESET;
   input        VALID_DATA_TAP_COUNT;
   input [5:0]  DATA_TAP_COUNT;   
                
   output       DATA_DLYINC;
   output       DATA_DLYCE;
   output       DATA_DLYRST;
   output       DATA_TAP_SEL_DONE;

   reg          DATA_DLYINC;
   reg          DATA_DLYCE;
   reg          DATA_DLYRST;   
   reg [5:0]    data_tap_inc_counter;
   reg          data_tap_sel_clk;
   reg          data_tap_sel_r1;
   
   assign DATA_TAP_SEL_DONE = data_tap_sel_r1;
   
   always @ (posedge CLK)
   begin
     if (RESET == 1'b1)
       begin
         data_tap_sel_clk  <= 1'b0;
       end
     else if (data_tap_inc_counter[5:0] == 6'b000001)
       begin
         data_tap_sel_clk  <= 1'b1;
       end
   end
   
   always @ (posedge CLK)
   begin
     if (RESET == 1'b1)
       data_tap_sel_r1 <= 1'b0;
     else
       data_tap_sel_r1 <= data_tap_sel_clk;
   end
   
   always @ (posedge CLK)
   begin
     if (RESET == 1'b1)        // Data IDELAY in no change mode
       begin
         DATA_DLYINC               <= 1'b0;
         DATA_DLYCE                <= 1'b0;
         DATA_DLYRST               <= 1'b0;
         data_tap_inc_counter[5:0] <= 6'b000000;
       end

     else if (VALID_DATA_TAP_COUNT == 1'b1)
       begin
         DATA_DLYINC               <= 1'b0;
         DATA_DLYCE                <= 1'b0;
         DATA_DLYRST               <= 1'b1;
         data_tap_inc_counter[5:0] <= DATA_TAP_COUNT[5:0];
       end
     else if (data_tap_inc_counter[5:0] != 6'b000000) //Data IDELAY incremented
       begin
         DATA_DLYINC               <= 1'b1;
         DATA_DLYCE                <= 1'b1;
         DATA_DLYRST               <= 1'b0;
         data_tap_inc_counter[5:0] <= data_tap_inc_counter[5:0] - 1'b1;
       end
     else                                         //Data IDELAY no change mode
       begin
         DATA_DLYINC               <= 1'b0;
         DATA_DLYCE                <= 1'b0;
         DATA_DLYRST               <= 1'b0;
         data_tap_inc_counter[5:0] <= 6'b000000;
       end
   end
   
endmodule