/********************************************************************************
**    
**  Xilinx, Inc. 2005                 www.xilinx.com
**    
**  Synthesizable DDR SDRAM Controller
**    
*********************************************************************************
**    
**  File name :   ddr_controller.v
**    
**  Description     : This module 
**                     1. provides the read enable signals to the Write Address,
**                        Write Data and the Read Address FIFOs
**                     2. has the controller state machine which supplies the
**                        right command signals to the ddr device. Auto Refresh 
**                        commands are generated by the controller taking the 
**                        Auto Refresh Command Interval into account. The controller
**                        also provides commands for enabling ODT and 
**                        OCD(Off chip driver calibration).
**                     3. provides the address signals to the ddr device.
**    
**    
**  Date - revision :
**                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund 
**    
**  Author :      Lakshmi Gopalakrishnan
**    
**  Contact :     e-mail  hotline@xilinx.com
**                phone   + 1 800 255 7778 
**    
**  Disclaimer:   LIMITED WARRANTY AND DISCLAMER. These designs are 
**                provided to you "as is". Xilinx and its licensors make and you 
**                receive no warranties or conditions, express, implied, 
**                statutory or otherwise, and Xilinx specifically disclaims any 
**                implied warranties of merchantability, non-infringement, or 
**                fitness for a particular purpose. Xilinx does not warrant that 
**                the functions contained in these designs will meet your 
**                requirements, or that the operation of these designs will be 
**                uninterrupted or error free, or that defects in the Designs 
**                will be corrected. Furthermore, Xilinx does not warrant or 
**                make any representations regarding use or the results of the 
**                use of the designs in terms of correctness, accuracy, 
**                reliability, or otherwise. 
**    
**                LIMITATION OF LIABILITY. In no event will Xilinx or its 
**                licensors be liable for any loss of data, lost profits, cost 
**                or procurement of substitute goods or services, or for any 
**                special, incidental, consequential, or indirect damages 
**                arising from the use or operation of the designs or 
**                accompanying documentation, however caused and on any theory 
**                of liability. This limitation will apply even if Xilinx 
**                has been advised of the possibility of such damage. This 
**                limitation shall apply not-withstanding the failure of the 
**                essential purpose of any limited remedies herein. 
**    
**  Copyright  2002 - 2005 Xilinx, Inc.
**  All rights reserved 
**    
******************************************************************************/

`define idle_ctrl                5'h00//0_0000
`define load_mode_reg            5'h01//0_0001
`define mode_register_wait       5'h02//0_0010      
`define precharge                5'h03//0_0011
`define precharge_wait           5'h04//0_0100 
`define auto_refresh             5'h05//0_0101
`define auto_refresh_wait        5'h06//0_0110
`define active                   5'h07//0_0111
`define active_wait              5'h08//0_1000
`define first_read               5'h09//0_1001
`define active2                 5'h0A//0_1010
`define read_wait                5'h0B//0_1011
`define first_write              5'h0C//0_1100
`define write_burst                5'h16
`define active2_wait              5'h0D//0_1101
`define write_wait               5'h0E//0_1110
`define initcount_200            5'h0F//0_1111
`define initcount_200_wait       5'h10//1_0000
`define dummy_read_cycles        5'h11//1_0001
`define dummy_active             5'h12//1_0010
`define dummy_active_wait        5'h13//1_0011
`define dummy_read               5'h14//1_0100
`define dummy_read_wait          5'h15//1_0101
//`define write_read               5'h16//1_0110
`define read_burst               5'h17//1_0111
`define dummy_first_read         5'h18//1_1000
`define holdoff_precharge        5'h19//1_1001
        
`timescale 1ns/100ps 
`include "parameters.v"

module ddr_controller(
                   clk266,
                   refresh_clk,
                   rst,
                   addr_fifo,   
                   cmd_fifo,   
                   phy_Dly_Slct_Done,
                   addr_fifo_empty,
                   cmd_fifo_empty,
                   ctrl_Dummyread_Start,
                   ctrl_af_RdEn, 
                   ctrl_Wdf_RdEn,
                   wdf_empty,
                   ctrl_cmd_RdEn, 
                   ctrl_Dqs_Rst,
                   ctrl_Dqs_En,
                   ctrl_Dm_En,
                   ctrl_WrEn,
                   ctrl_RdEn,
                   ctrl_init_done,
                   ddr_address,
                   ddr_ba,
                   ddr_ras_L,
                   ddr_cas_L,
                   ddr_we_L,
                   ddr_cs_L,
                   ddr_cke,
                   ddr_odt,
                   not_ready,
                   burst_half,
                   PCI_HIT,
                   PCI_HIT_EARLY,
                   PCI_BE,
                   PCI_ADDR_EARLY,
                   PCI_CBEA_EARLY,
                   first_read,
                   S_DONE
                   )/* synthesis syn_noprune=1 */;
      
// controller inputs
input          clk266;
input          refresh_clk;
input          rst;
// FIFO  signals
input [26:0]   addr_fifo;
input [23:0]   cmd_fifo;         

input          phy_Dly_Slct_Done;  
input          addr_fifo_empty;
input          cmd_fifo_empty;
output         ctrl_Dummyread_Start; 
// FIFO read enable signals
output         ctrl_af_RdEn;
output         ctrl_Wdf_RdEn;
output         ctrl_cmd_RdEn;
input          wdf_empty;
// Rst and Enable signals for DQS logic
output         ctrl_Dqs_Rst;
output         ctrl_Dqs_En;
output [15:0]  ctrl_Dm_En;
// Read and Write Enable signals to the phy interface
output         ctrl_WrEn;
output         ctrl_RdEn;
output         ctrl_init_done;
//    
output[11:0]   ddr_address; 
output[1:0]    ddr_ba;
output         ddr_ras_L;
output         ddr_cas_L;
output         ddr_we_L;
output[1:0]    ddr_cs_L;
output         ddr_cke;
output         ddr_odt;
output         not_ready;
input          burst_half;
input          PCI_HIT;
input          PCI_HIT_EARLY;
input  [7:0]   PCI_BE;
input  [26:0]  PCI_ADDR_EARLY;
input  [7:0]   PCI_CBEA_EARLY;
output         first_read;
input          S_DONE;


//synthesis attribute IOB of ddr_ras_r is "TRUE"
//synthesis attribute IOB of ddr_cas_r is "TRUE"
//synthesis attribute IOB of ddr_we_r is "TRUE"
//synthesis attribute IOB of ddr_address_1r is "TRUE"
//synthesis attribute IOB of ddr_ba_1r is "TRUE"
//synthesis attribute IOB of ddr_cs_1r is "TRUE"
      
// internal signals
reg [3:0]      init_count;
reg            init_memory;
reg [7:0]      count_200_cycle;
wire           ref_flag;
reg            ref_flag_266;
reg            ref_flag_266_r;
reg            auto_ref;
reg [4:0]      state;
reg [4:0]      state_2r;
reg [4:0]      state_3r;
reg [4:0]      state_4r;
reg [4:0]      state_5r;
reg [4:0]      state_6r;
reg [8:0]      col_addr_2r;
reg [8:0]      col_addr_3r;
reg [8:0]      col_addr_4r;
reg [8:0]      col_addr_5r;
reg [8:0]      col_addr_6r;
reg [11:0]     ddr_address_1r;
reg [1:0]      ddr_ba_1r;
      
// counters for ddr controller
reg            mrd_count;
reg [1:0]      rp_count;
reg [4:0]      rfc_count;
reg [1:0]      rcd_count;
reg [3:0]      rc_count;
reg [7:0]      refi_count;
reg [3:0]      wr_to_rd_count;
reg [2:0]      rd_to_wr_count;
reg [2:0]      cas_count;   
reg [2:0]      cas_check_count;
reg [3:0]      ras_count;
reg [1:0]      rtp_count; 
reg [3:0]      wtp_count;   
reg [8:0]      wrburst_cnt;
reg [8:0]      ctrl_WrEn_cnt;
reg [7:0]      rdburst_cnt;   
reg [2:0]      rd_cmd_cnt; 
reg [1:0]      bank_addr_2r;
reg            ddr_ras_2r;
reg            ddr_cas_2r;
reg            ddr_we_2r; 
reg            ddr_ras_r;
reg            ddr_cas_r;
reg            ddr_we_r; 
reg            ddr_ras_3r; 
reg            ddr_cas_3r; 
reg            ddr_we_3r;
wire            rd_cmd_issued;
wire            wr_cmd_issued;
wire            WR;
wire            RD;
reg             ctrl_Dummyread_Start_1r; 
reg             ctrl_Dummyread_Start_2r;
reg             ctrl_Dummyread_Start_3r;
reg             ctrl_Dummyread_Start_4r;
// FIFO read enable signals
wire            ctrl_cmd_RdEn;
wire            ctrl_af_RdEn;
// Rst and Enable signals for DQS logic
reg             ctrl_Dqs_Rst;
reg             ctrl_Dqs_En;
// Read and Write Enable signals to the phy interface
reg             ctrl_WrEn;
wire            ctrl_RdEn;
reg             ctrl_Dummyread_Start;
reg             dummy_read_en;
wire            ctrl_init_done;
wire            count_200cycle_done;
reg             init_done;
wire            phy_Dly_Slct_Done;
wire [7:0]      burst_cnt;
wire [1:0]      wdf_rcd_check;
reg  [4:0]      countdown; 
reg  [11:0]     ddr_address_2r, ddr_address_3r, ddr_address_4r;  
reg  [1:0]      ddr_ba_2r, ddr_ba_3r, ddr_ba_4r; 
reg             ddr_ras_4r;
reg             ddr_cas_4r;
reg             ddr_we_4r;   
reg  [1:0]      ddr_cs_1r, ddr_cs_2r, ddr_cs_3r, ddr_cs_4r;
wire            read_or_write;
wire             valid_command;
wire            burst_n_dword;
wire [26:0]     end_addr;
reg  [22:0]      calc_burst;
wire            first_write;
reg  [6:0]      burst_count_16B;     //how many 16B transfers
wire [26:0]     end_addr_16B;  
wire [6:0]      trunc_end_addr_16B;
wire [6:0]      trunc_start_addr_16B;
wire            same_row_bank;
reg             write_n_read;
reg             not_ready;
reg [26:0]      end_addr_r;
reg             valid_command_r;
reg             vld_cmd_hand;      //handshake signal for valid command
reg [11:0]       early_act_count;   //early active command counter
wire            ctrl_RdEn_w;
reg             ctrl_RdEn_r2, ctrl_RdEn_r1;
wire            burst_zero;
reg             burst_zero_r;


always@(posedge clk266)begin
    if(PCI_HIT_EARLY)
     write_n_read <= PCI_CBEA_EARLY[0];
    else
     write_n_read <= write_n_read;
end


assign first_read = (state_4r == `first_read);
assign same_row_bank = (addr_fifo[25:12] == end_addr[25:12]) ? 1'b1 : 1'b0;
assign first_write = (state == `first_write) ? 1'b1 : 1'b0;

//following logic for generating S_Retry
always@(posedge clk266)begin
   if(rst)begin
     not_ready <= 1'b0;
   end else if((PCI_HIT_EARLY || PCI_HIT) && (state != `idle_ctrl) 
                && !((state == `active_wait) && (ras_count>4)))begin
     not_ready <= 1'b1;
   end else if(state_3r == `idle_ctrl || S_DONE)begin
     not_ready <= 1'b0;
   end else   begin
     not_ready <= not_ready;
   end
end



//Instantiate the data mask generation logic
dm_gen dm_gen_inst(
                   .clk(clk266),
                   .rst(rst),
                   .addr_fifo(addr_fifo[26:0]),
                   .cmd_fifo(cmd_fifo[23:0]),
                   .wrburst_cnt(wrburst_cnt[8:0]),
                   .burst_n_dword(burst_n_dword),
                   .first_write(first_write),
                   .ctrl_Dm_En(ctrl_Dm_En[15:0]),
                   .pci_be(PCI_BE)
                   );

//fifo control signals
assign ctrl_af_RdEn = valid_command;  
assign ctrl_cmd_RdEn = valid_command;

assign WR = (cmd_fifo[20] === 1'b1) ? 1'b1 : 1'b0;
assign RD = (cmd_fifo[20] === 1'b0) ? 1'b1 : 1'b0;
assign burst_n_dword = cmd_fifo[23];
assign end_addr[26:0] = end_addr_r[26:0];

always@(posedge clk266)begin
    if(rst)begin
       end_addr_r[26:0] <= 0;
    end else if(cmd_fifo[19:8] == 0) begin
       end_addr_r[26:0] <= addr_fifo[26:0];
    end else begin
       end_addr_r[26:0] <= addr_fifo[26:0] + cmd_fifo[19:8] - 1;
    end
end


assign valid_command = PCI_HIT;
always @ (posedge clk266 or posedge rst) begin    
  if (rst) begin     
        valid_command_r <= 1'b0;
  end else begin 
      valid_command_r <= valid_command;
  end
end  

always @ (posedge clk266 or posedge rst) begin    
  if (rst) begin     
        vld_cmd_hand <= 1'b0;
  end else if(valid_command == 1'b1)begin 
      vld_cmd_hand <= 1'b1;
  end else if(state == `first_write || state == `first_read || 
              state == `precharge)begin
      vld_cmd_hand <= 1'b0;
  end else begin
      vld_cmd_hand <= vld_cmd_hand;
  end
end         

always @ ( posedge clk266) begin
   if (rst) begin
       early_act_count[11:0] <= 12'h000;
   end else if (PCI_HIT_EARLY) begin
       early_act_count[11:0] <= 12'h10f;
   end else if (early_act_count[11:0] != 12'h000) begin
       early_act_count[11:0] <= early_act_count[11:0] - 1;
   end else begin
       early_act_count[11:0] <= early_act_count[11:0];
   end
end 

//to initialize memory
always @ (posedge clk266 or posedge rst) begin
   if (rst) begin
        init_memory <= 1'b1;
   end else if (init_count == 4'hA) begin
        init_memory <= 1'b0;
   end  else begin
        init_memory <= init_memory;
   end
end   
      
// mrd count
always @ (posedge clk266) begin
   if (rst) begin
       mrd_count <= 1'b0;
   end else if (state == `load_mode_reg) begin
       mrd_count <= `mrd_count_value;
   end else if (mrd_count != 1'b0) begin
       mrd_count <= mrd_count - 1;
   end else
       mrd_count <= mrd_count;
end   
      
// rp count
always @ (posedge clk266) begin
   if (rst) begin
       rp_count[1:0] <= 2'b00;
   end else if (state == `precharge) begin
       rp_count[1:0] <= `rp_count_value;
   end else if (rp_count[1:0] != 2'b00) begin
       rp_count[1:0] <= rp_count[1:0] - 1;
   end else
       rp_count[1:0] <= rp_count[1:0];
end   

// rfc count
always @ ( posedge clk266) begin
   if (rst) begin
       rfc_count[4:0] <= 5'b00000;
   end else if (state == `auto_refresh) begin
       rfc_count[4:0] <= `rfc_count_value;
   end else if (rfc_count[4:0] != 5'b00000) begin
       rfc_count[4:0] <= rfc_count[4:0] - 1;
   end else
       rfc_count[4:0] <= rfc_count[4:0];
end 
                  
// rcd count - 20ns
always @ ( posedge clk266) begin
   if (rst) begin
       rcd_count[1:0] <= 2'b00;
   end else if ((state == `active || state == `active2 || state == `dummy_active)&&write_n_read) begin
       rcd_count[1:0] <= `rcd_count_value+2;
   end else if ((state == `active || state == `active2 || state == `dummy_active)&&!write_n_read) begin
       rcd_count[1:0] <= `rcd_count_value;
   end else if (rcd_count[1:0] != 2'b00) begin
       rcd_count[1:0] <= rcd_count[1:0] - 1;
   end else
       rcd_count[1:0] <= rcd_count[1:0];
end   
     
// rc count - active to active on same bank
always @ ( posedge clk266) begin
   if (rst) begin
       rc_count[3:0] <= 4'b0000;
   end else if (state == `active || state == `active2) begin
       rc_count[3:0] <= `rc_count_value;
   end else if (rc_count[3:0] != 4'b0000) begin
       rc_count[3:0] <= rc_count[3:0] - 1;
   end else
       rc_count[3:0] <= rc_count[3:0];
end   
      
// ras count - active to precharge
always @ ( posedge clk266) begin
   if (rst) begin
       ras_count[3:0] <= 4'b0000;
   end else if (state == `active || state == `active2) begin
       ras_count[3:0] <= `ras_count_value;
   end else if (ras_count[3:0] != 4'b0000) begin
       ras_count[3:0] <= ras_count[3:0] - 1;
   end else
       ras_count[3:0] <= ras_count[3:0];
end   
      
// rtp count - read to precharge
always @ ( posedge clk266) begin
   if (rst) begin
       rtp_count[1:0] <= 2'b00;
   end else if (state == `first_read) begin
       rtp_count[1:0] <= `rtp_count_value;
   end else if (rtp_count[1:0] != 2'b00) begin
       rtp_count[1:0] <= rtp_count[1:0] - 1;
   end else
       rtp_count[1:0] <= rtp_count[1:0];
end   
      
// wtp count - write to precharge
always @ ( posedge clk266) begin
   if (rst) begin
       wtp_count[3:0] <= 4'b0000;
   end else if (state == `first_write || state == `write_burst) begin
       wtp_count[3:0] <= `wtp_count_value;
   end else if (wtp_count[3:0] != 4'b0000) begin
       wtp_count[3:0] <= wtp_count[3:0] - 1;
   end else
       wtp_count[3:0] <= wtp_count[3:0];
end 
      
// write to read counter 
// write to read includes : write latency + burst time + tWTR
      
always @ (posedge clk266) begin
   if (rst) begin
       wr_to_rd_count[3:0] <= 4'b0000;
   end else if (state == `first_write) begin
       wr_to_rd_count[3:0] <= `wr_to_rd_count_value;
   end else if (wr_to_rd_count[3:0] != 4'b0000) begin
       wr_to_rd_count[3:0] <= wr_to_rd_count[3:0] - 1;
   end else
       wr_to_rd_count[3:0] <= wr_to_rd_count[3:0];
end         

// read to write counter
always @ (posedge clk266) begin
   if (rst) begin
      rd_to_wr_count[2:0] <= 3'b000;
   end else if (state == `first_read) begin
      rd_to_wr_count[2:0] <= `cas_latency_value;
   end else if (rd_to_wr_count[2:0] != 3'b000) begin
      rd_to_wr_count[2:0] <= rd_to_wr_count[2:0] - 1;
   end else rd_to_wr_count[2:0] <= 3'b000;
end
       

// auto refresh interval counter in refresh_clk domain
always @ (posedge refresh_clk) begin
   if (rst) begin
      refi_count <= 8'h00;
   end else if (refi_count == `max_ref_cnt) begin
      refi_count <= 8'h00;
   end else 
      refi_count <= refi_count + 1;
    
end 

assign ref_flag = (refi_count == `max_ref_cnt) ? 1'b1 : 1'b0;

// refresh detect in 266 MHz clock
always @ (posedge clk266) begin
   if (rst) begin
      ref_flag_266   <= 1'b0;
      ref_flag_266_r <= 1'b0;
   end else 
      ref_flag_266   <= ref_flag;
      ref_flag_266_r <= ref_flag_266;
end
      
//refresh flag detect
//auto_ref high indicates auto_refresh requirement
//auto_ref is held high until auto refresh command is issued.
always @(posedge clk266) begin
    if (rst) begin
       auto_ref <= 1'b0;
    end else if (ref_flag_266 == 1'b1 && ref_flag_266_r == 1'b0) begin
       auto_ref <= 1'b1;
    end else if (state == `auto_refresh) begin
       auto_ref <= 1'b0;
    end else auto_ref <= auto_ref;
end       
        
// 200 clocks counter - count value : C8
// required for initialization
always @ (posedge clk266) begin
   if (rst) begin
       count_200_cycle[7:0] <= 8'h00;
   end else if (state == `initcount_200) begin
       count_200_cycle[7:0] <= 8'hC8;
   end else if (count_200_cycle[7:0] != 8'h00) begin
       count_200_cycle[7:0] <= count_200_cycle[7:0] - 1;
   end else begin
       count_200_cycle[7:0] <= 8'h00;
   end
end   
      
      
assign count_200cycle_done = (init_memory == 1'b1 & count_200_cycle == 8'h00)? 1'b1 : 1'b0;

always @ (posedge clk266) begin
   if (rst) begin
      init_done <= 1'b0;
   end else if ((init_count == 4'h9) && (count_200cycle_done == 1'b1) && (`Phy_Mode == 1'b0)) begin
      init_done <= 1'b1;
   end else if ((`Phy_Mode == 1'b1) && (phy_Dly_Slct_Done ==1'b1)) begin
      init_done <= 1'b1;
   end else begin
      init_done <= init_done;
   end
end
                                                          
assign ctrl_init_done      = init_done;                          
                 
always @ (posedge clk266) begin
  if (rst) begin
      init_count[3:0] <= 4'b0000;
  end else if (init_memory) begin
              if (state == `load_mode_reg || state == `precharge || 
                  state == `auto_refresh || state == `dummy_read_cycles || 
                  state == `initcount_200) begin
                       init_count[3:0] <= init_count[3:0] + 1;
              end else init_count[3:0] <= init_count[3:0];
  end else begin       init_count[3:0] <= 4'b0000;
  end

end   

// write burst count
always @ (posedge clk266) begin
   if (rst) begin
       wrburst_cnt[8:0] <= 9'h000;
   end else if (state == `first_write  && burst_n_dword == 1'b0) begin
       wrburst_cnt[8:0] <=  9'b000000010;//burst_cnt[2:0]};
   end else if (state == `first_write  && burst_n_dword == 1'b1) begin
       wrburst_cnt[8:0] <= calc_burst[8:0];
   end else if (wrburst_cnt[8:0] != 9'h000) begin
       wrburst_cnt[8:0] <= wrburst_cnt[8:0] - 1'b1;
   end else begin wrburst_cnt[8:0] <= 9'h000; end
end
always@(end_addr[26:5] or addr_fifo[26:5])
   begin
      calc_burst[22:0] = (end_addr[26:5] - addr_fifo[26:5] + 1) << 1;
   end

// count to generate write enable to the data path
always @ (posedge clk266) begin
   if (rst) begin
       ctrl_WrEn_cnt[8:0] <= 9'b000000000;
   end else begin
       ctrl_WrEn_cnt[8:0] <= wrburst_cnt[8:0];
   end
end
         
//write enable to data path
always @ (posedge clk266) begin
   if (rst) begin
      ctrl_WrEn <= 1'b0;
   end else if ((ctrl_WrEn_cnt[8:0] != 9'b000000000) && (WR == 1'b1)) begin
      ctrl_WrEn <= 1'b1;
   end else begin
      ctrl_WrEn <= 1'b0;
      end
end

// DQS enable to data path
always @ (posedge clk266) begin
   if (rst) begin
      ctrl_Dqs_En <= 1'b0;
   end else if ((state == `first_write)  || 
                ((wrburst_cnt != 9'h000) && (WR == 1'b1))) begin
      ctrl_Dqs_En <= 1'b1;
   end else ctrl_Dqs_En <= 1'b0;         
end

// DQS reset to data path
always @ (posedge clk266) begin
   if (rst) begin
      ctrl_Dqs_Rst <= 1'b0;
   end else if (state == `first_write ) begin   
      ctrl_Dqs_Rst <= 1'b1;
   end else ctrl_Dqs_Rst <= 1'b0;
end

// cas count
always @ (posedge clk266) begin
   if (rst) begin
      cas_count[2:0] <= 3'b000;
   end else if (state == `first_read || state == `dummy_first_read) begin
      cas_count[2:0] <= `cas_latency_value;
   end else if (cas_count[2:0] != 3'b000) begin
      cas_count[2:0] <= cas_count[2:0] - 1;
   end else cas_count[2:0] <= 3'b000;
end

//dummy_read enable
always @ (posedge clk266) begin
   if (rst) begin
      dummy_read_en <= 1'b0;
   end else if (state == `dummy_read) begin
      dummy_read_en <= 1'b1;
   end else if (phy_Dly_Slct_Done == 1'b1) begin
      dummy_read_en <= 1'b0;
   end else dummy_read_en <= dummy_read_en;
end

  
// ctrl_Dummyread_Start signal generation to the data path module
always @ (posedge clk266) begin
   if (rst) begin
       ctrl_Dummyread_Start_1r <= 1'b0;
   end else if ((dummy_read_en == 1'b1) && (cas_count == 3'b000)) begin
       ctrl_Dummyread_Start_1r <= 1'b1;
   end else if (phy_Dly_Slct_Done == 1'b1) begin
       ctrl_Dummyread_Start_1r <= 1'b0;
   end else ctrl_Dummyread_Start_1r <= ctrl_Dummyread_Start_1r;
end   
 
// register ctrl_Dummyread_Start signal
always @ (posedge clk266) begin
   if (rst) begin
      ctrl_Dummyread_Start_2r <= 1'b0; 
      ctrl_Dummyread_Start_3r <= 1'b0;
      ctrl_Dummyread_Start_4r <= 1'b0;
      ctrl_Dummyread_Start <= 1'b0;
   end else 
      ctrl_Dummyread_Start_2r <=  ctrl_Dummyread_Start_1r;
      ctrl_Dummyread_Start_3r <=  ctrl_Dummyread_Start_2r;
      ctrl_Dummyread_Start_4r <=  ctrl_Dummyread_Start_3r;
      ctrl_Dummyread_Start <=  ctrl_Dummyread_Start_4r;
end
         
 
 
// read command count
always @ (posedge clk266) begin
   if (rst) begin
       rd_cmd_cnt[2:0] <= 3'b000;
   end else if (state == `first_read || state == `read_burst) begin
       rd_cmd_cnt[2:0] <= 3'b010;
   end else if (rd_cmd_cnt[2:0] != 3'b000) begin
       rd_cmd_cnt[2:0] <= rd_cmd_cnt[2:0] - 1;
   end else rd_cmd_cnt[2:0] <= 3'b000;
end     
 
// cas-count wait for generating read enable
always @ (posedge clk266) begin
   if (rst) begin
       cas_check_count[2:0] <= 3'b000;
   end else if (state_4r == `first_read) begin // in order to get the enable in the preamble
       cas_check_count[2:0] <= `cas_latency_value -1;
   end else if (cas_check_count[2:0] != 3'b000) begin
       cas_check_count[2:0] <= cas_check_count[2:0] - 1;
   end else cas_check_count[2:0] <= 3'b000;
end

// read burst count
always @ (posedge clk266) begin
   if (rst) begin
       rdburst_cnt[7:0] <= 8'h00;
   end else if (state == `first_read) begin
       rdburst_cnt[7:0] <= (calc_burst[8:0] >> 1);
   end else if (rdburst_cnt[7:0] != 8'h00 && 
               (state == `first_read || state == `read_burst)) begin
       rdburst_cnt[7:0] <= rdburst_cnt[7:0] - 1'b1;
   end else begin rdburst_cnt[7:0] <= rdburst_cnt[7:0]; end
end



//rd_cmd_cnt will work for this particular design
//for a different frequency ratio or burst length
//a separate counter may be needed 
assign ctrl_RdEn_w = (rd_cmd_cnt != 0 && RD == 1'b1)? 1'b1 : 1'b0;    
always @ (posedge clk266) begin
   if (rst) begin
      ctrl_RdEn_r1 <= 1'b0;
      ctrl_RdEn_r2 <= 1'b0;
   end else begin
      ctrl_RdEn_r1 <= ctrl_RdEn_w;
      ctrl_RdEn_r2 <= ctrl_RdEn_r1;
   end 
end


assign ctrl_RdEn = ctrl_RdEn_r1;   
assign wdf_rcd_check[1:0] = `rcd_count_value - 1;


// write data fifo read enable
   assign end_addr_16B[26:0] = end_addr[26:0] - 3;
   assign trunc_end_addr_16B[6:0] = end_addr_16B[10:4];
   assign trunc_start_addr_16B[6:0] = addr_fifo[10:4];

   always@(posedge clk266)begin
     if (rst)begin
        burst_count_16B[6:0] <= 7'b0000000;
     end   else if(state_2r == `first_write) begin
        burst_count_16B[6:0] <= trunc_end_addr_16B - trunc_start_addr_16B + 1;
     end   else if(burst_count_16B != 0) begin
        burst_count_16B <= burst_count_16B - 1;
     end
   end

//following logic used for corner case read/write of 0 bytes
assign burst_zero   = (burst_count_16B == 0) ? 1'b0 : 1'b1;

   always@(posedge clk266)begin
     if (rst)begin
        burst_zero_r <= 1'b0;   
     end   else begin
        burst_zero_r <= burst_zero;
     end   
   end


assign ctrl_Wdf_RdEn = (addr_fifo[4] == 0) ? burst_zero : burst_zero_r;

// main control state machine  
always @ (posedge clk266) begin
  if (rst) begin
      state <= `idle_ctrl;
    countdown <= 5'b11111;
  end else begin
      case (state)
      `idle_ctrl : begin
                if (init_memory) begin
                   case (init_count)
                    4'h0 : begin
                              state <= `initcount_200;
                           end
                           
                    4'h1 : begin
                              if (count_200cycle_done) 
                                  state <= `precharge;
                              else
                                  state <= `idle_ctrl;
                               
                           end
                       
                    4'h2 : begin
                              if (mrd_count == 2'b00) 
                                 state <= `load_mode_reg; //emr;
                              else
                                 state <= `idle_ctrl;
                              
                          end
                          
                    4'h3 : begin
                              if (mrd_count == 2'b00) 
                                  state <= `load_mode_reg; //lmr;
                              else
                                  state <= `idle_ctrl;
                              
                           end
                           
                    4'h4 : begin
                             if (mrd_count == 2'b00) 
                                 state <= `precharge;
                             else
                                 state <= `idle_ctrl;
                             
                          end
                          
                    4'h5 : begin
                             if (rp_count == 3'b000) 
                                 state <= `auto_refresh;
                             else
                                 state <= `idle_ctrl;
                             
                          end
                          
                    4'h6 : begin
                             if (rfc_count ==  5'b00000) 
                                 state <= `auto_refresh;
                             else
                                 state <= `idle_ctrl;
                             
                          end
                          
                    4'h7 : begin
                              if (rfc_count == 5'b00000) 
                                  state <= `load_mode_reg;
                              else
                                  state <= `idle_ctrl;
                              
                            end
                            
                    4'h8 : begin
                               if (mrd_count == 2'b00) 
                                  state <= `initcount_200;
                               else
                                  state <= `idle_ctrl;
                               
                             end
                             
                    4'h9 :  begin 
                               if (count_200cycle_done) 
                                 begin
                                    if (`Phy_Mode)  
                                        // direct clocking method requires dummy reads to train the 
                                        // incoming DQS to the system clock
                                      state <= `dummy_read_cycles;
                                    else 
                                      state <= `idle_ctrl;
                                  end 
                               else
                                  state <= `idle_ctrl;
                             end 
                    4'hA :  begin
                               // phy_Dly_Slct_Done is sent by the data path module
                               // after the delay has been selected
                               if (phy_Dly_Slct_Done) begin
                                  state <= `idle_ctrl;
                               end
                            end 
                   endcase
                                                                                                           
              end else if (auto_ref == 1'b1) begin
                       state <= `auto_refresh; 
            
             end else if (PCI_HIT_EARLY && !not_ready)begin      
                       state <= `active;                                       
              end
              end
                   
      `initcount_200 :     begin
                              state <= `initcount_200_wait;
                            end
                        
      `initcount_200_wait     : begin
                                  if (count_200_cycle == 8'hC8) 
                                   state <= `idle_ctrl;
                                  else
                                   state <= `initcount_200_wait;
                                end
                           
      `dummy_read_cycles  : begin
                               state <= `dummy_active;
                            end
      
      `dummy_active       : begin
                               state <= `dummy_active_wait;
                            end
                           
      `dummy_active_wait  : begin
                               if (rcd_count == 2'b00) 
                                   state <= `dummy_first_read;
                               else 
                                   state <= `dummy_active_wait;
                            end
                            
       `dummy_first_read  : begin
                               state <= `dummy_read_wait;
                            end
                           
      `dummy_read         : begin
                               state <= `dummy_read_wait;
                            end
                           
      `dummy_read_wait    : begin
                               if (phy_Dly_Slct_Done) 
                                   state <= `precharge;
                               else
                                   state <= `dummy_read;         
                            end
                                   
                            
      `precharge          : begin
                             state <= `precharge_wait;
                           end
                   
      `precharge_wait     : begin
                             if (rp_count == 2'b01) begin
                                  if (auto_ref == 1'b1) begin
                                     state <= `auto_refresh;
                                  end else  begin
                                      state <= `idle_ctrl;
                                  end
                              end else begin
                                 state <= `precharge_wait; 
                              end
                           end
                           
      `load_mode_reg      : begin
                              state <= `mode_register_wait;
                           end
                           
      `mode_register_wait : begin
                              if (mrd_count == 2'b01) 
                                 state <= `idle_ctrl;
                              else
                                 state <= `mode_register_wait;
                              
                           end   
                             
     `auto_refresh       : begin
                                 state <= `auto_refresh_wait;
                           end
     `auto_refresh_wait  : begin
                            if (rfc_count == 5'b00001) begin
                                 state <= `idle_ctrl;
                              end else begin
                                 state <= `auto_refresh_wait;
                              end
                           end
                           
      `active             : begin
                              state <= `active_wait;                                                
                            end
                           
      `active_wait        : begin
                               if(early_act_count == 0)begin
                                       state <= `precharge;
                               end else begin
                                   if(rcd_count > 2'b00) begin
                                       state   <= `active_wait;
                                   end else if (WR && burst_half && vld_cmd_hand) begin
                                       if((same_row_bank)&& !wdf_empty) begin 
                                           state <= `first_write;
                                       end else if(!same_row_bank) begin 
                                           state <= `active2;
                                       end
                                   end else if (RD && vld_cmd_hand) begin                               
                                             state <= `first_read;        
                                   end else begin
                                       state <= `active_wait;
                                   end  
                                end      
                            end
                                                                
      `active2            : begin                      
                               state <= `active2_wait;                            
                            end
                           
      `active2_wait       : begin                         
                              if ((rcd_count == 2'b00 
                                  && WR == 1'b1) && wdf_empty == 1'b0) begin 
                                  state <= `first_write;
                              end else if 
                                  (rcd_count <= 2'b00 && 
                                    RD == 1'b1) begin
                                   state <= `read_burst;
                              end else begin
                                  state <= `active2_wait;
                              end
                           end 
                                                       
      `first_write        : begin
                              state <= `write_wait;
                            end
                           
      `write_burst        : begin
                              state <= `write_wait; 
                           end 
                                                 
      `write_wait         : begin
                              if ((burst_n_dword == 1'b1) && (wrburst_cnt >= 9'h003)) begin 
                                   state <= `write_burst;
                              end else if (wtp_count == 4'b0000) begin
                                  state <= `precharge;                  
                              end else begin 
                                  state <= `write_wait;
                              end
                           end 
                                                     
      `first_read         : begin 
                              if(same_row_bank == 1'b0)
                                 state <= `active2;
                              else                       
                                 state <= `read_wait;
                           end

      `read_burst         : begin
                              state <= `read_wait;
                            end

      `read_wait          : begin 
                             if(rd_cmd_cnt !=0) begin
                                state <= `read_wait;                          
                             end else if ((burst_n_dword == 1'b1) && (rdburst_cnt >= 8'h02)) begin
                                state <= `read_burst;
                             end else if (rtp_count == 4'b0000) begin
                                state <= `precharge;
                             end
                           end                                                        
     endcase
   end
end                                                 
      
//register command outputs
always @ (posedge clk266) begin
   if (rst) begin
      state_2r <= 5'b00000;
      state_3r <= 5'b00000;
      state_4r <= 5'b00000;
      state_5r <= 5'b00000;
      state_6r <= 5'b00000;
   end else begin
      state_2r <= state;
      state_3r <= state_2r;
      state_4r <= state_3r;
      state_5r <= state_4r;
      state_6r <= state_5r;                   
   end
end

wire act1;
assign act1 = (state == `idle_ctrl && PCI_HIT_EARLY && !init_memory);
wire act2;
assign act2 = ((state == `first_read && !same_row_bank) ||
              (state == `active_wait && WR && rcd_count == 2'b00 && 
               burst_half && vld_cmd_hand && !same_row_bank));

// commands to the memory
always @ (posedge clk266) begin
   if (rst) begin
      ddr_ras_r <= 1'b1;
   end else if (state == `load_mode_reg || 
                state == `precharge || 
                act1 ||
                act2 ||
                state == `auto_refresh || 
                state == `dummy_active) begin
      ddr_ras_r <= 1'b0;
   end else ddr_ras_r <= 1'b1;
end

// commands to the memory
wire cas_cond;       
reg cas_cond_r1;       
reg cas_cond_r2;       
reg cas_cond_r3;       
reg cas_cond_r4;
reg cas_cond_r5;       

assign cas_cond = (state == `first_write || state == `write_burst);   

always @ (posedge clk266) begin       
   if (rst) begin         
       cas_cond_r1 <= 1'b0;         
       cas_cond_r2 <= 1'b0;
       cas_cond_r3 <= 1'b0;         
       cas_cond_r4 <= 1'b0;
       cas_cond_r5 <= 1'b0;                     
   end else begin            
      cas_cond_r1 <= cas_cond;   
      cas_cond_r2 <= cas_cond_r1;
      cas_cond_r3 <= cas_cond_r2;   
      cas_cond_r4 <= cas_cond_r3;
      cas_cond_r5 <= cas_cond_r4;            
   end                  
end               

wire read_cond;
assign read_cond = ((state == `active_wait && RD && rcd_count <= 0 && vld_cmd_hand && early_act_count !=0) ||
                    (state == `active2_wait && rcd_count <= 0 && RD) ||
                    (state == `read_wait && !rd_cmd_cnt && burst_n_dword && rdburst_cnt >= 8'h02));

always @ (posedge clk266) begin
   if (rst) begin
      ddr_cas_r <= 1'b1;
   end else if (state == `load_mode_reg || 
                cas_cond_r5 ||
                read_cond ||
                state == `dummy_first_read || 
                state == `auto_refresh || 
                state == `dummy_read) begin
      ddr_cas_r <= 1'b0;
   end else begin
      ddr_cas_r <= 1'b1;
   end 
end


wire we_write_cond;    
reg we_write_cond_r1;       
reg we_write_cond_r2;       
reg we_write_cond_r3;       
reg we_write_cond_r4;       
reg we_write_cond_r5;       
      
      
assign we_write_cond = (state == `first_write || state == `write_burst);   

always @ (posedge clk266) begin       
   if (rst) begin            
       we_write_cond_r1 <= 1'b0;      
       we_write_cond_r2 <= 1'b0;   
       we_write_cond_r3 <= 1'b0;      
       we_write_cond_r4 <= 1'b0;
       we_write_cond_r5 <= 1'b0;                                 
   end else begin         
      we_write_cond_r1 <= we_write_cond;   
      we_write_cond_r2 <= we_write_cond_r1;
      we_write_cond_r3 <= we_write_cond_r2;
      we_write_cond_r4 <= we_write_cond_r3;
      we_write_cond_r5 <= we_write_cond_r4;                     
   end                  
end                  

// commands to the memory
always @ (posedge clk266) begin
   if (rst) begin
      ddr_we_r <= 1'b1;
   end else if (state == `load_mode_reg || 
                we_write_cond_r5 ||
                state == `precharge) begin
      ddr_we_r <= 1'b0;
   end else ddr_we_r <= 1'b1;
end

//register commands to the memory
always @ (posedge clk266) begin
   if (rst) begin
      ddr_ras_2r <= 1'b1;
      ddr_cas_2r <= 1'b1;
      ddr_we_2r <= 1'b1;
    end else 
      ddr_ras_2r  <= ddr_ras_r; 
      ddr_cas_2r  <= ddr_cas_r;
      ddr_we_2r   <= ddr_we_r;
end
           
//register commands to the memory
always @ (posedge clk266) begin
   if (rst) begin
      ddr_ras_3r <= 1'b1;
      ddr_cas_3r <= 1'b1;
      ddr_we_3r  <= 1'b1;
   end else 
      ddr_ras_3r  <= ddr_ras_2r; 
      ddr_cas_3r  <= ddr_cas_2r;
      ddr_we_3r   <= ddr_we_2r;
end                    
 
// register address outputs

reg [11:0] active_row_addr;
reg [11:0] active2_row_addr;
reg [1:0] active_bank_addr;
reg [1:0] active2_bank_addr;


always @ (posedge clk266 or posedge rst) begin
  if (rst) begin        
     active_row_addr[11:0]  <= 12'h000;
     active_bank_addr[1:0]  <= 2'b00;  
     active2_row_addr[11:0]  <= 12'h000;
     active2_bank_addr[1:0]  <= 2'b00;  
  end else if(valid_command_r)begin                                
     active_row_addr[11:0]  <= addr_fifo[25:14];
     active_bank_addr[1:0]  <= addr_fifo[13:12];  
     active2_row_addr[11:0]  <= end_addr[25:14];
     active2_bank_addr[1:0]  <= end_addr[13:12];  
   end   
end  

reg [26:0] current_addr_r;
always @ (posedge clk266) begin
  if (rst) begin
       current_addr_r[26:0] <= 27'h0000000;
  end else if (state_2r == `active)begin//valid_command_r) begin 
       current_addr_r[26:0] <= {addr_fifo[26:5],5'b00000}; //start on 32B boundary
  end else if (burst_n_dword == 1'b1 && (state_2r == `first_write ||
               state_2r == `write_burst || state == `first_read ||
               state == `read_burst)) begin //(state == `read_wait || state == `write_wait)) begin
       current_addr_r[26:0] <= current_addr_r[26:0] + 6'b100000;
  end else begin
       current_addr_r[26:0] <= current_addr_r[26:0];
  end
end

always @ (posedge clk266) begin
  if (rst) begin
      col_addr_2r[8:0] <= 9'h000;
      col_addr_3r[8:0] <= 9'h000;
      col_addr_4r[8:0] <= 9'h000;
      col_addr_5r[8:0] <= 9'h000;
      col_addr_6r[8:0] <= 9'h000;
  end else begin
      col_addr_2r[8:0] <= current_addr_r[11:3];
      col_addr_3r[8:0] <= col_addr_2r[8:0];
      col_addr_4r[8:0] <= col_addr_3r[8:0];
      col_addr_5r[8:0] <= col_addr_4r[8:0];
      col_addr_6r[8:0] <= col_addr_5r[8:0];
  end
end
 
always @ (posedge clk266) begin
   if (rst) begin
       ddr_address_1r <= 13'h0000;
   end else if (init_memory) begin
                if (state == `precharge) begin
                    ddr_address_1r <= 12'h400; //A10 = 1 for precharge all
                end else if ( state == `load_mode_reg && init_count == 4'h3) begin
                    ddr_address_1r <= {5'b00010, 3'b110, 1'b0, 3'b010}; // A8 == 1 for DLL reset
                end else if (state == `load_mode_reg && init_count ==4'h7) begin
                    ddr_address_1r <= {5'b00000, 3'b110, 1'b0, 3'b010}; // OPE CAS Burst_Type Burst_Length
                end else
                    ddr_address_1r <= 12'h000;
   end else if (act2) begin
       ddr_address_1r <=  active2_row_addr[11:0];
   end else if (read_cond) begin  
       ddr_address_1r <= {3'b000,current_addr_r[11:3]};
   end else if (state_6r == `first_write || state_6r == `write_burst) begin
       ddr_address_1r <=    {3'b000, col_addr_6r[8:0]};
   end else if (state == `precharge) begin
       ddr_address_1r <= 12'h400;
   end else ddr_address_1r <= PCI_ADDR_EARLY[25:14];
end   

wire [1:0] current_cs;
assign current_cs = (current_addr_r[26] == 1'b0) ? 2'b01 : 2'b10;
reg [1:0] cs_addr_2r;
reg [1:0] cs_addr_3r;
reg [1:0] cs_addr_4r;
reg [1:0] cs_addr_5r;
reg [1:0] cs_addr_6r;

 always @ (posedge clk266) begin
  if (rst) begin
      cs_addr_2r[1:0] <= 2'b00;
      cs_addr_3r[1:0] <= 2'b00;
      cs_addr_4r[1:0] <= 2'b00;
      cs_addr_5r[1:0] <= 2'b00;
      cs_addr_6r[1:0] <= 2'b00;
  end else begin
      cs_addr_2r[1:0] <= current_cs[1:0];
      cs_addr_3r[1:0] <= cs_addr_2r[1:0];
      cs_addr_4r[1:0] <= cs_addr_3r[1:0];
      cs_addr_5r[1:0] <= cs_addr_4r[1:0];
      cs_addr_6r[1:0] <= cs_addr_5r[1:0];
  end
end     
always @ (posedge clk266) begin
   if (rst) begin
       ddr_cs_1r[1:0] <= 2'b11;
   end else if (read_cond || 
                state == `dummy_read || state == `dummy_read_wait || 
                state == `dummy_first_read) begin  
       ddr_cs_1r[1:0] <= current_cs[1:0]; 
   end else if (state_6r == `first_write || state_6r == `write_burst) begin
       ddr_cs_1r[1:0] <=    cs_addr_6r[1:0];
   end else begin
       ddr_cs_1r[1:0] <= 2'b00;
   end                    
end   
 
reg [1:0] bank_addr_3r;
reg [1:0] bank_addr_4r;
reg [1:0] bank_addr_5r;
reg [1:0] bank_addr_6r;

 always @ (posedge clk266) begin
  if (rst) begin
      bank_addr_2r[1:0] <= 2'b00;
      bank_addr_3r[1:0] <= 2'b00;
      bank_addr_4r[1:0] <= 2'b00;
      bank_addr_5r[1:0] <= 2'b00;
      bank_addr_6r[1:0] <= 2'b00;
  end else begin
      bank_addr_2r[1:0] <= current_addr_r[13:12];
      bank_addr_3r[1:0] <= bank_addr_2r[1:0];
      bank_addr_4r[1:0] <= bank_addr_3r[1:0];
      bank_addr_5r[1:0] <= bank_addr_4r[1:0];
      bank_addr_6r[1:0] <= bank_addr_5r[1:0];
  end
end     
always @ (posedge clk266) begin
   if (rst) begin
       ddr_ba_1r[1:0] <= 2'b00;
   end else if (init_memory == 1'b1 && state == `load_mode_reg ) begin
               if (init_count == 4'h2) begin
                   ddr_ba_1r[1:0] <= 2'b01; //emr2 2'b10 
               end else ddr_ba_1r[1:0] <= 2'b00; //lmr
   end else if (act2) begin
        ddr_ba_1r[1:0] <= active2_bank_addr[1:0];
   end else if (read_cond || state == `first_read || 
                state == `read_burst || state == `read_wait) begin  
       ddr_ba_1r[1:0] <= current_addr_r[13:12];
   end else if (state_6r == `first_write || state_6r == `write_burst || 
                state_6r == `write_wait) begin
       ddr_ba_1r[1:0] <=    bank_addr_6r[1:0];
   end else 
       ddr_ba_1r[1:0] <= PCI_ADDR_EARLY[13:12]; 
end

                           
// need to register commands and address signals inside IOBs
always @ ( posedge clk266) begin
   if (rst) begin
    ddr_address_2r[11:0] <= 12'b111111111111;  
    ddr_ba_2r[1:0]  <= 2'b11;
    ddr_cs_2r[1:0]  <= 2'b11;
    ddr_address_3r[11:0] <= 12'b111111111111;  
    ddr_ba_3r[1:0]  <= 2'b11;
    ddr_cs_3r[1:0]  <= 2'b11;
    ddr_address_4r[11:0] <= 12'b111111111111;  
    ddr_ba_4r[1:0]  <= 2'b11;
    ddr_cs_4r[1:0]  <= 2'b11;
    ddr_ras_4r    <= 1'b1;
    ddr_cas_4r <= 1'b1;
    ddr_we_4r <= 1'b1;
   end else begin
    ddr_address_2r[11:0] <= ddr_address_1r[11:0];  
    ddr_ba_2r[1:0]  <= ddr_ba_1r[1:0];
    ddr_cs_2r[1:0]  <= ddr_cs_1r[1:0];
    ddr_address_3r[11:0] <= ddr_address_2r[11:0];  
    ddr_ba_3r[1:0]  <= ddr_ba_2r[1:0];
    ddr_cs_3r[1:0]  <= ddr_cs_2r[1:0];
    ddr_address_4r[11:0] <= ddr_address_3r[11:0];  
    ddr_ba_4r[1:0]  <= ddr_ba_3r[1:0];
    ddr_cs_4r[1:0]  <= ddr_cs_3r[1:0];
    ddr_ras_4r    <= ddr_ras_3r;
    ddr_cas_4r <= ddr_cas_3r;
    ddr_we_4r <= ddr_we_3r;
    end


end 
                                 
assign ddr_odt  = 1'b0; 
assign ddr_cke  = 1'b1;
assign ddr_cs_L = ddr_cs_1r; 
assign ddr_address[11:0]  = ddr_address_1r[11:0];
assign ddr_ba[1:0]   = ddr_ba_1r[1:0];   
assign ddr_ras_L = ddr_ras_r;        
assign ddr_cas_L = ddr_cas_r;       
assign ddr_we_L  = ddr_we_r;          

//synthesis translate_off
reg [20*10:0] fsm_monitor;
always @(state)
case (state)
 `idle_ctrl              :    fsm_monitor = "IDLE"; 
 `load_mode_reg            :     fsm_monitor = "LMR";
 `mode_register_wait     :    fsm_monitor = "MRW";
 `precharge                :     fsm_monitor = "PRE";
 `precharge_wait         :    fsm_monitor = "PRE_WAIT";
 `auto_refresh           :      fsm_monitor = "AUTO_REF";
 `auto_refresh_wait      :      fsm_monitor = "AUTO_REF_WAIT";
 `active                 :      fsm_monitor = "ACTIVE";
 `active_wait            :      fsm_monitor = "ACTIVE_WAIT";
 `active2                 :      fsm_monitor = "ACTIVE2";
 `active2_wait            :      fsm_monitor = "ACTIVE2_WAIT";
 `first_read             :      fsm_monitor = "FIRST_READ";
 `read_wait              :      fsm_monitor = "READ_WAIT";
 `first_write            :      fsm_monitor = "FIRST_WRITE";
 `write_wait             :      fsm_monitor = "WRITE_WAIT";
 `initcount_200          :      fsm_monitor = "INIT_200";
 `initcount_200_wait     :      fsm_monitor = "INIT_200_WAIT";
 `dummy_read_cycles      :      fsm_monitor = "DUMMY_READ";
 `dummy_active           :      fsm_monitor = "DUMMY_ACTIVE";
 `dummy_active_wait      :      fsm_monitor = "DUMMY_ACTIVE_WAIT";
 `dummy_read             :      fsm_monitor = "DUMMY_READ";
 `dummy_read_wait        :      fsm_monitor = "DUMMY_READ_WAIT";
 `write_burst             :      fsm_monitor = "WRITE_BURST";
 `read_burst             :      fsm_monitor = "READ_BURST";
 `dummy_first_read       :      fsm_monitor = "DUMMY_FIRST_READ";
 `holdoff_precharge      :      fsm_monitor = "HOLDOFF_PRE";
 default                :    fsm_monitor = "ILLEGAL"; 
endcase 

reg [20*10:0] command_ascii;
always @(cmd_fifo[23:20])
case (cmd_fifo[23:20])
 4'h6             :    command_ascii = "DWORD READ"; 
 4'h7              :     command_ascii = "DWORD WRITE";
 4'he             :    command_ascii = "BURST READ";
 4'hf               :     command_ascii = "BURST WRITE";
 default          :    command_ascii = "ILLEGAL"; 
endcase 
//synthesis translate_on
                                        
endmodule 
              
