//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2005 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************


module ddr_dimm (Dq, Dqs, Addr, Ba, Clk0, Clk0_n, Clk1, Clk1_n, Clk2, Clk2_n ,Cke0, Cke1, Ras_n, Cas_n, We_n, Dm, S0_n, S1_n);

    parameter ADDR_BITS        =      12; // Set this parameter to control how many Address bits are used

    parameter DQ_BITS          =      16; // Set this parameter to control how many Data bits are used

    parameter DQS_BITS         =       2; // Set this parameter to control how many DQS bits are used

    parameter DM_BITS          =       2; // Set this parameter to control how many DM bits are used

    parameter COL_BITS         =       9; // Set this parameter to control how many Column bits are used


    inout     [4*DQ_BITS - 1 : 0] Dq;
    inout    [4*DQS_BITS - 1 : 0] Dqs;
    input     [ADDR_BITS - 1 : 0] Addr;
    input                 [1 : 0] Ba;
    input                         Clk0;
    input                         Clk0_n;
    input                         Clk1;
    input                         Clk1_n;
    input                         Clk2;
    input                         Clk2_n;
    input                         Cke0;
    input                         Cke1;
    input                         Ras_n;
    input                         Cas_n;
    input                         We_n;
    input     [4*DM_BITS - 1 : 0] Dm;
    input                         S0_n;
    input                         S1_n;
 
 ddr U1(
         .Dq(Dq[15:0]),
         .Dqs(Dqs[1:0]),
         .Addr(Addr[11:0]),
         .Ba(Ba[1:0]),
         .Clk(Clk0),
         .Clk_n(Clk0_n),
         .Cke(Cke0),
         .Cs_n(S0_n),
         .Ras_n(Ras_n),
         .Cas_n(Cas_n),
         .We_n(We_n),
         .Dm(Dm[1:0])
       );

 ddr U2(
         .Dq(Dq[31:16]),
         .Dqs(Dqs[3:2]),
         .Addr(Addr[11:0]),
         .Ba(Ba[1:0]),
         .Clk(Clk0),
         .Clk_n(Clk0_n),
         .Cke(Cke0),
         .Cs_n(S0_n),
         .Ras_n(Ras_n),
         .Cas_n(Cas_n),
         .We_n(We_n),
         .Dm(Dm[3:2])
       );


 ddr U3(
         .Dq(Dq[47:32]),
         .Dqs(Dqs[5:4]),
         .Addr(Addr[11:0]),
         .Ba(Ba[1:0]),
         .Clk(Clk1),
         .Clk_n(Clk1_n),
         .Cke(Cke0),
         .Cs_n(S0_n),
         .Ras_n(Ras_n),
         .Cas_n(Cas_n),
         .We_n(We_n),
         .Dm(Dm[5:4])
       );
       
 ddr U4(
         .Dq(Dq[63:48]),
         .Dqs(Dqs[7:6]),
         .Addr(Addr[11:0]),
         .Ba(Ba[1:0]),
         .Clk(Clk1),
         .Clk_n(Clk1_n),
         .Cke(Cke0),
         .Cs_n(S0_n),
         .Ras_n(Ras_n),
         .Cas_n(Cas_n),
         .We_n(We_n),
         .Dm(Dm[7:6])
       );
       
 ddr U5(
         .Dq(Dq[15:0]),
         .Dqs(Dqs[1:0]),
         .Addr(Addr[11:0]),
         .Ba(Ba[1:0]),
         .Clk(Clk0),
         .Clk_n(Clk0_n),
         .Cke(Cke1),
         .Cs_n(S1_n),
         .Ras_n(Ras_n),
         .Cas_n(Cas_n),
         .We_n(We_n),
         .Dm(Dm[1:0])
       );
       
 ddr U6(
         .Dq(Dq[31:16]),
         .Dqs(Dqs[3:2]),
         .Addr(Addr[11:0]),
         .Ba(Ba[1:0]),
         .Clk(Clk0),
         .Clk_n(Clk0_n),
         .Cke(Cke1),
         .Cs_n(S1_n),
         .Ras_n(Ras_n),
         .Cas_n(Cas_n),
         .We_n(We_n),
         .Dm(Dm[3:2])
       );
       
 ddr U7(
         .Dq(Dq[47:32]),
         .Dqs(Dqs[5:4]),
         .Addr(Addr[11:0]),
         .Ba(Ba[1:0]),
         .Clk(Clk1),
         .Clk_n(Clk1_n),
         .Cke(Cke1),
         .Cs_n(S1_n),
         .Ras_n(Ras_n),
         .Cas_n(Cas_n),
         .We_n(We_n),
         .Dm(Dm[5:4])
       );
       
 ddr U8(
         .Dq(Dq[63:48]),
         .Dqs(Dqs[7:6]),
         .Addr(Addr[11:0]),
         .Ba(Ba[1:0]),
         .Clk(Clk1),
         .Clk_n(Clk1_n),
         .Cke(Cke1),
         .Cs_n(S1_n),
         .Ras_n(Ras_n),
         .Cas_n(Cas_n),
         .We_n(We_n),
         .Dm(Dm[7:6])
       );

       endmodule
