//*******************************************************************************
//
//  File name :       ddr_top.v
//
//  Description :     This module is the top level for the physical layer. 
// 
//                    
//  Date - revision : 3/03/2004
//                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2002 - 2005 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************

`timescale 1ns/100ps

module ddr_top (
            DDR_CLK90, 
            DDR_CLKDIV,
            CLK200_N,
            CLK200_P,
            USER_CLK,
            SYS_RESET_IN,
            DDR_DIMM_RAS_L, 
            DDR_DIMM_CAS_L, 
            DDR_DIMM_WE_L, 
            DDR_DIMM_CKE0,   
            DDR_DIMM_CKE1, 
            DDR_DIMM_CS_N0_7,  // S0_n
            DDR_DIMM_CS_N8_15, // S1_n
            DDR_DIMM_DQ, 
            DDR_DIMM_DQS, 
            DDR_DIMM_CLK0_P, 
            DDR_DIMM_CLK0_N, 
            DDR_DIMM_CLK1_P, 
            DDR_DIMM_CLK1_N, 
            DDR_DIMM_CLK2_P, 
            DDR_DIMM_CLK2_N, 
            DDR_DIMM_BA, 
            DDR_DIMM_ADDRESS, 
            DDR_DIMM_SA, 
            DDR_DIMM_SDA, 
            DDR_DIMM_SCL, 
            DDR_DIMM_DM,
            PCI_ADDR,
            PCI_DATA_IN,
            PCI_WRITE_VALID,
            PCI_CBEA,
            PCI_WRITE,
            PCI_HIT,
            PCI_HIT_EARLY,
            READ_DATA_OUT,
            READ_DATA_VALID,
            DDR_NOT_READY,
            DCM_LOCK,
            S_WAIT,
            S_DONE
            );
            

input         DDR_CLK90; 
input         DDR_CLKDIV;
input         CLK200_N;
input         CLK200_P;
input         USER_CLK;
input         SYS_RESET_IN;
output        DDR_DIMM_RAS_L;
output        DDR_DIMM_CAS_L;
output        DDR_DIMM_WE_L;
output        DDR_DIMM_CKE0;
output        DDR_DIMM_CKE1;
output        DDR_DIMM_CS_N0_7;
output        DDR_DIMM_CS_N8_15;
output [7:0]  DDR_DIMM_DM;
inout  [63:0] DDR_DIMM_DQ;
inout  [7:0]  DDR_DIMM_DQS;
output        DDR_DIMM_CLK0_P;
output        DDR_DIMM_CLK0_N;
output        DDR_DIMM_CLK1_P;
output        DDR_DIMM_CLK1_N;
output        DDR_DIMM_CLK2_P;
output        DDR_DIMM_CLK2_N;
output[1:0]   DDR_DIMM_BA;
output[11:0]  DDR_DIMM_ADDRESS;
output[2:0]   DDR_DIMM_SA;     
inout         DDR_DIMM_SDA;     
output        DDR_DIMM_SCL;     
input [26:0]  PCI_ADDR;
input [63:0]  PCI_DATA_IN;
input         PCI_WRITE_VALID;
input [7:0]   PCI_CBEA;
input         PCI_WRITE;
input         PCI_HIT;
input         PCI_HIT_EARLY;
output [63:0] READ_DATA_OUT;
output        READ_DATA_VALID;
output        DDR_NOT_READY;
input         DCM_LOCK;
output        S_WAIT;
input         S_DONE;



wire[3:0]     DDR_DIMM_READ_EN_OUT;
wire          sys_clk_in;
wire          clk200_in;
wire          clk_0;
wire          clk_90;
wire          clk_200;
reg           system_rst;
reg           system_rst90;
wire [127:0]  wr_df_data;
wire          dq_tap_sel_done;
wire          rd_data_valid;
wire [127:0]  rd_data_fifo_out;
wire [63:0]   rd_data_rise;
wire [63:0]   rd_data_fall;
wire [3:0]    rd_en_delayed_rise;
wire [3:0]    rd_en_delayed_fall;
wire [3:0]    rising_first;
wire [31:0]   app_af_addr;
wire          app_af_wren;
wire          addr_fifo_empty;
wire [127:0]  app_wr_df_data;
wire          app_wr_df_wren;
wire [31:0]   app_cmd_addr;
wire          app_cmd_wren;
wire          wr_df_almost_full;
wire          rd_af_almost_full; 
wire          ref_clk;
wire [26:0]   wr_af_addr;
wire [23:0]   rd_af_addr;
wire [1:0]    cmd_register;
wire          ctrl_wr_af_rden;
wire          ctrl_wr_df_rden;
wire          ctrl_rd_af_rden;
wire          ctrl_initz_done;
wire          ctrl_dummy_rden;
wire          ctrl_dqs_enable;
wire          ctrl_dqs_reset;
wire          ctrl_wr_en;
wire[1:0]     ddr_cs_L;
wire          cmd_fifo_empty;
wire [15:0]   dm_data_w;
wire          wdf_empty;
wire          burst_half;
wire [7:0]    pci_be;
wire          pci_hit_ddr;
wire          s_done_ddr;



reg           sys_rst_0;
reg           sys_rst_1;
reg           sys_rst_2;
reg           sys_rst_3;
reg           sys_rst;
reg           sys_rst90_0;
reg           sys_rst90_1;
reg           sys_rst90_2;
reg           sys_rst90_3;
reg           sys_rst90;


wire [3:0]   b_length;
wire         ddr_odt;
wire         ctrl_rden;
wire [3:0]   state;
wire [2:0]   pbs;
wire         not_ready;
wire         raf_empty;
wire         system_rst_async;
reg          system_rst_sync_d1;
reg          system_rst90_sync_d1;
reg          system_rst_90;
wire         first_read;
wire  [5:0]  tap_count;

assign b_length           = 4'b0100;
assign system_rst_async   = SYS_RESET_IN;
assign DDR_DIMM_CKE1      = DDR_DIMM_CKE0;  
assign DDR_DIMM_CS_N0_7   = ddr_cs_L[0];
assign DDR_DIMM_CS_N8_15  = ddr_cs_L[1];
assign DDR_DIMM_SDA       = 1'b0;
assign DDR_DIMM_SCL       = 1'b0;
assign DDR_DIMM_SA        = 3'b000;
assign DDR_NOT_READY      = not_ready;

always@(posedge sys_clk_in)begin
      system_rst_sync_d1 <= system_rst_async;
      system_rst <= system_rst_sync_d1;
end
always@(posedge clk_90)begin
      system_rst_90 <= system_rst;
end


always @ (posedge clk_0)
begin
  if ((system_rst == 1'b1) || (DCM_LOCK == 1'b0))
    begin
      sys_rst_0 <= 1'b1;
      sys_rst_1 <= 1'b1;
      sys_rst_2 <= 1'b1;
      sys_rst_3 <= 1'b1;      
      sys_rst   <= 1'b1;
    end
  else
    begin
      sys_rst_0 <= 1'b0;
      sys_rst_1 <= sys_rst_0;
      sys_rst_2 <= sys_rst_1;
      sys_rst_3 <= sys_rst_2;
      sys_rst   <= sys_rst_3;
    end
end

always @ (posedge clk_90)
begin
  if ((system_rst_90 == 1'b1) || (DCM_LOCK == 1'b0))
    begin
      sys_rst90_0 <= 1'b1;
      sys_rst90_1 <= 1'b1;
      sys_rst90_2 <= 1'b1;
      sys_rst90_3 <= 1'b1;
      sys_rst90   <= 1'b1;
    end
  else
    begin
      sys_rst90_0 <= 1'b0;
      sys_rst90_1 <= sys_rst90_0;
      sys_rst90_2 <= sys_rst90_1;
      sys_rst90_3 <= sys_rst90_2;
      sys_rst90   <= sys_rst90_3;
    end
end

assign sys_clk_in = USER_CLK;

                                      
IBUFGDS_LVPECL_25 lvpecl_clk200_in (
                                    .O(clk200_in), 
                                    .I(CLK200_P), 
                                    .IB(CLK200_N)
                                    );                                      
                                      
clk_module clk_module0 (
                        .SYS_CLK_IN(sys_clk_in),//CLK0 input
                        .DDR_CLK90(DDR_CLK90),//CLK90 input
                        .DDR_CLKDIV(DDR_CLKDIV),//CLKDIV input                   
                        .REF_CLK200_IN(clk200_in),
                        .CLK(clk_0),
                        .CLK90(clk_90),
                        .CLK200(clk_200),
                        .REFRESH_CLK(ref_clk),//CLK_DIV output
                        .DDR_DIMM_CLK0_P(DDR_DIMM_CLK0_P),
                        .DDR_DIMM_CLK0_N(DDR_DIMM_CLK0_N),
                        .DDR_DIMM_CLK1_P(DDR_DIMM_CLK1_P),
                        .DDR_DIMM_CLK1_N(DDR_DIMM_CLK1_N),
                        .DDR_DIMM_CLK2_P(DDR_DIMM_CLK2_P),
                        .DDR_DIMM_CLK2_N(DDR_DIMM_CLK2_N)             
                        );
            
data_path data_path0 (
                      .CLK(clk_0),
                      .CLK90(clk_90),
                      .CLK200(clk_200),
                      .RESET0(sys_rst),
                      .RESET90(sys_rst90),
                      .CTRL_DUMMYREAD_START(ctrl_dummy_rden),
                      .WDF_DATA(wr_df_data),
                      .DM_DATA(dm_data_w[15:0]),
                      .CTRL_WREN(ctrl_wr_en),
                      .CTRL_DQS_RST(ctrl_dqs_reset),
                      .CTRL_DQS_EN(ctrl_dqs_enable),
                      .DDR_DQ(DDR_DIMM_DQ),
                      .DDR_DQS(DDR_DIMM_DQS[7:0]),
                      .DDR_DM(DDR_DIMM_DM[7:0]),
                      .SEL_DONE(dq_tap_sel_done),
                      .READ_DATA_RISE(rd_data_rise),
                      .READ_DATA_FALL(rd_data_fall),
                      .FIRST_RISING(rising_first),
                      .test_tap(tap_count)
                      );

                  
user_interface user_interface0 (
                                .USER_CLK(USER_CLK),              
                                .DDR_CLK(clk_0),                  //in
                                .RESET(sys_rst),                  //in
                                 // read data fifo
                                .FIRST_RISING(rising_first),      //in
                                .READ_DATA_RISE(rd_data_rise),    //in
                                .READ_DATA_FALL(rd_data_fall),    //in
                                .READ_DATA_VALID(rd_data_valid),  //out
                                // address fifo                                                 
                                .CTRL_AF_RDEN(ctrl_wr_af_rden),   //in
                                .AF_ADDR(wr_af_addr),             //out
                                .AF_EMPTY(addr_fifo_empty),       //out                       
                                // write data fifo
                                .APP_WDF_DATA(PCI_DATA_IN),       //in
                                .APP_WDF_WREN(PCI_WRITE_VALID && PCI_WRITE),//in                          
                                .CTRL_WDF_RDEN(ctrl_wr_df_rden),  //in
                                .WDF_DATA(wr_df_data),            //out
                                .WDF_ALMOST_FULL(wr_df_almost_full),//out
                                .WDF_EMPTY(wdf_empty),            //out
                                // command fifo 
                                .CTRL_CMD_RDEN(ctrl_rd_af_rden),  //in
                                .CMD_ADDR(rd_af_addr),            //out
                                .CMD_EMPTY(cmd_fifo_empty),       //out
                                // PCI SIGNALS
                                .PCI_ADDR(PCI_ADDR),              //in
                                .PCI_DATA_IN(PCI_DATA_IN),        //in
                                .PCI_CBEA(PCI_CBEA),              //in
                                .PCI_WRITE(PCI_WRITE),            //in
                                .PCI_HIT(PCI_HIT),                //in
                                .PCI_HIT_DDR(pci_hit_ddr),        //out
                                .PCI_HIT_EARLY(PCI_HIT_EARLY),    //in
                                .PCI_BE(pci_be[7:0]),             //out
                                .READ_DATA_OUT(READ_DATA_OUT),    //out                        
                                .BURST_HALF(burst_half),          //out
                                .FIRST_READ(first_read),          //in
                                .S_WAIT(S_WAIT),                  //out
                                .NOT_READY(not_ready),            //in   
                                .S_DONE(S_DONE),                   //in
                                .S_DONE_DDR(s_done_ddr)          //out
                                 );
 

   ddr_controller ddr_controller0 (
                                     .clk266(clk_0),             //in
                                     .refresh_clk(ref_clk),      //in
                                     .rst(sys_rst),              //in
                                     .addr_fifo(wr_af_addr),     //in
                                     .cmd_fifo(rd_af_addr),      //in
                                     .phy_Dly_Slct_Done(dq_tap_sel_done), //in
                                     .addr_fifo_empty(addr_fifo_empty),   //in
                                     .cmd_fifo_empty(cmd_fifo_empty),     //in
                                     .ctrl_Dummyread_Start(ctrl_dummy_rden),//out
                                     .ctrl_af_RdEn(ctrl_wr_af_rden),  //out
                                     .ctrl_Wdf_RdEn(ctrl_wr_df_rden), //out
                                     .ctrl_cmd_RdEn(ctrl_rd_af_rden), //out
                                     .wdf_empty(wdf_empty),           //in
                                     .ctrl_init_done(ctrl_initz_done),//out
                                     .ctrl_Dqs_Rst(ctrl_dqs_reset),   //out
                                     .ctrl_Dqs_En(ctrl_dqs_enable),   //out
                                     .ctrl_Dm_En(dm_data_w[15:0]),    //out
                                     .ctrl_WrEn(ctrl_wr_en),          //out
                                     .ctrl_RdEn(ctrl_rden),           //out
                                     .ddr_address(DDR_DIMM_ADDRESS),  //out
                                     .ddr_ba(DDR_DIMM_BA),            //out
                                     .ddr_ras_L(DDR_DIMM_RAS_L),      //out
                                     .ddr_cas_L(DDR_DIMM_CAS_L),      //out
                                     .ddr_we_L(DDR_DIMM_WE_L),        //out
                                     .ddr_cs_L(ddr_cs_L),             //out
                                     .ddr_cke(DDR_DIMM_CKE0),         //out
                                     .ddr_odt(ddr_odt),               //out
                                     .not_ready(not_ready),           //out
                                     .burst_half(burst_half),         //in
                                     .PCI_HIT(pci_hit_ddr),           //in
                                     .PCI_HIT_EARLY(PCI_HIT_EARLY),   //in //remove or sync this signal for diff. clock domain design
                                     .PCI_BE(pci_be[7:0]),            //in
                                     .PCI_ADDR_EARLY(PCI_ADDR),       //in   //remove or sync this signal for diff. clock domain design
                                     .PCI_CBEA_EARLY(PCI_CBEA),       //in   //remove or sync this signal for diff. clock domain design
                                     .first_read(first_read),         //out
                                     .S_DONE(s_done_ddr)                //in
                                     );

  				     
endmodule 

    
