//*******************************************************************************
//
//  File name :       rd_data_fifo.v
//
//  Description :     This module instantiates the FIFO16 primitive for read  
//                    data. One FIFO for rising edge data and the other for
//                    falling edge data.
//                    the read data has been assigned to FIFOs based on the proximity of 
//                    the data bytes to each other
//
// 
//                    
//  Date - revision : 6/23/2005
//
//  Author :          Kraig Lund
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2002 - 2005 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************

`timescale 1ns/100ps

module rd_data_fifo (
                     USER_CLK,
                     DDR_CLK,
                     RESET,
                     FIRST_RISING,
                     READ_DATA_RISE,
                     READ_DATA_FALL,
                     READ_DATA_OUT,
                     READ_DATA_VALID,
                     FIRST_READ
                     );
                     
   input          USER_CLK;
   input          DDR_CLK;
   input          RESET;
   input [3:0]    FIRST_RISING;
   input [63:0]   READ_DATA_RISE;
   input [63:0]   READ_DATA_FALL;
   input          FIRST_READ;
   
   output         READ_DATA_VALID;
   output [63:0]  READ_DATA_OUT;


reg [63:0] read_data_out_mux, READ_DATA_OUT;
reg [63:0] rise_data_r1, rise_data_r2, fall_data_r1;
reg [1:0] com_cnt;
wire com_sync;

assign com_sync = FIRST_READ;

always@(posedge DDR_CLK)begin
     rise_data_r1 <= READ_DATA_RISE;
     fall_data_r1 <= READ_DATA_FALL;
     rise_data_r2 <= rise_data_r1;
end 


always@(posedge DDR_CLK)begin
     if(RESET)begin
        com_cnt <= 0;
     end else if(com_sync)begin
        com_cnt <= 0;
     end else begin
        com_cnt <= com_cnt + 1;
     end
end

always@(READ_DATA_RISE, READ_DATA_FALL, fall_data_r1, 
        rise_data_r1, com_cnt)begin
     case(com_cnt)
         2'b00: read_data_out_mux = READ_DATA_RISE; //READ_DATA_FALL;
         2'b01: read_data_out_mux = READ_DATA_FALL; //rise_data_r1;
         2'b10: read_data_out_mux = rise_data_r1;   //fall_data_r1;
         2'b11: read_data_out_mux = fall_data_r1;   //rise_data_r2;
     endcase
end

always@(posedge DDR_CLK)begin
     if(RESET)begin
        READ_DATA_OUT <= 0;
     end else begin
        READ_DATA_OUT <= read_data_out_mux;
     end
end
 
endmodule

