//*******************************************************************************
//
//  File name :       user_interface.v
//
//  Description :     This module instantiates the FIFO16 primitive for read  
//                    data. One FIFO for rising edge data and the other for
//                    falling edge data.
// 
//                    
//  Date - revision : 3/24/2004
//                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
//
//  Author :          Maria George
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2002 - 2005 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************

`timescale 1ns/100ps

module user_interface (
                       USER_CLK,
                       DDR_CLK,
                       RESET,
                       FIRST_RISING,
                       READ_DATA_RISE,
                       READ_DATA_FALL,
                       READ_DATA_VALID,
                       CTRL_AF_RDEN,
                       AF_ADDR,
                       AF_EMPTY,
                       APP_WDF_DATA,
                       APP_WDF_WREN,
                       CTRL_WDF_RDEN,
                       WDF_DATA,
                       WDF_ALMOST_FULL,
                       WDF_EMPTY,
                       CTRL_CMD_RDEN,
                       CMD_ADDR,
                       CMD_EMPTY,
                       PCI_ADDR,
                       PCI_DATA_IN,
                       PCI_CBEA,
                       PCI_WRITE,
                       PCI_HIT,
                       PCI_HIT_DDR,
                       PCI_HIT_EARLY,
                       PCI_BE,
                       READ_DATA_OUT,
                       BURST_HALF,
                       FIRST_READ,
                       S_WAIT,
                       NOT_READY,
                       S_DONE,
                       S_DONE_DDR                     
                       );
                     
   input          USER_CLK;
   input          DDR_CLK;
   input          RESET;
   input [3:0]    FIRST_RISING;
   input [63:0]   READ_DATA_RISE;
   input [63:0]   READ_DATA_FALL;
   output         READ_DATA_VALID; 
   input          CTRL_AF_RDEN;  
   output [26:0]  AF_ADDR; 
   output         AF_EMPTY;      
   input [63:0]  APP_WDF_DATA;  
   input          APP_WDF_WREN;   
   input          CTRL_WDF_RDEN;  
   output [127:0] WDF_DATA;      
   output         WDF_ALMOST_FULL;
   output         WDF_EMPTY; 
   input          CTRL_CMD_RDEN;  
   output [23:0]  CMD_ADDR;  
   output         CMD_EMPTY;      
   input  [26:0]  PCI_ADDR;
   input  [63:0]  PCI_DATA_IN;
   input  [7:0]   PCI_CBEA;
   input            PCI_WRITE;
   input            PCI_HIT;
   output         PCI_HIT_DDR;
   input          PCI_HIT_EARLY;
   output [7:0]   PCI_BE;
   output [63:0]  READ_DATA_OUT;
   output         BURST_HALF;
   input          FIRST_READ; 
   output         S_WAIT;
   input          NOT_READY;
   input          S_DONE;
   output         S_DONE_DDR;

   wire           wr_af_empty;
   wire           rd_af_empty;
   wire [127:0]   wr_df_output;
   wire [11:0]    byte_count;   
   reg [26:0]     pci_addr_attr;  
   reg [26:0]     pci_addr_addr;
   reg [7:0]      pci_cbea_attr;
   reg [7:0]      pci_cbea_addr;
   reg [7:0]      PCI_BE;
   reg [63:0]     pci_data_in_r;
   reg            app_wdf_wren_r; 
   wire           app_wdf_wren_one;
   reg            steerer;
   reg [127:0]    pci_data_in_128;
   reg            pci_hit_early_r1; 
   reg            app_wdf_wren_one_r;
   reg [63:0]     pci_data_in_r2;    
   reg [26:0]     start_addr;
   reg [11:0]     byte_count_r; 
   //burst count calculation logic 
   reg [6:0]      burst_count;
   wire[26:0]     end_addr;
   wire[6:0]      trunc_end_addr;
   wire[6:0]      trunc_start_addr;
   reg [6:0]      burst_count_half;
   reg [3:0]      wait_cnt;
	reg [2:0]      blocker;

   assign WDF_DATA[127:0]  = wr_df_output[127:0];

   //following assigns would be sync registers for multi-clock design
    assign PCI_HIT_DDR = PCI_HIT;
   assign S_DONE_DDR = S_DONE;

   rd_data_fifo rd_data_fifo0 (
                               .USER_CLK(USER_CLK),
                               .DDR_CLK(DDR_CLK),
                               .RESET(RESET),
                               .FIRST_RISING(FIRST_RISING),
                               .READ_DATA_RISE(READ_DATA_RISE),
                               .READ_DATA_FALL(READ_DATA_FALL),
                               .READ_DATA_OUT(READ_DATA_OUT),
                               .READ_DATA_VALID(READ_DATA_VALID),
                               .FIRST_READ(FIRST_READ)
                               );
  
   always@(posedge USER_CLK)begin
      if(RESET)begin
       pci_hit_early_r1 <= 1'b0;
      end else begin
       pci_hit_early_r1 <= PCI_HIT_EARLY;
      end
   end
                               
   backend_fifos backend_fifos0 (
                                 .USER_CLK(USER_CLK),
                                 .DDR_CLK(DDR_CLK),
                                 .rst(RESET),
                                 //AF
                                 .app_af_addr({5'b00000,pci_addr_attr[26:0]}),
                                 .app_af_WrEn(pci_hit_early_r1 && !NOT_READY),
                                 .ctrl_af_RdEn(CTRL_AF_RDEN && !NOT_READY),
                                 .af_addr(AF_ADDR),
                                 .af_Almost_Empty(),
                                 .af_empty(AF_EMPTY),
                                 //WDF
                                 .app_Wdf_data(pci_data_in_128[127:0]),
                                 .app_Wdf_WrEn(burst_count[0]),
                                 .ctrl_Wdf_RdEn(CTRL_WDF_RDEN),
                                 .Wdf_data(wr_df_output),
                                 .Wdf_Almost_Full(WDF_ALMOST_FULL),
                                 .Wdf_Empty(WDF_EMPTY),
                                 //CMD
                                 .app_cmd_addr({7'b0000000, 
                                                pci_cbea_addr[3:0],
                                                byte_count[11:0],
                                                pci_cbea_attr[7:0]}),
                                 .app_cmd_WrEn(PCI_HIT),
                                 .ctrl_cmd_RdEn(CTRL_CMD_RDEN),
                                 .cmd_addr(CMD_ADDR),
                                 .cmd_Almost_Empty(rd_af_empty),
                                 .cmd_empty(CMD_EMPTY), 
                                 .cmd_Almost_Full()
                                 );

    //register PCI_ADDR and PCI_CBEA twice
    //When S_HIT is high, the first register will
    //contain the attribute phase and the second
    //will contain the address phase
     always @(posedge USER_CLK) begin
         pci_addr_attr[26:0] <= PCI_ADDR[26:0];  
         pci_addr_addr[26:0] <= pci_addr_attr[26:0];
         pci_cbea_attr[7:0] <= PCI_CBEA[7:0];
         pci_cbea_addr[7:0] <= pci_cbea_attr[7:0];
     end
    //for burst read/write the byte count is given as 
    //PCI_CBEA[3:0] and PCI_ADDR[7:0] during the PCI ATTR Phase
    assign byte_count[11:0] = {pci_cbea_attr[3:0],pci_addr_attr[7:0]};

     always @(posedge USER_CLK) begin
        if(RESET)
         PCI_BE[7:0] <= 8'h00;
        else if(APP_WDF_WREN)
         PCI_BE[7:0] <= PCI_CBEA[7:0]; 
        else 
         PCI_BE[7:0] <= PCI_BE[7:0]; 
     end

    //byte-steering logic for wd_fifo
    always @(posedge USER_CLK) begin
        pci_data_in_r[63:0] <= PCI_DATA_IN[63:0];
        pci_data_in_r2[63:0] <= pci_data_in_r[63:0];
        app_wdf_wren_r <= APP_WDF_WREN;
        app_wdf_wren_one_r <= app_wdf_wren_one;
    end

  // rising edge detector
   assign app_wdf_wren_one = APP_WDF_WREN & ~app_wdf_wren_r;

   always@(posedge USER_CLK)begin
     if (RESET)begin
        start_addr[26:0] <= 27'h0000000;
        byte_count_r[11:0] <= 12'h000;
     end   else if(PCI_HIT) begin
        start_addr[26:0] <= pci_addr_addr[26:0];
        byte_count_r[11:0] <= byte_count[11:0];
     end
   end

   assign end_addr[26:0] = start_addr[26:0] + byte_count_r[11:0] - 4;
   assign trunc_end_addr[6:0] = end_addr[10:4];
   assign trunc_start_addr[6:0] = start_addr[10:4];

// calculate the total number of 16 byte transfers
// multiply by 2 and use bit 0 as a write enable to the fifo
   always@(posedge USER_CLK)begin
     if (RESET)begin
        burst_count[6:0] <= 7'b0000000;
     end   else if(app_wdf_wren_one) begin
        burst_count[6:0] <= ((trunc_end_addr - trunc_start_addr )<<1) + 1;
     end   else if(burst_count != 0) begin
        burst_count <= burst_count - 1;
     end
   end

// the following code 
// purpose is to let the write data fifo fill up with 1/2 of the burst 
// before letting
// the ddr controller to start the burst because of DDR:SDR clock ratio 

   always@(posedge USER_CLK)begin
     if (RESET)begin
        burst_count_half[6:0] <= 7'b0000000;
     end   else if(app_wdf_wren_one) begin
        burst_count_half[6:0] <= (((trunc_end_addr - trunc_start_addr )<<1) + 1) >> 1;
     end
   end

 // blocker is used to "block" burst half during the first 5 cycles after PCI_HIT_EARLY
 // blocker not necessary in a design that doens't use PCI_HIT_EARLY  
 // The problem is that the DDR controller state machine is already in the active
 // state by the time it sees PCI_HIT; Since BURST_HALF may still be asserted
 // from the previous transaction the controller goes ahead and starts the
 // burst early.
   always @(posedge USER_CLK)begin
      if (RESET)begin
         blocker <= 0;
      end else if (PCI_HIT_EARLY)begin
            blocker <= 3'b110;
         end else if(blocker != 0)begin
            blocker <= blocker - 1;
         end else begin
			   blocker <= blocker;
         end
   end

   assign BURST_HALF = ((burst_count_half >= burst_count) && (blocker == 0)) ? 1'b1 : 1'b0;


     always @(posedge USER_CLK) begin
        if (RESET) begin
           steerer <= 1'b0;
        end else if(PCI_HIT)begin
             steerer <= pci_addr_addr[3];    
        end
     end

    always@(steerer or PCI_DATA_IN[63:0] or 
            pci_data_in_r[63:0] or pci_data_in_r2[63:0]) begin
        if(steerer)begin
           pci_data_in_128 = {pci_data_in_r2[31:0],pci_data_in_r2[63:32],
                              pci_data_in_r[31:0],pci_data_in_r[63:32]};
        end else begin
           pci_data_in_128 = {pci_data_in_r[31:0],pci_data_in_r[63:32],
                              PCI_DATA_IN[31:0],PCI_DATA_IN[63:32]};
        end
    end

/*************************************************************************
   Counter machine to drive S_WAIT   
**************************************************************************/


assign S_WAIT = (wait_cnt != 0) || PCI_HIT;
always@(posedge USER_CLK)begin
   if(RESET)begin
      wait_cnt <= 0;
   end else if(PCI_HIT == 1'b1 && PCI_WRITE == 1'b0 && !NOT_READY)begin
      case(pci_addr_addr[4:3])
         2'b00: wait_cnt <= 6;
         2'b01: wait_cnt <= 7;
         2'b10: wait_cnt <= 8;
         2'b11: wait_cnt <= 9;     
      endcase
   end else if(wait_cnt != 0)begin
         wait_cnt <= wait_cnt - 1;
   end else begin
         wait_cnt <= 0;
    end
end

endmodule                                 
