//*******************************************************************************
//
//  File name :       v4_dm_iob.v
//
//  Description :     This module instantiates the IDELAY primitive and IOB  
//                    flip-flops for the data mask I/Os.
// 
//                    
//  Date - revision : 1/23/2004
//
//  Author :          Kraig Lund
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2002 - 2005 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************
   

`timescale 1ns/100ps

module v4_dm_iob (
                  CLK90,
                  DM_RISE,
                  DM_FALL,
                  DDR_DM
                    );

   input        CLK90;               
   input        DM_RISE;
   input        DM_FALL;
   output       DDR_DM;
  
   wire         dm_out;
   wire         vcc;
   wire         gnd;
   wire         dm_rise_r;
   wire         dm_fall_r;
      
FD wrdata_rise_reg_dimm (
                         .Q(dm_rise_r),
                         .C(CLK90),
                         .D(DM_RISE)
                         ); 
                  
FD wrdata_fall_reg_dimm (
                         .Q(dm_fall_r),
                         .C(CLK90),
                         .D(DM_FALL)
                         );   
   
//synthesis translate_off
defparam oddr_dm.SRTYPE = "SYNC";
defparam oddr_dm.DDR_CLK_EDGE = "SAME_EDGE";
//synthesis translate_on

//synthesis attribute SRTYPE of oddr_dm is "SYNC"
//synthesis attribute DDR_CLK_EDGE of oddr_dm is "SAME_EDGE"    
ODDR oddr_dm (
               .Q(dm_out),
               .C(CLK90),
               .CE(1'b1),
               .D1(dm_rise_r),
               .D2(dm_fall_r),
               .R(1'b0),
               .S(1'b0)
               )/* synthesis xc_props="SRTYPE = SYNC,DDR_CLK_EDGE = SAME_EDGE"*/;
                                    
OBUF  obuf_dm (
                   .I(dm_out),
                   .O(DDR_DM)
                   );                                                                          
endmodule 