//*******************************************************************************
//
//  File name :       v4_dq_iob.v
//
//  Description :     This module instantiates the IDELAY primitive and IOB  
//                    flip-flops for the bi-directional data.
// 
//                    
//  Date - revision : 1/23/2004
//                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
//
//  Author :          Maria George
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2002 - 2005 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************
   

`timescale 1ns/100ps

module v4_dq_iob (
                  CLK,
                  CLK90,
                  RESET,
                  DATA_DLYINC, 
                  DATA_DLYCE,  
                  DATA_DLYRST,
                  WRITE_DATA_RISE,
                  WRITE_DATA_FALL,
                  CTRL_WREN,
                  DDR_DQ,
                  READ_DATA_RISE,
                  READ_DATA_FALL 
                  );

   input        CLK;
   input        CLK90;               
   input        RESET;
   input        DATA_DLYINC;
   input        DATA_DLYCE;
   input        DATA_DLYRST;
   input        WRITE_DATA_RISE;
   input        WRITE_DATA_FALL;
   input        CTRL_WREN;  
   inout        DDR_DQ;
   output       READ_DATA_RISE;
   output       READ_DATA_FALL;
   
   wire         dq_in;
   wire         dq_out;
   wire         dq_delayed;
   wire         write_en_L;
   wire         write_en_L_r1;
   wire         vcc;
   wire         gnd;
   wire         rd_data_rise;
   wire         rd_data_fall;
   wire         wr_data_rise;
   wire         wr_data_fall;
   wire         write_en_L_r;
  
   assign vcc        = 1'b1;
   assign gnd        = 1'b0;

   assign write_en_L = ~CTRL_WREN;
   
FD wrdata_rise_reg_dimm (
                         .Q(wr_data_rise),
                         .C(CLK90),
                         .D(WRITE_DATA_RISE)
                         ); 
                  
FD wrdata_fall_reg_dimm (
                         .Q(wr_data_fall),
                         .C(CLK90),
                         .D(WRITE_DATA_FALL)
                         );   
   
//synthesis translate_off
defparam oddr_dq.SRTYPE = "SYNC";
defparam oddr_dq.DDR_CLK_EDGE = "SAME_EDGE";
//synthesis translate_on

//synthesis attribute SRTYPE of oddr_dq is "SYNC"
//synthesis attribute DDR_CLK_EDGE of oddr_dq is "SAME_EDGE"    

ODDR oddr_dq (
               .Q(dq_out),
               .C(CLK90),
               .CE(vcc),
               .D1(wr_data_rise),
               .D2(wr_data_fall),
               .R(gnd),
               .S(gnd)
               )/* synthesis xc_props="SRTYPE = SYNC,DDR_CLK_EDGE = SAME_EDGE"*/;
               
FD ctrl_wren_reg_dimm (
                         .Q(write_en_L_r),
                         .C(CLK90),
                         .D(write_en_L)
                         );               

//synthesis attribute IOB of tri_state_dq is "TRUE"
FDCE tri_state_dq (
                   .D(write_en_L_r), 
                   .CLR(RESET), 
                   .C(CLK90), 
                   .Q(write_en_L_r1), 
                   .CE(vcc)
                  );            
            
IOBUF  iobuf_dq (
                   .I(dq_out),
                   .T(write_en_L_r1),
                   .IO(DDR_DQ),
                   .O(dq_in)
                   );                    
                   
//synthesis translate_off
defparam idelay_dq.IOBDELAY_TYPE = "VARIABLE";
defparam idelay_dq.IOBDELAY_VALUE = 0;
//synthesis translate_on

//synthesis attribute IOBDELAY_TYPE of idelay_dq is "VARIABLE"
//synthesis attribute IOBDELAY_VALUE of idelay_dq is "0"

IDELAY idelay_dq (
                   .O(dq_delayed),
                   .I(dq_in),
                   .C(CLK),   
                   .CE(DATA_DLYCE),
                   .INC(DATA_DLYINC),
                   .RST(DATA_DLYRST)
                   )/* synthesis xc_props="IOBDELAY_TYPE = VARIABLE,IOBDELAY_VALUE = 0"*/;

//synthesis translate_off
defparam iddr_dq.SRTYPE = "SYNC";
defparam iddr_dq.DDR_CLK_EDGE = "SAME_EDGE";   
//synthesis translate_on

//synthesis attribute SRTYPE of iddr_dq is "SYNC"
//synthesis attribute DDR_CLK_EDGE of iddr_dq is "SAME_EDGE"

IDDR iddr_dq (
              .Q1(READ_DATA_RISE),
              .Q2(READ_DATA_FALL),
              .C(CLK),
              .CE(vcc),           
              .D(dq_delayed),
              .R(gnd),
              .S(gnd)
              )/* synthesis xc_props="SRTYPE = SYNC,DDR_CLK_EDGE = SAME_EDGE"*/;
              
                                  
endmodule 