//*******************************************************************************
//
//  File name :       v4_dqs_iob.v
//
//  Description :     This module instantiates the IDELAY primitive and IOB  
//                    flip-flops for the bi-directional strobe.
// 
//                    
//  Date - revision : 1/23/2004
//                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
//
//  Author :          Maria George
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778 
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2002 - 2005 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************

`timescale 1ns/100ps

module v4_dqs_iob (
                   CLK,
                   DLYINC, 
                   DLYCE,  
                   DLYRST,
                   CTRL_DQS_RST,
                   CTRL_DQS_EN,
                   DDR_DQS,
                   DQS 
                    );

   input        CLK;               
   input        DLYINC;
   input        DLYCE;
   input        DLYRST;
   input        CTRL_DQS_RST;
   input        CTRL_DQS_EN;   
   inout        DDR_DQS;   
   output       DQS;
   
   wire         dqs_in;
   wire         dqs_out;
   wire         ctrl_dqs_rst_r;
   wire         ctrl_dqs_en_r;
   wire         ctrl_dqs_en_r1;
   wire         ctrl_dqsl_en_r1;
   wire         vcc;
   wire         gnd;
   wire         clk180;
   reg          data1;
   reg          data2;
   
   
   assign vcc         = 1'b1;
   assign gnd         = 1'b0;
   assign clk180      = ~CLK;   


always @ (negedge CLK)
begin
  if (ctrl_dqs_rst_r == 1'b1)     
    data1 <= 1'b0;
  else
    data1 <= 1'b1;
end

always @ (negedge CLK)
begin
  if (ctrl_dqs_rst_r == 1'b1)
    data2 <= 1'b1;
  else
    data2 <= 1'b0;
end

FD dqs_rst_reg_dimm (
                     .D(CTRL_DQS_RST),
                     .Q(ctrl_dqs_rst_r),
                     .C(clk180)
                     );
                     
FD dqs_en_reg_dimm (
                     .D(CTRL_DQS_EN),
                     .Q(ctrl_dqs_en_r),
                     .C(clk180)
                     );                        

//synthesis translate_off
defparam idelay_dqs.IOBDELAY_TYPE = "VARIABLE";
defparam idelay_dqs.IOBDELAY_VALUE = 0;
//synthesis translate_on

//synthesis attribute IOBDELAY_TYPE of idelay_dqs is "VARIABLE"
//synthesis attribute IOBDELAY_VALUE of idelay_dqs is "0"

IDELAY idelay_dqs (
                   .O(DQS),
                   .I(dqs_in),
                   .C(CLK),   
                   .CE(DLYCE),
                   .INC(DLYINC),
                   .RST(DLYRST)
                   )/* synthesis xc_props="IOBDELAY_TYPE = VARIABLE,IOBDELAY_VALUE = 0"*/;
                   
//synthesis translate_off
defparam oddr_dqs.SRTYPE = "SYNC";
defparam oddr_dqs.DDR_CLK_EDGE = "OPPOSITE_EDGE";
//synthesis translate_on

//synthesis attribute SRTYPE of oddr_dqs is "SYNC"
//synthesis attribute DDR_CLK_EDGE of oddr_dqs is "OPPOSITE_EDGE"

ODDR oddr_dqs (
               .Q(dqs_out),
               .C(clk180),
               .CE(vcc),
               .D1(data1),
               .D2(gnd),
               .R(gnd),
               .S(gnd)
               )/* synthesis xc_props="SRTYPE = SYNC,DDR_CLK_EDGE = OPPOSITE_EDGE"*/;

//synthesis attribute IOB of tri_state_dqs is "TRUE"               
FD tri_state_dqs (
                  .D(ctrl_dqs_en_r),
                  .Q(ctrl_dqs_en_r1),
                  .C(clk180)
                  );
                                                    
IOBUF  iobuf_dqs (
                   .I(dqs_out),
                   .T(ctrl_dqs_en_r1),
                   .IO(DDR_DQS),
                   .O(dqs_in)
                   );            
                   
endmodule   