--*******************************************************************************
-- *******************************************************************************
-- **
-- **  Xilinx, Inc. 2005                 www.xilinx.com
-- **
-- **  Synthesizable DDR SDRAM Controller
-- **
-- *********************************************************************************
-- **
-- **  File name :   backend_fifos.vhd
-- **
-- **  Description     : This module instantiates the backend fifos required for the 
-- **                       ddr interface. This includes the write address and data fifos,
-- **                       and the read address and data fifos. The FIFOs are implemented
-- **                       using the Virtex-4 block rams which can also be used as FIFOs
-- **                        
-- **
-- **  Date - revision : 11/10/2005 modified for use in XAPP708 - Kraig Lund
-- **
-- **  Author :      Lakshmi Gopalakrishnan
-- **
-- **  Contact :     e-mail  hotline@xilinx.com
-- **                phone   + 1 800 255 7778 
-- **
-- **  Disclaimer:   LIMITED WARRANTY AND DISCLAMER. These designs are 
-- **                provided to you "as is". Xilinx and its licensors make and you 
-- **                receive no warranties or conditions, express, implied, 
-- **                statutory or otherwise, and Xilinx specifically disclaims any 
-- **                implied warranties of merchantability, non-infringement, or 
-- **                fitness for a particular purpose. Xilinx does not warrant that 
-- **                the functions contained in these designs will meet your 
-- **                requirements, or that the operation of these designs will be 
-- **                uninterrupted or error free, or that defects in the Designs 
-- **                will be corrected. Furthermore, Xilinx does not warrant or 
-- **                make any representations regarding use or the results of the 
-- **                use of the designs in terms of correctness, accuracy, 
-- **                reliability, or otherwise. 
-- **
-- **                LIMITATION OF LIABILITY. In no event will Xilinx or its 
-- **                licensors be liable for any loss of data, lost profits, cost 
-- **                or procurement of substitute goods or services, or for any 
-- **                special, incidental, consequential, or indirect damages 
-- **                arising from the use or operation of the designs or 
-- **                accompanying documentation, however caused and on any theory 
-- **                of liability. This limitation will apply even if Xilinx 
-- **                has been advised of the possibility of such damage. This 
-- **                limitation shall apply not-withstanding the failure of the 
-- **                essential purpose of any limited remedies herein. 
-- **
-- **  Copyright  2002 - 2005 Xilinx, Inc.
-- **  All rights reserved 
-- ** 
-- *****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY backend_fifos IS
   PORT (
      USER_CLK                : IN std_logic;   
      DDR_CLK                 : IN std_logic;   
      rst                     : IN std_logic;   
      app_af_addr             : IN std_logic_vector(31 DOWNTO 0);   
      app_af_WrEn             : IN std_logic;   
      ctrl_af_RdEn            : IN std_logic;   
      af_addr                 : OUT std_logic_vector(26 DOWNTO 0);   
      af_Almost_Empty         : OUT std_logic;   
      af_empty                : OUT std_logic;   
      --wdf signals

      app_Wdf_data            : IN std_logic_vector(127 DOWNTO 0);   
      app_Wdf_WrEn            : IN std_logic;   
      ctrl_Wdf_RdEn           : IN std_logic;   
      Wdf_data                : OUT std_logic_vector(127 DOWNTO 0);   
      Wdf_Empty               : OUT std_logic;   
      Wdf_Almost_Full         : OUT std_logic;   
      --cmd signals

      app_cmd_addr            : IN std_logic_vector(31 DOWNTO 0);   
      app_cmd_WrEn            : IN std_logic;   
      ctrl_cmd_RdEn           : IN std_logic;   
      cmd_addr                : OUT std_logic_vector(23 DOWNTO 0);   
      cmd_Almost_Empty        : OUT std_logic;   
      cmd_empty               : OUT std_logic;   
      cmd_Almost_Full         : OUT std_logic);   
END backend_fifos;

ARCHITECTURE rtl OF backend_fifos IS

component coregen_wr_fifo
	port (
	din: IN std_logic_VECTOR(31 downto 0);
	rd_clk: IN std_logic;
	rd_en: IN std_logic;
	rst: IN std_logic;
	wr_clk: IN std_logic;
	wr_en: IN std_logic;
	dout: OUT std_logic_VECTOR(31 downto 0);
	empty: OUT std_logic;
	full: OUT std_logic);
end component;

   SIGNAL fifo_input_write_addr    :  std_logic_vector(31 DOWNTO 0);   
   SIGNAL fifo_input_read_addr     :  std_logic_vector(31 DOWNTO 0);   
   SIGNAL addr_fifo_output         :  std_logic_vector(31 DOWNTO 0);   
   SIGNAL cmd_fifo_output          :  std_logic_vector(31 DOWNTO 0);   
   SIGNAL wr_df_almost_full_1      :  std_logic;   
   SIGNAL wr_df_almost_full_2      :  std_logic;   
   SIGNAL wr_df_almost_full_3      :  std_logic;   
   SIGNAL wr_df_almost_full_4      :  std_logic;   
   SIGNAL wr_df_almost_full_5      :  std_logic;   
   SIGNAL wr_df_almost_full_6      :  std_logic;   
   SIGNAL wr_df_almost_full_7      :  std_logic;   
   SIGNAL wr_df_almost_full_8      :  std_logic;   
   SIGNAL wr_df_almost_full        :  std_logic;   
   SIGNAL wr_af_almost_empty       :  std_logic;   
   SIGNAL wr_af_rderr              :  std_logic;   
   SIGNAL rd_af_almost_empty       :  std_logic;   
   SIGNAL rd_af_rderr              :  std_logic;   
   SIGNAL rd_af_almost_full        :  std_logic;   
   SIGNAL rd_af_wrerr              :  std_logic;   
   SIGNAL wr_af_almost_full        :  std_logic;   
   SIGNAL wr_df_wrerr_1            :  std_logic;   
   SIGNAL wr_df_wrerr_2            :  std_logic;   
   SIGNAL wr_df_wrerr_3            :  std_logic;   
   SIGNAL wr_df_wrerr_4            :  std_logic;   
   SIGNAL wr_df_wrerr_5            :  std_logic;   
   SIGNAL wr_df_wrerr_6            :  std_logic;   
   SIGNAL wr_df_wrerr_7            :  std_logic;   
   SIGNAL wr_df_wrerr_8            :  std_logic;   
   SIGNAL wr_df_wrerr              :  std_logic;   
   SIGNAL test_wr_pointer          :  std_logic_vector(11 DOWNTO 0);   
   SIGNAL test_rd_pointer          :  std_logic_vector(11 DOWNTO 0);   
   SIGNAL wrcount_1                :  std_logic_vector(11 DOWNTO 0);   
   SIGNAL rdcount_1                :  std_logic_vector(11 DOWNTO 0);   
    

BEGIN
   af_addr <= addr_fifo_output(26 DOWNTO 0) ;
   af_Almost_Empty <= wr_af_almost_empty OR wr_af_rderr ;
   Wdf_Almost_Full <= wr_df_almost_full OR wr_df_wrerr ;
                      wr_df_almost_full <= ((wr_df_almost_full_1 AND wr_df_almost_full_2) 
                      AND wr_df_almost_full_3) AND wr_df_almost_full_4 ;
   cmd_addr <= cmd_fifo_output(23 DOWNTO 0) ;
   cmd_Almost_Empty <= rd_af_almost_empty OR rd_af_rderr ;
   cmd_Almost_Full <= rd_af_almost_full OR rd_af_wrerr ;
   wr_df_wrerr <= ((wr_df_wrerr_1 AND wr_df_wrerr_2) AND wr_df_wrerr_3) 
   AND wr_df_wrerr_4 ;

   -- Address and Command fifos replaced with registers   
   -- necessary for 133 PCI-X in order to meet 16 clock latency spec for reads
   
   PROCESS (USER_CLK)
   BEGIN
      IF (USER_CLK'EVENT AND USER_CLK = '1') THEN
         IF (rst = '1') THEN
            addr_fifo_output <= X"00000000";    
         ELSE
            IF (app_af_WrEn = '1') THEN
               addr_fifo_output <= app_af_addr;    
            ELSE
               addr_fifo_output <= addr_fifo_output;    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (USER_CLK)
   BEGIN
      IF (USER_CLK'EVENT AND USER_CLK = '1') THEN
         IF (rst = '1') THEN
            cmd_fifo_output <= X"00000000";    
         ELSE
            IF (app_cmd_WrEn = '1') THEN
               cmd_fifo_output <= app_cmd_addr;    
            ELSE
               cmd_fifo_output <= cmd_fifo_output;    
            END IF;
         END IF;
      END IF;
   END PROCESS;



Wdf_1 : coregen_wr_fifo
		port map (
			din => app_Wdf_data(31 DOWNTO 0),
			rd_clk => DDR_CLK,
			rd_en =>  ctrl_Wdf_RdEn,
			rst => rst,
			wr_clk => USER_CLK,
			wr_en => app_Wdf_WrEn,
			dout => Wdf_data(31 DOWNTO 0),
			empty =>  Wdf_Empty,
			full => open);

Wdf_2 : coregen_wr_fifo
		port map (
			din => app_Wdf_data(63 DOWNTO 32),
			rd_clk => DDR_CLK,
			rd_en =>  ctrl_Wdf_RdEn,
			rst => rst,
			wr_clk => USER_CLK,
			wr_en => app_Wdf_WrEn,
			dout => Wdf_data(63 DOWNTO 32),
			empty =>  open,
			full => open);

Wdf_3 : coregen_wr_fifo
		port map (
			din => app_Wdf_data(95 DOWNTO 64),
			rd_clk => DDR_CLK,
			rd_en =>  ctrl_Wdf_RdEn,
			rst => rst,
			wr_clk => USER_CLK,
			wr_en => app_Wdf_WrEn,
			dout => Wdf_data(95 DOWNTO 64),
			empty =>  open,
			full => open);

Wdf_4 : coregen_wr_fifo
		port map (
			din => app_Wdf_data(127 DOWNTO 96),
			rd_clk => DDR_CLK,
			rd_en =>  ctrl_Wdf_RdEn,
			rst => rst,
			wr_clk => USER_CLK,
			wr_en => app_Wdf_WrEn,
			dout => Wdf_data(127 DOWNTO 96),
			empty =>  open,
			full => open);

--   Wdf_1 : FIFO16 
--      GENERIC MAP (
--         ALMOST_EMPTY_OFFSET => X"006",        
--         ALMOST_FULL_OFFSET => X"00F",
--         DATA_WIDTH => 36)      
--      PORT MAP (
--         ALMOSTEMPTY => open,
--         ALMOSTFULL => wr_df_almost_full_1,
--         DO => Wdf_data(31 DOWNTO 0),
--         DOP => open,
--         EMPTY => Wdf_Empty,
--         FULL => open,
--         RDCOUNT => open,
--         RDERR => open,
--         WRCOUNT => open,
--         WRERR => wr_df_wrerr_1,
--         DI => app_Wdf_data(31 DOWNTO 0),
--         DIP => "0000",
--         RDCLK => DDR_CLK,
--         RDEN => ctrl_Wdf_RdEn,
--         RST => rst,
--         WRCLK => USER_CLK,
--         WREN => app_Wdf_WrEn);   
--   
--   Wdf_2 : FIFO16 
--      GENERIC MAP (
--         ALMOST_EMPTY_OFFSET => X"006",        
--        ALMOST_FULL_OFFSET => X"00F",
--         DATA_WIDTH => 36)       
--      PORT MAP (
--         ALMOSTEMPTY => open,
--         ALMOSTFULL => wr_df_almost_full_2,
--         DO => Wdf_data(63 DOWNTO 32),
--         DOP => open,
--         EMPTY => open,
--         FULL => open,
--         RDCOUNT => open,
--         RDERR => open,
--         WRCOUNT => open,
--         WRERR => wr_df_wrerr_2,
--         DI => app_Wdf_data(63 DOWNTO 32),
--         DIP => "0000",
--         RDCLK => DDR_CLK,
--         RDEN => ctrl_Wdf_RdEn,
--         RST => rst,
--         WRCLK => USER_CLK,
--         WREN => app_Wdf_WrEn);   
--   

--   Wdf_3 : FIFO16 
--      GENERIC MAP (
--         ALMOST_EMPTY_OFFSET => X"006",        
--         ALMOST_FULL_OFFSET => X"00F",
--         DATA_WIDTH => 36)    
--      PORT MAP (
--         ALMOSTEMPTY => open,
--         ALMOSTFULL => wr_df_almost_full_3,
--         DO => Wdf_data(95 DOWNTO 64),
--         DOP => open,
--         EMPTY => open,
--         FULL => open,
--         RDCOUNT => open,
--         RDERR => open,
--         WRCOUNT => open,
--         WRERR => wr_df_wrerr_3,
--         DI => app_Wdf_data(95 DOWNTO 64),
--         DIP => "0000",
--         RDCLK => DDR_CLK,
--         RDEN => ctrl_Wdf_RdEn,
--         RST => rst,
--         WRCLK => USER_CLK,
--         WREN => app_Wdf_WrEn);   
--   

--   Wdf_4 : FIFO16 
--      GENERIC MAP (
--         ALMOST_EMPTY_OFFSET => X"006",        
--         ALMOST_FULL_OFFSET => X"00F",
--         DATA_WIDTH => 36)    
--      PORT MAP (
--         ALMOSTEMPTY => open,
--         ALMOSTFULL => wr_df_almost_full_4,
--         DO => Wdf_data(127 DOWNTO 96),
--         DOP => open,
--         EMPTY => open,
--         FULL => open,
--         RDCOUNT => open,
--         RDERR => open,
--         WRCOUNT => open,
--         WRERR => wr_df_wrerr_4,
--         DI => app_Wdf_data(127 DOWNTO 96),
--         DIP => "0000",
--         RDCLK => DDR_CLK,
--         RDEN => ctrl_Wdf_RdEn,
--         RST => rst,
--         WRCLK => USER_CLK,
--         WREN => app_Wdf_WrEn);   
   
END rtl;
