--------------------------------------------------------------------------
--
--  File:   cfg_test_x.vhd
--  Rev:    5.0.0
--  Notes:  Modified for XAPP708 
--  This is the user configurable options file for Xilinx's PCIX LogiCORE.
--  This file is normally generated by the CORE Generator tool.
--
--  Copyright (c) 2005 Xilinx, Inc.  All rights reserved.
--
--------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;


-- Declare the entity.

entity CFG is
  port (
        CFG             : out   std_logic_vector (511 downto 0)
  );
end CFG;


architecture RTL of CFG is


  signal cfg_int        : bit_vector (511 downto 0);
  

  -- Declare literals for use later.

  constant DISABLE      : bit := '0';
  constant ENABLE       : bit := '1';


  begin


  --------------------------------------------------------------
  -- Constant fields in configuration space.
  --------------------------------------------------------------

  -- Vendor Identification.
  cfg_int( 15 downto   0) <= X"10ee";

  -- Device Identification.
  cfg_int( 31 downto  16) <= X"0500";

  -- Revision Identification.
  cfg_int( 39 downto  32) <= X"00";

  -- Class Code.
  cfg_int( 63 downto  40) <= X"005000";

  -- Cardbus CIS Pointer.
  cfg_int(287 downto 256) <= X"00000000";

  -- Subvendor Identification.
  cfg_int(303 downto 288) <= X"10ee";

  -- Subsystem Identification.
  cfg_int(319 downto 304) <= X"0500";

  -- Capabilities Pointer.
  cfg_int(359 downto 352) <= X"40";

  -- Minimum Grant.
  cfg_int(367 downto 360) <= X"00";

  -- Maximum Latency.
  cfg_int(375 downto 368) <= X"00";

  -- PCI Device 66 MHz Capable.
  cfg_int(480) <= '0';

  -- PCI Device Decode Speed.
  cfg_int(482 downto 481) <= "10";


  --------------------------------------------------------------
  -- Power management in extended config space, 40h.
  --------------------------------------------------------------

  -- Power Management Next Pointer.
  cfg_int(383 downto 376) <= X"48";

  -- Support State D1.
  cfg_int(483) <= '0';

  -- Support State D2.
  cfg_int(484) <= '0';

  -- Support Soft Reset.
  cfg_int(485) <= '1';


  --------------------------------------------------------------
  -- Message signaled interrupts in extended config space, 48h.
  --------------------------------------------------------------

  -- Message Signaled Interrupts Next Pointer.
  cfg_int(391 downto 384) <= X"58";

  -- Multiple Message Capable Bit 0.
  cfg_int(486) <= '0';

  -- Multiple Message Capable Bit 1.
  cfg_int(487) <= '0';

  -- Multiple Message Capable Bit 2.
  cfg_int(488) <= '0';


  --------------------------------------------------------------
  -- PCI-X capabilities in extended config space, 58h.
  --------------------------------------------------------------

  -- PCI-X Next Pointer.
  cfg_int(399 downto 392) <= X"00";

  -- PCI-X Interface Design Information.
  cfg_int(407 downto 400) <= "00000000";

  -- PCI-X Device 64-Bit.
  cfg_int(489) <= '1';

  -- PCI-X Device 133 MHz Capable.
  cfg_int(490) <= '1';

  -- PCI-X Device Complex.
  cfg_int(491) <= '0';

  -- PCI-X Max Outstanding Split Command Register Bit 0. 
  cfg_int(496) <= '0'; 
 
  -- PCI-X Max Outstanding Split Command Register Bit 1. 
  cfg_int(498) <= '0'; 
 
  -- PCI-X Max Outstanding Split Command Register Bit 2.
  cfg_int(499) <= '0'; 


  --------------------------------------------------------------
  -- Generic target address space configuration.
  --------------------------------------------------------------

  -- BAR0.
  cfg_int( 95 downto  64) <= X"00000000";
  cfg_int(408) <= '0';

  -- BAR1.
  cfg_int(127 downto  96) <= X"00000000";
  cfg_int(409) <= '0';

  -- BAR2.
  cfg_int(159 downto 128) <= X"f8000004";
  cfg_int(410) <= '1';

  -- BAR3.
  cfg_int(191 downto 160) <= X"ffffffff";
  cfg_int(411) <= '1';

  -- BAR4.
  cfg_int(223 downto 192) <= X"00000000";
  cfg_int(412) <= '0';

  -- BAR5.
  cfg_int(255 downto 224) <= X"00000000";
  cfg_int(413) <= '0';

  -- Expansion ROM BAR.
  cfg_int(351 downto 320) <= X"00000000";
  cfg_int(414) <= '0';

  -- Split Transaction Target.
  cfg_int(415) <= '0';


  --------------------------------------------------------------
  -- Special target address space configuration options.
  --------------------------------------------------------------

  -- Autoconfigure BAR0.
  cfg_int(492) <= '0';
  cfg_int(447 downto 416) <= X"00000000";

  -- Autoconfigure BAR1.
  cfg_int(493) <= '0';
  cfg_int(479 downto 448) <= X"00000000";

  -- Enable Instant On.
  cfg_int(494) <= '0';
  cfg_int(501) <= '0';

  -- Enable Interrupt Acknowledge.
  cfg_int(495) <= '0';

  -- Enable User Configuration Space.
  cfg_int(497) <= '0';


  --------------------------------------------------------------
  -- Other configuration options.
  --------------------------------------------------------------
 
  -- Initiator Uses 32-Bit Data Objects. 
  cfg_int(500) <= '0';
 
  -- Bus Width Detect Disable.
  cfg_int(502) <= '0';

  -- Bus Width Manual as 32-Bit. 
  cfg_int(503) <= '0';
 
  -- Bus Mode Detect Disable. 
  cfg_int(504) <= '1';
 
  -- Bus Mode Manual as PCI. 
  cfg_int(505) <= '0';
 
  -- Combinational M_SPLIT Signal.
  cfg_int(509) <= '0';
 
  -- Reserved, Do Not Modify. 
  cfg_int(506) <= '0';
  cfg_int(507) <= '1';
  cfg_int(508) <= '0';
 
  -- Reserved, Do Not Modify.
  cfg_int(510) <= '0';
  cfg_int(511) <= '0';


  --------------------------------------------------------------
  --
  --------------------------------------------------------------

  CFG <= to_stdlogicvector(cfg_int);


end RTL;
