--*******************************************************************************
--
--  File name :       clk_module.vhd
--
--  Description :     This module instantiates DCM_BASE primitive for the memory
--                    interface and the 200 MHz clock for the IDELAYCTRL module. 
-- 
--                    
--  Date - revision : 3/01/2004
--                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
--
--  Author :          Maria George
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 - 2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY clk_module IS
   PORT (
         SYS_CLK_IN              : IN std_logic;   
         DDR_CLK90               : IN std_logic;   
         DDR_CLKDIV              : IN std_logic;   
         REF_CLK200_IN           : IN std_logic;   
         CLK                     : OUT std_logic;   
         CLK90                   : OUT std_logic;   
         CLK200                  : OUT std_logic;   
         REFRESH_CLK             : OUT std_logic;   
         DDR_DIMM_CLK0_P         : OUT std_logic;   
         DDR_DIMM_CLK0_N         : OUT std_logic;   
         DDR_DIMM_CLK1_P         : OUT std_logic;   
         DDR_DIMM_CLK1_N         : OUT std_logic;   
         DDR_DIMM_CLK2_P         : OUT std_logic;   
         DDR_DIMM_CLK2_N         : OUT std_logic);   
END clk_module;

ARCHITECTURE rtl OF clk_module IS


   SIGNAL clk0_bufg1_out           :  std_logic;   
   SIGNAL vcc                      :  std_logic;   
   SIGNAL gnd                      :  std_logic;    
  
BEGIN
   CLK <= SYS_CLK_IN ;
   CLK90 <= DDR_CLK90 ;
   CLK200 <= clk0_bufg1_out ;
   REFRESH_CLK <= DDR_CLKDIV;
   vcc <= '1' ;
   gnd <= '0' ;

   dcm1_clk0 : BUFG 
      PORT MAP (
         O => clk0_bufg1_out,
         I => REF_CLK200_IN);   
   
   oddr_clk0 : ODDR 
      GENERIC MAP (
         DDR_CLK_EDGE => "OPPOSITE_EDGE",         
         SRTYPE => "SYNC")
      PORT MAP (
         Q => DDR_DIMM_CLK0_N,
         C => SYS_CLK_IN,
         CE => vcc,
         D1 => gnd,
         D2 => vcc,
         R => gnd,
         S => gnd);   
   

   oddr_clk0n : ODDR 
      GENERIC MAP (
         DDR_CLK_EDGE => "OPPOSITE_EDGE",         
         SRTYPE => "SYNC")
      PORT MAP (
         Q => DDR_DIMM_CLK0_P,
         C => SYS_CLK_IN,
         CE => vcc,
         D1 => vcc,
         D2 => gnd,
         R => gnd,
         S => gnd);   
   

   oddr_clk1 : ODDR 
      GENERIC MAP (
         DDR_CLK_EDGE => "OPPOSITE_EDGE",         
         SRTYPE => "SYNC")
      PORT MAP (
         Q => DDR_DIMM_CLK1_N,
         C => SYS_CLK_IN,
         CE => vcc,
         D1 => gnd,
         D2 => vcc,
         R => gnd,
         S => gnd);   
   

   oddr_clk1n : ODDR 
      GENERIC MAP (
         DDR_CLK_EDGE => "OPPOSITE_EDGE",         
         SRTYPE => "SYNC")
      PORT MAP (
         Q => DDR_DIMM_CLK1_P,
         C => SYS_CLK_IN,
         CE => vcc,
         D1 => vcc,
         D2 => gnd,
         R => gnd,
         S => gnd);   
   
   oddr_clk2 : ODDR 
      GENERIC MAP (
         DDR_CLK_EDGE => "OPPOSITE_EDGE",         
         SRTYPE => "SYNC")
      PORT MAP (
         Q => DDR_DIMM_CLK2_N,
         C => SYS_CLK_IN,
         CE => vcc,
         D1 => gnd,
         D2 => vcc,
         R => gnd,
         S => gnd);   
   

   oddr_clk2n : ODDR 
      GENERIC MAP (
         DDR_CLK_EDGE => "OPPOSITE_EDGE",         
         SRTYPE => "SYNC")
      PORT MAP (
         Q => DDR_DIMM_CLK2_P,
         C => SYS_CLK_IN,
         CE => vcc,
         D1 => vcc,
         D2 => gnd,
         R => gnd,
         S => gnd);   
   

END rtl;
