--*******************************************************************************
--
--  File name :       data_path.vhd
--
--  Description :     This module is the top level for the physical layer and  
--                    instantiates the following modules: tap_ctrl, data_tap_inc,
--                    idelay_ctrl, idelay_rd_en, v4_dqs_iob, v4_dq_iob, and
--                    rd_data_fifo.
-- 
--                    
--  Date - revision : 1/27/2004
--                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
--
--  Author :          Maria George
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 - 2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY data_path IS
   PORT (
      CLK                     : IN std_logic;   
      CLK90                   : IN std_logic;   
      CLK200                  : IN std_logic;   
      RESET0                  : IN std_logic;   
      RESET90                 : IN std_logic;   
      CTRL_DUMMYREAD_START    : IN std_logic;   --  dummy read enable does not need to be normalized
      WDF_DATA                : IN std_logic_vector(127 DOWNTO 0);   
      DM_DATA                 : IN std_logic_vector(15 DOWNTO 0);   
      CTRL_WREN               : IN std_logic;   
      CTRL_DQS_RST            : IN std_logic;   
      CTRL_DQS_EN             : IN std_logic;   
      DDR_DQ                  : INOUT std_logic_vector(63 DOWNTO 0);   
      DDR_DQS                 : INOUT std_logic_vector(7 DOWNTO 0);   
      DDR_DM                  : OUT std_logic_vector(7 DOWNTO 0);   
      SEL_DONE                : OUT std_logic;   
      READ_DATA_RISE          : OUT std_logic_vector(63 DOWNTO 0);   
      READ_DATA_FALL          : OUT std_logic_vector(63 DOWNTO 0);   
      FIRST_RISING            : OUT std_logic_vector(3 DOWNTO 0);   
      test_tap                : OUT std_logic_vector(5 DOWNTO 0));   
END data_path;

ARCHITECTURE rtl OF data_path IS
 
COMPONENT idelay_ctrl
   PORT (
      CLK200                  : IN std_logic;   
      RESET                   : IN std_logic;   
      RDY_STATUS              : OUT std_logic);   
END COMPONENT;

COMPONENT data_tap_inc
   PORT (
      CLK                     : IN std_logic;   
      RESET                   : IN std_logic;   
      DATA_DLYINC             : OUT std_logic;   
      DATA_DLYCE              : OUT std_logic;   
      DATA_DLYRST             : OUT std_logic;   
      DATA_TAP_SEL_DONE       : OUT std_logic;   
      VALID_DATA_TAP_COUNT    : IN std_logic;   
      DATA_TAP_COUNT          : IN std_logic_vector(5 DOWNTO 0));   
END COMPONENT;

COMPONENT tap_ctrl 
   PORT (
      CLK                     : IN std_logic;   
      RESET                   : IN std_logic;   
      RDY_STATUS              : IN std_logic;   
      DQS                     : IN std_logic;   
      CTRL_DUMMYREAD_START    : IN std_logic;   
      DLYINC                  : OUT std_logic;   
      DLYCE                   : OUT std_logic;   
      DLYRST                  : OUT std_logic;   
      SEL_DONE                : OUT std_logic;   
      FIRST_RISING            : OUT std_logic;   
      VALID_DATA_TAP_COUNT    : OUT std_logic;   
      DATA_TAP_COUNT          : OUT std_logic_vector(5 DOWNTO 0));   
END COMPONENT;

COMPONENT v4_dm_iob 
   PORT (
      CLK90                   : IN std_logic;   
      DM_RISE                 : IN std_logic;   
      DM_FALL                 : IN std_logic;   
      DDR_DM                  : OUT std_logic);   
END COMPONENT;

COMPONENT v4_dq_iob 
   PORT (
      CLK                     : IN std_logic;   
      CLK90                   : IN std_logic;   
      RESET                   : IN std_logic;     
      DATA_DLYINC             : IN std_logic;   
      DATA_DLYCE              : IN std_logic;   
      DATA_DLYRST             : IN std_logic;   
      WRITE_DATA_RISE         : IN std_logic;   
      WRITE_DATA_FALL         : IN std_logic;   
      CTRL_WREN               : IN std_logic;    
      DDR_DQ                  : INOUT std_logic;   
      READ_DATA_RISE          : OUT std_logic;   
      READ_DATA_FALL          : OUT std_logic);   
END COMPONENT;

COMPONENT v4_dqs_iob 
   PORT (
      CLK                     : IN std_logic;   
      DLYINC                  : IN std_logic;   
      DLYCE                   : IN std_logic;     
      DLYRST                  : IN std_logic;   
      CTRL_DQS_RST            : IN std_logic;   
      CTRL_DQS_EN             : IN std_logic;    
      DDR_DQS                 : INOUT std_logic;   
      DQS                     : OUT std_logic);   
END COMPONENT;

   SIGNAL data_idelay_inc          :  std_logic_vector(7 DOWNTO 0);  
   SIGNAL data_idelay_ce           :  std_logic_vector(7 DOWNTO 0);  
   SIGNAL data_idelay_rst          :  std_logic_vector(7 DOWNTO 0);  
   SIGNAL rd_data_rise             :  std_logic_vector(63 DOWNTO 0); 
   SIGNAL rd_data_fall             :  std_logic_vector(63 DOWNTO 0); 
   SIGNAL dqs_idelay_inc           :  std_logic_vector(7 DOWNTO 0);  
   SIGNAL dqs_idelay_ce            :  std_logic_vector(7 DOWNTO 0);  
   SIGNAL dqs_idelay_rst           :  std_logic_vector(7 DOWNTO 0);  
   SIGNAL dqs_delayed              :  std_logic_vector(7 DOWNTO 0);  
   SIGNAL idelay_ctrl_rdy          :  std_logic;   
   SIGNAL data_tap_select          :  std_logic_vector(7 DOWNTO 0);  
   SIGNAL dqs_tap_sel_done         :  std_logic_vector(7 DOWNTO 0);  
   SIGNAL rising_first             :  std_logic_vector(7 DOWNTO 0);  
   SIGNAL valid_tap_count          :  std_logic_vector(7 DOWNTO 0);  
   SIGNAL data_tap_count0          :  std_logic_vector(5 DOWNTO 0);  
   SIGNAL data_tap_count1          :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL data_tap_count2          :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL data_tap_count3          :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL data_tap_count4          :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL data_tap_count5          :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL data_tap_count6          :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL data_tap_count7          :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL first_rising_detected    :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL rd_en_delayed_rise       :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL rd_en_delayed_fall       :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL wr_data                  :  std_logic_vector(127 DOWNTO 0);   
   SIGNAL wr_data_r1               :  std_logic_vector(127 DOWNTO 0);   
   SIGNAL wr_data_rise             :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL wr_data_fall             :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL mask_data                :  std_logic_vector(15 DOWNTO 0);   
   SIGNAL mask_data_r1             :  std_logic_vector(15 DOWNTO 0);   
   SIGNAL mask_data_rise           :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL mask_data_fall           :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL dqs_rst_r1               :  std_logic;   
   SIGNAL dqs_rst_r2               :  std_logic;   
   SIGNAL dqs_rst_r3               :  std_logic;   
   SIGNAL dqs_en_r1                :  std_logic;   
   SIGNAL dqs_en_r2                :  std_logic;   
   SIGNAL dqs_en_r3                :  std_logic;   
   SIGNAL dqs_en_r4                :  std_logic;   
   SIGNAL wr_en_clk270_r1          :  std_logic;   
   SIGNAL wr_en_clk180_r2          :  std_logic;   
   SIGNAL wr_en_clk90_r3           :  std_logic;   
   SIGNAL DDR_DM_sig             :  std_logic_vector(7 DOWNTO 0);   
BEGIN
   test_tap <= data_tap_count0 ;
	-- For controller to stop dummy reads
   DDR_DM <= DDR_DM_sig;
   SEL_DONE <= data_tap_select(0) and 
	            data_tap_select(1) and
					data_tap_select(2) and
					data_tap_select(3) and
					data_tap_select(4) and
					data_tap_select(5) and
					data_tap_select(6) and
					data_tap_select(7);
   READ_DATA_RISE <=  rd_data_rise(63 DOWNTO 0) ;
   READ_DATA_FALL <= rd_data_fall(63 DOWNTO 0) ;
   FIRST_RISING <= first_rising_detected(3 DOWNTO 0);

   rd_en_delayed_fall(3 DOWNTO 2) <= rd_en_delayed_fall(0) & 
   rd_en_delayed_fall(0) ;
   first_rising_detected(0) <= rising_first(0) AND rising_first(1) ;
   first_rising_detected(1) <= rising_first(2) AND rising_first(3) ;
   first_rising_detected(2) <= rising_first(4) AND rising_first(5) ;
   first_rising_detected(3) <= rising_first(6) AND rising_first(7) ;

   PROCESS (CLK90)
   BEGIN
      IF (CLK90'EVENT AND CLK90 = '0') THEN
         IF (RESET90 = '1') THEN
            wr_data(127 DOWNTO 0) <= X"00000000000000000000000000000000";    
            mask_data(15 DOWNTO 0) <= "0000000000000000";    
            wr_en_clk270_r1 <= '0';    
            dqs_rst_r1 <= '0';    
            dqs_en_r1 <= '0';    
         ELSE
            wr_data(63 DOWNTO 0) <= WDF_DATA(63 DOWNTO 0);    
            wr_data(127 DOWNTO 64) <= WDF_DATA(127 DOWNTO 64);    
            mask_data(7 DOWNTO 0) <= DM_DATA(7 DOWNTO 0);    
            mask_data(15 DOWNTO 8) <= DM_DATA(15 DOWNTO 8);    
            wr_en_clk270_r1 <= CTRL_WREN;    
            dqs_rst_r1 <= CTRL_DQS_RST;    
            dqs_en_r1 <= NOT CTRL_DQS_EN;    
         END IF;
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '0') THEN
         IF (RESET0 = '1') THEN
            wr_en_clk180_r2 <= '0';    
            wr_data_r1(127 DOWNTO 0) <= X"00000000000000000000000000000000";    
            mask_data_r1(15 DOWNTO 0) <= "0000000000000000";    
            dqs_rst_r2 <= '0';    
            dqs_rst_r3 <= '0';    
            dqs_en_r2 <= '0';    
            dqs_en_r3 <= '0';    
            dqs_en_r4 <= '0';    
         ELSE
            wr_en_clk180_r2 <= wr_en_clk270_r1;    
            wr_data_r1(127 DOWNTO 0) <= wr_data(127 DOWNTO 0);    
            mask_data_r1(15 DOWNTO 0) <= mask_data(15 DOWNTO 0);    
            dqs_rst_r2 <= dqs_rst_r1;    
            dqs_rst_r3 <= dqs_rst_r2;    
            dqs_en_r2 <= dqs_en_r1;    
            dqs_en_r3 <= dqs_en_r2;    
            dqs_en_r4 <= dqs_en_r3;    
         END IF;
      END IF;
   END PROCESS;

   PROCESS (CLK90)
   BEGIN
      IF (CLK90'EVENT AND CLK90 = '1') THEN
         IF (RESET90 = '1') THEN
            wr_en_clk90_r3 <= '0';    
            wr_data_rise(63 DOWNTO 0) <= X"0000000000000000"; 
            wr_data_fall(63 DOWNTO 0) <= X"0000000000000000";    
            mask_data_rise(7 DOWNTO 0) <= "00000000";    
            mask_data_fall(7 DOWNTO 0) <= "00000000";    
         ELSE
            wr_en_clk90_r3 <= wr_en_clk180_r2;    
            wr_data_rise(63 DOWNTO 0) <= wr_data_r1(127 DOWNTO 64);    
            wr_data_fall(63 DOWNTO 0) <= wr_data_r1(63 DOWNTO 0);    
            mask_data_rise(7 DOWNTO 0) <= mask_data_r1(15 DOWNTO 8);    
            mask_data_fall(7 DOWNTO 0) <= mask_data_r1(7 DOWNTO 0);    
         END IF;
      END IF;
   END PROCESS;
   
   --/////////////////////////////////////////////////////////////////////////////   
   -- IDELAYCTRL instantiation  
   --/////////////////////////////////////////////////////////////////////////////
   idelay_ctrl0 : idelay_ctrl 
      PORT MAP (
         CLK200 => CLK200,
         RESET => RESET0,
         RDY_STATUS => idelay_ctrl_rdy);   
   
   
   --/////////////////////////////////////////////////////////////////////////////   
   -- 36 DQS instances   
   --/////////////////////////////////////////////////////////////////////////////
   dqs0 : v4_dqs_iob 
      PORT MAP (
         CLK => CLK,
         DLYINC => dqs_idelay_inc(0),
         DLYCE => dqs_idelay_ce(0),
         DLYRST => dqs_idelay_rst(0),
         CTRL_DQS_RST => dqs_rst_r3,
         CTRL_DQS_EN => dqs_en_r4,
         DDR_DQS => DDR_DQS(0),
         DQS => dqs_delayed(0));   
   

   dqs1 : v4_dqs_iob 
      PORT MAP (
         CLK => CLK,
         DLYINC => dqs_idelay_inc(1),
         DLYCE => dqs_idelay_ce(1),
         DLYRST => dqs_idelay_rst(1),
         CTRL_DQS_RST => dqs_rst_r3,
         CTRL_DQS_EN => dqs_en_r4,
         DDR_DQS => DDR_DQS(1),
         DQS => dqs_delayed(1));   
   
   dqs2 : v4_dqs_iob 
      PORT MAP (
         CLK => CLK,
         DLYINC => dqs_idelay_inc(2),
         DLYCE => dqs_idelay_ce(2),
         DLYRST => dqs_idelay_rst(2),
         CTRL_DQS_RST => dqs_rst_r3,
         CTRL_DQS_EN => dqs_en_r4,
         DDR_DQS => DDR_DQS(2),
         DQS => dqs_delayed(2));   
   
   dqs3 : v4_dqs_iob 
      PORT MAP (
         CLK => CLK,
         DLYINC => dqs_idelay_inc(3),
         DLYCE => dqs_idelay_ce(3),
         DLYRST => dqs_idelay_rst(3),
         CTRL_DQS_RST => dqs_rst_r3,
         CTRL_DQS_EN => dqs_en_r4,
         DDR_DQS => DDR_DQS(3),
         DQS => dqs_delayed(3));   
   
   dqs4 : v4_dqs_iob 
      PORT MAP (
         CLK => CLK,
         DLYINC => dqs_idelay_inc(4),
         DLYCE => dqs_idelay_ce(4),
         DLYRST => dqs_idelay_rst(4),
         CTRL_DQS_RST => dqs_rst_r3,
         CTRL_DQS_EN => dqs_en_r4,
         DDR_DQS => DDR_DQS(4),
         DQS => dqs_delayed(4));   
   
   dqs5 : v4_dqs_iob 
      PORT MAP (
         CLK => CLK,
         DLYINC => dqs_idelay_inc(5),
         DLYCE => dqs_idelay_ce(5),
         DLYRST => dqs_idelay_rst(5),
         CTRL_DQS_RST => dqs_rst_r3,
         CTRL_DQS_EN => dqs_en_r4,
         DDR_DQS => DDR_DQS(5),
         DQS => dqs_delayed(5));   
   
   dqs6 : v4_dqs_iob 
      PORT MAP (
         CLK => CLK,
         DLYINC => dqs_idelay_inc(6),
         DLYCE => dqs_idelay_ce(6),
         DLYRST => dqs_idelay_rst(6),
         CTRL_DQS_RST => dqs_rst_r3,
         CTRL_DQS_EN => dqs_en_r4,
         DDR_DQS => DDR_DQS(6),
         DQS => dqs_delayed(6));   
   
   dqs7 : v4_dqs_iob 
      PORT MAP (
         CLK => CLK,
         DLYINC => dqs_idelay_inc(7),
         DLYCE => dqs_idelay_ce(7),
         DLYRST => dqs_idelay_rst(7),
         CTRL_DQS_RST => dqs_rst_r3,
         CTRL_DQS_EN => dqs_en_r4,
         DDR_DQS => DDR_DQS(7),
         DQS => dqs_delayed(7));   
   
   
   --/////////////////////////////////////////////////////////////////////////////   
   -- 36 tap_ctrl instances for 36 DDR_DQS strobes   
   --/////////////////////////////////////////////////////////////////////////////
   tap_ctrl_dqs0 : tap_ctrl 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         RDY_STATUS => idelay_ctrl_rdy,
         DQS => dqs_delayed(0),
         CTRL_DUMMYREAD_START => CTRL_DUMMYREAD_START,
         DLYINC => dqs_idelay_inc(0),
         DLYCE => dqs_idelay_ce(0),
         DLYRST => dqs_idelay_rst(0),
         SEL_DONE => dqs_tap_sel_done(0),
         FIRST_RISING => rising_first(0),
         VALID_DATA_TAP_COUNT => valid_tap_count(0),
         DATA_TAP_COUNT => data_tap_count0(5 DOWNTO 0));   
   
   tap_ctrl_dqs1 : tap_ctrl 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         RDY_STATUS => idelay_ctrl_rdy,
         DQS => dqs_delayed(1),
         CTRL_DUMMYREAD_START => CTRL_DUMMYREAD_START,
         DLYINC => dqs_idelay_inc(1),
         DLYCE => dqs_idelay_ce(1),
         DLYRST => dqs_idelay_rst(1),
         SEL_DONE => dqs_tap_sel_done(1),
         FIRST_RISING => rising_first(1),
         VALID_DATA_TAP_COUNT => valid_tap_count(1),
         DATA_TAP_COUNT => data_tap_count1(5 DOWNTO 0));   
   
   tap_ctrl_dqs2 : tap_ctrl 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         RDY_STATUS => idelay_ctrl_rdy,
         DQS => dqs_delayed(2),
         CTRL_DUMMYREAD_START => CTRL_DUMMYREAD_START,
         DLYINC => dqs_idelay_inc(2),
         DLYCE => dqs_idelay_ce(2),
         DLYRST => dqs_idelay_rst(2),
         SEL_DONE => dqs_tap_sel_done(2),
         FIRST_RISING => rising_first(2),
         VALID_DATA_TAP_COUNT => valid_tap_count(2),
         DATA_TAP_COUNT => data_tap_count2(5 DOWNTO 0));   
   
   tap_ctrl_dqs3 : tap_ctrl 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         RDY_STATUS => idelay_ctrl_rdy,
         DQS => dqs_delayed(3),
         CTRL_DUMMYREAD_START => CTRL_DUMMYREAD_START,
         DLYINC => dqs_idelay_inc(3),
         DLYCE => dqs_idelay_ce(3),
         DLYRST => dqs_idelay_rst(3),
         SEL_DONE => dqs_tap_sel_done(3),
         FIRST_RISING => rising_first(3),
         VALID_DATA_TAP_COUNT => valid_tap_count(3),
         DATA_TAP_COUNT => data_tap_count3(5 DOWNTO 0));   
   
   tap_ctrl_dqs4 : tap_ctrl 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         RDY_STATUS => idelay_ctrl_rdy,
         DQS => dqs_delayed(4),
         CTRL_DUMMYREAD_START => CTRL_DUMMYREAD_START,
         DLYINC => dqs_idelay_inc(4),
         DLYCE => dqs_idelay_ce(4),
         DLYRST => dqs_idelay_rst(4),
         SEL_DONE => dqs_tap_sel_done(4),
         FIRST_RISING => rising_first(4),
         VALID_DATA_TAP_COUNT => valid_tap_count(4),
         DATA_TAP_COUNT => data_tap_count4(5 DOWNTO 0));   
   
   tap_ctrl_dqs5 : tap_ctrl 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         RDY_STATUS => idelay_ctrl_rdy,
         DQS => dqs_delayed(5),
         CTRL_DUMMYREAD_START => CTRL_DUMMYREAD_START,
         DLYINC => dqs_idelay_inc(5),
         DLYCE => dqs_idelay_ce(5),
         DLYRST => dqs_idelay_rst(5),
         SEL_DONE => dqs_tap_sel_done(5),
         FIRST_RISING => rising_first(5),
         VALID_DATA_TAP_COUNT => valid_tap_count(5),
         DATA_TAP_COUNT => data_tap_count5(5 DOWNTO 0));   
   
   tap_ctrl_dqs6 : tap_ctrl 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         RDY_STATUS => idelay_ctrl_rdy,
         DQS => dqs_delayed(6),
         CTRL_DUMMYREAD_START => CTRL_DUMMYREAD_START,
         DLYINC => dqs_idelay_inc(6),
         DLYCE => dqs_idelay_ce(6),
         DLYRST => dqs_idelay_rst(6),
         SEL_DONE => dqs_tap_sel_done(6),
         FIRST_RISING => rising_first(6),
         VALID_DATA_TAP_COUNT => valid_tap_count(6),
         DATA_TAP_COUNT => data_tap_count6(5 DOWNTO 0));   
   
   tap_ctrl_dqs7 : tap_ctrl 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         RDY_STATUS => idelay_ctrl_rdy,
         DQS => dqs_delayed(7),
         CTRL_DUMMYREAD_START => CTRL_DUMMYREAD_START,
         DLYINC => dqs_idelay_inc(7),
         DLYCE => dqs_idelay_ce(7),
         DLYRST => dqs_idelay_rst(7),
         SEL_DONE => dqs_tap_sel_done(7),
         FIRST_RISING => rising_first(7),
         VALID_DATA_TAP_COUNT => valid_tap_count(7),
         DATA_TAP_COUNT => data_tap_count7(5 DOWNTO 0));   
   
   
   --/////////////////////////////////////////////////////////////////////////////   
   -- 36 instances of data_tap_inc for each dqs and associated tap_ctrl   
   --/////////////////////////////////////////////////////////////////////////////
   data_tap_inc_dq7_0 : data_tap_inc 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         DATA_DLYINC => data_idelay_inc(0),
         DATA_DLYCE => data_idelay_ce(0),
         DATA_DLYRST => data_idelay_rst(0),
         DATA_TAP_SEL_DONE => data_tap_select(0),
         VALID_DATA_TAP_COUNT => valid_tap_count(0),
         DATA_TAP_COUNT => data_tap_count0(5 DOWNTO 0));   
   
   data_tap_inc_dq15_8 : data_tap_inc 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         DATA_DLYINC => data_idelay_inc(1),
         DATA_DLYCE => data_idelay_ce(1),
         DATA_DLYRST => data_idelay_rst(1),
         DATA_TAP_SEL_DONE => data_tap_select(1),
         VALID_DATA_TAP_COUNT => valid_tap_count(1),
         DATA_TAP_COUNT => data_tap_count1(5 DOWNTO 0));   
   
   data_tap_inc_dq23_16 : data_tap_inc 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         DATA_DLYINC => data_idelay_inc(2),
         DATA_DLYCE => data_idelay_ce(2),
         DATA_DLYRST => data_idelay_rst(2),
         DATA_TAP_SEL_DONE => data_tap_select(2),
         VALID_DATA_TAP_COUNT => valid_tap_count(2),
         DATA_TAP_COUNT => data_tap_count2(5 DOWNTO 0));   
   
   data_tap_inc_dq31_24 : data_tap_inc 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         DATA_DLYINC => data_idelay_inc(3),
         DATA_DLYCE => data_idelay_ce(3),
         DATA_DLYRST => data_idelay_rst(3),
         DATA_TAP_SEL_DONE => data_tap_select(3),
         VALID_DATA_TAP_COUNT => valid_tap_count(3),
         DATA_TAP_COUNT => data_tap_count3(5 DOWNTO 0));   
   
   data_tap_inc_dq39_32 : data_tap_inc 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         DATA_DLYINC => data_idelay_inc(4),
         DATA_DLYCE => data_idelay_ce(4),
         DATA_DLYRST => data_idelay_rst(4),
         DATA_TAP_SEL_DONE => data_tap_select(4),
         VALID_DATA_TAP_COUNT => valid_tap_count(4),
         DATA_TAP_COUNT => data_tap_count4(5 DOWNTO 0));   
   
   data_tap_inc_dq47_40 : data_tap_inc 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         DATA_DLYINC => data_idelay_inc(5),
         DATA_DLYCE => data_idelay_ce(5),
         DATA_DLYRST => data_idelay_rst(5),
         DATA_TAP_SEL_DONE => data_tap_select(5),
         VALID_DATA_TAP_COUNT => valid_tap_count(5),
         DATA_TAP_COUNT => data_tap_count5(5 DOWNTO 0));   
   
   data_tap_inc_dq55_48 : data_tap_inc 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         DATA_DLYINC => data_idelay_inc(6),
         DATA_DLYCE => data_idelay_ce(6),
         DATA_DLYRST => data_idelay_rst(6),
         DATA_TAP_SEL_DONE => data_tap_select(6),
         VALID_DATA_TAP_COUNT => valid_tap_count(6),
         DATA_TAP_COUNT => data_tap_count6(5 DOWNTO 0));   
   
   data_tap_inc_dq63_56 : data_tap_inc 
      PORT MAP (
         CLK => CLK,
         RESET => RESET0,
         DATA_DLYINC => data_idelay_inc(7),
         DATA_DLYCE => data_idelay_ce(7),
         DATA_DLYRST => data_idelay_rst(7),
         DATA_TAP_SEL_DONE => data_tap_select(7),
         VALID_DATA_TAP_COUNT => valid_tap_count(7),
         DATA_TAP_COUNT => data_tap_count7(5 DOWNTO 0));   
   
   
   --///////////////////////////////////////////////////////////////////////////// 
   -- DM instances   
   --/////////////////////////////////////////////////////////////////////////////
   dm_bit0 : v4_dm_iob 
      PORT MAP (
         CLK90 => CLK90,
         DM_RISE => mask_data_rise(0),
         DM_FALL => mask_data_fall(0),
         DDR_DM => DDR_DM_sig(0));   
   
   dm_bit1 : v4_dm_iob 
      PORT MAP (
         CLK90 => CLK90,
         DM_RISE => mask_data_rise(1),
         DM_FALL => mask_data_fall(1),
         DDR_DM => DDR_DM_sig(1));   
   
   dm_bit2 : v4_dm_iob 
      PORT MAP (
         CLK90 => CLK90,
         DM_RISE => mask_data_rise(2),
         DM_FALL => mask_data_fall(2),
         DDR_DM => DDR_DM_sig(2));   
   
   dm_bit3 : v4_dm_iob 
      PORT MAP (
         CLK90 => CLK90,
         DM_RISE => mask_data_rise(3),
         DM_FALL => mask_data_fall(3),
         DDR_DM => DDR_DM_sig(3));   
   
   dm_bit4 : v4_dm_iob 
      PORT MAP (
         CLK90 => CLK90,
         DM_RISE => mask_data_rise(4),
         DM_FALL => mask_data_fall(4),
         DDR_DM => DDR_DM_sig(4));   
   
   dm_bit5 : v4_dm_iob 
      PORT MAP (
         CLK90 => CLK90,
         DM_RISE => mask_data_rise(5),
         DM_FALL => mask_data_fall(5),
         DDR_DM => DDR_DM_sig(5));   
   
   dm_bit6 : v4_dm_iob 
      PORT MAP (
         CLK90 => CLK90,
         DM_RISE => mask_data_rise(6),
         DM_FALL => mask_data_fall(6),
         DDR_DM => DDR_DM_sig(6));   
   
   dm_bit7 : v4_dm_iob 
      PORT MAP (
         CLK90 => CLK90,
         DM_RISE => mask_data_rise(7),
         DM_FALL => mask_data_fall(7),
         DDR_DM => DDR_DM_sig(7));   
   
   
   --/////////////////////////////////////////////////////////////////////////////   
   -- 36 instances of DQ_IOB4 instances  
   --/////////////////////////////////////////////////////////////////////////////

	 dq0_7: for N in 0 to 7 generate
    dq_bit : v4_dq_iob 
      PORT MAP (
         CLK => CLK,
         CLK90 => CLK90,
         RESET => RESET90,
         DATA_DLYINC => data_idelay_inc(0),
         DATA_DLYCE => data_idelay_ce(0),
         DATA_DLYRST => data_idelay_rst(0),
         WRITE_DATA_RISE => wr_data_rise(N),
         WRITE_DATA_FALL => wr_data_fall(N),
         CTRL_WREN => wr_en_clk90_r3,
         DDR_DQ => DDR_DQ(N),
         READ_DATA_RISE => rd_data_rise(N),
         READ_DATA_FALL => rd_data_fall(N));   
      end generate;

  	 dq8_15: for N in 8 to 15 generate
    dq_bit : v4_dq_iob 
      PORT MAP (
         CLK => CLK,
         CLK90 => CLK90,
         RESET => RESET90,
         DATA_DLYINC => data_idelay_inc(1),
         DATA_DLYCE => data_idelay_ce(1),
         DATA_DLYRST => data_idelay_rst(1),
         WRITE_DATA_RISE => wr_data_rise(N),
         WRITE_DATA_FALL => wr_data_fall(N),
         CTRL_WREN => wr_en_clk90_r3,
         DDR_DQ => DDR_DQ(N),
         READ_DATA_RISE => rd_data_rise(N),
         READ_DATA_FALL => rd_data_fall(N));   
      end generate;

  	 dq16_23: for N in 16 to 23 generate
    dq_bit : v4_dq_iob 
      PORT MAP (
         CLK => CLK,
         CLK90 => CLK90,
         RESET => RESET90,
         DATA_DLYINC => data_idelay_inc(2),
         DATA_DLYCE => data_idelay_ce(2),
         DATA_DLYRST => data_idelay_rst(2),
         WRITE_DATA_RISE => wr_data_rise(N),
         WRITE_DATA_FALL => wr_data_fall(N),
         CTRL_WREN => wr_en_clk90_r3,
         DDR_DQ => DDR_DQ(N),
         READ_DATA_RISE => rd_data_rise(N),
         READ_DATA_FALL => rd_data_fall(N));   
      end generate;

 	 dq24_31: for N in 24 to 31 generate
    dq_bit : v4_dq_iob 
      PORT MAP (
         CLK => CLK,
         CLK90 => CLK90,
         RESET => RESET90,
         DATA_DLYINC => data_idelay_inc(3),
         DATA_DLYCE => data_idelay_ce(3),
         DATA_DLYRST => data_idelay_rst(3),
         WRITE_DATA_RISE => wr_data_rise(N),
         WRITE_DATA_FALL => wr_data_fall(N),
         CTRL_WREN => wr_en_clk90_r3,
         DDR_DQ => DDR_DQ(N),
         READ_DATA_RISE => rd_data_rise(N),
         READ_DATA_FALL => rd_data_fall(N));   
      end generate;

    dq32_39: for N in 32 to 39 generate
    dq_bit : v4_dq_iob 
      PORT MAP (
         CLK => CLK,
         CLK90 => CLK90,
         RESET => RESET90,
         DATA_DLYINC => data_idelay_inc(4),
         DATA_DLYCE => data_idelay_ce(4),
         DATA_DLYRST => data_idelay_rst(4),
         WRITE_DATA_RISE => wr_data_rise(N),
         WRITE_DATA_FALL => wr_data_fall(N),
         CTRL_WREN => wr_en_clk90_r3,
         DDR_DQ => DDR_DQ(N),
         READ_DATA_RISE => rd_data_rise(N),
         READ_DATA_FALL => rd_data_fall(N));   
      end generate;

	dq40_47: for N in 40 to 47 generate
    dq_bit : v4_dq_iob 
      PORT MAP (
         CLK => CLK,
         CLK90 => CLK90,
         RESET => RESET90,
         DATA_DLYINC => data_idelay_inc(5),
         DATA_DLYCE => data_idelay_ce(5),
         DATA_DLYRST => data_idelay_rst(5),
         WRITE_DATA_RISE => wr_data_rise(N),
         WRITE_DATA_FALL => wr_data_fall(N),
         CTRL_WREN => wr_en_clk90_r3,
         DDR_DQ => DDR_DQ(N),
         READ_DATA_RISE => rd_data_rise(N),
         READ_DATA_FALL => rd_data_fall(N));   
      end generate;

	 dq48_55: for N in 48 to 55 generate
    dq_bit : v4_dq_iob 
      PORT MAP (
         CLK => CLK,
         CLK90 => CLK90,
         RESET => RESET90,
         DATA_DLYINC => data_idelay_inc(6),
         DATA_DLYCE => data_idelay_ce(6),
         DATA_DLYRST => data_idelay_rst(6),
         WRITE_DATA_RISE => wr_data_rise(N),
         WRITE_DATA_FALL => wr_data_fall(N),
         CTRL_WREN => wr_en_clk90_r3,
         DDR_DQ => DDR_DQ(N),
         READ_DATA_RISE => rd_data_rise(N),
         READ_DATA_FALL => rd_data_fall(N));   
      end generate;

	 dq56_63: for N in 56 to 63 generate
    dq_bit : v4_dq_iob 
      PORT MAP (
         CLK => CLK,
         CLK90 => CLK90,
         RESET => RESET90,
         DATA_DLYINC => data_idelay_inc(7),
         DATA_DLYCE => data_idelay_ce(7),
         DATA_DLYRST => data_idelay_rst(7),
         WRITE_DATA_RISE => wr_data_rise(N),
         WRITE_DATA_FALL => wr_data_fall(N),
         CTRL_WREN => wr_en_clk90_r3,
         DDR_DQ => DDR_DQ(N),
         READ_DATA_RISE => rd_data_rise(N),
         READ_DATA_FALL => rd_data_fall(N));   
      end generate;
END rtl;
