--******************************************************************************
--
--  Xilinx, Inc. 2005                 www.xilinx.com
--
--
--*******************************************************************************
--
--  File name :       data_tap_inc.vhd
--
--  Description :     This module implements the tap selection controller 
--                    for data bits associated with a strobe.
-- 
--                    
--  Date - revision : 1/16/2004
--                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
--
--  Author :          Maria George
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 - 2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  USE IEEE.std_logic_unsigned.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY data_tap_inc IS
   PORT (
      CLK                     : IN std_logic;   
      RESET                   : IN std_logic;   
      DATA_DLYINC             : OUT std_logic;   
      DATA_DLYCE              : OUT std_logic;   
      DATA_DLYRST             : OUT std_logic;   
      DATA_TAP_SEL_DONE       : OUT std_logic;   
      VALID_DATA_TAP_COUNT    : IN std_logic;   
      DATA_TAP_COUNT          : IN std_logic_vector(5 DOWNTO 0));   
END data_tap_inc;

ARCHITECTURE rtl OF data_tap_inc IS

   SIGNAL data_tap_inc_counter     :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL data_tap_sel_clk         :  std_logic;   
   SIGNAL data_tap_sel_r1          :  std_logic;   
   SIGNAL DATA_DLYINC_sig          :  std_logic;   
   SIGNAL DATA_DLYCE_sig         :  std_logic;   
   SIGNAL DATA_DLYRST_sig        :  std_logic;   
  
BEGIN
   DATA_DLYINC <= DATA_DLYINC_sig;
   DATA_DLYCE <= DATA_DLYCE_sig;
   DATA_DLYRST <= DATA_DLYRST_sig;
   DATA_TAP_SEL_DONE <= data_tap_sel_r1;
   
   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (RESET = '1') THEN
            data_tap_sel_clk <= '0';    
         ELSE
            IF (data_tap_inc_counter(5 DOWNTO 0) = "000001") THEN
               data_tap_sel_clk <= '1';    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (RESET = '1') THEN
            data_tap_sel_r1 <= '0';    
         ELSE
            data_tap_sel_r1 <= data_tap_sel_clk;    
         END IF;
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (RESET = '1') THEN
            -- Data IDELAY in no change mode
            
            DATA_DLYINC_sig <= '0';    
            DATA_DLYCE_sig <= '0';    
            DATA_DLYRST_sig <= '0';    
            data_tap_inc_counter(5 DOWNTO 0) <= "000000";    
         ELSE
            IF (VALID_DATA_TAP_COUNT = '1') THEN
               DATA_DLYINC_sig <= '0';    
               DATA_DLYCE_sig <= '0';    
               DATA_DLYRST_sig <= '1';    
               data_tap_inc_counter(5 DOWNTO 0) <= DATA_TAP_COUNT(5 
               DOWNTO 0);    
            ELSE
               IF (data_tap_inc_counter(5 DOWNTO 0) /= "000000") THEN
                  --Data IDELAY incremented
                  
                  DATA_DLYINC_sig <= '1';    
                  DATA_DLYCE_sig <= '1';    
                  DATA_DLYRST_sig <= '0';    
                  data_tap_inc_counter(5 DOWNTO 0) <= 
                  data_tap_inc_counter(5 DOWNTO 0) - "000001";    
               ELSE
                  --Data IDELAY no change mode
                  
                  DATA_DLYINC_sig <= '0';    
                  DATA_DLYCE_sig <= '0';    
                  DATA_DLYRST_sig <= '0';    
                  data_tap_inc_counter(5 DOWNTO 0) <= "000000";    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

END rtl;
