--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 - 2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY ddr_dimm IS
         PORT (
      Dq                      : INOUT std_logic_vector(63 DOWNTO 0);   
      Dqs                     : INOUT std_logic_vector(7 DOWNTO 0);   
      Addr                    : IN std_logic_vector(11 DOWNTO 0);   
      Ba                      : IN std_logic_vector(1 DOWNTO 0);   
      Clk0                    : IN std_logic;   
      Clk0_n                  : IN std_logic;   
      Clk1                    : IN std_logic;   
      Clk1_n                  : IN std_logic;   
      Clk2                    : IN std_logic;   
      Clk2_n                  : IN std_logic;   
      Cke0                    : IN std_logic;   
      Cke1                    : IN std_logic;   
      Ras_n                   : IN std_logic;   
      Cas_n                   : IN std_logic;   
      We_n                    : IN std_logic;   
      Dm                      : IN std_logic_vector(7 DOWNTO 0);   
      S0_n                    : IN std_logic;   
      S1_n                    : IN std_logic);   
END ddr_dimm;

ARCHITECTURE translated OF ddr_dimm IS

COMPONENT MT46V8M16 
    GENERIC (                                   -- Timing for -75Z CL2
        tCK       : TIME    :=  7.500 ns;
        tCH       : TIME    :=  3.375 ns;       -- 0.45*tCK
        tCL       : TIME    :=  3.375 ns;       -- 0.45*tCK
        tDH       : TIME    :=  0.500 ns;
        tDS       : TIME    :=  0.500 ns;
        tIH       : TIME    :=  0.900 ns;
        tIS       : TIME    :=  0.900 ns;
        tMRD      : TIME    := 15.000 ns;
        tRAS      : TIME    := 40.000 ns;
        --tRAP      : TIME    := 20.000 ns;
        tRC       : TIME    := 65.000 ns;
        tRFC      : TIME    := 75.000 ns;
        tRCD      : TIME    := 20.000 ns;
        tRP       : TIME    := 20.000 ns;
        tRRD      : TIME    := 15.000 ns;
        tWR       : TIME    := 15.000 ns;
        addr_bits : INTEGER := 12;
        data_bits : INTEGER := 16;
        cols_bits : INTEGER :=  9
    );
    PORT (
        Dq    : INOUT STD_LOGIC_VECTOR (data_bits - 1 DOWNTO 0) := (OTHERS => 'Z');
        Dqs   : INOUT STD_LOGIC_VECTOR (1 DOWNTO 0) := "ZZ";
        Addr  : IN    STD_LOGIC_VECTOR (addr_bits - 1 DOWNTO 0);
        Ba    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0);
        Clk   : IN    STD_LOGIC;
        Clk_n : IN    STD_LOGIC;
        Cke   : IN    STD_LOGIC;
        Cs_n  : IN    STD_LOGIC;
        Ras_n : IN    STD_LOGIC;
        Cas_n : IN    STD_LOGIC;
        We_n  : IN    STD_LOGIC;
        Dm    : IN    STD_LOGIC_VECTOR (1 DOWNTO 0)
    );
END COMPONENT;

BEGIN
 
   U1 : MT46V8M16 
      PORT MAP (
         Dq => Dq(15 DOWNTO 0),
         Dqs => Dqs(1 DOWNTO 0),
         Addr => Addr(11 DOWNTO 0),
         Ba => Ba(1 DOWNTO 0),
         Clk => Clk0,
         Clk_n => Clk0_n,
         Cke => Cke0,
         Cs_n => S0_n,
         Ras_n => Ras_n,
         Cas_n => Cas_n,
         We_n => We_n,
         Dm => Dm(1 DOWNTO 0));   
   
   -- <<X-HDL>> Can't find translated component 'ddr'. Port & generic names and types may not match. No template will be included
   U2 : MT46V8M16 
      PORT MAP (
         Dq => Dq(31 DOWNTO 16),
         Dqs => Dqs(3 DOWNTO 2),
         Addr => Addr(11 DOWNTO 0),
         Ba => Ba(1 DOWNTO 0),
         Clk => Clk0,
         Clk_n => Clk0_n,
         Cke => Cke0,
         Cs_n => S0_n,
         Ras_n => Ras_n,
         Cas_n => Cas_n,
         We_n => We_n,
         Dm => Dm(3 DOWNTO 2));   
   
   -- <<X-HDL>> Can't find translated component 'ddr'. Port & generic names and types may not match. No template will be included
   U3 : MT46V8M16 
      PORT MAP (
         Dq => Dq(47 DOWNTO 32),
         Dqs => Dqs(5 DOWNTO 4),
         Addr => Addr(11 DOWNTO 0),
         Ba => Ba(1 DOWNTO 0),
         Clk => Clk1,
         Clk_n => Clk1_n,
         Cke => Cke0,
         Cs_n => S0_n,
         Ras_n => Ras_n,
         Cas_n => Cas_n,
         We_n => We_n,
         Dm => Dm(5 DOWNTO 4));   
   
   -- <<X-HDL>> Can't find translated component 'ddr'. Port & generic names and types may not match. No template will be included
   U4 : MT46V8M16 
      PORT MAP (
         Dq => Dq(63 DOWNTO 48),
         Dqs => Dqs(7 DOWNTO 6),
         Addr => Addr(11 DOWNTO 0),
         Ba => Ba(1 DOWNTO 0),
         Clk => Clk1,
         Clk_n => Clk1_n,
         Cke => Cke0,
         Cs_n => S0_n,
         Ras_n => Ras_n,
         Cas_n => Cas_n,
         We_n => We_n,
         Dm => Dm(7 DOWNTO 6));   
   
   -- <<X-HDL>> Can't find translated component 'ddr'. Port & generic names and types may not match. No template will be included
   U5 : MT46V8M16 
      PORT MAP (
         Dq => Dq(15 DOWNTO 0),
         Dqs => Dqs(1 DOWNTO 0),
         Addr => Addr(11 DOWNTO 0),
         Ba => Ba(1 DOWNTO 0),
         Clk => Clk0,
         Clk_n => Clk0_n,
         Cke => Cke1,
         Cs_n => S1_n,
         Ras_n => Ras_n,
         Cas_n => Cas_n,
         We_n => We_n,
         Dm => Dm(1 DOWNTO 0));   
   
   -- <<X-HDL>> Can't find translated component 'ddr'. Port & generic names and types may not match. No template will be included
   U6 : MT46V8M16 
      PORT MAP (
         Dq => Dq(31 DOWNTO 16),
         Dqs => Dqs(3 DOWNTO 2),
         Addr => Addr(11 DOWNTO 0),
         Ba => Ba(1 DOWNTO 0),
         Clk => Clk0,
         Clk_n => Clk0_n,
         Cke => Cke1,
         Cs_n => S1_n,
         Ras_n => Ras_n,
         Cas_n => Cas_n,
         We_n => We_n,
         Dm => Dm(3 DOWNTO 2));   
   
   -- <<X-HDL>> Can't find translated component 'ddr'. Port & generic names and types may not match. No template will be included
   U7 : MT46V8M16 
      PORT MAP (
         Dq => Dq(47 DOWNTO 32),
         Dqs => Dqs(5 DOWNTO 4),
         Addr => Addr(11 DOWNTO 0),
         Ba => Ba(1 DOWNTO 0),
         Clk => Clk1,
         Clk_n => Clk1_n,
         Cke => Cke1,
         Cs_n => S1_n,
         Ras_n => Ras_n,
         Cas_n => Cas_n,
         We_n => We_n,
         Dm => Dm(5 DOWNTO 4));   
   
   -- <<X-HDL>> Can't find translated component 'ddr'. Port & generic names and types may not match. No template will be included
   U8 : MT46V8M16 
      PORT MAP (
         Dq => Dq(63 DOWNTO 48),
         Dqs => Dqs(7 DOWNTO 6),
         Addr => Addr(11 DOWNTO 0),
         Ba => Ba(1 DOWNTO 0),
         Clk => Clk1,
         Clk_n => Clk1_n,
         Cke => Cke1,
         Cs_n => S1_n,
         Ras_n => Ras_n,
         Cas_n => Cas_n,
         We_n => We_n,
         Dm => Dm(7 DOWNTO 6));   
   

END translated;
