--*******************************************************************************
--
--  File name :       top.vhd
--
--  Description :     This module is the top level for the physical layer. 
-- 
--                    
--  Date - revision : 3/03/2004
--                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 - 2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY ddr_top IS
   PORT (
         DDR_CLK90               : IN std_logic;   
         DDR_CLKDIV              : IN std_logic;   
         CLK200_N                : IN std_logic;   
         CLK200_P                : IN std_logic;   
         USER_CLK                : IN std_logic;   
         SYS_RESET_IN            : IN std_logic;   
         DDR_DIMM_RAS_L          : OUT std_logic;   
         DDR_DIMM_CAS_L          : OUT std_logic;   
         DDR_DIMM_WE_L           : OUT std_logic;   
         DDR_DIMM_CKE0           : OUT std_logic;   
         DDR_DIMM_CKE1           : OUT std_logic;   
         DDR_DIMM_CS_N0_7        : OUT std_logic;   
         DDR_DIMM_CS_N8_15       : OUT std_logic;   
         DDR_DIMM_DQ             : INOUT std_logic_vector(63 DOWNTO 0);   
         DDR_DIMM_DQS            : INOUT std_logic_vector(7 DOWNTO 0);   
         DDR_DIMM_CLK0_P         : OUT std_logic;   
         DDR_DIMM_CLK0_N         : OUT std_logic;   
         DDR_DIMM_CLK1_P         : OUT std_logic;   
         DDR_DIMM_CLK1_N         : OUT std_logic;   
         DDR_DIMM_CLK2_P         : OUT std_logic;   
         DDR_DIMM_CLK2_N         : OUT std_logic;   
         DDR_DIMM_BA             : OUT std_logic_vector(1 DOWNTO 0);   
         DDR_DIMM_ADDRESS        : OUT std_logic_vector(11 DOWNTO 0);   
         DDR_DIMM_SA             : OUT std_logic_vector(2 DOWNTO 0);   
         DDR_DIMM_SDA            : INOUT std_logic;   
         DDR_DIMM_SCL            : OUT std_logic;   
         DDR_DIMM_DM             : OUT std_logic_vector(7 DOWNTO 0);   
         PCI_ADDR                : IN std_logic_vector(26 DOWNTO 0);   
         PCI_DATA_IN             : IN std_logic_vector(63 DOWNTO 0);   
         PCI_WRITE_VALID         : IN std_logic;   
         PCI_CBEA                : IN std_logic_vector(7 DOWNTO 0);   
         PCI_WRITE               : IN std_logic;   
         PCI_HIT                 : IN std_logic;   
         PCI_HIT_EARLY           : IN std_logic;   
         READ_DATA_OUT           : OUT std_logic_vector(63 DOWNTO 0);   
         READ_DATA_VALID         : OUT std_logic;   
         DDR_NOT_READY           : OUT std_logic;   
         DCM_LOCK                : IN std_logic;   
         S_WAIT                  : OUT std_logic;   
         S_DONE                  : IN std_logic);   
END ddr_top;

ARCHITECTURE rtl OF ddr_top IS

   COMPONENT clk_module
    PORT (
         SYS_CLK_IN              : IN std_logic;   
         DDR_CLK90               : IN std_logic;   
         DDR_CLKDIV              : IN std_logic;   
         REF_CLK200_IN           : IN std_logic;   
         CLK                     : OUT std_logic;   
         CLK90                   : OUT std_logic;   
         CLK200                  : OUT std_logic;   
         REFRESH_CLK             : OUT std_logic;   
         DDR_DIMM_CLK0_P         : OUT std_logic;   
         DDR_DIMM_CLK0_N         : OUT std_logic;   
         DDR_DIMM_CLK1_P         : OUT std_logic;   
         DDR_DIMM_CLK1_N         : OUT std_logic;   
         DDR_DIMM_CLK2_P         : OUT std_logic;   
         DDR_DIMM_CLK2_N         : OUT std_logic);  
   END COMPONENT;

   COMPONENT data_path
   PORT (
      CLK                     : IN std_logic;   
      CLK90                   : IN std_logic;   
      CLK200                  : IN std_logic;   
      RESET0                  : IN std_logic;   
      RESET90                 : IN std_logic;   
      CTRL_DUMMYREAD_START    : IN std_logic;   --  dummy read enable does not need to be normalized
      WDF_DATA                : IN std_logic_vector(127 DOWNTO 0);   
      DM_DATA                 : IN std_logic_vector(15 DOWNTO 0);   
      CTRL_WREN               : IN std_logic;   
      CTRL_DQS_RST            : IN std_logic;   
      CTRL_DQS_EN             : IN std_logic;   
      DDR_DQ                  : INOUT std_logic_vector(63 DOWNTO 0);   
      DDR_DQS                 : INOUT std_logic_vector(7 DOWNTO 0);   
      DDR_DM                  : OUT std_logic_vector(7 DOWNTO 0);   
      SEL_DONE                : OUT std_logic;   
      READ_DATA_RISE          : OUT std_logic_vector(63 DOWNTO 0);   
      READ_DATA_FALL          : OUT std_logic_vector(63 DOWNTO 0);   
      FIRST_RISING            : OUT std_logic_vector(3 DOWNTO 0);   
      test_tap                : OUT std_logic_vector(5 DOWNTO 0));  
   END COMPONENT;

   COMPONENT ddr_controller
   PORT (
      clk266                  : IN std_logic;   
      refresh_clk             : IN std_logic;   
      rst                     : IN std_logic;   
      addr_fifo               : IN std_logic_vector(26 DOWNTO 0);   
      cmd_fifo                : IN std_logic_vector(23 DOWNTO 0);   
      phy_Dly_Slct_Done       : IN std_logic;   
      addr_fifo_empty         : IN std_logic;   
      cmd_fifo_empty          : IN std_logic;   
      ctrl_Dummyread_Start    : OUT std_logic;   
      ctrl_af_RdEn            : OUT std_logic;   
      ctrl_Wdf_RdEn           : OUT std_logic;   
      wdf_empty               : IN std_logic;   
      ctrl_cmd_RdEn           : OUT std_logic;   
      ctrl_Dqs_Rst            : OUT std_logic;   
      ctrl_Dqs_En             : OUT std_logic;   
      ctrl_Dm_En              : OUT std_logic_vector(15 DOWNTO 0);   
      ctrl_WrEn               : OUT std_logic;   
      ctrl_RdEn               : OUT std_logic;   
      ctrl_init_done          : OUT std_logic;   
      ddr_address             : OUT std_logic_vector(11 DOWNTO 0);   
      ddr_ba                  : OUT std_logic_vector(1 DOWNTO 0);   
      ddr_ras_L               : OUT std_logic;   
      ddr_cas_L               : OUT std_logic;   
      ddr_we_L                : OUT std_logic;   
      ddr_cs_L                : OUT std_logic_vector(1 DOWNTO 0);   
      ddr_cke                 : OUT std_logic;   
      ddr_odt                 : OUT std_logic;   
      not_ready               : OUT std_logic;   
      burst_half              : IN std_logic;   
      PCI_HIT                 : IN std_logic;   
      PCI_HIT_EARLY           : IN std_logic;   
      PCI_BE                  : IN std_logic_vector(7 DOWNTO 0);   
      PCI_ADDR_EARLY          : IN std_logic_vector(26 DOWNTO 0);   
      PCI_CBEA_EARLY          : IN std_logic_vector(7 DOWNTO 0);   
      first_read              : OUT std_logic;   
      S_DONE                  : IN std_logic);   
   END COMPONENT;

   COMPONENT user_interface
   PORT (
      USER_CLK                : IN std_logic;   
      DDR_CLK                 : IN std_logic;   
      RESET                   : IN std_logic;   
      FIRST_RISING            : IN std_logic_vector(3 DOWNTO 0);   
      READ_DATA_RISE          : IN std_logic_vector(63 DOWNTO 0);   
      READ_DATA_FALL          : IN std_logic_vector(63 DOWNTO 0);   
      READ_DATA_VALID         : OUT std_logic;   
      CTRL_AF_RDEN            : IN std_logic;   
      AF_ADDR                 : OUT std_logic_vector(26 DOWNTO 0);   
      AF_EMPTY                : OUT std_logic;   
      APP_WDF_DATA            : IN std_logic_vector(63 DOWNTO 0);   
      APP_WDF_WREN            : IN std_logic;   
      CTRL_WDF_RDEN           : IN std_logic;   
      WDF_DATA                : OUT std_logic_vector(127 DOWNTO 0);   
      WDF_ALMOST_FULL         : OUT std_logic;   
      WDF_EMPTY               : OUT std_logic;   
      CTRL_CMD_RDEN           : IN std_logic;   
      CMD_ADDR                : OUT std_logic_vector(23 DOWNTO 0);   
      CMD_EMPTY               : OUT std_logic;   
      PCI_ADDR                : IN std_logic_vector(26 DOWNTO 0);   
      PCI_DATA_IN             : IN std_logic_vector(63 DOWNTO 0);   
      PCI_CBEA                : IN std_logic_vector(7 DOWNTO 0);   
      PCI_WRITE               : IN std_logic;   
      PCI_HIT                 : IN std_logic;   
      PCI_HIT_DDR             : OUT std_logic;   
      PCI_HIT_EARLY           : IN std_logic;   
      PCI_BE                  : OUT std_logic_vector(7 DOWNTO 0);   
      READ_DATA_OUT           : OUT std_logic_vector(63 DOWNTO 0);   
      BURST_HALF              : OUT std_logic;   
      FIRST_READ              : IN std_logic;   
      S_WAIT                  : OUT std_logic;   
      NOT_READY               : IN std_logic;   
      S_DONE                  : IN std_logic;   
      S_DONE_DDR              : OUT std_logic);   
   END COMPONENT;


   SIGNAL DDR_DIMM_READ_EN_OUT     :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL sys_clk_in               :  std_logic;   
   SIGNAL clk200_in                :  std_logic;   
   SIGNAL clk_0                    :  std_logic;   
   SIGNAL clk_90                   :  std_logic;   
   SIGNAL clk_200                  :  std_logic;   
   SIGNAL system_rst               :  std_logic;   
   SIGNAL system_rst90             :  std_logic;   
   SIGNAL wr_df_data               :  std_logic_vector(127 DOWNTO 0);   
   SIGNAL dq_tap_sel_done          :  std_logic;   
   SIGNAL rd_data_valid            :  std_logic;   
   SIGNAL rd_data_fifo_out         :  std_logic_vector(127 DOWNTO 0);   
   SIGNAL rd_data_rise             :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL rd_data_fall             :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL rd_en_delayed_rise       :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL rd_en_delayed_fall       :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL rising_first             :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL app_af_addr              :  std_logic_vector(31 DOWNTO 0);   
   SIGNAL app_af_wren              :  std_logic;   
   SIGNAL addr_fifo_empty          :  std_logic;   
   SIGNAL app_wr_df_data           :  std_logic_vector(127 DOWNTO 0);   
   SIGNAL app_wr_df_wren           :  std_logic;   
   SIGNAL app_cmd_addr             :  std_logic_vector(31 DOWNTO 0);   
   SIGNAL app_cmd_wren             :  std_logic;   
   SIGNAL wr_df_almost_full        :  std_logic;   
   SIGNAL rd_af_almost_full        :  std_logic;   
   SIGNAL ref_clk                  :  std_logic;   
   SIGNAL wr_af_addr               :  std_logic_vector(26 DOWNTO 0);   
   SIGNAL rd_af_addr               :  std_logic_vector(23 DOWNTO 0);   
   SIGNAL cmd_register             :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL ctrl_wr_af_rden          :  std_logic;   
   SIGNAL ctrl_wr_df_rden          :  std_logic;   
   SIGNAL ctrl_rd_af_rden          :  std_logic;   
   SIGNAL ctrl_initz_done          :  std_logic;   
   SIGNAL ctrl_dummy_rden          :  std_logic;   
   SIGNAL ctrl_dqs_enable          :  std_logic;   
   SIGNAL ctrl_dqs_reset           :  std_logic;   
   SIGNAL ctrl_wr_en               :  std_logic;   
   SIGNAL ddr_cs_L                 :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL cmd_fifo_empty           :  std_logic;   
   SIGNAL dm_data_w                :  std_logic_vector(15 DOWNTO 0);   
   SIGNAL wdf_empty                :  std_logic;   
   SIGNAL burst_half               :  std_logic;   
   SIGNAL pci_be                   :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL pci_hit_ddr              :  std_logic;   
   SIGNAL s_done_ddr               :  std_logic;   
   SIGNAL sys_rst_0                :  std_logic;   
   SIGNAL sys_rst_1                :  std_logic;   
   SIGNAL sys_rst_2                :  std_logic;   
   SIGNAL sys_rst_3                :  std_logic;   
   SIGNAL sys_rst                  :  std_logic;   
   SIGNAL sys_rst90_0              :  std_logic;   
   SIGNAL sys_rst90_1              :  std_logic;   
   SIGNAL sys_rst90_2              :  std_logic;   
   SIGNAL sys_rst90_3              :  std_logic;   
   SIGNAL sys_rst90                :  std_logic;   
   SIGNAL b_length                 :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL ddr_odt                  :  std_logic;   
   SIGNAL ctrl_rden                :  std_logic;   
   SIGNAL control0                 :  std_logic_vector(35 DOWNTO 0);   
   SIGNAL data1                    :  std_logic_vector(149 DOWNTO 0);   
   SIGNAL trig1                    :  std_logic_vector(2 DOWNTO 0);   
   SIGNAL state                    :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL pbs                      :  std_logic_vector(2 DOWNTO 0);   
   SIGNAL not_ready                :  std_logic;   
   SIGNAL raf_empty                :  std_logic;   
   SIGNAL system_rst_async         :  std_logic;   
   SIGNAL system_rst_sync_d1       :  std_logic;   
   SIGNAL system_rst90_sync_d1     :  std_logic;   
   SIGNAL system_rst_90            :  std_logic;   
   SIGNAL first_read               :  std_logic;   
   SIGNAL tap_count                :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL pci_write_temp           :  std_logic;   
   SIGNAL ddr_dimm_cke0_temp       :  std_logic; 
	 
BEGIN

   DDR_DIMM_CKE0 <= ddr_dimm_cke0_temp;
   DDR_DIMM_CKE1 <= ddr_dimm_cke0_temp;
   DDR_DIMM_CS_N0_7 <= ddr_cs_L(0);
   DDR_DIMM_CS_N8_15 <= ddr_cs_L(1);
   DDR_DIMM_SA <= "000";
   DDR_DIMM_SCL <= '0';
   DDR_NOT_READY <= not_ready;
   b_length <= "0100" ;
   system_rst_async <= SYS_RESET_IN ;
   DDR_DIMM_SDA <= '0' ;


   PROCESS (sys_clk_in)
   BEGIN
      IF (sys_clk_in'EVENT AND sys_clk_in = '1') THEN
         system_rst_sync_d1 <= system_rst_async;    
         system_rst <= system_rst_sync_d1;    
      END IF;
   END PROCESS;

   PROCESS (clk_90)
   BEGIN
      IF (clk_90'EVENT AND clk_90 = '1') THEN
         system_rst_90 <= system_rst;    
      END IF;
   END PROCESS;

   PROCESS (clk_0)
   BEGIN
      IF (clk_0'EVENT AND clk_0 = '1') THEN
         IF ((system_rst = '1') OR (DCM_LOCK = '0')) THEN
            sys_rst_0 <= '1';    
            sys_rst_1 <= '1';    
            sys_rst_2 <= '1';    
            sys_rst_3 <= '1';    
            sys_rst <= '1';    
         ELSE
            sys_rst_0 <= '0';    
            sys_rst_1 <= sys_rst_0;    
            sys_rst_2 <= sys_rst_1;    
            sys_rst_3 <= sys_rst_2;    
            sys_rst <= sys_rst_3;    
         END IF;
      END IF;
   END PROCESS;

   PROCESS (clk_90)
   BEGIN
      IF (clk_90'EVENT AND clk_90 = '1') THEN
         IF ((system_rst_90 = '1') OR (DCM_LOCK = '0')) THEN
            sys_rst90_0 <= '1';    
            sys_rst90_1 <= '1';    
            sys_rst90_2 <= '1';    
            sys_rst90_3 <= '1';    
            sys_rst90 <= '1';    
         ELSE
            sys_rst90_0 <= '0';    
            sys_rst90_1 <= sys_rst90_0;    
            sys_rst90_2 <= sys_rst90_1;    
            sys_rst90_3 <= sys_rst90_2;    
            sys_rst90 <= sys_rst90_3;    
         END IF;
      END IF;
   END PROCESS;

   sys_clk_in <= USER_CLK ;
   lvpecl_clk200_in : IBUFGDS_LVPECL_25 
      PORT MAP (
         O => clk200_in,
         I => CLK200_P,
         IB => CLK200_N);   
   
   
   clk_module0 : clk_module 
      PORT MAP (
         SYS_CLK_IN => sys_clk_in,
         DDR_CLK90 => DDR_CLK90,
         DDR_CLKDIV => DDR_CLKDIV,
         REF_CLK200_IN => clk200_in,
         CLK => clk_0,
         CLK90 => clk_90,
         CLK200 => clk_200,
         REFRESH_CLK => ref_clk,
         DDR_DIMM_CLK0_P => DDR_DIMM_CLK0_P,
         DDR_DIMM_CLK0_N => DDR_DIMM_CLK0_N,
         DDR_DIMM_CLK1_P => DDR_DIMM_CLK1_P,
         DDR_DIMM_CLK1_N => DDR_DIMM_CLK1_N,
         DDR_DIMM_CLK2_P => DDR_DIMM_CLK2_P,
         DDR_DIMM_CLK2_N => DDR_DIMM_CLK2_N);   
   
   data_path0 : data_path 
      PORT MAP (
         CLK => clk_0,
         CLK90 => clk_90,
         CLK200 => clk_200,
         RESET0 => sys_rst,
         RESET90 => sys_rst90,
         CTRL_DUMMYREAD_START => ctrl_dummy_rden,
         WDF_DATA => wr_df_data,
         DM_DATA => dm_data_w(15 DOWNTO 0),
         CTRL_WREN => ctrl_wr_en,
         CTRL_DQS_RST => ctrl_dqs_reset,
         CTRL_DQS_EN => ctrl_dqs_enable,
         DDR_DQ => DDR_DIMM_DQ,
         DDR_DQS => DDR_DIMM_DQS(7 DOWNTO 0),
         DDR_DM => DDR_DIMM_DM(7 DOWNTO 0),
         SEL_DONE => dq_tap_sel_done,
         READ_DATA_RISE => rd_data_rise,
         READ_DATA_FALL => rd_data_fall,
         FIRST_RISING => rising_first,
         test_tap => tap_count);   
   
   pci_write_temp <= (PCI_WRITE_VALID AND PCI_WRITE);
   
   user_interface0 : user_interface 
      PORT MAP (
         USER_CLK => USER_CLK,
         DDR_CLK => clk_0,
         RESET => sys_rst,
         FIRST_RISING => rising_first,
         READ_DATA_RISE => rd_data_rise,
         READ_DATA_FALL => rd_data_fall,
         READ_DATA_VALID => rd_data_valid,
         CTRL_AF_RDEN => ctrl_wr_af_rden,
         AF_ADDR => wr_af_addr,
         AF_EMPTY => addr_fifo_empty,
         APP_WDF_DATA => PCI_DATA_IN,
         APP_WDF_WREN => pci_write_temp,
         CTRL_WDF_RDEN => ctrl_wr_df_rden,
         WDF_DATA => wr_df_data,
         WDF_ALMOST_FULL => wr_df_almost_full,
         WDF_EMPTY => wdf_empty,
         CTRL_CMD_RDEN => ctrl_rd_af_rden,
         CMD_ADDR => rd_af_addr,
         CMD_EMPTY => cmd_fifo_empty,
         PCI_ADDR => PCI_ADDR,
         PCI_DATA_IN => PCI_DATA_IN,
         PCI_CBEA => PCI_CBEA,
         PCI_WRITE => PCI_WRITE,
         PCI_HIT => PCI_HIT,
         PCI_HIT_DDR => pci_hit_ddr,
         PCI_HIT_EARLY => PCI_HIT_EARLY,
         PCI_BE => pci_be(7 DOWNTO 0),
         READ_DATA_OUT => READ_DATA_OUT,
         BURST_HALF => burst_half,
         FIRST_READ => first_read,
         S_WAIT => S_WAIT,
         NOT_READY => not_ready,
         S_DONE => S_DONE,
         S_DONE_DDR => s_done_ddr);   
   
   
   ddr_controller0 : ddr_controller 
      PORT MAP (
         clk266 => clk_0,
         refresh_clk => ref_clk,
         rst => sys_rst,
         addr_fifo => wr_af_addr,
         cmd_fifo => rd_af_addr,
         phy_Dly_Slct_Done => dq_tap_sel_done,
         addr_fifo_empty => addr_fifo_empty,
         cmd_fifo_empty => cmd_fifo_empty,
         ctrl_Dummyread_Start => ctrl_dummy_rden,
         ctrl_af_RdEn => ctrl_wr_af_rden,
         ctrl_Wdf_RdEn => ctrl_wr_df_rden,
         ctrl_cmd_RdEn => ctrl_rd_af_rden,
         wdf_empty => wdf_empty,
         ctrl_init_done => ctrl_initz_done,
         ctrl_Dqs_Rst => ctrl_dqs_reset,
         ctrl_Dqs_En => ctrl_dqs_enable,
         ctrl_Dm_En => dm_data_w(15 DOWNTO 0),
         ctrl_WrEn => ctrl_wr_en,
         ctrl_RdEn => ctrl_rden,
         ddr_address => DDR_DIMM_ADDRESS,
         ddr_ba => DDR_DIMM_BA,
         ddr_ras_L => DDR_DIMM_RAS_L,
         ddr_cas_L => DDR_DIMM_CAS_L,
         ddr_we_L => DDR_DIMM_WE_L,
         ddr_cs_L => ddr_cs_L,
         ddr_cke => ddr_dimm_cke0_temp,
         ddr_odt => ddr_odt,
         not_ready => not_ready,
         burst_half => burst_half,
         PCI_HIT => pci_hit_ddr,
         PCI_HIT_EARLY => PCI_HIT_EARLY,
         PCI_BE => pci_be(7 DOWNTO 0),
         PCI_ADDR_EARLY => PCI_ADDR,
         PCI_CBEA_EARLY => PCI_CBEA,
         first_read => first_read,
         S_DONE => s_done_ddr);   
   

END rtl;
