--*******************************************************************************
--
--  File name :       dm_gen.vhd
--
--  Description :     This module contains logic to generate the data mask
--                    for a DDR-DIMM
--
-- 
--                    
--  Date - revision : 11/9/2005
--
--  Author :          Kraig Lund
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  USE IEEE.std_logic_unsigned.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY dm_gen IS
   PORT (
      clk                     : IN std_logic;   
      rst                     : IN std_logic;   
      addr_fifo               : IN std_logic_vector(26 DOWNTO 0);   
      cmd_fifo                : IN std_logic_vector(23 DOWNTO 0);   
      wrburst_cnt             : IN std_logic_vector(8 DOWNTO 0);   
      burst_n_dword           : IN std_logic;   
      first_write             : IN std_logic;         
      ctrl_Dm_En              : OUT std_logic_vector(15 DOWNTO 0);         
      pci_be                  : IN std_logic_vector(7 DOWNTO 0));   
END ENTITY dm_gen;

ARCHITECTURE rtl OF dm_gen IS
	type dm_state is (idle_dm, first_dm, second_dm, middle_burst_dm, 
	                  second_to_last_dm,last_dm);          

   SIGNAL ctrl_Dm_En_r             :  std_logic_vector(15 DOWNTO 0);   
   SIGNAL ctrl_Dm_En_r1            :  std_logic_vector(15 DOWNTO 0);   
   SIGNAL state_dm                 :  dm_state;   
   SIGNAL first_addr               :  std_logic_vector(13 DOWNTO 0);   
   SIGNAL last_addr                :  std_logic_vector(13 DOWNTO 0);   
   SIGNAL byte_count               :  std_logic_vector(11 DOWNTO 0);   
   SIGNAL byte_plus_addr           :  std_logic_vector(4 DOWNTO 2);   
   SIGNAL dm_sel                   :  std_logic_vector(2 DOWNTO 0);     
   --Data Mask (DM) Generation
   SIGNAL first_dm_beat            :  std_logic_vector(31 DOWNTO 0);   
   SIGNAL last_dm_beat             :  std_logic_vector(31 DOWNTO 0);   
   SIGNAL first_addr_w             :  std_logic_vector(14 DOWNTO 0);   
   SIGNAL last_addr_w              :  std_logic_vector(14 DOWNTO 0);     

BEGIN
   ctrl_Dm_En <= (ctrl_Dm_En_r1 OR pci_be(3 DOWNTO 0) & pci_be(7 DOWNTO 4)
	               & pci_be(3 DOWNTO 0) & pci_be(7 DOWNTO 4)) 
						WHEN (burst_n_dword = '0') 
						ELSE ctrl_Dm_En_r1;

   byte_plus_addr(4 DOWNTO 2) <= addr_fifo(4 DOWNTO 2) + byte_count(4 DOWNTO 2) ;
   byte_count <= cmd_fifo(19 DOWNTO 8) ;

   PROCESS (byte_count(11 DOWNTO 0), addr_fifo(4 DOWNTO 2), 
            byte_plus_addr(4 DOWNTO 2))
   BEGIN
      IF ((byte_count >= "000000000000") AND (byte_count < "000000100100"))    --less than 36 bytes
      THEN         
         first_addr(13 DOWNTO 0) <= '0' & "00" & byte_count(4 DOWNTO 2) 
		       	                      & addr_fifo(4 DOWNTO 2) & "00000";    
         last_addr(13 DOWNTO 0) <= '1' & "00" & byte_count(4 DOWNTO 2) 
			                           & addr_fifo(4 DOWNTO 2) & "00000";    
      ELSE
         first_addr(13 DOWNTO 0) <= '0' & '0' & "1001" 
			                            & addr_fifo(4 DOWNTO 2) & "00000";    
         last_addr(13 DOWNTO 0) <= '1' & '0' & "1001" 
			                           & byte_plus_addr(4 DOWNTO 2) & "00000";    
      END IF;
   END PROCESS;

   --State machine to drive DM Mux 
   
   PROCESS (clk)
   BEGIN
      IF (clk'EVENT AND clk = '1') THEN
         IF (rst = '1') THEN
            state_dm <= idle_dm;    
            dm_sel <= "000";    
         ELSE
            CASE state_dm IS
               WHEN idle_dm =>
                        dm_sel <= "000";    
                        IF (first_write = '1') THEN
                           state_dm <= first_dm;    
                        ELSE
                           state_dm <= idle_dm;    
                        END IF;
               WHEN first_dm =>
                        dm_sel <= "010";    
                        state_dm <= second_dm;    
               WHEN second_dm =>
                        dm_sel <= "011";    
                        IF (burst_n_dword = '0') THEN
                           state_dm <= idle_dm;    
                        ELSE
                           IF (wrburst_cnt > "000000011") THEN
                              state_dm <= middle_burst_dm;    
                           ELSE
                              state_dm <= second_to_last_dm;    
                           END IF;
                        END IF;
               WHEN middle_burst_dm =>
                        dm_sel <= "001";    
                        IF (wrburst_cnt > "000000011") THEN
                           state_dm <= middle_burst_dm;    
                        ELSE
                           state_dm <= second_to_last_dm;    
                        END IF;
               WHEN second_to_last_dm =>
                        dm_sel <= "100";    
                        state_dm <= last_dm;    
               WHEN last_dm =>
                        dm_sel <= "101";    
                        state_dm <= idle_dm;    
               WHEN OTHERS =>
                        NULL;
               
            END CASE;
         END IF;
      END IF;
   END PROCESS;

   --DM mux
   
   PROCESS (dm_sel, first_dm_beat, last_dm_beat)
   BEGIN
      CASE dm_sel IS
         WHEN "000" =>
                  ctrl_Dm_En_r(15 DOWNTO 0) <= "1111111111111111"; 
         WHEN "001" =>
                  ctrl_Dm_En_r(15 DOWNTO 0) <= "0000000000000000"; 
         WHEN "010" =>
                  ctrl_Dm_En_r(15 DOWNTO 0) <= first_dm_beat(31 
                  DOWNTO 16);    
         WHEN "011" =>
                  ctrl_Dm_En_r(15 DOWNTO 0) <= first_dm_beat(15 
                  DOWNTO 0);    
         WHEN "100" =>
                  ctrl_Dm_En_r(15 DOWNTO 0) <= last_dm_beat(31 
                  DOWNTO 16);    
         WHEN "101" =>
                  ctrl_Dm_En_r(15 DOWNTO 0) <= last_dm_beat(15 
                  DOWNTO 0);    
         WHEN OTHERS  =>
                  ctrl_Dm_En_r(15 DOWNTO 0) <= "1111111111111111"; 
         
      END CASE;
   END PROCESS;

   PROCESS (clk)
   BEGIN
      IF (clk'EVENT AND clk = '1') THEN
         ctrl_Dm_En_r1 <= ctrl_Dm_En_r;    
      END IF;
   END PROCESS;


   first_addr_w <= '0' & first_addr;
   last_addr_w <= '0' & last_addr;
  
   RAMB16_inst : RAMB16 
	   generic map (
      DOA_REG => 0, -- Optional output registers on the A port (0 or 1)
      DOB_REG => 0, -- Optional output registers on the B port (0 or 1)
      INIT_A => X"FFFFFFFFF", --  Initial values on A output port
      INIT_B => X"FFFFFFFFF", --  Initial values on B output port
      INVERT_CLK_DOA_REG => FALSE, -- Invert clock on A port output registers (TRUE or FALSE)
      INVERT_CLK_DOB_REG => FALSE, -- Invert clock on B port output registers (TRUE or FALSE)
      RAM_EXTENSION_A => "NONE", -- "UPPER", "LOWER" or "NONE" when cascaded
      RAM_EXTENSION_B => "NONE", -- "UPPER", "LOWER" or "NONE" when cascaded
      READ_WIDTH_A => 36, -- Valid values are 1,2,4,9,18 or 36 
      READ_WIDTH_B => 36, -- Valid values are 1,2,4,9,18 or 36 
      SIM_COLLISION_CHECK => "ALL", -- Collision check enable "ALL", "WARNING_ONLY", "GENERATE_X_ONLY" 
                                   -- or "NONE" 
      SRVAL_A => X"FFFFFFFFF", --  Port A ouput value upon SSR assertion
      SRVAL_B => X"FFFFFFFFF", --  Port B ouput value upon SSR assertion
      WRITE_MODE_A => "WRITE_FIRST", --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      WRITE_MODE_B => "WRITE_FIRST", --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      WRITE_WIDTH_A => 36, -- Valid values are 1,2,4,9,18 or 36
      WRITE_WIDTH_B => 36, -- Valid values are 1,2,4,9,18 or 36 
      INIT_00 => X"00000000000000000000000000000000000000000000000000000000_ffffffff",
      INIT_01 => X"fffffff0_ffffff0f_fffff0ff_ffff0fff_fff0ffff_ff0fffff_f0ffffff_0fffffff",
      INIT_02 => X"fffffff0_ffffff00_fffff00f_ffff00ff_fff00fff_ff00ffff_f00fffff_00ffffff",
      INIT_03 => X"fffffff0_ffffff00_fffff000_ffff000f_fff000ff_ff000fff_f000ffff_000fffff",
      INIT_04 => X"fffffff0_ffffff00_fffff000_ffff0000_fff0000f_ff0000ff_f0000fff_0000ffff",
      INIT_05 => X"fffffff0_ffffff00_fffff000_ffff0000_fff00000_ff00000f_f00000ff_00000fff",
      INIT_06 => X"fffffff0_ffffff00_fffff000_ffff0000_fff00000_ff000000_f000000f_000000ff",
      INIT_07 => X"fffffff0_ffffff00_fffff000_ffff0000_fff00000_ff000000_f0000000_0000000f",
      INIT_08 => X"fffffff0_ffffff00_fffff000_ffff0000_fff00000_ff000000_f0000000_00000000",
      INIT_09 => X"fffffff0_ffffff00_fffff000_ffff0000_fff00000_ff000000_f0000000_00000000",
      INIT_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_10 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_11 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_12 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_13 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_14 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_15 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_16 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_17 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_18 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_19 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_1A => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_1B => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_1C => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_1D => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_1E => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_1F => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_21 => X"ffffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff",
      INIT_22 => X"0fffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff",
      INIT_23 => X"00ffffff_0fffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff",
      INIT_24 => X"000fffff_00ffffff_0fffffff_ffffffff_ffffffff_ffffffff_ffffffff_ffffffff",
      INIT_25 => X"0000ffff_000fffff_00ffffff_0fffffff_ffffffff_ffffffff_ffffffff_ffffffff",
      INIT_26 => X"00000fff_0000ffff_000fffff_00ffffff_0fffffff_ffffffff_ffffffff_ffffffff",
      INIT_27 => X"000000ff_00000fff_0000ffff_000fffff_00ffffff_0fffffff_ffffffff_ffffffff",
      INIT_28 => X"0000000f_000000ff_00000fff_0000ffff_000fffff_00ffffff_0fffffff_ffffffff",
      INIT_29 => X"0000000f_000000ff_00000fff_0000ffff_000fffff_00ffffff_0fffffff_00000000",
      INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
      INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
      INITP_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INITP_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INITP_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INITP_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
      INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      PORT MAP (
         DOA => first_dm_beat,
         DOB => last_dm_beat,
         DOPA => open,
         DOPB => open,
         ADDRA => first_addr_w,
         ADDRB => last_addr_w,
         CASCADEINA => '0',
         CASCADEINB => '0',
         CLKA => clk,
         CLKB => clk,
         DIA => X"00000000",
         DIB => X"00000000",
         DIPA => "0000",
         DIPB => "0000",
         ENA => '1',
         ENB => '1',
         REGCEA => '0',
         REGCEB => '0',
         SSRA => rst,
         SSRB => rst,
         WEA => "0000",
         WEB => "0000");   
   
END ARCHITECTURE rtl;
