--*******************************************************************************
--
--  File name :       pcb_delay.vhd
--
--  Description :     This module creates a physical delay for a bi-directional
--                    signal - not synthesizable
-- 
--                    
--  Date - revision : 11/11/2005
--
--  Author :          Kraig Lund
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

library IEEE; 
use IEEE.Std_Logic_1164.all; 

entity pcb_delay is 
generic (board_delay : time); 
port (a : inout Std_Logic := 'Z';
      b : inout Std_Logic := 'Z'); 
		end pcb_delay; 


		
architecture behavioral of pcb_delay is 
type semaphore_state is (NoPass, PassAtoB, PassBtoA);
signal semaphore : semaphore_state := NoPass;

begin 

process(a,b)
begin
-- semaphore selects which inout gets to drive the signal
if(semaphore = NoPass)then
   if(a='Z' and b='Z')then 
     semaphore <= NoPass; --if in NoPass state and both are 'Z' then stay in NoPass state
   else
     if((a = '0' or a = '1') and b = 'Z')then
       semaphore <= PassAtoB; --if in NoPass state A is being driven then give control to A
     else
       if((b = '0' or b ='1') and a = 'Z')then
         semaphore <= PassBtoA; -- if in NoPass state and B is being drive then give control to B
       end if;
     end if;
   end if;
end if;

if(semaphore = PassAtoB)then
   if (a='Z' and b = 'Z')then
      semaphore <= NoPass; -- if A has control but both inout are 'Z' then go back to NoPass
   else
      semaphore <= PassAtoB; -- otherwise A still gets control
   end if;
end if;

if(semaphore = PassBtoA)then
   if (a = 'Z' and b ='Z')then
       semaphore <= NoPass; -- if B has control but both inout are 'Z' then go back to NoPass
   else
       semaphore <= PassBtoA; -- otherwise B still gets control
   end if;
end if;
end process;



process(semaphore,a,b)
begin

if(semaphore = NoPass)then
a <= 'Z'; -- Both inouts get high impedance when in NoPass
b <= 'Z';
end if;
if(semaphore = PassAtoB)then
b <= transport a after board_delay; -- create delay on b
a <= 'Z'; -- drive a with 'Z'
end if;
if(semaphore = PassBtoA)then
a <= transport b after board_delay; -- create delay on a
b <= 'Z'; -- drive b with 'Z'
end if;
end process;

end behavioral; 
