--------------------------------------------------------------------------
--
--  File:   pcix_lc.vhd
--  Rev:    5.0.0
--
--  This is a lower-level VHDL module which serves as a wrapper
--  for the PCIX interface.  This module makes use of Unified Library
--  Primitives.  Do not modify this file.
--
--  Copyright (c) 2005 Xilinx, Inc.  All rights reserved.
--
--------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;



library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

-- Declare the entity.

entity PCIX_LC is
  port (
        AD_IO           : inout std_logic_vector( 63 downto 0);
        CBE_IO          : inout std_logic_vector(  7 downto 0);
        PAR_IO          : inout std_logic;
        PAR64_IO        : inout std_logic;
        FRAME_IO        : inout std_logic;
        REQ64_IO        : inout std_logic;
        TRDY_IO         : inout std_logic;
        IRDY_IO         : inout std_logic;
        STOP_IO         : inout std_logic;
        DEVSEL_IO       : inout std_logic;
        ACK64_IO        : inout std_logic;
        IDSEL_I         : in    std_logic;
        PERR_IO         : inout std_logic;
        SERR_IO         : inout std_logic;
        INT_O           : out   std_logic;
        PME_O           : out   std_logic;
        REQ_O           : out   std_logic;
        GNT_I           : in    std_logic;
        RST_I           : in    std_logic;
        CLK_I           : in    std_logic;
        S_DATA_OUT      : out   std_logic_vector( 63 downto 0);
        S_CBEA_OUT      : out   std_logic_vector(  7 downto 0);
        S_DATA_IN       : in    std_logic_vector( 63 downto 0);
        S_ADDL_VLD      : out   std_logic;
        S_ADDH_VLD      : out   std_logic;
        S_ATTR_VLD      : out   std_logic;
        S_DATL_VLD      : out   std_logic;
        S_DATH_VLD      : out   std_logic;
        S_DATA_NXT      : out   std_logic;
        S_TABORT        : in    std_logic;
        S_RETRY         : in    std_logic;
        S_DISCON        : in    std_logic;
        S_NXTADB        : in    std_logic;
        S_WAIT          : in    std_logic;
        S_SPLIT         : in    std_logic;
        S_WRITE         : out   std_logic;
        S_DONE          : out   std_logic;
        S_HIT           : out   std_logic_vector( 15 downto 0);
        M_DATA_OUT      : out   std_logic_vector( 63 downto 0);
        M_CBEA_IN       : in    std_logic_vector(  7 downto 0);
        M_DATA_IN       : in    std_logic_vector( 63 downto 0);
        M_ADDL_VLD      : out   std_logic;
        M_ADDH_VLD      : out   std_logic;
        M_ATTR_VLD      : out   std_logic;
        M_DATL_VLD      : out   std_logic;
        M_DATH_VLD      : out   std_logic;
        M_DATA_NXT      : out   std_logic;
        M_MABORT        : out   std_logic;
        M_RETRY         : out   std_logic;
        M_NXTADB        : out   std_logic;
        M_DISCON        : out   std_logic;
        M_TABORT        : out   std_logic;
        M_SPLIT         : out   std_logic;
        M_FINISH        : in    std_logic;
        M_DONE          : out   std_logic;
        M_REQ           : in    std_logic;
        PERR_N          : out   std_logic;
        SERR_N          : out   std_logic;
        INT_N           : in    std_logic;
        MS_STAT         : out   std_logic_vector( 95 downto 0);
        PME_N           : in    std_logic;
        PM_STAT         : out   std_logic_vector( 47 downto 0);
        CSR             : out   std_logic_vector( 31 downto 0);
        CSR_SYSTEMERR_N : in    std_logic; 
        CSRX_UNEXPECTED : in    std_logic; 
        CSRX_DISCARDED  : in    std_logic; 
        CSRX            : out   std_logic_vector( 47 downto 0);
        PCIW_EN         : out   std_logic;
        PCIX_EN         : out   std_logic;
        RTR             : out   std_logic;
        CFG             : in    std_logic_vector(511 downto 0);
        RST             : out   std_logic;
        CLK             : inout std_logic;
		  CLK90           : out   std_logic;
		  CLKDIV          : out   std_logic;
		  DCM_LOCK        : out   std_logic
  );
end PCIX_LC;


architecture WRAPPER of PCIX_LC is

  attribute syn_edif_bit_format : string; 
  attribute syn_edif_scalar_format : string;
  attribute syn_noclockbuf : boolean; 
  attribute syn_hier : string; 
  attribute syn_edif_bit_format of WRAPPER : architecture is "%u<%i>"; 
  attribute syn_edif_scalar_format of WRAPPER : architecture is "%u"; 
  attribute syn_noclockbuf of WRAPPER : architecture is true; 
  attribute syn_hier of WRAPPER : architecture is "hard";   

  component IOBUF_PCIX
    port (
        O               : out   std_logic;
        IO              : inout std_logic;
        I               : in    std_logic;
        T               : in    std_logic
    );
  end component;

  component IBUF_PCIX
    port (
        O               : out   std_logic;
        I               : in    std_logic
    );
  end component; 

  component OBUFT_PCIX
    port (
        O               : out   std_logic;
        I               : in    std_logic;
        T               : in    std_logic
    );
  end component; 

  component IBUFG_PCIX
    port ( 
        O               : out   std_logic;
        I               : in    std_logic
    ); 
  end component;  

  component BUFG
    port ( 
        O               : out   std_logic;
        I               : in    std_logic
    ); 
  end component;  

  component INV
    port ( 
        O               : out   std_logic;
        I               : in    std_logic
    ); 
  end component;

-- component DCM_BASE
--    port (
--        CLKIN           : in    std_logic;
--        CLKFB           : in    std_logic;
--        RST             : in    std_logic;
--        CLK0            : out   std_logic;
--        CLK90           : out   std_logic;
--        CLK180          : out   std_logic;
--        CLK270          : out   std_logic;
--        CLK2X           : out   std_logic;
--        CLKDV           : out   std_logic;
--        LOCKED          : out   std_logic
--    );
--  end component;

  component PCIX_CORE
    port (
        ADO             : out   std_logic_vector ( 63 downto 0);
        ADI             : in    std_logic_vector ( 63 downto 0);
        ADT             : out   std_logic_vector ( 63 downto 0);
        CBO             : out   std_logic_vector (  7 downto 0);
        CBI             : in    std_logic_vector (  7 downto 0);
        CBT             : out   std_logic_vector (  7 downto 0);
        PARO            : out   std_logic;
        PARI            : in    std_logic;
        PART            : out   std_logic;
        PAR64O          : out   std_logic;
        PAR64I          : in    std_logic;
        PAR64T          : out   std_logic;
        FRAMEO          : out   std_logic;
        FRAMEI          : in    std_logic;
        FRAMET          : out   std_logic;
        REQ64O          : out   std_logic;
        REQ64I          : in    std_logic;
        REQ64T          : out   std_logic;
        TRDYO           : out   std_logic;
        TRDYI           : in    std_logic;
        TRDYT           : out   std_logic;
        IRDYO           : out   std_logic;
        IRDYI           : in    std_logic;
        IRDYT           : out   std_logic;
        STOPO           : out   std_logic;
        STOPI           : in    std_logic;
        STOPT           : out   std_logic;
        DEVSELO         : out   std_logic;
        DEVSELI         : in    std_logic;
        DEVSELT         : out   std_logic;
        ACK64O          : out   std_logic;
        ACK64I          : in    std_logic;
        ACK64T          : out   std_logic;
        PERRO           : out   std_logic;
        PERRI           : in    std_logic;
        PERRT           : out   std_logic;
        SERRO           : out   std_logic;
        SERRI           : in    std_logic;
        SERRT           : out   std_logic;
        INTO            : out   std_logic;
        INTT            : out   std_logic;
        PMEO            : out   std_logic;
        PMET            : out   std_logic;
        REQO            : out   std_logic;
        REQT            : out   std_logic;
        GNTI            : in    std_logic;
        IDSELI          : in    std_logic;
        S_DATA_OUT      : out   std_logic_vector ( 63 downto 0);
        S_CBEA_OUT      : out   std_logic_vector (  7 downto 0);
        S_DATA_IN       : in    std_logic_vector ( 63 downto 0);
        S_ADDL_VLD      : out   std_logic;
        S_ADDH_VLD      : out   std_logic;
        S_ATTR_VLD      : out   std_logic;
        S_DATL_VLD      : out   std_logic;
        S_DATH_VLD      : out   std_logic;
        S_DATA_NXT      : out   std_logic;
        S_TABORT        : in    std_logic;
        S_RETRY         : in    std_logic;
        S_DISCON        : in    std_logic;
        S_NXTADB        : in    std_logic;
        S_WAIT          : in    std_logic;
        S_SPLIT         : in    std_logic;
        S_WRITE         : out   std_logic;
        S_DONE          : out   std_logic;
        S_HIT           : out   std_logic_vector ( 15 downto 0);
        M_DATA_OUT      : out   std_logic_vector ( 63 downto 0);
        M_CBEA_IN       : in    std_logic_vector (  7 downto 0);
        M_DATA_IN       : in    std_logic_vector ( 63 downto 0);
        M_ADDL_VLD      : out   std_logic;
        M_ADDH_VLD      : out   std_logic;
        M_ATTR_VLD      : out   std_logic;
        M_DATL_VLD      : out   std_logic;
        M_DATH_VLD      : out   std_logic;
        M_DATA_NXT      : out   std_logic;
        M_MABORT        : out   std_logic;
        M_RETRY         : out   std_logic;
        M_NXTADB        : out   std_logic;
        M_DISCON        : out   std_logic;
        M_TABORT        : out   std_logic;
        M_SPLIT         : out   std_logic;
        M_FINISH        : in    std_logic;
        M_DONE          : out   std_logic;
        M_REQ           : in    std_logic;
        PERR_N          : out   std_logic;
        SERR_N          : out   std_logic;
        INT_N           : in    std_logic;
        MS_STAT         : out   std_logic_vector ( 95 downto 0);
        PME_N           : in    std_logic;
        PM_STAT         : out   std_logic_vector ( 47 downto 0);
        CSR             : out   std_logic_vector ( 31 downto 0);
        CSR_SYSTEMERR_N : in    std_logic; 
        CSRX_UNEXPECTED : in    std_logic; 
        CSRX_DISCARDED  : in    std_logic; 
        CSRX            : out   std_logic_vector ( 47 downto 0);
        PCIW_EN         : out   std_logic;
        PCIX_EN         : out   std_logic;
        BUS_RST         : in    std_logic;
        RTR             : out   std_logic;
        CFG             : in    std_logic_vector (511 downto 0);
        RST             : out   std_logic;
        BYP             : out   std_logic;
        LCK             : in    std_logic;
        CLK             : in    std_logic
    );
  end component;


  signal RST_INV        : std_logic;
  signal BUS_RST        : std_logic;
  signal CLK_NUB        : std_logic;
  signal CLK_OUT        : std_logic;
  signal BYP            : std_logic;
  signal LCK            : std_logic;
  signal LO             : std_logic;
  signal HI             : std_logic;
  signal NC90           : std_logic;
  signal NC180          : std_logic;
  signal NC270          : std_logic;
  signal NC2X           : std_logic;
  signal NCDV           : std_logic;
  signal NCLCK          : std_logic;
  signal ADO            : std_logic_vector (63 downto 0);
  signal ADI            : std_logic_vector (63 downto 0);
  signal ADT            : std_logic_vector (63 downto 0);
  signal CBO            : std_logic_vector ( 7 downto 0);
  signal CBI            : std_logic_vector ( 7 downto 0);
  signal CBT            : std_logic_vector ( 7 downto 0);
  signal PARO           : std_logic;
  signal PARI           : std_logic;
  signal PART           : std_logic;
  signal PAR64O         : std_logic;
  signal PAR64I         : std_logic;
  signal PAR64T         : std_logic;
  signal FRAMEO         : std_logic;
  signal FRAMEI         : std_logic;
  signal FRAMET         : std_logic;
  signal REQ64O         : std_logic;
  signal REQ64I         : std_logic;
  signal REQ64T         : std_logic;
  signal TRDYO          : std_logic;
  signal TRDYI          : std_logic;
  signal TRDYT          : std_logic;
  signal IRDYO          : std_logic;
  signal IRDYI          : std_logic;
  signal IRDYT          : std_logic;
  signal STOPO          : std_logic;
  signal STOPI          : std_logic;
  signal STOPT          : std_logic;
  signal DEVSELO        : std_logic;
  signal DEVSELI        : std_logic;
  signal DEVSELT        : std_logic;
  signal ACK64O         : std_logic;
  signal ACK64I         : std_logic;
  signal ACK64T         : std_logic;
  signal PERRO          : std_logic;
  signal PERRI          : std_logic;
  signal PERRT          : std_logic;
  signal SERRO          : std_logic;
  signal SERRI          : std_logic;
  signal SERRT          : std_logic;
  signal INTO           : std_logic;
  signal INTT           : std_logic;
  signal PMEO           : std_logic;
  signal PMET           : std_logic;
  signal REQO           : std_logic;
  signal REQT           : std_logic;
  signal GNTI           : std_logic;
  signal IDSELI         : std_logic;
  signal CLK90_OUT      : std_logic;
  signal CLKDIV_OUT     : std_logic;


begin


  -- I/O instances for clock, reset, power, and ground.

  XPCI_CLK1 : IBUFG_PCIX port map
              ( O => CLK_NUB, I => CLK_I );
  XPCI_DLL : DCM_BASE generic map
              (DLL_FREQUENCY_MODE =>"LOW",
               DUTY_CYCLE_CORRECTION => TRUE,
               CLKDV_DIVIDE => 16.0)
				  port map
              ( CLKIN => CLK_NUB, CLKFB => CLK, RST => BUS_RST, CLK0 => CLK_OUT,
                CLK90 => CLK90_OUT, CLK180 => NC180, CLK270 => NC270,
                CLK2X => NC2X, CLKDV => CLKDIV_OUT,
                LOCKED => NCLCK);
  XPCI_CLK2 : BUFG port map
              ( O => CLK, I => CLK_OUT );
  XPCI_RST1 : IBUF_PCIX port map
              ( O => RST_INV, I => RST_I );
  XPCI_RST2 : INV port map
              ( O => BUS_RST, I => RST_INV );
  DDR_CLK90 : BUFG port map
              ( O => CLK90, I => CLK90_OUT );
  DDR_CLKDIV : BUFG port map
              ( O => CLKDIV, I => CLKDIV_OUT );

  LCK <= NCLCK;
  DCM_LOCK <= LCK;
  LO <= '0';
  HI <= '1';


  -- I/O instances for address and data.

  XPCI_AD63 : IOBUF_PCIX port map
              ( O => ADI(63), IO => AD_IO(63), I => ADO(63), T => ADT(63) );
  XPCI_AD62 : IOBUF_PCIX port map
              ( O => ADI(62), IO => AD_IO(62), I => ADO(62), T => ADT(62) );
  XPCI_AD61 : IOBUF_PCIX port map
              ( O => ADI(61), IO => AD_IO(61), I => ADO(61), T => ADT(61) );
  XPCI_AD60 : IOBUF_PCIX port map
              ( O => ADI(60), IO => AD_IO(60), I => ADO(60), T => ADT(60) );
  XPCI_AD59 : IOBUF_PCIX port map
              ( O => ADI(59), IO => AD_IO(59), I => ADO(59), T => ADT(59) );
  XPCI_AD58 : IOBUF_PCIX port map
              ( O => ADI(58), IO => AD_IO(58), I => ADO(58), T => ADT(58) );
  XPCI_AD57 : IOBUF_PCIX port map
              ( O => ADI(57), IO => AD_IO(57), I => ADO(57), T => ADT(57) );
  XPCI_AD56 : IOBUF_PCIX port map
              ( O => ADI(56), IO => AD_IO(56), I => ADO(56), T => ADT(56) );
  XPCI_AD55 : IOBUF_PCIX port map
              ( O => ADI(55), IO => AD_IO(55), I => ADO(55), T => ADT(55) );
  XPCI_AD54 : IOBUF_PCIX port map
              ( O => ADI(54), IO => AD_IO(54), I => ADO(54), T => ADT(54) );
  XPCI_AD53 : IOBUF_PCIX port map
              ( O => ADI(53), IO => AD_IO(53), I => ADO(53), T => ADT(53) );
  XPCI_AD52 : IOBUF_PCIX port map
              ( O => ADI(52), IO => AD_IO(52), I => ADO(52), T => ADT(52) );
  XPCI_AD51 : IOBUF_PCIX port map
              ( O => ADI(51), IO => AD_IO(51), I => ADO(51), T => ADT(51) );
  XPCI_AD50 : IOBUF_PCIX port map
              ( O => ADI(50), IO => AD_IO(50), I => ADO(50), T => ADT(50) );
  XPCI_AD49 : IOBUF_PCIX port map
              ( O => ADI(49), IO => AD_IO(49), I => ADO(49), T => ADT(49) );
  XPCI_AD48 : IOBUF_PCIX port map
              ( O => ADI(48), IO => AD_IO(48), I => ADO(48), T => ADT(48) );
  XPCI_AD47 : IOBUF_PCIX port map
              ( O => ADI(47), IO => AD_IO(47), I => ADO(47), T => ADT(47) );
  XPCI_AD46 : IOBUF_PCIX port map
              ( O => ADI(46), IO => AD_IO(46), I => ADO(46), T => ADT(46) );
  XPCI_AD45 : IOBUF_PCIX port map
              ( O => ADI(45), IO => AD_IO(45), I => ADO(45), T => ADT(45) );
  XPCI_AD44 : IOBUF_PCIX port map
              ( O => ADI(44), IO => AD_IO(44), I => ADO(44), T => ADT(44) );
  XPCI_AD43 : IOBUF_PCIX port map
              ( O => ADI(43), IO => AD_IO(43), I => ADO(43), T => ADT(43) );
  XPCI_AD42 : IOBUF_PCIX port map
              ( O => ADI(42), IO => AD_IO(42), I => ADO(42), T => ADT(42) );
  XPCI_AD41 : IOBUF_PCIX port map
              ( O => ADI(41), IO => AD_IO(41), I => ADO(41), T => ADT(41) );
  XPCI_AD40 : IOBUF_PCIX port map
              ( O => ADI(40), IO => AD_IO(40), I => ADO(40), T => ADT(40) );
  XPCI_AD39 : IOBUF_PCIX port map
              ( O => ADI(39), IO => AD_IO(39), I => ADO(39), T => ADT(39) );
  XPCI_AD38 : IOBUF_PCIX port map
              ( O => ADI(38), IO => AD_IO(38), I => ADO(38), T => ADT(38) );
  XPCI_AD37 : IOBUF_PCIX port map
              ( O => ADI(37), IO => AD_IO(37), I => ADO(37), T => ADT(37) );
  XPCI_AD36 : IOBUF_PCIX port map
              ( O => ADI(36), IO => AD_IO(36), I => ADO(36), T => ADT(36) );
  XPCI_AD35 : IOBUF_PCIX port map
              ( O => ADI(35), IO => AD_IO(35), I => ADO(35), T => ADT(35) );
  XPCI_AD34 : IOBUF_PCIX port map
              ( O => ADI(34), IO => AD_IO(34), I => ADO(34), T => ADT(34) );
  XPCI_AD33 : IOBUF_PCIX port map
              ( O => ADI(33), IO => AD_IO(33), I => ADO(33), T => ADT(33) );
  XPCI_AD32 : IOBUF_PCIX port map
              ( O => ADI(32), IO => AD_IO(32), I => ADO(32), T => ADT(32) );
  XPCI_AD31 : IOBUF_PCIX port map
              ( O => ADI(31), IO => AD_IO(31), I => ADO(31), T => ADT(31) );
  XPCI_AD30 : IOBUF_PCIX port map
              ( O => ADI(30), IO => AD_IO(30), I => ADO(30), T => ADT(30) );
  XPCI_AD29 : IOBUF_PCIX port map
              ( O => ADI(29), IO => AD_IO(29), I => ADO(29), T => ADT(29) );
  XPCI_AD28 : IOBUF_PCIX port map
              ( O => ADI(28), IO => AD_IO(28), I => ADO(28), T => ADT(28) );
  XPCI_AD27 : IOBUF_PCIX port map
              ( O => ADI(27), IO => AD_IO(27), I => ADO(27), T => ADT(27) );
  XPCI_AD26 : IOBUF_PCIX port map
              ( O => ADI(26), IO => AD_IO(26), I => ADO(26), T => ADT(26) );
  XPCI_AD25 : IOBUF_PCIX port map
              ( O => ADI(25), IO => AD_IO(25), I => ADO(25), T => ADT(25) );
  XPCI_AD24 : IOBUF_PCIX port map
              ( O => ADI(24), IO => AD_IO(24), I => ADO(24), T => ADT(24) );
  XPCI_AD23 : IOBUF_PCIX port map
              ( O => ADI(23), IO => AD_IO(23), I => ADO(23), T => ADT(23) );
  XPCI_AD22 : IOBUF_PCIX port map
              ( O => ADI(22), IO => AD_IO(22), I => ADO(22), T => ADT(22) );
  XPCI_AD21 : IOBUF_PCIX port map
              ( O => ADI(21), IO => AD_IO(21), I => ADO(21), T => ADT(21) );
  XPCI_AD20 : IOBUF_PCIX port map
              ( O => ADI(20), IO => AD_IO(20), I => ADO(20), T => ADT(20) );
  XPCI_AD19 : IOBUF_PCIX port map
              ( O => ADI(19), IO => AD_IO(19), I => ADO(19), T => ADT(19) );
  XPCI_AD18 : IOBUF_PCIX port map
              ( O => ADI(18), IO => AD_IO(18), I => ADO(18), T => ADT(18) );
  XPCI_AD17 : IOBUF_PCIX port map
              ( O => ADI(17), IO => AD_IO(17), I => ADO(17), T => ADT(17) );
  XPCI_AD16 : IOBUF_PCIX port map
              ( O => ADI(16), IO => AD_IO(16), I => ADO(16), T => ADT(16) );
  XPCI_AD15 : IOBUF_PCIX port map
              ( O => ADI(15), IO => AD_IO(15), I => ADO(15), T => ADT(15) );
  XPCI_AD14 : IOBUF_PCIX port map
              ( O => ADI(14), IO => AD_IO(14), I => ADO(14), T => ADT(14) );
  XPCI_AD13 : IOBUF_PCIX port map
              ( O => ADI(13), IO => AD_IO(13), I => ADO(13), T => ADT(13) );
  XPCI_AD12 : IOBUF_PCIX port map
              ( O => ADI(12), IO => AD_IO(12), I => ADO(12), T => ADT(12) );
  XPCI_AD11 : IOBUF_PCIX port map
              ( O => ADI(11), IO => AD_IO(11), I => ADO(11), T => ADT(11) );
  XPCI_AD10 : IOBUF_PCIX port map
              ( O => ADI(10), IO => AD_IO(10), I => ADO(10), T => ADT(10) );
  XPCI_AD9  : IOBUF_PCIX port map
              ( O => ADI( 9), IO => AD_IO( 9), I => ADO( 9), T => ADT( 9) );
  XPCI_AD8  : IOBUF_PCIX port map
              ( O => ADI( 8), IO => AD_IO( 8), I => ADO( 8), T => ADT( 8) );
  XPCI_AD7  : IOBUF_PCIX port map
              ( O => ADI( 7), IO => AD_IO( 7), I => ADO( 7), T => ADT( 7) );
  XPCI_AD6  : IOBUF_PCIX port map
              ( O => ADI( 6), IO => AD_IO( 6), I => ADO( 6), T => ADT( 6) );
  XPCI_AD5  : IOBUF_PCIX port map
              ( O => ADI( 5), IO => AD_IO( 5), I => ADO( 5), T => ADT( 5) );
  XPCI_AD4  : IOBUF_PCIX port map
              ( O => ADI( 4), IO => AD_IO( 4), I => ADO( 4), T => ADT( 4) );
  XPCI_AD3  : IOBUF_PCIX port map
              ( O => ADI( 3), IO => AD_IO( 3), I => ADO( 3), T => ADT( 3) );
  XPCI_AD2  : IOBUF_PCIX port map
              ( O => ADI( 2), IO => AD_IO( 2), I => ADO( 2), T => ADT( 2) );
  XPCI_AD1  : IOBUF_PCIX port map
              ( O => ADI( 1), IO => AD_IO( 1), I => ADO( 1), T => ADT( 1) );
  XPCI_AD0  : IOBUF_PCIX port map
              ( O => ADI( 0), IO => AD_IO( 0), I => ADO( 0), T => ADT( 0) );


  -- I/O instances for command and byte enable.

  XPCI_CB7 : IOBUF_PCIX port map
             ( O => CBI( 7), IO => CBE_IO( 7), I => CBO( 7), T => CBT( 7) );
  XPCI_CB6 : IOBUF_PCIX port map
             ( O => CBI( 6), IO => CBE_IO( 6), I => CBO( 6), T => CBT( 6) );
  XPCI_CB5 : IOBUF_PCIX port map
             ( O => CBI( 5), IO => CBE_IO( 5), I => CBO( 5), T => CBT( 5) );
  XPCI_CB4 : IOBUF_PCIX port map
             ( O => CBI( 4), IO => CBE_IO( 4), I => CBO( 4), T => CBT( 4) );
  XPCI_CB3 : IOBUF_PCIX port map
             ( O => CBI( 3), IO => CBE_IO( 3), I => CBO( 3), T => CBT( 3) );
  XPCI_CB2 : IOBUF_PCIX port map
             ( O => CBI( 2), IO => CBE_IO( 2), I => CBO( 2), T => CBT( 2) );
  XPCI_CB1 : IOBUF_PCIX port map
             ( O => CBI( 1), IO => CBE_IO( 1), I => CBO( 1), T => CBT( 1) );
  XPCI_CB0 : IOBUF_PCIX port map
             ( O => CBI( 0), IO => CBE_IO( 0), I => CBO( 0), T => CBT( 0) );


  -- I/O instances for parity.

  XPCI_PAR    : IOBUF_PCIX port map
                ( O => PARI, IO => PAR_IO, I => PARO, T => PART );
  XPCI_PAR64  : IOBUF_PCIX port map
                ( O => PAR64I, IO => PAR64_IO, I => PAR64O, T => PAR64T );


  -- I/O instances for control.

  XPCI_FRAME  : IOBUF_PCIX port map
              ( O => FRAMEI, IO => FRAME_IO, I => FRAMEO, T => FRAMET );
  XPCI_REQ64  : IOBUF_PCIX port map
              ( O => REQ64I, IO => REQ64_IO, I => REQ64O, T => REQ64T );
  XPCI_TRDY   : IOBUF_PCIX port map
              ( O => TRDYI, IO => TRDY_IO, I => TRDYO, T => TRDYT );
  XPCI_IRDY   : IOBUF_PCIX port map
              ( O => IRDYI, IO => IRDY_IO, I => IRDYO, T => IRDYT );
  XPCI_STOP   : IOBUF_PCIX port map
              ( O => STOPI, IO => STOP_IO, I => STOPO, T => STOPT );
  XPCI_DEVSEL : IOBUF_PCIX port map
              ( O => DEVSELI, IO => DEVSEL_IO, I => DEVSELO, T => DEVSELT );
  XPCI_ACK64  : IOBUF_PCIX port map
              ( O => ACK64I, IO => ACK64_IO, I => ACK64O, T => ACK64T );
  XPCI_PERR   : IOBUF_PCIX port map
              ( O => PERRI, IO => PERR_IO, I => PERRO, T => PERRT );
  XPCI_SERR   : IOBUF_PCIX port map
              ( O => SERRI, IO => SERR_IO, I => SERRO, T => SERRT );

  XPCI_INT    : OBUFT_PCIX port map
              ( O => INT_O, I => INTO, T => INTT );
  XPCI_PME    : OBUFT_PCIX port map
              ( O => PME_O, I => PMEO, T => PMET );
  XPCI_REQ    : OBUFT_PCIX port map
              ( O => REQ_O, I => REQO, T => REQT );

  XPCI_GNT    : IBUF_PCIX port map
              ( O => GNTI, I => GNT_I );
  XPCI_IDSEL  : IBUF_PCIX port map
              ( O => IDSELI, I => IDSEL_I );


  -- Instantiate PCIX interface.

  XPCI_CORE : PCIX_CORE port map (
    ADO                 => ADO,
    ADI                 => ADI,
    ADT                 => ADT,
    CBO                 => CBO,
    CBI                 => CBI,
    CBT                 => CBT,
    PARO                => PARO,
    PARI                => PARI,
    PART                => PART,
    PAR64O              => PAR64O,
    PAR64I              => PAR64I,
    PAR64T              => PAR64T,
    FRAMEO              => FRAMEO,
    FRAMEI              => FRAMEI,
    FRAMET              => FRAMET,
    REQ64O              => REQ64O,
    REQ64I              => REQ64I,
    REQ64T              => REQ64T,
    TRDYO               => TRDYO,
    TRDYI               => TRDYI,
    TRDYT               => TRDYT,
    IRDYO               => IRDYO,
    IRDYI               => IRDYI,
    IRDYT               => IRDYT,
    STOPO               => STOPO,
    STOPI               => STOPI,
    STOPT               => STOPT,
    DEVSELO             => DEVSELO,
    DEVSELI             => DEVSELI,
    DEVSELT             => DEVSELT,
    ACK64O              => ACK64O,
    ACK64I              => ACK64I,
    ACK64T              => ACK64T,
    PERRO               => PERRO,
    PERRI               => PERRI,
    PERRT               => PERRT,
    SERRO               => SERRO,
    SERRI               => SERRI,
    SERRT               => SERRT,
    INTO                => INTO,
    INTT                => INTT,
    PMEO                => PMEO,
    PMET                => PMET,
    REQO                => REQO,
    REQT                => REQT,
    GNTI                => GNTI,
    IDSELI              => IDSELI,
    S_DATA_OUT          => S_DATA_OUT,
    S_CBEA_OUT          => S_CBEA_OUT,
    S_DATA_IN           => S_DATA_IN,
    S_ADDL_VLD          => S_ADDL_VLD,
    S_ADDH_VLD          => S_ADDH_VLD,
    S_ATTR_VLD          => S_ATTR_VLD,
    S_DATL_VLD          => S_DATL_VLD,
    S_DATH_VLD          => S_DATH_VLD,
    S_DATA_NXT          => S_DATA_NXT,
    S_TABORT            => S_TABORT,
    S_RETRY             => S_RETRY,
    S_DISCON            => S_DISCON,
    S_NXTADB            => S_NXTADB,
    S_WAIT              => S_WAIT,
    S_SPLIT             => S_SPLIT,
    S_WRITE             => S_WRITE,
    S_DONE              => S_DONE,
    S_HIT               => S_HIT,
    M_DATA_OUT          => M_DATA_OUT,
    M_CBEA_IN           => M_CBEA_IN,
    M_DATA_IN           => M_DATA_IN,
    M_ADDL_VLD          => M_ADDL_VLD,
    M_ADDH_VLD          => M_ADDH_VLD,
    M_ATTR_VLD          => M_ATTR_VLD,
    M_DATL_VLD          => M_DATL_VLD,
    M_DATH_VLD          => M_DATH_VLD,
    M_DATA_NXT          => M_DATA_NXT,
    M_MABORT            => M_MABORT,
    M_RETRY             => M_RETRY,
    M_NXTADB            => M_NXTADB,
    M_DISCON            => M_DISCON,
    M_TABORT            => M_TABORT,
    M_SPLIT             => M_SPLIT,
    M_FINISH            => M_FINISH,
    M_DONE              => M_DONE,
    M_REQ               => M_REQ,
    PERR_N              => PERR_N,
    SERR_N              => SERR_N,
    INT_N               => INT_N,
    MS_STAT             => MS_STAT,
    PME_N               => PME_N,
    PM_STAT             => PM_STAT,
    CSR                 => CSR,
    CSR_SYSTEMERR_N     => CSR_SYSTEMERR_N,
    CSRX_UNEXPECTED     => CSRX_UNEXPECTED,
    CSRX_DISCARDED      => CSRX_DISCARDED,
    CSRX                => CSRX,
    PCIW_EN             => PCIW_EN,
    PCIX_EN             => PCIX_EN,
    BUS_RST             => BUS_RST,
    RTR                 => RTR,
    CFG                 => CFG,
    RST                 => RST,
    BYP                 => BYP,
    LCK                 => LCK,
    CLK                 => CLK
  );


end WRAPPER;
