-- **********************************************************************
--   File:   pcix_top.v
--   Rev:    5.0.0
--   This is the top-level template file for VHDL designs.
--   The user should place his backend application design in the
--   userapp module.
--   Copyright (c) 2003 - 2005 Xilinx, Inc.  All rights reserved.
-- **********************************************************************
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
 --              provided to you "as is". Xilinx and its licensors make and you 
 --              receive no warranties or conditions, express, implied, 
 --              statutory or otherwise, and Xilinx specifically disclaims any 
 --              implied warranties of merchantability, non-infringement, or 
 --              fitness for a particular purpose. Xilinx does not warrant that 
 --              the functions contained in these designs will meet your 
 --              requirements, or that the operation of these designs will be 
 --              uninterrupted or error free, or that defects in the Designs 
 --              will be corrected. Furthermore, Xilinx does not warrant or 
 --              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 - 2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************
-- Declare the module.

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY PCIX_TOP IS
   PORT (
      AD                      : INOUT std_logic_vector(63 DOWNTO 0);   
      CBE                     : INOUT std_logic_vector(7 DOWNTO 0);   
      PAR                     : INOUT std_logic;   
      PAR64                   : INOUT std_logic;   
      FRAME_N                 : INOUT std_logic;   
      REQ64_N                 : INOUT std_logic;   
      TRDY_N                  : INOUT std_logic;   
      IRDY_N                  : INOUT std_logic;   
      STOP_N                  : INOUT std_logic;   
      DEVSEL_N                : INOUT std_logic;   
      ACK64_N                 : INOUT std_logic;   
      IDSEL                   : IN std_logic;   
      PERR_N                  : INOUT std_logic;   
      SERR_N                  : INOUT std_logic;   
      INTA_N                  : OUT std_logic;   
      PMEA_N                  : OUT std_logic;   
      REQ_N                   : OUT std_logic;   
      GNT_N                   : IN std_logic;   
      RST_N                   : IN std_logic;   
      PCLK                    : IN std_logic;   
      FPGA_RTR                : OUT std_logic;   
      --DDR interface

      CLK200_N                : IN std_logic;   
      CLK200_P                : IN std_logic;   
      DDR_DIMM_RAS_L          : OUT std_logic;   
      DDR_DIMM_CAS_L          : OUT std_logic;   
      DDR_DIMM_WE_L           : OUT std_logic;   
      DDR_DIMM_CKE0           : OUT std_logic;   
      DDR_DIMM_CKE1           : OUT std_logic;   
      DDR_DIMM_CS_N0_7        : OUT std_logic;   
      DDR_DIMM_CS_N8_15       : OUT std_logic;   
      DDR_DIMM_DQ             : INOUT std_logic_vector(63 DOWNTO 0);   
      DDR_DIMM_DQS            : INOUT std_logic_vector(7 DOWNTO 0);   
      DDR_DIMM_CLK0_P         : OUT std_logic;   
      DDR_DIMM_CLK0_N         : OUT std_logic;   
      DDR_DIMM_CLK1_P         : OUT std_logic;   
      DDR_DIMM_CLK1_N         : OUT std_logic;   
      DDR_DIMM_CLK2_P         : OUT std_logic;   
      DDR_DIMM_CLK2_N         : OUT std_logic;   
      DDR_DIMM_BA             : OUT std_logic_vector(1 DOWNTO 0);   
      DDR_DIMM_ADDRESS        : OUT std_logic_vector(11 DOWNTO 0);   
      DDR_DIMM_SA             : OUT std_logic_vector(2 DOWNTO 0);   
      DDR_DIMM_SDA            : INOUT std_logic;   
      DDR_DIMM_SCL            : OUT std_logic;   
      DDR_DIMM_DM             : OUT std_logic_vector(7 DOWNTO 0));   
END PCIX_TOP;

ARCHITECTURE rtl OF PCIX_TOP IS

  attribute syn_edif_bit_format : string; 
  attribute syn_edif_scalar_format : string;
  attribute syn_noclockbuf : boolean; 
  attribute syn_hier : string; 
  attribute syn_edif_bit_format of RTL : architecture is "%u<%i>"; 
  attribute syn_edif_scalar_format of RTL : architecture is "%u"; 
  attribute syn_noclockbuf of RTL : architecture is true; 
  attribute syn_hier of RTL : architecture is "hard";  

  component CFG
    port (
        CFG             : out   std_logic_vector (511 downto 0)
    );
  end component;

  component PCIX_LC
    port (
       AD_IO           : inout std_logic_vector( 63 downto 0);
        CBE_IO          : inout std_logic_vector(  7 downto 0);
        PAR_IO          : inout std_logic;
        PAR64_IO        : inout std_logic;
        FRAME_IO        : inout std_logic;
        REQ64_IO        : inout std_logic;
        TRDY_IO         : inout std_logic;
        IRDY_IO         : inout std_logic;
        STOP_IO         : inout std_logic;
        DEVSEL_IO       : inout std_logic;
        ACK64_IO        : inout std_logic;
        IDSEL_I         : in    std_logic;
        PERR_IO         : inout std_logic;
        SERR_IO         : inout std_logic;
        INT_O           : out   std_logic;
        PME_O           : out   std_logic;
        REQ_O           : out   std_logic;
        GNT_I           : in    std_logic;
        RST_I           : in    std_logic;
        CLK_I           : in    std_logic;
        S_DATA_OUT      : out   std_logic_vector( 63 downto 0);
        S_CBEA_OUT      : out   std_logic_vector(  7 downto 0);
        S_DATA_IN       : in    std_logic_vector( 63 downto 0);
        S_ADDL_VLD      : out   std_logic;
        S_ADDH_VLD      : out   std_logic;
        S_ATTR_VLD      : out   std_logic;
        S_DATL_VLD      : out   std_logic;
        S_DATH_VLD      : out   std_logic;
        S_DATA_NXT      : out   std_logic;
        S_TABORT        : in    std_logic;
        S_RETRY         : in    std_logic;
        S_DISCON        : in    std_logic;
        S_NXTADB        : in    std_logic;
        S_WAIT          : in    std_logic;
        S_SPLIT         : in    std_logic;
        S_WRITE         : out   std_logic;
        S_DONE          : out   std_logic;
        S_HIT           : out   std_logic_vector( 15 downto 0);
        M_DATA_OUT      : out   std_logic_vector( 63 downto 0);
        M_CBEA_IN       : in    std_logic_vector(  7 downto 0);
        M_DATA_IN       : in    std_logic_vector( 63 downto 0);
        M_ADDL_VLD      : out   std_logic;
        M_ADDH_VLD      : out   std_logic;
        M_ATTR_VLD      : out   std_logic;
        M_DATL_VLD      : out   std_logic;
        M_DATH_VLD      : out   std_logic;
        M_DATA_NXT      : out   std_logic;
        M_MABORT        : out   std_logic;
        M_RETRY         : out   std_logic;
        M_NXTADB        : out   std_logic;
        M_DISCON        : out   std_logic;
        M_TABORT        : out   std_logic;
        M_SPLIT         : out   std_logic;
        M_FINISH        : in    std_logic;
        M_DONE          : out   std_logic;
        M_REQ           : in    std_logic;
        PERR_N          : out   std_logic;
        SERR_N          : out   std_logic;
        INT_N           : in    std_logic;
        MS_STAT         : out   std_logic_vector( 95 downto 0);
        PME_N           : in    std_logic;
        PM_STAT         : out   std_logic_vector( 47 downto 0);
        CSR             : out   std_logic_vector( 31 downto 0);
        CSR_SYSTEMERR_N : in    std_logic; 
        CSRX_UNEXPECTED : in    std_logic; 
        CSRX_DISCARDED  : in    std_logic; 
        CSRX            : out   std_logic_vector( 47 downto 0);
        PCIW_EN         : out   std_logic;
        PCIX_EN         : out   std_logic;
        RTR             : out   std_logic;
        CFG             : in    std_logic_vector(511 downto 0);
        RST             : out   std_logic;
        CLK             : inout std_logic;
		  CLK90           : out   std_logic;
		  CLKDIV          : out   std_logic;
		  DCM_LOCK        : out   std_logic
    );
  end component;

  component USERAPP
   PORT (
      S_DATA_OUT              : IN std_logic_vector(63 DOWNTO 0);   --  target address, attribute and write data bus
      S_CBEA_OUT              : IN std_logic_vector(7 DOWNTO 0);   --  command byte enables
      S_DATA_IN               : OUT std_logic_vector(63 DOWNTO 0);   --  target read data bus
      S_ADDL_VLD              : IN std_logic;   --  lower 32-bit address valid
      S_ADDH_VLD              : IN std_logic;   --  upper 32-bit address valid
      S_ATTR_VLD              : IN std_logic;   --  target attributes valid
      S_DATL_VLD              : IN std_logic;   --  lower 32-bit data valid
      S_DATH_VLD              : IN std_logic;   --  upper 32-bit data valid
      S_DATA_NXT              : IN std_logic;   --  next data requested
      S_TABORT                : OUT std_logic;   --  slave target abort
      S_RETRY                 : OUT std_logic;   --  slave retry
      S_DISCON                : OUT std_logic;   --  slave disconnect request
      S_NXTADB                : OUT std_logic;   
      S_WAIT                  : OUT std_logic;   --  target wait: PCI-X only prior to 1st data transfer
      S_SPLIT                 : OUT std_logic;   --  split transaction requested
      S_WRITE                 : IN std_logic;   --  target write request
      S_DONE                  : IN std_logic;   --  target done
      S_HIT                   : IN std_logic_vector(15 DOWNTO 0);   --  
      M_DATA_OUT              : IN std_logic_vector(63 DOWNTO 0);   
      M_CBEA_IN               : OUT std_logic_vector(7 DOWNTO 0);   
      M_DATA_IN               : OUT std_logic_vector(63 DOWNTO 0);   
      M_ADDL_VLD              : IN std_logic;   
      M_ADDH_VLD              : IN std_logic;   
      M_ATTR_VLD              : IN std_logic;   
      M_DATL_VLD              : IN std_logic;   
      M_DATH_VLD              : IN std_logic;   
      M_DATA_NXT              : IN std_logic;   
      M_MABORT                : IN std_logic;   
      M_RETRY                 : IN std_logic;   
      M_NXTADB                : IN std_logic;   
      M_DISCON                : IN std_logic;   
      M_TABORT                : IN std_logic;   
      M_SPLIT                 : IN std_logic;   
      M_FINISH                : OUT std_logic;   
      M_DONE                  : IN std_logic;   
      M_REQ                   : OUT std_logic;   
      PERR_N                  : IN std_logic;   
      SERR_N                  : IN std_logic;   
      INT_N                   : OUT std_logic;   
      MS_STAT                 : IN std_logic_vector(95 DOWNTO 0);   
      PME_N                   : OUT std_logic;   
      PM_STAT                 : IN std_logic_vector(47 DOWNTO 0);   
      CSR                     : IN std_logic_vector(31 DOWNTO 0);   
      CSR_SYSTEMERR_N         : OUT std_logic;   
      CSRX_UNEXPECTED         : OUT std_logic;   
      CSRX_DISCARDED          : OUT std_logic;   
      CSRX                    : IN std_logic_vector(47 DOWNTO 0);   
      PCIW_EN                 : IN std_logic;   
      PCIX_EN                 : IN std_logic;   
      RTR                     : IN std_logic;   
      CFG                     : IN std_logic_vector(511 DOWNTO 0);   
      RST                     : IN std_logic;   
      CLK                     : IN std_logic;   
      FPGA_RTR                : OUT std_logic;   
      --DDR interface

      DDR_CLK90               : IN std_logic;   
      DDR_CLKDIV              : IN std_logic;   
      CLK200_N                : IN std_logic;   
      CLK200_P                : IN std_logic;   
      USER_CLK                : IN std_logic;   
      SYS_RESET_IN            : IN std_logic;   
      DDR_DIMM_RAS_L          : OUT std_logic;   
      DDR_DIMM_CAS_L          : OUT std_logic;   
      DDR_DIMM_WE_L           : OUT std_logic;   
      DDR_DIMM_CKE0           : OUT std_logic;   
      DDR_DIMM_CKE1           : OUT std_logic;   
      DDR_DIMM_CS_N0_7        : OUT std_logic;   
      DDR_DIMM_CS_N8_15       : OUT std_logic;   
      DDR_DIMM_DQ             : INOUT std_logic_vector(63 DOWNTO 0);   
      DDR_DIMM_DQS            : INOUT std_logic_vector(7 DOWNTO 0);   
      DDR_DIMM_CLK0_P         : OUT std_logic;   
      DDR_DIMM_CLK0_N         : OUT std_logic;   
      DDR_DIMM_CLK1_P         : OUT std_logic;   
      DDR_DIMM_CLK1_N         : OUT std_logic;   
      DDR_DIMM_CLK2_P         : OUT std_logic;   
      DDR_DIMM_CLK2_N         : OUT std_logic;   
      DDR_DIMM_BA             : OUT std_logic_vector(1 DOWNTO 0);   
      DDR_DIMM_ADDRESS        : OUT std_logic_vector(11 DOWNTO 0);   
      DDR_DIMM_SA             : OUT std_logic_vector(2 DOWNTO 0);   
      DDR_DIMM_SDA            : INOUT std_logic;   
      DDR_DIMM_SCL            : OUT std_logic;   
      DDR_DIMM_DM             : OUT std_logic_vector(7 DOWNTO 0);   
      DCM_LOCK                : IN std_logic);  
  end component;

   -- Internal wiring to connect instances.
   SIGNAL S_DATA_OUT               :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL S_CBEA_OUT               :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL S_DATA_IN                :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL S_ADDL_VLD               :  std_logic;   
   SIGNAL S_ADDH_VLD               :  std_logic;   
   SIGNAL S_ATTR_VLD               :  std_logic;   
   SIGNAL S_DATL_VLD               :  std_logic;   
   SIGNAL S_DATH_VLD               :  std_logic;   
   SIGNAL S_DATA_NXT               :  std_logic;   
   SIGNAL S_TABORT                 :  std_logic;   
   SIGNAL S_RETRY                  :  std_logic;   
   SIGNAL S_DISCON                 :  std_logic;   
   SIGNAL S_NXTADB                 :  std_logic;   
   SIGNAL S_WAIT                   :  std_logic;   
   SIGNAL S_SPLIT                  :  std_logic;   
   SIGNAL S_WRITE                  :  std_logic;   
   SIGNAL S_DONE                   :  std_logic;   
   SIGNAL S_HIT                    :  std_logic_vector(15 DOWNTO 0);   
   SIGNAL M_DATA_OUT               :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL M_CBEA_IN                :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL M_DATA_IN                :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL M_ADDL_VLD               :  std_logic;   
   SIGNAL M_ADDH_VLD               :  std_logic;   
   SIGNAL M_ATTR_VLD               :  std_logic;   
   SIGNAL M_DATL_VLD               :  std_logic;   
   SIGNAL M_DATH_VLD               :  std_logic;   
   SIGNAL M_DATA_NXT               :  std_logic;   
   SIGNAL M_MABORT                 :  std_logic;   
   SIGNAL M_RETRY                  :  std_logic;   
   SIGNAL M_NXTADB                 :  std_logic;   
   SIGNAL M_DISCON                 :  std_logic;   
   SIGNAL M_TABORT                 :  std_logic;   
   SIGNAL M_SPLIT                  :  std_logic;   
   SIGNAL M_FINISH                 :  std_logic;   
   SIGNAL M_DONE                   :  std_logic;   
   SIGNAL M_REQ                    :  std_logic;   
   SIGNAL UPERR_N                  :  std_logic;   
   SIGNAL USERR_N                  :  std_logic;   
   SIGNAL INT_N                    :  std_logic;   
   SIGNAL MS_STAT                  :  std_logic_vector(95 DOWNTO 0);   
   SIGNAL PME_N                    :  std_logic;   
   SIGNAL PM_STAT                  :  std_logic_vector(47 DOWNTO 0);   
   SIGNAL CSR                      :  std_logic_vector(31 DOWNTO 0);   
   SIGNAL CSR_SYSTEMERR_N          :  std_logic;   
   SIGNAL CSRX_UNEXPECTED          :  std_logic;   
   SIGNAL CSRX_DISCARDED           :  std_logic;   
   SIGNAL CSRX                     :  std_logic_vector(47 DOWNTO 0);   
   SIGNAL PCIW_EN                  :  std_logic;   
   SIGNAL PCIX_EN                  :  std_logic;   
   SIGNAL RTR                      :  std_logic;   
   SIGNAL CFG_BUS                  :  std_logic_vector(511 DOWNTO 0);   
   SIGNAL RST                      :  std_logic;   
   SIGNAL CLK                      :  std_logic;   
   SIGNAL CLK90                    :  std_logic;   
   SIGNAL CLKDIV                   :  std_logic;   
   SIGNAL dcm_lock                 :  std_logic;   
   
BEGIN
   
   XPCI_CFG : CFG 
      PORT MAP (
         CFG => CFG_BUS);   
      
   -- Instantiation of the PCIX interface.
   XPCI_WRAP : PCIX_LC 
      PORT MAP (
         AD_IO => AD,
         CBE_IO => CBE,
         PAR_IO => PAR,
         PAR64_IO => PAR64,
         FRAME_IO => FRAME_N,
         REQ64_IO => REQ64_N,
         TRDY_IO => TRDY_N,
         IRDY_IO => IRDY_N,
         STOP_IO => STOP_N,
         DEVSEL_IO => DEVSEL_N,
         ACK64_IO => ACK64_N,
         IDSEL_I => IDSEL,
         PERR_IO => PERR_N,
         SERR_IO => SERR_N,
         INT_O => INTA_N,
         PME_O => PMEA_N,
         REQ_O => REQ_N,
         GNT_I => GNT_N,
         RST_I => RST_N,
         CLK_I => PCLK,
         S_DATA_OUT => S_DATA_OUT,
         S_CBEA_OUT => S_CBEA_OUT,
         S_DATA_IN => S_DATA_IN,
         S_ADDL_VLD => S_ADDL_VLD,
         S_ADDH_VLD => S_ADDH_VLD,
         S_ATTR_VLD => S_ATTR_VLD,
         S_DATL_VLD => S_DATL_VLD,
         S_DATH_VLD => S_DATH_VLD,
         S_DATA_NXT => S_DATA_NXT,
         S_TABORT => S_TABORT,
         S_RETRY => S_RETRY,
         S_DISCON => S_DISCON,
         S_NXTADB => S_NXTADB,
         S_WAIT => S_WAIT,
         S_SPLIT => S_SPLIT,
         S_WRITE => S_WRITE,
         S_DONE => S_DONE,
         S_HIT => S_HIT,
         M_DATA_OUT => M_DATA_OUT,
         M_CBEA_IN => M_CBEA_IN,
         M_DATA_IN => M_DATA_IN,
         M_ADDL_VLD => M_ADDL_VLD,
         M_ADDH_VLD => M_ADDH_VLD,
         M_ATTR_VLD => M_ATTR_VLD,
         M_DATL_VLD => M_DATL_VLD,
         M_DATH_VLD => M_DATH_VLD,
         M_DATA_NXT => M_DATA_NXT,
         M_MABORT => M_MABORT,
         M_RETRY => M_RETRY,
         M_NXTADB => M_NXTADB,
         M_DISCON => M_DISCON,
         M_TABORT => M_TABORT,
         M_SPLIT => M_SPLIT,
         M_FINISH => M_FINISH,
         M_DONE => M_DONE,
         M_REQ => M_REQ,
         PERR_N => UPERR_N,
         SERR_N => USERR_N,
         INT_N => INT_N,
         MS_STAT => MS_STAT,
         PME_N => PME_N,
         PM_STAT => PM_STAT,
         CSR => CSR,
         CSR_SYSTEMERR_N => CSR_SYSTEMERR_N,
         CSRX_UNEXPECTED => CSRX_UNEXPECTED,
         CSRX_DISCARDED => CSRX_DISCARDED,
         CSRX => CSRX,
         PCIW_EN => PCIW_EN,
         PCIX_EN => PCIX_EN,
         RTR => RTR,
         CFG => CFG_BUS,
         RST => RST,
         CLK => CLK,
         CLK90 => CLK90,
         CLKDIV => CLKDIV,
         DCM_LOCK => dcm_lock);   
   
   
   -- Instantiation of the user application.
  
   XPCI_USER : USERAPP 
      PORT MAP (
         S_DATA_OUT => S_DATA_OUT,
         S_CBEA_OUT => S_CBEA_OUT,
         S_DATA_IN => S_DATA_IN,
         S_ADDL_VLD => S_ADDL_VLD,
         S_ADDH_VLD => S_ADDH_VLD,
         S_ATTR_VLD => S_ATTR_VLD,
         S_DATL_VLD => S_DATL_VLD,
         S_DATH_VLD => S_DATH_VLD,
         S_DATA_NXT => S_DATA_NXT,
         S_TABORT => S_TABORT,
         S_RETRY => S_RETRY,
         S_DISCON => S_DISCON,
         S_NXTADB => S_NXTADB,
         S_WAIT => S_WAIT,
         S_SPLIT => S_SPLIT,
         S_WRITE => S_WRITE,
         S_DONE => S_DONE,
         S_HIT => S_HIT,
         M_DATA_OUT => M_DATA_OUT,
         M_CBEA_IN => M_CBEA_IN,
         M_DATA_IN => M_DATA_IN,
         M_ADDL_VLD => M_ADDL_VLD,
         M_ADDH_VLD => M_ADDH_VLD,
         M_ATTR_VLD => M_ATTR_VLD,
         M_DATL_VLD => M_DATL_VLD,
         M_DATH_VLD => M_DATH_VLD,
         M_DATA_NXT => M_DATA_NXT,
         M_MABORT => M_MABORT,
         M_RETRY => M_RETRY,
         M_NXTADB => M_NXTADB,
         M_DISCON => M_DISCON,
         M_TABORT => M_TABORT,
         M_SPLIT => M_SPLIT,
         M_FINISH => M_FINISH,
         M_DONE => M_DONE,
         M_REQ => M_REQ,
         PERR_N => UPERR_N,
         SERR_N => USERR_N,
         INT_N => INT_N,
         MS_STAT => MS_STAT,
         PME_N => PME_N,
         PM_STAT => PM_STAT,
         CSR => CSR,
         CSR_SYSTEMERR_N => CSR_SYSTEMERR_N,
         CSRX_UNEXPECTED => CSRX_UNEXPECTED,
         CSRX_DISCARDED => CSRX_DISCARDED,
         CSRX => CSRX,
         PCIW_EN => PCIW_EN,
         PCIX_EN => PCIX_EN,
         RTR => RTR,
         CFG => CFG_BUS,
         RST => RST,
         CLK => CLK,
         FPGA_RTR => FPGA_RTR,
         DDR_CLK90 => CLK90,
         DDR_CLKDIV => CLKDIV,
         CLK200_N => CLK200_N,
         CLK200_P => CLK200_P,
         USER_CLK => CLK,
         SYS_RESET_IN => RST,
         DDR_DIMM_RAS_L => DDR_DIMM_RAS_L,
         DDR_DIMM_CAS_L => DDR_DIMM_CAS_L,
         DDR_DIMM_WE_L => DDR_DIMM_WE_L,
         DDR_DIMM_CKE0 => DDR_DIMM_CKE0,
         DDR_DIMM_CKE1 => DDR_DIMM_CKE1,
         DDR_DIMM_CS_N0_7 => DDR_DIMM_CS_N0_7,
         DDR_DIMM_CS_N8_15 => DDR_DIMM_CS_N8_15,
         DDR_DIMM_DQ => DDR_DIMM_DQ,
         DDR_DIMM_DQS => DDR_DIMM_DQS,
         DDR_DIMM_CLK0_P => DDR_DIMM_CLK0_P,
         DDR_DIMM_CLK0_N => DDR_DIMM_CLK0_N,
         DDR_DIMM_CLK1_P => DDR_DIMM_CLK1_P,
         DDR_DIMM_CLK1_N => DDR_DIMM_CLK1_N,
         DDR_DIMM_CLK2_P => DDR_DIMM_CLK2_P,
         DDR_DIMM_CLK2_N => DDR_DIMM_CLK2_N,
         DDR_DIMM_BA => DDR_DIMM_BA,
         DDR_DIMM_ADDRESS => DDR_DIMM_ADDRESS,
         DDR_DIMM_SA => DDR_DIMM_SA,
         DDR_DIMM_SDA => DDR_DIMM_SDA,
         DDR_DIMM_SCL => DDR_DIMM_SCL,
         DDR_DIMM_DM => DDR_DIMM_DM,
         DCM_LOCK => dcm_lock);   
   
END rtl;
