--*******************************************************************************
--
--  File name :       rd_data_fifo.vhd
--
--  Description :     This module instantiates the FIFO16 primitive for read  
--                    data. One FIFO for rising edge data and the other for
--                    falling edge data.
--                    the read data has been assigned to FIFOs based on the proximity of 
--                    the data bytes to each other
--
-- 
--                    
--  Date - revision : 6/23/2005
--
--  Author :          Kraig Lund
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  USE IEEE.std_logic_unsigned.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY rd_data_fifo IS
   PORT (
      USER_CLK                : IN std_logic;   
      DDR_CLK                 : IN std_logic;   
      RESET                   : IN std_logic;     
      FIRST_RISING            : IN std_logic_vector(3 DOWNTO 0);         
      READ_DATA_RISE          : IN std_logic_vector(63 DOWNTO 0);        
      READ_DATA_FALL          : IN std_logic_vector(63 DOWNTO 0);         
      READ_DATA_OUT           : OUT std_logic_vector(63 DOWNTO 0);         
      READ_DATA_VALID         : OUT std_logic;   
      FIRST_READ              : IN std_logic);   
END ENTITY rd_data_fifo;

ARCHITECTURE rtl OF rd_data_fifo IS


   SIGNAL read_data_out_mux        :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL rise_data_r1             :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL rise_data_r2             :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL fall_data_r1             :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL com_cnt                  :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL com_sync                 :  std_logic;   
 
BEGIN
   com_sync <= FIRST_READ ;

   PROCESS (DDR_CLK)
   BEGIN
      IF (DDR_CLK'EVENT AND DDR_CLK = '1') THEN
         rise_data_r1 <= READ_DATA_RISE;    
         fall_data_r1 <= READ_DATA_FALL;    
         rise_data_r2 <= rise_data_r1;    
      END IF;
   END PROCESS;

   PROCESS (DDR_CLK)
   BEGIN
      IF (DDR_CLK'EVENT AND DDR_CLK = '1') THEN
         IF (RESET = '1') THEN
            com_cnt <= "00";    
         ELSE
            IF (com_sync = '1') THEN
               com_cnt <= "00";    
            ELSE
               com_cnt <= com_cnt + "01";    
            END IF;
         END IF;
      END IF;
   END PROCESS;


	PROCESS (READ_DATA_RISE(63 downto 0), READ_DATA_FALL(63 downto 0), fall_data_r1(63 downto 0), rise_data_r1(63 downto 0), com_cnt(1 downto 0))
	BEGIN
	      case com_cnt is
           when "00" => read_data_out_mux <= READ_DATA_RISE; --READ_DATA_FALL;
           when "01" => read_data_out_mux <= READ_DATA_FALL; --rise_data_r1;
           when "10" => read_data_out_mux <= rise_data_r1;   --fall_data_r1;
           when "11" => read_data_out_mux <= fall_data_r1;   --rise_data_r2;
           when others => read_data_out_mux <= READ_DATA_RISE;
			end case;
   END PROCESS;
   
   PROCESS (DDR_CLK)
   BEGIN
      IF (DDR_CLK'EVENT AND DDR_CLK = '1') THEN
         IF (RESET = '1') THEN
            READ_DATA_OUT <= X"0000000000000000";    
         ELSE
            READ_DATA_OUT <= read_data_out_mux;    
         END IF;
      END IF;
   END PROCESS;

END ARCHITECTURE rtl;
