/***********************************************************************
 
  File:   stimulus.v
  Rev:    5.0.0

  This is a functional simulation model for a stimulus generator.
  This is not synthesizable.  This file is only for simulation.

  Copyright (c) 2006 Xilinx, Inc.  All rights reserved.

***********************************************************************/
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
//              provided to you "as is". Xilinx and its licensors make and you 
//              receive no warranties or conditions, express, implied, 
//              statutory or otherwise, and Xilinx specifically disclaims any 
//              implied warranties of merchantability, non-infringement, or 
//              fitness for a particular purpose. Xilinx does not warrant that 
//              the functions contained in these designs will meet your 
//              requirements, or that the operation of these designs will be 
//              uninterrupted or error free, or that defects in the Designs 
//              will be corrected. Furthermore, Xilinx does not warrant or 
//              make any representations regarding use or the results of the 
//              use of the designs in terms of correctness, accuracy, 
//              reliability, or otherwise. 
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its 
//              licensors be liable for any loss of data, lost profits, cost 
//              or procurement of substitute goods or services, or for any 
//              special, incidental, consequential, or indirect damages 
//              arising from the use or operation of the designs or 
//              accompanying documentation, however caused and on any theory 
//              of liability. This limitation will apply even if Xilinx 
//              has been advised of the possibility of such damage. This 
//              limitation shall apply not-withstanding the failure of the 
//              essential purpose of any limited remedies herein. 
//
//  Copyright  2006 Xilinx, Inc.
//  All rights reserved 
// 
//*****************************************************************************


`timescale 1 ns / 1 ps

module STIMULUS (
                AD,
                CBE,
                PAR,
                PAR64,
                FRAME_N,
                REQ64_N,
                TRDY_N,
                IRDY_N,
                STOP_N,
                DEVSEL_N,
                ACK64_N,
                IDSEL,
                INTA_N,
                PMEA_N,
                PERR_N,
                SERR_N,
                REQ_N,
                GNT_N,
                RST_N,
                CLK,
                FPGA_RTR
                );


  inout  [63:0] AD;
  inout   [7:0] CBE;
  inout         PAR;
  inout         PAR64;
  inout         FRAME_N;
  inout         REQ64_N;
  inout         TRDY_N;
  inout         IRDY_N;
  inout         STOP_N;
  inout         DEVSEL_N;
  inout         ACK64_N;
  output        IDSEL;
  input         INTA_N;
  input         PMEA_N;
  input         PERR_N;
  input         SERR_N;
  input         REQ_N;
  inout         GNT_N;
  output        RST_N;
  output        CLK;
  input         FPGA_RTR;


  // Define Parameters.

  parameter TCKO = 2;
  parameter FAST = 1;
  parameter WIDE = 1;
  parameter XXXX = 1;


  // Define Internal Registers.

  reg           pciclk;
  reg           clk_en;
  reg     [1:0] clk_sp;
  reg  [8*12:1] operation;
  reg     [3:0] status_code;
  reg    [63:0] reg_ad;
  reg           adh_oe;
  reg           adl_oe;
  reg     [7:0] reg_cbe;
  reg           cbeh_oe;
  reg           cbel_oe;
  reg           reg_par;
  reg           par_oe;
  reg           reg_par64;
  reg           par64_oe;
  reg           reg_frame_n;
  reg           frame_oe;
  reg           reg_req64_n;
  reg           req64_oe;
  reg           reg_trdy_n;
  reg           trdy_oe;
  reg           reg_irdy_n;
  reg           irdy_oe;
  reg           reg_stop_n;
  reg           stop_oe;
  reg           reg_devsel_n;
  reg           devsel_oe;
  reg           reg_ack64_n;
  reg           ack64_oe;
  reg           reg_idsel;
  reg           reg_rst_n;
  reg           reg_gnt_n;
  reg           reg_gnt_oe;

  // Define port hookup.

  assign #TCKO AD[63:32] = adh_oe ? reg_ad[63:32] : 32'bz;
  assign #TCKO AD[31: 0] = adl_oe ? reg_ad[31: 0] : 32'bz;
  assign #TCKO CBE[7:4] = cbeh_oe ? reg_cbe[7:4] : 4'bz;
  assign #TCKO CBE[3:0] = cbel_oe ? reg_cbe[3:0] : 4'bz;
  assign #TCKO PAR = par_oe ? reg_par : 1'bz;
  assign #TCKO PAR64 = par64_oe ? reg_par64 : 1'bz;

  assign #TCKO FRAME_N = frame_oe ? reg_frame_n : 1'bz;
  assign #TCKO REQ64_N = req64_oe ? reg_req64_n : 1'bz;
  assign #TCKO TRDY_N = trdy_oe ? reg_trdy_n : 1'bz;
  assign #TCKO IRDY_N = irdy_oe ? reg_irdy_n : 1'bz;
  assign #TCKO STOP_N = stop_oe ? reg_stop_n : 1'bz;
  assign #TCKO DEVSEL_N = devsel_oe ? reg_devsel_n : 1'bz;
  assign #TCKO ACK64_N = ack64_oe ? reg_ack64_n : 1'bz;
  assign #TCKO IDSEL = reg_idsel;
  assign #TCKO RST_N = reg_rst_n;
  assign #TCKO GNT_N = reg_gnt_n;



  // Clock generation.

  always
  begin
    pciclk <= 0;
    #(3.75/2);//#3.75;
    if (clk_sp == 0) #(11.25/2);//#11.25;
    if (clk_sp == 1) #(3.75/2);//#3.75;
    if (clk_sp == 2) #(3.75/2);//#3.75;
    pciclk <= clk_en;
    #(3.75/2);//#3.75;
    if (clk_sp == 0) #(11.25/2);//#11.25;
    if (clk_sp == 1) #(3.75/2);//#3.75;
    if (clk_sp == 2) #(3.75/2);//#3.75;
  end

  assign CLK = pciclk;


  // PCI Grant Generation.

  always @(posedge pciclk)
  begin
    reg_gnt_n <= reg_gnt_oe ? REQ_N : 1'b1;
  end


  // PCI Parity Generation.

  always @(posedge pciclk)
  begin
    // Always computed, selectively enabled.
    reg_par <= (^ {AD[31:0], CBE[3:0]} );
    reg_par64 <= (^ {AD[63:32], CBE[7:4]} );
  end

  wire driveh;
  wire drivel;

  assign #TCKO driveh = adh_oe;
  assign #TCKO drivel = adl_oe;

  always @(posedge pciclk)
  begin
    par_oe   <= drivel;
    par64_oe <= driveh;
  end


  // Task for resetting the bus in various
  // busmodes.  This must be called to
  // properly initialize the core.

  task RESET_BUS_AS;
    input opt_fast;
    input opt_wide;
    input opt_xxxx;
  begin
    // Housekeeping.
    operation <= "SYSTEM RESET";
    clk_sp <= {opt_xxxx, opt_fast};
    clk_en <= 1'b1;
    // Assign initial values.
    reg_ad  <= 64'h0;
    adh_oe  <= 0; 
    adl_oe  <= 0;
    reg_cbe <= 8'h0;
    cbeh_oe <= 0;
    cbel_oe <= 0;
    reg_frame_n <= 1;
    frame_oe <= 0;
    reg_req64_n <= 1;
    req64_oe <= 0;
    reg_trdy_n <= 1;
    trdy_oe <= 0;
    reg_irdy_n <= 1;
    irdy_oe <= 0;
    reg_stop_n <= 1;
    stop_oe <= 0;
    reg_devsel_n <= 1;
    devsel_oe <= 0;
    reg_ack64_n <= 1;
    ack64_oe <= 0;
    reg_idsel <= 0;
    reg_rst_n <= 0;
    reg_gnt_oe <= 0;
    // Let the dust settle.
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    $display(" ");
    $display("Beginning System Reset...");
    // Make busmode specific changes.
    if (opt_wide)
    begin
      reg_req64_n <= 1'b0;
      req64_oe <= 1'b1;
      $display("     Bus is wide.");
    end
    if (opt_xxxx)
    begin
      reg_trdy_n <= 1'b0;
      trdy_oe <= 1'b1;
      $display("     Bus is xxxx.");
    end
    if (opt_fast)
    begin
      reg_stop_n <= !opt_xxxx;
      stop_oe <= opt_xxxx;
      $display("     Bus is fast.");
    end
    // Let the dust settle.
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    // Deassert reset signal.
    reg_rst_n <= 1;
    // Let the dust settle.
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    // Reassign initial values.
    reg_frame_n <= 1;
    frame_oe <= 0;
    reg_req64_n <= 1;
    req64_oe <= 0;
    reg_trdy_n <= 1;
    trdy_oe <= 0;
    reg_irdy_n <= 1;
    irdy_oe <= 0;
    reg_stop_n <= 1;
    stop_oe <= 0;
    reg_devsel_n <= 1;
    devsel_oe <= 0;
    reg_ack64_n <= 1;
    ack64_oe <= 0;
    reg_idsel <= 0;
    $display("System Reset Complete...");
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
  end
  endtask


  // Task for writing a DWORD in PCI mode.

  task WRITE_DWORD_PCI;
    input [63:0] address;
    input [31:0] data;
    input  [3:0] cmd;
    input  [3:0] cbe;
    integer dsel_cnt;
    integer trdy_cnt;
  begin 
    $display(" ");
    $display("Writing DWORD in PCI mode...");
    operation <= "WR_DWORD    ";
    dsel_cnt <= 0;
    trdy_cnt <= 0;
    @(posedge pciclk);
    reg_frame_n <= 0;
    frame_oe <= 1;
    if (address[63:32] == 32'h00000000)
    begin
      reg_ad <= {32'bz,address[31:0]};
      reg_cbe <= {4'bz,cmd[3:0]};
      reg_idsel <=(cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
    end
    else
    begin
      reg_ad <= {32'bz,address[31:0]};
      reg_cbe <= {4'bz,4'b1101};
      reg_idsel <=(cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
      @(posedge pciclk);
      reg_ad <= {32'bz,address[63:32]};
      reg_cbe <= {4'bz,cmd[3:0]};
      reg_idsel <= (cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
      reg_irdy_n <= 1;
      irdy_oe <= 1;
    end
    @(posedge pciclk);
    reg_frame_n <= 1;
    frame_oe <= 1;
    reg_irdy_n <= 0;
    irdy_oe <= 1;
    reg_ad <= {32'bz,data};
    reg_cbe <= {4'bz,cbe};
    reg_idsel <= 0;
    adl_oe <= 1;
    cbel_oe <= 1;

    while(DEVSEL_N && (dsel_cnt < 10))
    begin
      @(posedge pciclk);
      dsel_cnt = dsel_cnt + 1;
    end

    while(TRDY_N && STOP_N && ((trdy_cnt < 16) && (dsel_cnt < 10)))
    begin
      trdy_cnt = trdy_cnt + 1;
      @(posedge pciclk);
    end

    if (dsel_cnt < 10)
    begin
      if (trdy_cnt <= 16)
      begin
        if (TRDY_N == 0 && STOP_N == 1)
        $display("  STM-->DUT: Normal, Data Transferred");
        else if (TRDY_N == 0 && STOP_N == 0)
        $display("  STM-->DUT: Disconnect, Data Transferred");
        else if (TRDY_N==1 && STOP_N == 0 && DEVSEL_N == 0)
        $display("  STM-->DUT: Retry, No Data Transferred");
        else if (TRDY_N==1 && STOP_N == 0 && DEVSEL_N == 1)
        $display("  STM-->DUT: Target Abort, No Data Transferred");
        else $display("  ERROR: Check Design");
      end
      else $display("  ERROR: Latency Violation");
    end
    else $display("  ERROR: Master Abort");

    reg_frame_n <= 1;
    frame_oe <= 0;
    reg_irdy_n <= 1;
    irdy_oe <= 1;
    adl_oe <= 0;
    cbel_oe <= 0;
    @(posedge pciclk);
    reg_irdy_n <= 1;
    irdy_oe <= 0;
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
  end
  endtask


  // Task for reading a DWORD in PCI mode.

  task READ_DWORD_PCI;
    input [63:0] address;
    input  [3:0] cmd;
    input  [3:0] cbe;
    integer dsel_cnt;
    integer trdy_cnt;
  begin 
    $display(" ");
    $display("Reading DWORD in PCI mode...");
    operation <= "RD_DWORD    ";
    dsel_cnt <= 0;
    trdy_cnt <= 0;
    @(posedge pciclk);
    reg_frame_n <= 0;
    frame_oe <= 1;
    if (address[63:32] == 32'h00000000)
    begin
      reg_ad <= {32'bz,address[31:0]};
      reg_cbe <= {4'bz,cmd[3:0]};
      reg_idsel <= (cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
    end
    else
    begin
      reg_ad <= {32'bz,address[31:0]};
      reg_cbe <= {4'bz,4'b1101};
      reg_idsel <= (cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
      @(posedge pciclk);
      reg_ad <= {32'bz,address[63:32]};
      reg_cbe <= {4'bz,cmd[3:0]};
      reg_idsel <= (cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
      reg_irdy_n <= 1;
      irdy_oe <= 1;
    end
    @(posedge pciclk);
    reg_frame_n <= 1;
    frame_oe <= 1;
    reg_irdy_n <= 0;
    irdy_oe <= 1;
    reg_ad <= {32'bz,32'bz};
    reg_cbe <= {4'bz,cbe};
    reg_idsel <= 0;
    adl_oe <= 0;
    cbel_oe <= 1;

    while(DEVSEL_N && (dsel_cnt < 10))
    begin
      @(posedge pciclk);
      dsel_cnt = dsel_cnt + 1;
    end

    while(TRDY_N && STOP_N && ((trdy_cnt < 16) && (dsel_cnt < 10)))
    begin
      trdy_cnt = trdy_cnt + 1;
      @(posedge pciclk);
    end

    if (dsel_cnt < 10)
    begin
      if (trdy_cnt <= 16)
      begin
        if (TRDY_N == 0 && STOP_N == 1)
        $display("  STM<--DUT: Normal, Data Transferred");
        else if (TRDY_N == 0 && STOP_N == 0)
        $display("  STM<--DUT: Disconnect, Data Transferred");
        else if (TRDY_N==1 && STOP_N == 0 && DEVSEL_N == 0)
        $display("  STM<--DUT: Retry, No Data Transferred");
        else if (TRDY_N==1 && STOP_N == 0 && DEVSEL_N == 1)
        $display("  STM<--DUT: Target Abort, No Data Transferred");
        else $display("  ERROR: Check Design");
      end
      else $display("  ERROR: Latency Violation");
    end
    else $display("  ERROR: Master Abort");

    reg_frame_n <= 1;
    frame_oe <= 0;
    reg_irdy_n <= 1;
    irdy_oe <= 1;
    adl_oe <= 0;
    cbel_oe <= 0;
    @(posedge pciclk);
    reg_irdy_n <= 1;
    irdy_oe <= 0;
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
  end
  endtask


  // Task for writing a DWORD in PCIX mode.

  task WRITE_DWORD_PCIX;
    input [63:0] address;
    input [31:0] data;
    input  [3:0] cmd;
    input [31:0] attrl;
    input  [3:0] attrh;
    integer dsel_cnt;
    integer trdy_cnt;
  begin 
    $display(" ");
    $display("Writing DWORD in PCIX mode...");
    operation <= "WR_DWORD_X  ";
    dsel_cnt <= 0;
    trdy_cnt <= 0;
    @(posedge pciclk);
    reg_frame_n <= 0;
    frame_oe <= 1;
    if (address[63:32] == 32'h00000000)
    begin
      reg_ad <= {32'bz,address[31:0]};
      reg_cbe <= {4'bz,cmd[3:0]};
      reg_idsel <= (cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
    end
    else
    begin
      reg_ad <= {32'bz,address[31:0]};
      reg_cbe <= {4'bz,4'b1101};
      reg_idsel <= (cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
      @(posedge pciclk);
      reg_ad <= {32'bz,address[63:32]};
      reg_cbe <= {4'bz,cmd[3:0]};
      reg_idsel <= (cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
      reg_irdy_n <= 1;
      irdy_oe <= 1;
    end
    @(posedge pciclk);
    reg_ad <= {32'bz,attrl[31:0]};
    reg_cbe <= {4'bz,attrh[3:0]};
    reg_idsel <= 0 ;
    adl_oe <= 1;
    cbel_oe <= 1;
    reg_irdy_n <= 1;
    irdy_oe <= 1;
    @(posedge pciclk);
    reg_ad <= {32'bz,data};
    reg_cbe <= {4'bz,((cmd == 4'b0111) ? attrh : 4'b1111)};
    adl_oe <= 1;
    cbel_oe <= 1;
    @(posedge pciclk);
    reg_irdy_n <= 0;
    irdy_oe <= 1;

    while(DEVSEL_N && (dsel_cnt < 10))
    begin
      @(posedge pciclk);
      dsel_cnt = dsel_cnt + 1;
    end

    while(!DEVSEL_N && STOP_N && TRDY_N && ((trdy_cnt < 16) && (dsel_cnt < 10)))
    begin
      trdy_cnt = trdy_cnt + 1;
      @(posedge pciclk);
    end
    if (dsel_cnt < 10)
    begin
      if (trdy_cnt <= 16)
      begin
        if (DEVSEL_N == 1 && STOP_N == 1 && TRDY_N == 0)
        $display("  STM-->DUT: Split, No Data Transferred");
        else if (DEVSEL_N == 1 && STOP_N == 0 && TRDY_N == 1)
        $display("  STM-->DUT: Target Abort, No Data Transferred");
        else if (DEVSEL_N == 1 && STOP_N == 0 && TRDY_N == 0)
        $display("  STM-->DUT: Immediate Disconnect, Data Transferred");
        else if (DEVSEL_N == 0 && STOP_N == 1 && TRDY_N == 0)
        $display("  STM-->DUT: Normal, Data Transferred");
        else if (DEVSEL_N == 0 && STOP_N == 0 && TRDY_N == 1)
        $display("  STM-->DUT: Retry, No Data Transferred");
        else if (DEVSEL_N == 0 && STOP_N == 0 && TRDY_N == 0)
        $display("  STM-->DUT: Next ADB Disconnect, Data Transferred");
        else $display("  ERROR: Check Design");
      end
      else $display("  ERROR: Latency Violation");
    end
    else $display("  ERROR: Master Abort");

    @(posedge pciclk);
    reg_frame_n <= 1;
    frame_oe <= 1;
    reg_irdy_n <= 1;
    irdy_oe <= 1;
    @(posedge pciclk);
    frame_oe <= 0;
    adl_oe <= 0;
    cbel_oe <= 0;
   // @(posedge pciclk);
    irdy_oe <= 0;
   // @(posedge pciclk);
   // @(posedge pciclk);
   // @(posedge pciclk);
  end
  endtask

  // Task for writing a byte using DWORD  in PCIX mode.

  task WRITE_BYTE_PCIX;
    input [63:0] address;
    input [31:0] data;
    input  [3:0] cmd;
    input [31:0] attrl;
    input  [3:0] attrh;
    input  [3:0] byte;
    integer dsel_cnt;
    integer trdy_cnt;
  begin 
    $display(" ");
    $display("Writing BYTE mode in PCIX mode...");
    operation <= "WR_BYTE_X  ";
    dsel_cnt <= 0;
    trdy_cnt <= 0;
    @(posedge pciclk);
    reg_frame_n <= 0;
    frame_oe <= 1;
    if (address[63:32] == 32'h00000000)
    begin
      reg_ad <= {32'bz,address[31:0]};
      reg_cbe <= {4'bz,cmd[3:0]};
      reg_idsel <= (cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
    end
    else
    begin
      reg_ad <= {32'bz,address[31:0]};
      reg_cbe <= {4'bz,4'b1101};
      reg_idsel <= (cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
      @(posedge pciclk);
      reg_ad <= {32'bz,address[63:32]};
      reg_cbe <= {4'bz,cmd[3:0]};
      reg_idsel <= (cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
      reg_irdy_n <= 1;
      irdy_oe <= 1;
    end
    @(posedge pciclk);
    reg_ad <= {32'bz,attrl[31:0]};
    reg_cbe <= {4'bz,attrh[3:0]};
    reg_idsel <= 0 ;
    adl_oe <= 1;
    cbel_oe <= 1;
    reg_irdy_n <= 1;
    irdy_oe <= 1;
    @(posedge pciclk);
    reg_ad <= {32'bz,data};
    reg_cbe <= {4'bz,((cmd == 4'b0111) ? byte : 4'b1111)};
    adl_oe <= 1;
    cbel_oe <= 1;
    @(posedge pciclk);
    reg_irdy_n <= 0;
    irdy_oe <= 1;

    while(DEVSEL_N && (dsel_cnt < 10))
    begin
      @(posedge pciclk);
      dsel_cnt = dsel_cnt + 1;
    end

    while(!DEVSEL_N && STOP_N && TRDY_N && ((trdy_cnt < 16) && (dsel_cnt < 10)))
    begin
      trdy_cnt = trdy_cnt + 1;
      @(posedge pciclk);
    end
    if (dsel_cnt < 10)
    begin
      if (trdy_cnt <= 16)
      begin
        if (DEVSEL_N == 1 && STOP_N == 1 && TRDY_N == 0)
        $display("  STM-->DUT: Split, No Data Transferred");
        else if (DEVSEL_N == 1 && STOP_N == 0 && TRDY_N == 1)
        $display("  STM-->DUT: Target Abort, No Data Transferred");
        else if (DEVSEL_N == 1 && STOP_N == 0 && TRDY_N == 0)
        $display("  STM-->DUT: Immediate Disconnect, Data Transferred");
        else if (DEVSEL_N == 0 && STOP_N == 1 && TRDY_N == 0)
        $display("  STM-->DUT: Normal, Data Transferred");
        else if (DEVSEL_N == 0 && STOP_N == 0 && TRDY_N == 1)
        $display("  STM-->DUT: Retry, No Data Transferred");
        else if (DEVSEL_N == 0 && STOP_N == 0 && TRDY_N == 0)
        $display("  STM-->DUT: Next ADB Disconnect, Data Transferred");
        else $display("  ERROR: Check Design");
      end
      else $display("  ERROR: Latency Violation");
    end
    else $display("  ERROR: Master Abort");

    @(posedge pciclk);
    reg_frame_n <= 1;
    frame_oe <= 1;
    reg_irdy_n <= 1;
    irdy_oe <= 1;
    @(posedge pciclk);
    frame_oe <= 0;
    adl_oe <= 0;
    cbel_oe <= 0;
    @(posedge pciclk);
    irdy_oe <= 0;
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
  end
  endtask

  // Task for reading a DWORD in PCIX mode.

  task READ_DWORD_PCIX;
    input [63:0] address;
    input  [3:0] cmd;
    input [31:0] attrl;
    input  [3:0] attrh;
    integer dsel_cnt;
    integer trdy_cnt;
  begin 
    $display(" ");
    $display("Reading DWORD in PCIX mode...");
    operation <= "RD_DWORD_X  ";
    dsel_cnt <= 0;
    trdy_cnt <= 0;
    @(posedge pciclk);
    reg_frame_n <= 0;
    frame_oe <= 1;
    if (address[63:32] == 32'h00000000)
    begin
      reg_ad <= {32'bz,address[31:0]};
      reg_cbe <= {4'bz,cmd[3:0]};
      reg_idsel <= (cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
    end
    else
    begin
      reg_ad <= {32'bz,address[31:0]};
      reg_cbe <= {4'bz,4'b1101};
      reg_idsel <= (cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
      @(posedge pciclk);
      reg_ad <= {32'bz,address[63:32]};
      reg_cbe <= {4'bz,cmd[3:0]};
      reg_idsel <= (cmd[3:1] == 3'b101);
      adl_oe <= 1;
      cbel_oe <= 1;
      reg_irdy_n <= 1;
      irdy_oe <= 1;
    end
    @(posedge pciclk);
    reg_ad <= {32'bz,attrl[31:0]};
    reg_cbe <= {4'bz,attrh[3:0]};
    reg_idsel <= 0 ;
    adl_oe <= 1;
    cbel_oe <= 1;
    reg_irdy_n <= 1;
    irdy_oe <= 1;
    @(posedge pciclk);
    reg_cbe <= {4'bz,4'b1111};
    adl_oe <= 0;
    cbel_oe <= 1;
    @(posedge pciclk);
    reg_irdy_n <= 0;
    irdy_oe <= 1;

    while(DEVSEL_N && (dsel_cnt < 10))
    begin
      @(posedge pciclk);
      dsel_cnt = dsel_cnt + 1;
    end

    while(!DEVSEL_N && STOP_N && TRDY_N && ((trdy_cnt < 16) && (dsel_cnt < 10)))
    begin
      trdy_cnt = trdy_cnt + 1;
      @(posedge pciclk);
    end

    if (dsel_cnt < 10)
    begin
      if (trdy_cnt <= 16)
      begin
        if (DEVSEL_N == 1 && STOP_N == 1 && TRDY_N == 0)
        $display("  STM<--DUT: Split, No Data Transferred");
        else if (DEVSEL_N == 1 && STOP_N == 0 && TRDY_N == 1)
        $display("  STM<--DUT: Target Abort, No Data Transferred");
        else if (DEVSEL_N == 1 && STOP_N == 0 && TRDY_N == 0)
        $display("  STM<--DUT: Immediate Disconnect, Data Transferred");
        else if (DEVSEL_N == 0 && STOP_N == 1 && TRDY_N == 0)
        $display("  STM<--DUT: Normal, Data Transferred");
        else if (DEVSEL_N == 0 && STOP_N == 0 && TRDY_N == 1)
        $display("  STM<--DUT: Retry, No Data Transferred");
        else if (DEVSEL_N == 0 && STOP_N == 0 && TRDY_N == 0)
        $display("  STM<--DUT: Next ADB Disconnect, Data Transferred");
        else $display("  ERROR: Check Design");
      end
      else $display("  ERROR: Latency Violation");
    end
    else $display("  ERROR: Master Abort");

    @(posedge pciclk);
    reg_frame_n <= 1;
    frame_oe <= 1;
    reg_irdy_n <= 1;
    irdy_oe <= 1;
    @(posedge pciclk);
    frame_oe <= 0;
    cbel_oe <= 0;
    @(posedge pciclk);
    irdy_oe <= 0;
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
  end
  endtask


  // Task for waiting.

  task INTERCYCLE_GAP;
  begin
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
  end
  endtask




/////////////////////////////////////////////////////////////
// Add PCI-X Burst Command Tasks (ST: 2/22/2005)           //
/////////////////////////////////////////////////////////////

//Create a memory array for the split competions
  reg    [63:0] cmd_mem [0:65535];
  integer       loop_var;
  reg    [31:0] counter;
  reg           ad_oe;
  reg           cbe_oe;
  


// Initialize Internal Memory
  always @(posedge RST_N)
  begin
    for (loop_var = 0; loop_var < 65536; loop_var = loop_var + 1)
    begin
      cmd_mem[loop_var] = { (loop_var[15:0]+1), ~(loop_var[15:0]+1), ~loop_var[15:0], loop_var[15:0]};
    end
  end   

  // Task for burst write in PCIX mode. Used for Split Completions
  reg [12:0] reg_byte_cnter;
  
  task PCIX_BURST_WR;
    input [26:0] l_address;
    input [3:0] cmd;
//    input [31:0] data;
//    input [31:0] attrl;
//    input  [3:0] attrh;
    input [7:0] l_byte_cnt;
    input [3:0] u_byte_cnt;
    integer dsel_cnt;
    integer trdy_cnt;
  begin 
    dsel_cnt <= 0;
    trdy_cnt <= 0;

    if(l_address[2] == 0)
        reg_byte_cnter <= {u_byte_cnt,l_byte_cnt};
    else
          reg_byte_cnter <= {u_byte_cnt,l_byte_cnt} + 4
        ;
    counter = (l_address >> 2) & 32'h0000ffff;
    @(posedge pciclk); 
    //Address Phase
      reg_req64_n <= 0;
      req64_oe <= 1;
      reg_frame_n <= 0;
      frame_oe <= 1;
      reg_ad <= { 32'bz, 5'b00110, l_address};
      reg_cbe <= {cmd[3:0], cmd[3:0]};  //64-bit transactions have command on both BE's
      adl_oe <= 1;
      adh_oe <= 1;
      cbel_oe <= 1;
      cbeh_oe <= 1;

    @(posedge pciclk);
    // Attributes Phase
      reg_ad <= {32'b0, 1'b0,1'b0,1'b0,5'b00000,8'b00000000,5'b00000,3'b000,l_byte_cnt}; //was 32'bz
      reg_cbe <= u_byte_cnt;
      reg_irdy_n <= 1;
      irdy_oe <= 1;
    @(posedge pciclk);
    // Data Phase (wait for TRDY)
    // Must drive CBE == 0xFF for burst write cycles
      reg_ad = cmd_mem[counter];
      reg_cbe <= 8'hff;
   /*   if(l_address[2])
         reg_cbe <= 8'hF0;
      else
         reg_cbe <= 8'hFF;   */
    @(posedge pciclk);
    // Assert INITIATOR READY
    begin
      reg_irdy_n <= 0;
      irdy_oe <= 1;
    end
    
    while(DEVSEL_N && (dsel_cnt < 10))
    // Wait for TARGET to claim transaction
    begin
      @(posedge pciclk);
        dsel_cnt = dsel_cnt + 1;
    end

    while(!DEVSEL_N && STOP_N && TRDY_N && ((trdy_cnt < 16) && (dsel_cnt < 10)))
    begin
      trdy_cnt = trdy_cnt + 1;
      @(posedge pciclk);
    end

    reg_byte_cnter <= reg_byte_cnter - 13'b0000000001000;
    counter <= (counter + 1) & 32'h0000ffff;
    
    while (!DEVSEL_N && !TRDY_N && (reg_byte_cnter != 13'b0000000100000))
    begin
       @(posedge pciclk);
         counter <= (counter + 1) & 32'h0000ffff;
         reg_ad = cmd_mem[counter];
         reg_byte_cnter <= reg_byte_cnter - 13'b0000000001000;
         reg_cbe <= 8'hff;
    end
    
    //finish up, assumed all transfers are more than 4 dataphases.
    @(posedge pciclk);
      counter <= (counter + 1) & 32'h0000ffff;
      reg_ad = cmd_mem[counter];
      reg_req64_n <= 1;
      req64_oe <= 1;
      reg_frame_n <= 1;
      reg_irdy_n <= 0;
      irdy_oe <= 1;
      frame_oe <= 1;
      adl_oe <= 1;
      cbel_oe <= 1;
    @(posedge pciclk);
      counter <= (counter + 1) & 32'h0000ffff;
      reg_ad = cmd_mem[(counter + 1)];
      reg_req64_n <= 1;
      reg_frame_n <= 1;
      reg_irdy_n <= 0;
      irdy_oe <= 1;
      req64_oe <= 1;
      frame_oe <= 0;
      adl_oe <= 1;
      cbel_oe <= 1;    
    @(posedge pciclk);
      reg_irdy_n <= 1;
      irdy_oe <= 1;
      req64_oe <= 0;
      frame_oe <= 0;
      adl_oe <= 0;
      cbel_oe <= 0;
    @(posedge pciclk);
      irdy_oe <= 0;
      req64_oe <= 0;
      frame_oe <= 0;
      adl_oe <= 0;
      cbel_oe <= 0;
  $write("     Host wrote 0x%h bytes with starting lower addr 0x%h:  ", l_byte_cnt, l_address);
    if (dsel_cnt < 10)
    begin
      if (trdy_cnt <= 16)
      begin
        $write("     Host completed Split write!");
        //okay = 1'b1;
      end
      else
      begin
        $display("  ERROR: Latency Violation");
        //okay = 1'b0;
      end
    end
    else $display("  ERROR: Master Abort");

    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
  end
  endtask




// PCI-X BURST READ task
  task PCIX_BURST_RD;
    input  [26:0]  l_address;
    input  [3:0]  cmd;
    input [31:0]  attrl;
    input  [3:0]  attrh;
    input  [7:0]  l_byte_cnt;
    input  [3:0]  u_byte_cnt;

    integer dsel_cnt;
    integer trdy_cnt;

  begin 
    dsel_cnt <= 0;
    trdy_cnt <= 0;

    if(l_address[2] == 0)
        reg_byte_cnter <= {u_byte_cnt,l_byte_cnt};
    else
          reg_byte_cnter <= {u_byte_cnt,l_byte_cnt} + 4;

    counter = (l_address >> 2) & 32'h0000ffff;
    @(posedge pciclk); 
    //Address Phase
      reg_req64_n <= 0;
      req64_oe <= 1;
      reg_frame_n <= 0;
      frame_oe <= 1;
      reg_ad <= { 32'hFFFFFFFF, 5'b00110, l_address};
      reg_cbe <= { 4'hF, cmd[3:0] };
      reg_irdy_n <= 1;
      adl_oe <= 1;
      adh_oe <= 1;
      cbel_oe <= 1;
      cbeh_oe <= 1;
    @(posedge pciclk);
    // Attributes Phase
      reg_ad <= {32'hFFFFFFFF, 1'b0,1'b0,1'b0,5'b00000,8'b00000000,5'b00000,3'b000,l_byte_cnt};
      reg_cbe <= { 4'h0, u_byte_cnt };
      reg_irdy_n <= 1;
      irdy_oe <= 1;
    @(posedge pciclk);
    // Data Phase (wait for TRDY)
    // Must drive CBE == 0xFF for burst read cycles
      reg_ad = 64'hffffffffffffffff;
      reg_cbe <= 8'hff;
      reg_irdy_n <= 1;
      adl_oe <= 0;    // tristate initiator bus drivers
      adh_oe <= 0;
      cbel_oe <= 1;   // drive CBE=0xFF during read cycles (maintain parity)
      cbeh_oe <= 1;   
    while(DEVSEL_N && (dsel_cnt < 10))
    // Wait for TARGET to claim transaction
    begin
      @(posedge pciclk);
        dsel_cnt = dsel_cnt + 1;
    end

    while(!DEVSEL_N && STOP_N && TRDY_N && ((trdy_cnt < 16) && (dsel_cnt < 10)))
    begin
      trdy_cnt = trdy_cnt + 1;
      @(posedge pciclk);
      reg_irdy_n <= 0;
    end
    
    while (!DEVSEL_N && !TRDY_N && (reg_byte_cnter != 13'b0000000110000))
    begin
       @(posedge pciclk);
         counter <= (counter + 1) & 32'h0000ffff;
         reg_irdy_n <= 0;
         reg_byte_cnter <= reg_byte_cnter - 13'b0000000001000;
    end
    
    //finish up, assumed all transfers are more than 4 dataphases.
    @(posedge pciclk);
      counter <= (counter + 1) & 32'h0000ffff;
      reg_req64_n <= 1;
      reg_frame_n <= 1;
      reg_irdy_n <= 0;
      req64_oe <= 1;
      frame_oe <= 1;
    @(posedge pciclk);
      counter <= (counter + 1) & 32'h0000ffff;
      reg_frame_n <= 1;
      reg_req64_n <= 1;
      reg_irdy_n <= 0;
      irdy_oe <= 1;
      req64_oe <= 1;
      frame_oe <= 0;    
    @(posedge pciclk);
      reg_irdy_n <= 1;
      irdy_oe <= 1;
      frame_oe <= 0;
      req64_oe <= 0;
    @(posedge pciclk);
      irdy_oe <= 0;

  $write("     Host read 0x%h bytes with starting lower addr 0x%h:  ", l_byte_cnt, l_address);
    if (dsel_cnt < 10)
    begin
      if (trdy_cnt <= 16)
      begin
        $write("     Host completed READ!");
      end
      else
      begin
        $display("  ERROR: Latency Violation");
      end
    end
    else $display("  ERROR: Master Abort");

    @(posedge pciclk);
    @(posedge pciclk);
    @(posedge pciclk);
  end
  endtask










///////////////////////////////////////////////////////////////
// Begin the actual simulation sequence.                     //
///////////////////////////////////////////////////////////////



  initial
  begin
 /*
    // reset the system
    RESET_BUS_AS (~FAST,  WIDE, ~XXXX);

    // skip if unsupported
    if (FPGA_RTR == 0)
    begin

      // read device and vendor id
      READ_DWORD_PCI(0, 10, 0);

      // setup bar0
      READ_DWORD_PCI(16, 10, 0);
      WRITE_DWORD_PCI(16, 32'hffffffff, 11, 0);
      READ_DWORD_PCI(16, 10, 0);
      WRITE_DWORD_PCI(16, 32'h10000000, 11, 0);
      READ_DWORD_PCI(16, 10, 0);

      // setup bar1
      READ_DWORD_PCI(20, 10, 0);
      WRITE_DWORD_PCI(20, 32'hffffffff, 11, 0);
      READ_DWORD_PCI(20, 10, 0);
      WRITE_DWORD_PCI(20, 32'h20000000, 11, 0);
      READ_DWORD_PCI(20, 10, 0);

      // setup bar2
      READ_DWORD_PCI(24, 10, 0);
      READ_DWORD_PCI(28, 10, 0);
      WRITE_DWORD_PCI(24, 32'hffffffff, 11, 0);
      WRITE_DWORD_PCI(28, 32'hffffffff, 11, 0);
      READ_DWORD_PCI(24, 10, 0);
      READ_DWORD_PCI(28, 10, 0);
      WRITE_DWORD_PCI(24, 32'h30000000, 11, 0);
      WRITE_DWORD_PCI(28, 32'h00000000, 11, 0);
      READ_DWORD_PCI(24, 10, 0);
      READ_DWORD_PCI(28, 10, 0);

      // setup command register
      READ_DWORD_PCI(4, 10, 0);
      WRITE_DWORD_PCI(4, 32'hffff0147, 11, 0);
      READ_DWORD_PCI(4, 10, 0);

      // access bar0 space
      READ_DWORD_PCI(32'h10000000, 2, 0);
      WRITE_DWORD_PCI(32'h10000000, 32'hba000ba0, 3, 0);
      READ_DWORD_PCI(32'h10000000, 2, 0);

      // access bar1 space
      READ_DWORD_PCI(32'h20000000, 6, 0);
      WRITE_DWORD_PCI(32'h20000000, 32'hba100ba1, 7, 0);
      READ_DWORD_PCI(32'h20000000, 6, 0);

      // access bar2 space even
      READ_DWORD_PCI(32'h30000000, 6, 0);
      WRITE_DWORD_PCI(32'h30000000, 32'hba2aaba2, 7, 0);
      READ_DWORD_PCI(32'h30000000, 6, 0);

      // access bar2 space odd
      READ_DWORD_PCI(32'h30000004, 6, 0);
      WRITE_DWORD_PCI(32'h30000004, 32'hba2bbba2, 7, 0);
      READ_DWORD_PCI(32'h30000004, 6, 0);

      // just checking...
      reg_gnt_oe <= 1'b1;
      INTERCYCLE_GAP;
      INTERCYCLE_GAP;
      INTERCYCLE_GAP;
      INTERCYCLE_GAP;

    end
*/
    // reset the system
    RESET_BUS_AS (~FAST,  WIDE,  XXXX);

    // skip if unsupported
//    if (FPGA_RTR == 0)
    begin

      // read device and vendor id
      READ_DWORD_PCIX(0, 10, 0, 0);

      // setup bar0
      READ_DWORD_PCIX(16, 10, 0, 0);
      WRITE_DWORD_PCIX(16, 32'hffffffff, 11, 0, 0); //address/data/cmd/attrl/addrh
      READ_DWORD_PCIX(16, 10, 0, 0);
      WRITE_DWORD_PCIX(16, 32'h10000000, 11, 0, 0);
      READ_DWORD_PCIX(16, 10, 0, 0);

      // setup bar1
      READ_DWORD_PCIX(20, 10, 0, 0);
      WRITE_DWORD_PCIX(20, 32'hffffffff, 11, 0, 0);
      READ_DWORD_PCIX(20, 10, 0, 0);
      WRITE_DWORD_PCIX(20, 32'h20000000, 11, 0, 0);
      READ_DWORD_PCIX(20, 10, 0, 0);

      // setup bar2
      READ_DWORD_PCIX(24, 10, 0, 0);
      READ_DWORD_PCIX(28, 10, 0, 0);
      WRITE_DWORD_PCIX(24, 32'hffffffff, 11, 0, 0);
      WRITE_DWORD_PCIX(28, 32'hffffffff, 11, 0, 0);
      READ_DWORD_PCIX(24, 10, 0, 0);
      READ_DWORD_PCIX(28, 10, 0, 0);
      WRITE_DWORD_PCIX(24, 32'h30000000, 11, 0, 0);
      WRITE_DWORD_PCIX(28, 32'h00000000, 11, 0, 0);
      READ_DWORD_PCIX(24, 10, 0, 0);
      READ_DWORD_PCIX(28, 10, 0, 0);

      // setup command register
      READ_DWORD_PCIX(4, 10, 0, 0);
      WRITE_DWORD_PCIX(4, 32'hffff0147, 11, 0, 0);
      READ_DWORD_PCIX(4, 10, 0, 0);

      // access bar0 space
 //     READ_DWORD_PCIX(32'h10000000, 2, 0, 0);
 //     WRITE_DWORD_PCIX(32'h10000000, 32'hba000ba0, 3, 0, 0);
 //     READ_DWORD_PCIX(32'h10000000, 2, 0, 0);

      // access bar1 space
 //     READ_DWORD_PCIX(32'h20000000, 6, 0, 0);
 //     WRITE_DWORD_PCIX(32'h20000000, 32'hba100ba1, 7, 4, 0);
 //     READ_DWORD_PCIX(32'h20000000, 6, 0, 0);


      #(3000*pciclk);
      // access bar2 space even
      WRITE_DWORD_PCIX(32'h37fffffC, 32'hfafafafa, 7, 4, 0);
         #(300*pciclk);
      READ_DWORD_PCIX(32'h37fffffC, 6, 0, 0);
         #(300*pciclk);
      WRITE_DWORD_PCIX(32'h30000000, 32'hfeedface, 7, 4, 0);
         #(300*pciclk);
      READ_DWORD_PCIX(32'h30000000, 6, 0, 0);
         #(300*pciclk);
      READ_DWORD_PCIX(32'h37fffffC, 6, 0, 0);
      #(300*pciclk);
      READ_DWORD_PCIX(32'h37fffffC, 6, 0, 0);
      //test byte writes
      #(300*pciclk);    
      WRITE_BYTE_PCIX(32'h30000000, 32'h0000dd00, 7, 4, 0, 4'hd);  //write a byte
      #(300*pciclk);
      WRITE_BYTE_PCIX(32'h30000004, 32'h0000ff00, 7, 4, 0, 4'hd);  //write a byte
      #(300*pciclk);
      WRITE_DWORD_PCIX(32'h30000000, 32'h76543210, 7, 4, 0);
      #(300*pciclk);   
      //test the RETRY Logic        
      WRITE_DWORD_PCIX(32'h30000004, 32'hfedcba98, 7, 4, 0);
       WRITE_DWORD_PCIX(32'h30000004, 32'hfedcba98, 7, 4, 0);
      WRITE_DWORD_PCIX(32'h30000004, 32'hfedcba98, 7, 4, 0);
      WRITE_DWORD_PCIX(32'h30000008, 32'hdeadbeef, 7, 4, 0);
      WRITE_DWORD_PCIX(32'h30000008, 32'hdeadbeef, 7, 4, 0);
      WRITE_DWORD_PCIX(32'h30000008, 32'hdeadbeef, 7, 4, 0);   
      WRITE_DWORD_PCIX(32'h3000000C, 32'hfeedface, 7, 4, 0);
      WRITE_DWORD_PCIX(32'h3000000C, 32'hfeedface, 7, 4, 0);
      WRITE_DWORD_PCIX(32'h3000000C, 32'hfeedface, 7, 4, 0);
      //test plain old DWORD writes to various addresses
      #(300*pciclk);
      WRITE_DWORD_PCIX(32'h30000010, 32'h13121110, 7, 4, 0);
      #(300*pciclk);
      WRITE_DWORD_PCIX(32'h30000014, 32'h17161514, 7, 4, 0);
      #(300*pciclk);
      WRITE_DWORD_PCIX(32'h30000018, 32'h1b1a1918, 7, 4, 0);
      #(300*pciclk);
      WRITE_DWORD_PCIX(32'h3000001C, 32'h1f1e1d1c, 7, 4, 0);
      #(300*pciclk);  
      //test DWORD reads to various addresses
      READ_DWORD_PCIX(32'h30000000, 6, 0, 0);
      #(300*pciclk);
      READ_DWORD_PCIX(32'h30000004, 6, 0, 0);
      #(300*pciclk);
      READ_DWORD_PCIX(32'h30000008, 6, 0, 0);
      #(300*pciclk);
      READ_DWORD_PCIX(32'h3000000c, 6, 0, 0);
      #(300*pciclk); 
      READ_DWORD_PCIX(32'h30000010, 6, 0, 0);
      #(300*pciclk);
      READ_DWORD_PCIX(32'h30000014, 6, 0, 0);
      #(300*pciclk);
      READ_DWORD_PCIX(32'h30000018, 6, 0, 0);
      #(300*pciclk);
      READ_DWORD_PCIX(32'h3000001c, 6, 0, 0);
      #(300*pciclk);

//Write and Read top of 512x64 BRAM
      WRITE_DWORD_PCIX(32'h30000FF8, 32'h30000ff8, 7, 4, 0);
      #(300*pciclk);
      READ_DWORD_PCIX(32'h30000FF8, 6, 0, 0);
      #(300*pciclk);
      WRITE_DWORD_PCIX(32'h30000FFC, 32'h30000ffc, 7, 4, 0);
      #(300*pciclk);
      READ_DWORD_PCIX(32'h30000FFC, 6, 0, 0);
      #(300*pciclk);

      PCIX_BURST_WR(7'b0011100, 4'hf, 8'h8, 4'h0);   //1
      #(300*pciclk);

// Burst PCIX Write to BRAM memory
// Add task for PCIX_BURST_WR in order to proceed with debug
// Got example code from John Ayer
      //PCIX_BURST_WR(7'b0000000, 4'hf, 8'hF8, 4'hF);
      PCIX_BURST_WR(27'b0000000, 4'hf, 8'h4, 4'h0);   //1
      #(300*pciclk);
      PCIX_BURST_WR(27'b0000000, 4'hf, 8'h1C, 4'h0);//7
      #(300*pciclk);
      PCIX_BURST_WR(27'b0000000, 4'hf, 8'h24, 4'h0);//9
      #(300*pciclk);
      PCIX_BURST_WR(27'b0000000, 4'hf, 8'h30, 4'h0);//12
      #(300*pciclk);
      PCIX_BURST_WR(27'b0000000, 4'hf, 8'h34, 4'h0);//13

      PCIX_BURST_RD(27'h0000000, 4'he, 32'b0, 4'b0, 8'h4, 4'h0);   //1
      #(300*pciclk);
      PCIX_BURST_RD(27'h0000000, 4'he, 32'b0, 4'b0, 8'h1C, 4'h0);//7
      #(300*pciclk);
      PCIX_BURST_RD(27'h0000000, 4'he, 32'b0, 4'b0, 8'h24, 4'h0);//9
      #(300*pciclk);
      PCIX_BURST_RD(27'h0000000, 4'he, 32'b0, 4'b0, 8'h30, 4'h0);//12
      #(300*pciclk);
      PCIX_BURST_RD(27'h0000000, 4'he, 32'b0, 4'b0, 8'h34, 4'h0);//13
      #(300*pciclk);
      PCIX_BURST_WR(27'h0001fff, 4'hf, 8'h30, 4'h2);//12
      #(300*pciclk);
      PCIX_BURST_RD(27'h0001fff, 4'he, 32'b0, 4'b0, 8'h30, 4'h2);//13
      #(300*pciclk);
      PCIX_BURST_RD(27'h0000000, 4'he, 32'b0, 4'b0, 8'h30, 4'h0);//12
      #(300*pciclk);
      PCIX_BURST_RD(27'h0001fff, 4'he, 32'b0, 4'b0, 8'h30, 4'h2);//13

      #(1000*pciclk);
      PCIX_BURST_WR(27'h0000004, 4'hf, 8'h4, 4'h0);   //1
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000004, 4'hf, 8'h1C, 4'h0);//7
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000004, 4'hf, 8'h24, 4'h0);//9
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000004, 4'hf, 8'h30, 4'h0);//12
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000004, 4'hf, 8'h34, 4'h0);//13




      #(1000*pciclk);
      PCIX_BURST_WR(27'h0000008, 4'hf, 8'h4, 4'h0);   //1
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000008, 4'hf, 8'h1C, 4'h0);//7
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000008, 4'hf, 8'h24, 4'h0);//9
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000000, 4'hf, 8'h30, 4'h0);//12
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000008, 4'hf, 8'h34, 4'h0);//13


       #(1000*pciclk);
      PCIX_BURST_WR(27'h000000c, 4'hf, 8'h4, 4'h0);   //1
      #(300*pciclk);
      PCIX_BURST_WR(27'h000000c, 4'hf, 8'h1C, 4'h0);//7
      #(300*pciclk);
      PCIX_BURST_WR(27'h000000c, 4'hf, 8'h24, 4'h0);//9
      #(300*pciclk);
      PCIX_BURST_WR(27'h000000c, 4'hf, 8'h30, 4'h0);//12
      #(300*pciclk);
      PCIX_BURST_WR(27'h000000c, 4'hf, 8'h34, 4'h0);//13


      #(1000*pciclk);
      PCIX_BURST_WR(27'h0000010, 4'hf, 8'h4, 4'h0);   //1
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000010, 4'hf, 8'h1C, 4'h0);//7
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000010, 4'hf, 8'h24, 4'h0);//9
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000010, 4'hf, 8'h30, 4'h0);//12
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000010, 4'hf, 8'h34, 4'h0);//13

      #(1000*pciclk);
      PCIX_BURST_WR(27'h0000014, 4'hf, 8'h4, 4'h0);   //1
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000014, 4'hf, 8'h1C, 4'h0);//7
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000014, 4'hf, 8'h24, 4'h0);//9
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000014, 4'hf, 8'h30, 4'h0);//12
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000014, 4'hf, 8'h34, 4'h0);//13

      #(1000*pciclk);
      PCIX_BURST_WR(27'h0000018, 4'hf, 8'h4, 4'h0);   //1
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000018, 4'hf, 8'h1C, 4'h0);//7
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000018, 4'hf, 8'h24, 4'h0);//9
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000018, 4'hf, 8'h30, 4'h0);//12
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000018, 4'hf, 8'h34, 4'h0);//13

      #(1000*pciclk);
      PCIX_BURST_WR(27'h000001c, 4'hf, 8'h4, 4'h0);   //1
      #(300*pciclk);
      PCIX_BURST_WR(27'h000001c, 4'hf, 8'h1C, 4'h0);//7
      #(300*pciclk);
      PCIX_BURST_WR(27'h000001c, 4'hf, 8'h24, 4'h0);//9
      #(300*pciclk);
      PCIX_BURST_WR(27'h000001c, 4'hf, 8'h30, 4'h0);//12
      #(300*pciclk);
      PCIX_BURST_WR(27'h000001c, 4'hf, 8'h34, 4'h0);//13

      #(1000*pciclk);
      PCIX_BURST_WR(27'h000005c, 4'hf, 8'h4, 4'h0);   //1
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000004, 4'hf, 8'h1C, 4'h0);//7
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000050, 4'hf, 8'h30, 4'h1);//9
      #(300*pciclk);
      PCIX_BURST_WR(27'h0001fff, 4'hf, 8'h30, 4'h2);//12
      #(300*pciclk);
      PCIX_BURST_WR(27'h3fffffd, 4'hf, 8'h30, 4'h2);//12
      #(300*pciclk);
      PCIX_BURST_WR(27'h000001c, 4'hf, 8'h34, 4'h1);//13
      #(1000*pciclk);

      PCIX_BURST_RD(27'h0000008, 4'he, 32'b0, 4'b0, 8'hF8, 4'hF);
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000009, 4'hf, 8'h0c, 4'h0);
      #(300*pciclk);
      PCIX_BURST_WR(27'h0000009, 4'hf, 8'hff, 4'h0);
      #(300*pciclk);

      // just checking...
      reg_gnt_oe <= 1'b1;
      INTERCYCLE_GAP;
      INTERCYCLE_GAP;
      INTERCYCLE_GAP;
      INTERCYCLE_GAP;

    end

    // let the dust settle
    $display(" ");
    INTERCYCLE_GAP;
    INTERCYCLE_GAP;
    $display(" ");
    $display("Simulation complete...");
    $display(" ");

    // finish up
    clk_en <= 0;
    $finish;
  end


endmodule
