--******************************************************************************
--
--  Xilinx, Inc. 2005                 www.xilinx.com
--
--
--*******************************************************************************
--
--  File name :       tap_ctrl.vhd
--
--  Description :     This module detects two transitions of a dqs (strobe) signal 
--                    and determines the tap delay required for the associated
--                    data bits in order to center them with respect to internal
--                    FPGA clock, CLK.
-- 
--                    
--  Date - revision : 3/11/2004
--                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
--
--  Author :          Maria George
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 - 2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  use IEEE.std_logic_unsigned.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY tap_ctrl IS
   PORT (
      CLK                     : IN std_logic;   
      RESET                   : IN std_logic;   
      RDY_STATUS              : IN std_logic;   
      DQS                     : IN std_logic;   
      CTRL_DUMMYREAD_START    : IN std_logic;   
      DLYINC                  : OUT std_logic;   
      DLYCE                   : OUT std_logic;   
      DLYRST                  : OUT std_logic;   
      SEL_DONE                : OUT std_logic;   
      FIRST_RISING            : OUT std_logic;   
      VALID_DATA_TAP_COUNT    : OUT std_logic;   
      DATA_TAP_COUNT          : OUT std_logic_vector(5 DOWNTO 0));   
END tap_ctrl;

ARCHITECTURE rtl OF tap_ctrl IS

	attribute syn_noprune : boolean;
	attribute IOB : string;

	attribute syn_noprune of rtl : architecture is true;
--   attribute syn_noprune of pulse_width_tap_count : signal is true;
--	attribute syn_noprune of pulse_center_tap_count : signal is true;
   attribute IOB of dqs_r1: label is "TRUE"; 
   attribute IOB of dqs_r2: label is "TRUE"; 

   SIGNAL prev_dqs_level           :  std_logic;   
   SIGNAL dly_inc                  :  std_logic;   
   SIGNAL dly_ce                   :  std_logic;   
   SIGNAL dly_rst                  :  std_logic;   
   SIGNAL transition               :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL rising                   :  std_logic;   
   SIGNAL rising_edge              :  std_logic;   
   SIGNAL first_edge               :  std_logic;   
   SIGNAL second_edge              :  std_logic;   
   SIGNAL second_edge_r1           :  std_logic;   
   SIGNAL second_edge_r2           :  std_logic;   
   SIGNAL second_edge_r3           :  std_logic;   
   SIGNAL transition_rst           :  std_logic;   
   SIGNAL sel_complete             :  std_logic;   
   SIGNAL tap_counter              :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL first_edge_tap_count     :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL second_edge_tap_count    :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL pulse_width_tap_count    :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL data_bit_tap_count       :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL state                    :  std_logic_vector(2 DOWNTO 0);   
   SIGNAL idelay_rst_idle          :  std_logic;   
   SIGNAL idelay_rst_idle_r1       :  std_logic;   
   SIGNAL idelay_inc_idle          :  std_logic;   
   SIGNAL idelay_inc_idle_r1       :  std_logic;   
   SIGNAL detect_edge_idle         :  std_logic;   
   SIGNAL detect_edge_idle_r1      :  std_logic;   
   SIGNAL dly_after_first_cnt      :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL pulse_center_tap_count   :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL valid_data_count         :  std_logic;   
   SIGNAL data_count_valid         :  std_logic;   
   SIGNAL dly_after_first          :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL curr_dqs_level           :  std_logic;   
   SIGNAL dqs_int                  :  std_logic;   
   SIGNAL delay_sel_done           :  std_logic;   
   SIGNAL reset_int                :  std_logic;   
   SIGNAL gnd                      :  std_logic;   
   SIGNAL unused_data_in           :  std_logic_vector(7 DOWNTO 0);   

BEGIN
   DLYINC <= dly_inc;
   DLYCE <= dly_ce;
   DLYRST <=  dly_rst;
   SEL_DONE <=  sel_complete;
   FIRST_RISING <= rising_edge;
   VALID_DATA_TAP_COUNT <= valid_data_count;
   DATA_TAP_COUNT <= data_bit_tap_count ;  
	reset_int <= (NOT RDY_STATUS) OR RESET ;
   gnd <= '0' ;
   unused_data_in(7 DOWNTO 0) <= "11111111" ;

DCV: process(second_edge_r3, tap_counter(5 DOWNTO 0))
    begin
	   if((second_edge_r3 = '1') OR (tap_counter(5 DOWNTO 0) = "111111"))then
		   data_count_valid <= '1';
      else
		   data_count_valid <= '0';
      end if;
    end process DCV;

DSD: process(second_edge_r3, tap_counter(5 DOWNTO 0), CTRL_DUMMYREAD_START, sel_complete)
    begin
	   if((second_edge_r3 = '1') OR (tap_counter(5 DOWNTO 0) = "111111"))then
		   delay_sel_done <= '1';
      elsif(CTRL_DUMMYREAD_START = '0')then 
		   delay_sel_done <= '0';
      else
		   delay_sel_done	<= sel_complete;
      end if;
    end process DSD;

DAF: process(transition(1 DOWNTO 0),first_edge,dly_inc,dly_after_first_cnt(3 DOWNTO 0))
    begin
	   if((transition(1 DOWNTO 0) = "01") AND (first_edge = '0'))then
		   dly_after_first(3 DOWNTO 0) <= "1001";
      elsif((dly_after_first_cnt(3 DOWNTO 0) /= "0000") AND (dly_inc = '1'))then 
		   dly_after_first(3 DOWNTO 0) <= dly_after_first_cnt(3 downto 0) - "0001";
      else
		   dly_after_first(3 DOWNTO 0) <= dly_after_first_cnt(3 DOWNTO 0);
      end if;
    end process DAF;
	    
   -- Two pipeline stages for DQS output from IDELAY block
   dqs_r1 : FD 
      PORT MAP (
         Q => dqs_int,
         C => CLK,
         D => DQS);   
      
   dqs_r2 : FD 
      PORT MAP (
         Q => curr_dqs_level,
         C => CLK,
         D => dqs_int);   
   

   -- Shift registers for controls   
   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (reset_int = '1') THEN
            second_edge_r1 <= '0';    
            second_edge_r2 <= '0';    
            second_edge_r3 <= '0';    
            idelay_rst_idle_r1 <= '0';    
            idelay_inc_idle_r1 <= '0';    
            detect_edge_idle_r1 <= '0';    
            valid_data_count <= '0';    
         ELSE
            second_edge_r1 <= second_edge;    
            second_edge_r2 <= second_edge_r1;    
            second_edge_r3 <= second_edge_r2;    
            idelay_rst_idle_r1 <= idelay_rst_idle;    
            idelay_inc_idle_r1 <= idelay_inc_idle;    
            detect_edge_idle_r1 <= detect_edge_idle;    
            valid_data_count <= data_count_valid;    
         END IF;
      END IF;
   END PROCESS;

   -- Tap Delay Selection Complete for Data bus associated with a DQS                              
   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (reset_int = '1') THEN
            sel_complete <= '0';    
         ELSE
            sel_complete <= delay_sel_done;    
         END IF;
      END IF;
   END PROCESS;

   -- Start detection of second transition only after 10 taps  
   -- from first transition  
   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (reset_int = '1') THEN
            dly_after_first_cnt(3 DOWNTO 0) <= "0000";    
         ELSE
            dly_after_first_cnt(3 DOWNTO 0) <= dly_after_first(3 DOWNTO 0);    
         END IF;
      END IF;
   END PROCESS;

   -- Tap Counter       
   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF ((reset_int = '1') OR (tap_counter(5 DOWNTO 0) = "111111")) 
         THEN
            tap_counter(5 DOWNTO 0) <= "000000";    
         ELSE
            IF (dly_inc = '1') THEN
               tap_counter(5 DOWNTO 0) <= tap_counter(5 DOWNTO 0) + 
               "000001";    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- Tap value for Data IDELAY circuit
   
   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (reset_int = '1') THEN
            first_edge_tap_count(5 DOWNTO 0) <= "000000";    
         ELSE
            IF ((transition(1 DOWNTO 0) = "01") AND (first_edge = '0')) 
            THEN
               first_edge_tap_count(5 DOWNTO 0) <= tap_counter(5 DOWNTO 
               0) - "000010";    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (reset_int = '1') THEN
            second_edge_tap_count(5 DOWNTO 0) <= "000000";    
         ELSE
            IF ((transition(1 DOWNTO 0) = "10") AND (second_edge = '0')) 
            THEN
               second_edge_tap_count(5 DOWNTO 0) <= tap_counter(5 DOWNTO 
               0) - "000010";    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (reset_int = '1') THEN
            pulse_width_tap_count(5 DOWNTO 0) <= "000000";    
         ELSE
            IF (second_edge_r1 = '1') THEN
               pulse_width_tap_count(5 DOWNTO 0) <= 
               second_edge_tap_count(5 DOWNTO 0) - 
               first_edge_tap_count(5 DOWNTO 0);    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (reset_int = '1') THEN
            pulse_center_tap_count(5 DOWNTO 0) <= "000000";       
         ELSE
            IF (second_edge_r2 = '1') THEN
               -- Shift right to divide by 2 and find pulse center              
                pulse_center_tap_count(5 DOWNTO 0) <= '0' & pulse_center_tap_count(5 DOWNTO 1);
            END IF;
         END IF;
      END IF;
   END PROCESS;


   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (reset_int = '1') THEN
            data_bit_tap_count(5 DOWNTO 0) <= "000000";  
           --  data_bit_tap_count(5 DOWNTO 0) <= "110111";   
         ELSE
            IF (second_edge_r3 = '1') THEN
               -- 2 edges detected               
               data_bit_tap_count(5 DOWNTO 0) <= first_edge_tap_count(5 
               DOWNTO 0) + pulse_center_tap_count(5 DOWNTO 0);    
            ELSE
               IF ((transition(1 DOWNTO 0) = "01") AND (tap_counter(5 
               DOWNTO 0) = "111111")) THEN
                  -- Only 1 edge detected                  
                  -- The default is + 6'b010000;                 
                  -- For 133MHz DDR such as on the ML-455 a better number would be 25 taps                 
                  -- (0x19) or half the bit-rate        
                  -- By subtracting instead of adding this value we get the best                   
                  -- possible latency                  
                  -- For other designs it may not be possible to subtract depending                  
                  -- on the amount of signal flight time                  
                  -- in which case it will be necesarry to add instead of subtract                   
                  data_bit_tap_count(5 DOWNTO 0) <= 
                  first_edge_tap_count(5 DOWNTO 0) - "011001"; 
                  -- data_bit_tap_count(5 DOWNTO 0) <= "110111";      
               ELSE
                  IF ((transition(1 DOWNTO 0) = "00") AND (tap_counter(5 
                  DOWNTO 0) = "111111")) THEN
                     -- No edges detected                     
                     data_bit_tap_count(5 DOWNTO 0) <= "100000"; 
                     -- data_bit_tap_count(5 DOWNTO 0) <= "110111";      
                  END IF;
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- First and second edge assignment logic 
   
   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (reset_int = '1') THEN
            transition(1 DOWNTO 0) <= "00";    
            rising <= '0';    
         ELSE
            IF ((dly_after_first_cnt = "0000") AND (state(2 DOWNTO 0) = 
            "011")) THEN
               IF ((curr_dqs_level /= prev_dqs_level) AND 
               (transition_rst = '0')) THEN
                  transition(1 DOWNTO 0) <= transition(1 DOWNTO 0) + 
                  "01";    
                  IF ((curr_dqs_level = '0') AND (prev_dqs_level = '1')) 
                  THEN
                     rising <= '1';    
                  ELSE
                     rising <= '0';    
                  END IF;
               END IF;
            ELSE
               IF (transition_rst = '1') THEN
                  transition(1 DOWNTO 0) <= "00";    
               ELSE
                  transition(1 DOWNTO 0) <= transition(1 DOWNTO 0);    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (reset_int = '1') THEN
            transition_rst <= '0';    
            first_edge <= '0';    
            second_edge <= '0';    
            rising_edge <= '0';    
         ELSE
            CASE transition(1 DOWNTO 0) IS
               WHEN "01" =>
                        first_edge <= '1';    
                        IF (rising = '1') THEN
                           rising_edge <= '1';    
                        ELSE
                           rising_edge <= '0';    
                        END IF;
               WHEN "10" =>
                        IF (transition_rst = '1') THEN
                           second_edge <= '0';    
                           transition_rst <= '0';    
                        ELSE
                           second_edge <= '1';    
                           transition_rst <= '1';    
                        END IF;
               WHEN OTHERS  =>
                        first_edge <= '0';    
                        second_edge <= '0';    
               
            END CASE;
         END IF;
      END IF;
   END PROCESS;

   -- State Machine for edge detection and midpoint determination   
   
   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (reset_int = '1') THEN
            -- DQS IDELAY in reset            
            dly_rst <= '1';    
            dly_ce <= '0';    
            dly_inc <= '0';    
            idelay_rst_idle <= '0';    
            detect_edge_idle <= '0';    
            idelay_inc_idle <= '0';    
            prev_dqs_level <= curr_dqs_level;    
            state(2 DOWNTO 0) <= "000";    
         ELSE
            IF ((CTRL_DUMMYREAD_START = '1') AND (sel_complete = '0')) 
            THEN
               CASE state(2 DOWNTO 0) IS
                  WHEN "000" =>
                           -- idelay_rst                            
                           dly_rst <= '1';    
                           dly_ce <= '0';    
                           dly_inc <= '0';    
                           idelay_rst_idle <= '1';    
                           state(2 DOWNTO 0) <= "001";    
                  WHEN "001" =>
                           -- idle                           
                           dly_rst <= '0';    
                           dly_ce <= '0';    
                           dly_inc <= '0';    
                           idelay_rst_idle <= '0';    
                           detect_edge_idle <= '0';    
                           idelay_inc_idle <= '0';    
                           IF (idelay_rst_idle_r1 = '1') THEN
                              state(2 DOWNTO 0) <= "010";    
                           ELSE
                              IF ((idelay_inc_idle_r1 = '1') OR 
                              (((detect_edge_idle_r1 = '1') AND 
                              (second_edge_r1 = '0')) AND (tap_counter(5 
                              DOWNTO 0) /= "111111"))) THEN
                                 state(2 DOWNTO 0) <= "011";    
                              ELSE
                                 state(2 DOWNTO 0) <= "001";    
                              END IF;
                           END IF;
                  WHEN "010" =>
                           -- idelay_inc                            
                           dly_rst <= '0';    
                           dly_ce <= '1';    
                           dly_inc <= '1';    
                           idelay_inc_idle <= '1';    
                           state(2 DOWNTO 0) <= "001";    
                           prev_dqs_level <= curr_dqs_level;    
                  WHEN "011" =>
                           -- detect_edge                            
                           dly_rst <= '0';    
                           dly_ce <= '1';    
                           dly_inc <= '1';    
                           detect_edge_idle <= '1';    
                           state(2 DOWNTO 0) <= "001";    
                           prev_dqs_level <= curr_dqs_level;    
                  WHEN OTHERS  =>
                           dly_rst <= '0';    
                           dly_ce <= '0';    
                           dly_inc <= '0';    
                           idelay_rst_idle <= '0';    
                           detect_edge_idle <= '0';    
                           idelay_inc_idle <= '0';    
                           prev_dqs_level <= curr_dqs_level;    
                           state(2 DOWNTO 0) <= "001";    
                  
               END CASE;
            ELSE
               dly_rst <= '0';    
               dly_ce <= '0';    
               dly_inc <= '0';    
               idelay_rst_idle <= '0';    
               detect_edge_idle <= '0';    
               idelay_inc_idle <= '0';    
               prev_dqs_level <= curr_dqs_level;    
               state(2 DOWNTO 0) <= "000";    
            END IF;
         END IF;
      END IF;
   END PROCESS;

END rtl;
