--*******************************************************************************
--
--  File name :       user_interface.vhd
--
--  Description :     This module instantiates the FIFO16 primitive for read  
--                    data. One FIFO for rising edge data and the other for
--                    falling edge data.
-- 
--                    
--  Date - revision : 3/24/2004
--                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
--
--  Author :          Maria George
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 - 2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  USE IEEE.std_logic_unsigned.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY user_interface IS
   PORT (
      USER_CLK                : IN std_logic;   
      DDR_CLK                 : IN std_logic;   
      RESET                   : IN std_logic;   
      FIRST_RISING            : IN std_logic_vector(3 DOWNTO 0);   
      READ_DATA_RISE          : IN std_logic_vector(63 DOWNTO 0);   
      READ_DATA_FALL          : IN std_logic_vector(63 DOWNTO 0);   
      READ_DATA_VALID         : OUT std_logic;   
      CTRL_AF_RDEN            : IN std_logic;   
      AF_ADDR                 : OUT std_logic_vector(26 DOWNTO 0);   
      AF_EMPTY                : OUT std_logic;   
      APP_WDF_DATA            : IN std_logic_vector(63 DOWNTO 0);   
      APP_WDF_WREN            : IN std_logic;   
      CTRL_WDF_RDEN           : IN std_logic;   
      WDF_DATA                : OUT std_logic_vector(127 DOWNTO 0);   
      WDF_ALMOST_FULL         : OUT std_logic;   
      WDF_EMPTY               : OUT std_logic;   
      CTRL_CMD_RDEN           : IN std_logic;   
      CMD_ADDR                : OUT std_logic_vector(23 DOWNTO 0);   
      CMD_EMPTY               : OUT std_logic;   
      PCI_ADDR                : IN std_logic_vector(26 DOWNTO 0);   
      PCI_DATA_IN             : IN std_logic_vector(63 DOWNTO 0);   
      PCI_CBEA                : IN std_logic_vector(7 DOWNTO 0);   
      PCI_WRITE               : IN std_logic;   
      PCI_HIT                 : IN std_logic;   
      PCI_HIT_DDR             : OUT std_logic;   
      PCI_HIT_EARLY           : IN std_logic;   
      PCI_BE                  : OUT std_logic_vector(7 DOWNTO 0);   
      READ_DATA_OUT           : OUT std_logic_vector(63 DOWNTO 0);   
      BURST_HALF              : OUT std_logic;   
      FIRST_READ              : IN std_logic;   
      S_WAIT                  : OUT std_logic;   
      NOT_READY               : IN std_logic;   
      S_DONE                  : IN std_logic;   
      S_DONE_DDR              : OUT std_logic);   
END user_interface;

ARCHITECTURE rtl OF user_interface IS

   COMPONENT backend_fifos
    PORT (
      USER_CLK                : IN std_logic;   
      DDR_CLK                 : IN std_logic;   
      rst                     : IN std_logic;   
      app_af_addr             : IN std_logic_vector(31 DOWNTO 0);   
      app_af_WrEn             : IN std_logic;   
      ctrl_af_RdEn            : IN std_logic;   
      af_addr                 : OUT std_logic_vector(26 DOWNTO 0);   
      af_Almost_Empty         : OUT std_logic;   
      af_empty                : OUT std_logic;   
      app_Wdf_data            : IN std_logic_vector(127 DOWNTO 0);   
      app_Wdf_WrEn            : IN std_logic;   
      ctrl_Wdf_RdEn           : IN std_logic;   
      Wdf_data                : OUT std_logic_vector(127 DOWNTO 0);   
      Wdf_Empty               : OUT std_logic;   
      Wdf_Almost_Full         : OUT std_logic;   
      app_cmd_addr            : IN std_logic_vector(31 DOWNTO 0);   
      app_cmd_WrEn            : IN std_logic;   
      ctrl_cmd_RdEn           : IN std_logic;   
      cmd_addr                : OUT std_logic_vector(23 DOWNTO 0);   
      cmd_Almost_Empty        : OUT std_logic;   
      cmd_empty               : OUT std_logic;   
      cmd_Almost_Full         : OUT std_logic);   
   END COMPONENT;

   COMPONENT rd_data_fifo 
     PORT (
      USER_CLK                : IN std_logic;   
      DDR_CLK                 : IN std_logic;   
      RESET                   : IN std_logic;     
      FIRST_RISING            : IN std_logic_vector(3 DOWNTO 0);         
      READ_DATA_RISE          : IN std_logic_vector(63 DOWNTO 0);        
      READ_DATA_FALL          : IN std_logic_vector(63 DOWNTO 0);         
      READ_DATA_OUT           : OUT std_logic_vector(63 DOWNTO 0);         
      READ_DATA_VALID         : OUT std_logic;   
      FIRST_READ              : IN std_logic);   
   END COMPONENT;

   SIGNAL wr_af_empty              :  std_logic;   
   SIGNAL rd_af_empty              :  std_logic;   
   SIGNAL wr_df_output             :  std_logic_vector(127 DOWNTO 0);   
   SIGNAL byte_count               :  std_logic_vector(11 DOWNTO 0);   
   SIGNAL pci_addr_attr            :  std_logic_vector(26 DOWNTO 0);   
   SIGNAL pci_addr_addr            :  std_logic_vector(26 DOWNTO 0);   
   SIGNAL pci_cbea_attr            :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL pci_cbea_addr            :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL pci_data_in_r            :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL app_wdf_wren_r           :  std_logic;   
   SIGNAL app_wdf_wren_one         :  std_logic;   
   SIGNAL steerer                  :  std_logic;   
   SIGNAL pci_data_in_128          :  std_logic_vector(127 DOWNTO 0);   
   SIGNAL pci_hit_early_r1         :  std_logic;   
   SIGNAL app_wdf_wren_one_r       :  std_logic;   
   SIGNAL pci_data_in_r2           :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL start_addr               :  std_logic_vector(26 DOWNTO 0);   
   SIGNAL byte_count_r             :  std_logic_vector(11 DOWNTO 0);   
   --burst count calculation logic 
   SIGNAL burst_count              :  std_logic_vector(6 DOWNTO 0);   
   SIGNAL end_addr                 :  std_logic_vector(26 DOWNTO 0);   
   SIGNAL trunc_end_addr           :  std_logic_vector(6 DOWNTO 0);   
   SIGNAL trunc_start_addr         :  std_logic_vector(6 DOWNTO 0);   
   SIGNAL burst_count_half         :  std_logic_vector(6 DOWNTO 0);   
   SIGNAL wait_cnt                 :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL app_af_addr_temp         :  std_logic_vector(31 DOWNTO 0);   
   SIGNAL app_af_WrEn_temp         :  std_logic;   
   SIGNAL ctrl_af_RdEn_temp        :  std_logic;   
   SIGNAL app_cmd_addr_temp        :  std_logic_vector(31 DOWNTO 0);     
   SIGNAL pci_be_reg               :  std_logic_vector(7 DOWNTO 0);     
   SIGNAL trunc_diff               :  std_logic_vector(6 DOWNTO 0); 
	SIGNAL trunc_diff_times_two     :  std_logic_vector(6 DOWNTO 0); 
   SIGNAL trunc_temp               :  std_logic_vector(6 DOWNTO 0); 
   SIGNAL blocker                  :  std_logic_vector(2 DOWNTO 0); 

BEGIN
   WDF_DATA <= wr_df_output(127 DOWNTO 0);
   PCI_HIT_DDR <= PCI_HIT;
   PCI_BE <= pci_be_reg;
   S_DONE_DDR <= S_DONE;
 
   rd_data_fifo0 : rd_data_fifo 
      PORT MAP (
         USER_CLK => USER_CLK,
         DDR_CLK => DDR_CLK,
         RESET => RESET,
         FIRST_RISING => FIRST_RISING,
         READ_DATA_RISE => READ_DATA_RISE,
         READ_DATA_FALL => READ_DATA_FALL,
         READ_DATA_OUT => READ_DATA_OUT,
         READ_DATA_VALID => READ_DATA_VALID,
         FIRST_READ => FIRST_READ);   
  
   PROCESS (USER_CLK)
   BEGIN
      IF (USER_CLK'EVENT AND USER_CLK = '1') THEN
         IF (RESET = '1') THEN
            pci_hit_early_r1 <= '0';    
         ELSE
            pci_hit_early_r1 <= PCI_HIT_EARLY;    
         END IF;
      END IF;
   END PROCESS;

   app_af_addr_temp <= "00000" & pci_addr_attr(26 DOWNTO 0);
   app_af_WrEn_temp <= (pci_hit_early_r1 AND NOT NOT_READY);
   ctrl_af_RdEn_temp <= (CTRL_AF_RDEN AND NOT NOT_READY);
   app_cmd_addr_temp <= "00000000" & pci_cbea_addr(3 DOWNTO 0) &
	                      byte_count(11 DOWNTO 0) & pci_cbea_attr(7 DOWNTO 0);

   backend_fifos0 : backend_fifos 
      PORT MAP (
         USER_CLK => USER_CLK,
         DDR_CLK => DDR_CLK,
         rst => RESET,
         app_af_addr => app_af_addr_temp,
         app_af_WrEn => app_af_WrEn_temp,
         ctrl_af_RdEn => ctrl_af_RdEn_temp,
         af_addr => AF_ADDR,
         af_Almost_Empty => open,
         af_empty => AF_EMPTY,
         app_Wdf_data => pci_data_in_128(127 DOWNTO 0),
         app_Wdf_WrEn => burst_count(0),
         ctrl_Wdf_RdEn => CTRL_WDF_RDEN,
         Wdf_data => wr_df_output,
         Wdf_Almost_Full => WDF_ALMOST_FULL,
         Wdf_Empty => WDF_EMPTY,
         app_cmd_addr => app_cmd_addr_temp,
         app_cmd_WrEn => PCI_HIT,
         ctrl_cmd_RdEn => CTRL_CMD_RDEN,
         cmd_addr => CMD_ADDR,
         cmd_Almost_Empty => rd_af_empty,
         cmd_empty => CMD_EMPTY,
         cmd_Almost_Full => open);   
   

   --register PCI_ADDR and PCI_CBEA twice
   --When S_HIT is high, the first register will   
   --contain the attribute phase and the second   
   --will contain the address phase
   
   PROCESS (USER_CLK)
   BEGIN
      IF (USER_CLK'EVENT AND USER_CLK = '1') THEN
         pci_addr_attr(26 DOWNTO 0) <= PCI_ADDR(26 DOWNTO 0);    
         pci_addr_addr(26 DOWNTO 0) <= pci_addr_attr(26 DOWNTO 0);    
         pci_cbea_attr(7 DOWNTO 0) <= PCI_CBEA(7 DOWNTO 0);    
         pci_cbea_addr(7 DOWNTO 0) <= pci_cbea_attr(7 DOWNTO 0);    
      END IF;
   END PROCESS;

   --for burst read/write the byte count is given as    
   --PCI_CBEA[3:0] and PCI_ADDR[7:0] during the PCI ATTR Phase
   byte_count(11 DOWNTO 0) <= pci_cbea_attr(3 DOWNTO 0) & 
   pci_addr_attr(7 DOWNTO 0) ;

   PROCESS (USER_CLK)
   BEGIN
      IF (USER_CLK'EVENT AND USER_CLK = '1') THEN
         IF (RESET = '1') THEN
            pci_be_reg(7 DOWNTO 0) <= "00000000";    
         ELSE
            IF (APP_WDF_WREN = '1') THEN
               pci_be_reg(7 DOWNTO 0) <= PCI_CBEA(7 DOWNTO 0);    
            ELSE
               pci_be_reg(7 DOWNTO 0) <= pci_be_reg(7 DOWNTO 0);   
            END IF;
         END IF;
      END IF;
   END PROCESS;

   --byte-steering logic for wd_fifo  
   PROCESS (USER_CLK)
   BEGIN
      IF (USER_CLK'EVENT AND USER_CLK = '1') THEN
         pci_data_in_r(63 DOWNTO 0) <= PCI_DATA_IN(63 DOWNTO 0);    
         pci_data_in_r2(63 DOWNTO 0) <= pci_data_in_r(63 DOWNTO 0);    
         app_wdf_wren_r <= APP_WDF_WREN;    
         app_wdf_wren_one_r <= app_wdf_wren_one;    
      END IF;
   END PROCESS;
   -- rising edge detector
   app_wdf_wren_one <= APP_WDF_WREN AND NOT app_wdf_wren_r ;

   PROCESS (USER_CLK)
   BEGIN
      IF (USER_CLK'EVENT AND USER_CLK = '1') THEN
         IF (RESET = '1') THEN
            start_addr(26 DOWNTO 0) <= "000000000000000000000000000";    
            byte_count_r(11 DOWNTO 0) <= "000000000000";    
         ELSE
            IF (PCI_HIT = '1') THEN
               start_addr(26 DOWNTO 0) <= pci_addr_addr(26 DOWNTO 0);    
               byte_count_r(11 DOWNTO 0) <= byte_count(11 DOWNTO 0);    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   end_addr(26 DOWNTO 0) <= start_addr(26 DOWNTO 0) + 
	                         byte_count_r(11 DOWNTO 0) - 
									 "000000000000000000000000100" ;
   trunc_end_addr(6 DOWNTO 0) <= end_addr(10 DOWNTO 4) ;
   trunc_start_addr(6 DOWNTO 0) <= start_addr(10 DOWNTO 4) ;

   -- calculate the total number of 16 byte transfers   
   -- multiply by 2 and use bit 0 as a write enable to the fifo
	trunc_diff(6 DOWNTO 0) <= trunc_end_addr(6 DOWNTO 0) - trunc_start_addr(6 DOWNTO 0);
	trunc_diff_times_two(6 DOWNTO 0) <= trunc_diff(5 downto 0) & '0';
	   
   PROCESS (USER_CLK)
   BEGIN
      IF (USER_CLK'EVENT AND USER_CLK = '1') THEN
         IF (RESET = '1') THEN
            burst_count(6 DOWNTO 0) <= "0000000";    
         ELSE
            IF (app_wdf_wren_one = '1') THEN
               burst_count(6 DOWNTO 0) <= trunc_diff_times_two(6 DOWNTO 0) + 1;  
            ELSE
               IF (burst_count /= "0000000") THEN
                  burst_count(6 DOWNTO 0) <= burst_count(6 DOWNTO 0) - "0000001";    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   -- the following code    
   -- purpose is to let the write data fifo fill up with 1/2 of the burst    
   -- before letting   
   -- the ddr controller to start the burst because of DDR:SDR clock ratio 
   trunc_temp(6 DOWNTO 0) <= trunc_diff_times_two(6 DOWNTO 0) + "0000001";

   PROCESS (USER_CLK)
   BEGIN
      IF (USER_CLK'EVENT AND USER_CLK = '1') THEN
         IF (RESET = '1') THEN
            burst_count_half(6 DOWNTO 0) <= "0000000";    
         ELSE
            IF (app_wdf_wren_one = '1') THEN
               burst_count_half(6 DOWNTO 0) <= '0' & trunc_temp(6 downto 1);   
            END IF;
         END IF;
      END IF;
   END PROCESS;


 -- blocker is used to "block" burst half during the first 5 cycles after PCI_HIT_EARLY
 -- blocker not necessary in a design that doens't use PCI_HIT_EARLY  
 -- The problem is that the DDR controller state machine is already in the active
 -- state by the time it sees PCI_HIT; Since BURST_HALF may still be asserted
 -- from the previous transaction the controller goes ahead and starts the
 -- burst early.

   PROCESS (USER_CLK)
   BEGIN
      IF (USER_CLK'EVENT AND USER_CLK = '1') THEN
         IF (RESET = '1') THEN
            blocker <= "000";
         ELSE 
            IF (PCI_HIT_EARLY = '1')THEN
               blocker <= "110";
            ELSE 
               IF(blocker /= "000")THEN
                  blocker <= blocker - '1';
               ELSE
	          blocker <= blocker;
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;


   BURST_HALF <= '1' WHEN ((burst_count_half >= burst_count) and (blocker = "000")) ELSE '0';

   PROCESS (USER_CLK)
   BEGIN
      IF (USER_CLK'EVENT AND USER_CLK = '1') THEN
         IF (RESET = '1') THEN
            steerer <= '0';    
         ELSE
            IF (PCI_HIT = '1') THEN
               steerer <= pci_addr_addr(3);    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (steerer, PCI_DATA_IN(63 DOWNTO 0), pci_data_in_r(63 DOWNTO 0),
            pci_data_in_r2(63 DOWNTO 0))
   BEGIN
      IF (steerer = '1') THEN
         pci_data_in_128 <= pci_data_in_r2(31 DOWNTO 0) & 
         pci_data_in_r2(63 DOWNTO 32) & pci_data_in_r(31 DOWNTO 0) & 
         pci_data_in_r(63 DOWNTO 32);    
      ELSE
         pci_data_in_128 <= pci_data_in_r(31 DOWNTO 0) & 
         pci_data_in_r(63 DOWNTO 32) & PCI_DATA_IN(31 DOWNTO 0) & 
         PCI_DATA_IN(63 DOWNTO 32);    
      END IF;
   END PROCESS;

   -- ************************************************************************
   --    Counter machine to drive S_WAIT   
   -- *************************************************************************
   PROCESS (wait_cnt, PCI_HIT)
   BEGIN
      IF ((wait_cnt /= "0000") OR (PCI_HIT = '1')) THEN
         S_WAIT <= '1';    
      ELSE
         S_WAIT <= '0';    
      END IF;
   END PROCESS;

   PROCESS (USER_CLK)
   BEGIN
      IF (USER_CLK'EVENT AND USER_CLK = '1') THEN
         IF (RESET = '1') THEN
            wait_cnt <= "0000";    
         ELSE
            IF ((PCI_HIT = '1') AND (PCI_WRITE = '0') AND (NOT_READY = '0')) THEN
               CASE pci_addr_addr(4 DOWNTO 3) IS
                  WHEN "00" =>
                           wait_cnt <= "0110";    
                  WHEN "01" =>
                           wait_cnt <= "0111";    
                  WHEN "10" =>
                           wait_cnt <= "1000";    
                  WHEN "11" =>
                           wait_cnt <= "1001";    
                  WHEN OTHERS =>
                           NULL;
                  
               END CASE;
            ELSE
               IF (wait_cnt /= "0000") THEN
                  wait_cnt <= wait_cnt - "0001";    
               ELSE
                  wait_cnt <= "0000";    
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

END rtl;
