--***********************************************************************
--
--  File:   userapp.v
--  Rev:    5.0.0
--
--  This is an example template for the user backend application.
--
--  Copyright (c) 2003 Xilinx, Inc.  All rights reserved.
--
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 - 2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;


ENTITY USERAPP IS
   PORT (
      S_DATA_OUT              : IN std_logic_vector(63 DOWNTO 0);   --  target address, attribute and write data bus
      S_CBEA_OUT              : IN std_logic_vector(7 DOWNTO 0);   --  command byte enables
      S_DATA_IN               : OUT std_logic_vector(63 DOWNTO 0);   --  target read data bus
      S_ADDL_VLD              : IN std_logic;   --  lower 32-bit address valid
      S_ADDH_VLD              : IN std_logic;   --  upper 32-bit address valid
      S_ATTR_VLD              : IN std_logic;   --  target attributes valid
      S_DATL_VLD              : IN std_logic;   --  lower 32-bit data valid
      S_DATH_VLD              : IN std_logic;   --  upper 32-bit data valid
      S_DATA_NXT              : IN std_logic;   --  next data requested
      S_TABORT                : OUT std_logic;   --  slave target abort
      S_RETRY                 : OUT std_logic;   --  slave retry
      S_DISCON                : OUT std_logic;   --  slave disconnect request
      S_NXTADB                : OUT std_logic;   
      S_WAIT                  : OUT std_logic;   --  target wait: PCI-X only prior to 1st data transfer
      S_SPLIT                 : OUT std_logic;   --  split transaction requested
      S_WRITE                 : IN std_logic;   --  target write request
      S_DONE                  : IN std_logic;   --  target done
      S_HIT                   : IN std_logic_vector(15 DOWNTO 0);   --  
      M_DATA_OUT              : IN std_logic_vector(63 DOWNTO 0);   
      M_CBEA_IN               : OUT std_logic_vector(7 DOWNTO 0);   
      M_DATA_IN               : OUT std_logic_vector(63 DOWNTO 0);   
      M_ADDL_VLD              : IN std_logic;   
      M_ADDH_VLD              : IN std_logic;   
      M_ATTR_VLD              : IN std_logic;   
      M_DATL_VLD              : IN std_logic;   
      M_DATH_VLD              : IN std_logic;   
      M_DATA_NXT              : IN std_logic;   
      M_MABORT                : IN std_logic;   
      M_RETRY                 : IN std_logic;   
      M_NXTADB                : IN std_logic;   
      M_DISCON                : IN std_logic;   
      M_TABORT                : IN std_logic;   
      M_SPLIT                 : IN std_logic;   
      M_FINISH                : OUT std_logic;   
      M_DONE                  : IN std_logic;   
      M_REQ                   : OUT std_logic;   
      PERR_N                  : IN std_logic;   
      SERR_N                  : IN std_logic;   
      INT_N                   : OUT std_logic;   
      MS_STAT                 : IN std_logic_vector(95 DOWNTO 0);   
      PME_N                   : OUT std_logic;   
      PM_STAT                 : IN std_logic_vector(47 DOWNTO 0);   
      CSR                     : IN std_logic_vector(31 DOWNTO 0);   
      CSR_SYSTEMERR_N         : OUT std_logic;   
      CSRX_UNEXPECTED         : OUT std_logic;   
      CSRX_DISCARDED          : OUT std_logic;   
      CSRX                    : IN std_logic_vector(47 DOWNTO 0);   
      PCIW_EN                 : IN std_logic;   
      PCIX_EN                 : IN std_logic;   
      RTR                     : IN std_logic;   
      CFG                     : IN std_logic_vector(511 DOWNTO 0);   
      RST                     : IN std_logic;   
      CLK                     : IN std_logic;   
      FPGA_RTR                : OUT std_logic;   
      --DDR interface

      DDR_CLK90               : IN std_logic;   
      DDR_CLKDIV              : IN std_logic;   
      CLK200_N                : IN std_logic;   
      CLK200_P                : IN std_logic;   
      USER_CLK                : IN std_logic;   
      SYS_RESET_IN            : IN std_logic;   
      DDR_DIMM_RAS_L          : OUT std_logic;   
      DDR_DIMM_CAS_L          : OUT std_logic;   
      DDR_DIMM_WE_L           : OUT std_logic;   
      DDR_DIMM_CKE0           : OUT std_logic;   
      DDR_DIMM_CKE1           : OUT std_logic;   
      DDR_DIMM_CS_N0_7        : OUT std_logic;   
      DDR_DIMM_CS_N8_15       : OUT std_logic;   
      DDR_DIMM_DQ             : INOUT std_logic_vector(63 DOWNTO 0);   
      DDR_DIMM_DQS            : INOUT std_logic_vector(7 DOWNTO 0);   
      DDR_DIMM_CLK0_P         : OUT std_logic;   
      DDR_DIMM_CLK0_N         : OUT std_logic;   
      DDR_DIMM_CLK1_P         : OUT std_logic;   
      DDR_DIMM_CLK1_N         : OUT std_logic;   
      DDR_DIMM_CLK2_P         : OUT std_logic;   
      DDR_DIMM_CLK2_N         : OUT std_logic;   
      DDR_DIMM_BA             : OUT std_logic_vector(1 DOWNTO 0);   
      DDR_DIMM_ADDRESS        : OUT std_logic_vector(11 DOWNTO 0);   
      DDR_DIMM_SA             : OUT std_logic_vector(2 DOWNTO 0);   
      DDR_DIMM_SDA            : INOUT std_logic;   
      DDR_DIMM_SCL            : OUT std_logic;   
      DDR_DIMM_DM             : OUT std_logic_vector(7 DOWNTO 0);   
      DCM_LOCK                : IN std_logic);   
END ENTITY USERAPP;

ARCHITECTURE rtl OF USERAPP IS

  attribute syn_edif_bit_format : string; 
  attribute syn_edif_scalar_format : string;
  attribute syn_noclockbuf : boolean; 
  attribute syn_hier : string; 
  attribute syn_edif_bit_format of RTL : architecture is "%u<%i>"; 
  attribute syn_edif_scalar_format of RTL : architecture is "%u"; 
  attribute syn_noclockbuf of RTL : architecture is true; 
  attribute syn_hier of RTL : architecture is "hard";   

COMPONENT ddr_top 
   PORT (
         DDR_CLK90               : IN std_logic;   
         DDR_CLKDIV              : IN std_logic;   
         CLK200_N                : IN std_logic;   
         CLK200_P                : IN std_logic;   
         USER_CLK                : IN std_logic;   
         SYS_RESET_IN            : IN std_logic;   
         DDR_DIMM_RAS_L          : OUT std_logic;   
         DDR_DIMM_CAS_L          : OUT std_logic;   
         DDR_DIMM_WE_L           : OUT std_logic;   
         DDR_DIMM_CKE0           : OUT std_logic;   
         DDR_DIMM_CKE1           : OUT std_logic;   
         DDR_DIMM_CS_N0_7        : OUT std_logic;   
         DDR_DIMM_CS_N8_15       : OUT std_logic;   
         DDR_DIMM_DQ             : INOUT std_logic_vector(63 DOWNTO 0);   
         DDR_DIMM_DQS            : INOUT std_logic_vector(7 DOWNTO 0);   
         DDR_DIMM_CLK0_P         : OUT std_logic;   
         DDR_DIMM_CLK0_N         : OUT std_logic;   
         DDR_DIMM_CLK1_P         : OUT std_logic;   
         DDR_DIMM_CLK1_N         : OUT std_logic;   
         DDR_DIMM_CLK2_P         : OUT std_logic;   
         DDR_DIMM_CLK2_N         : OUT std_logic;   
         DDR_DIMM_BA             : OUT std_logic_vector(1 DOWNTO 0);   
         DDR_DIMM_ADDRESS        : OUT std_logic_vector(11 DOWNTO 0);   
         DDR_DIMM_SA             : OUT std_logic_vector(2 DOWNTO 0);   
         DDR_DIMM_SDA            : INOUT std_logic;   
         DDR_DIMM_SCL            : OUT std_logic;   
         DDR_DIMM_DM             : OUT std_logic_vector(7 DOWNTO 0);   
         PCI_ADDR                : IN std_logic_vector(26 DOWNTO 0);   
         PCI_DATA_IN             : IN std_logic_vector(63 DOWNTO 0);   
         PCI_WRITE_VALID         : IN std_logic;   
         PCI_CBEA                : IN std_logic_vector(7 DOWNTO 0);   
         PCI_WRITE               : IN std_logic;   
         PCI_HIT                 : IN std_logic;   
         PCI_HIT_EARLY           : IN std_logic;   
         READ_DATA_OUT           : OUT std_logic_vector(63 DOWNTO 0);   
         READ_DATA_VALID         : OUT std_logic;   
         DDR_NOT_READY           : OUT std_logic;   
         DCM_LOCK                : IN std_logic;   
         S_WAIT                  : OUT std_logic;   
         S_DONE                  : IN std_logic);   
END COMPONENT;

   SIGNAL bar0                     :  std_logic;   
   SIGNAL bar1                     :  std_logic;   
   SIGNAL bar2                     :  std_logic;   
   SIGNAL bar0_rd_cs               :  std_logic;   
   SIGNAL bar1_rd_cs               :  std_logic;   
   SIGNAL bar2_rd_cs               :  std_logic;   
   SIGNAL bar0_wr_cs               :  std_logic;   
   SIGNAL bar1_wr_cs               :  std_logic;   
   SIGNAL bar2_wr_cs               :  std_logic;   
   SIGNAL pci_hit_early            :  std_logic;   
   SIGNAL s_wait_w                 :  std_logic;   
   SIGNAL DDR_NOT_READY            :  std_logic;   
   SIGNAL RAMDOUT                  :  std_logic_vector(63 DOWNTO 0);   
   SIGNAL BRAMWE                   :  std_logic_vector(7 DOWNTO 0);   

   SIGNAL reg_s_tabort             :  std_logic;   
   SIGNAL reg_s_retry              :  std_logic;   
   SIGNAL reg_s_discon             :  std_logic;   
   SIGNAL reg_s_nxtadb             :  std_logic;   
   SIGNAL reg_s_wait               :  std_logic;   
   SIGNAL reg_s_split              :  std_logic;   
   SIGNAL reg_int_n                :  std_logic;   
   SIGNAL reg_pme_n                :  std_logic;   
   SIGNAL reg_m_finish             :  std_logic;   
   SIGNAL s_data_keep              :  std_logic_vector(15 DOWNTO 0);   
   SIGNAL bar2_state               :  std_logic_vector(1 DOWNTO 0);   
   SIGNAL upper_addr               :  std_logic_vector(4 DOWNTO 0);   
   SIGNAL S_RETRY_temp            :  std_logic;   
 
BEGIN

   S_RETRY <= S_RETRY_temp;

   --******************************************************************//
   -- This section contains unused signals.                            //
   --******************************************************************//
   unused_signals_s : PROCESS (CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         reg_s_tabort <= '1';    
         reg_s_retry <= '1';    
         reg_s_discon <= '1';    
         reg_s_nxtadb <= '1';    
         reg_s_wait <= '1';    
         reg_s_split <= '1';    
         reg_int_n <= '1';    
         reg_pme_n <= '1';    
      ELSIF (CLK'EVENT AND CLK = '1') THEN
         reg_s_tabort <= '0';    
         reg_s_retry <= '0';    
         reg_s_discon <= '0';    
         reg_s_nxtadb <= '0';    
         reg_s_wait <= '0';    
         reg_s_split <= '0';    
         reg_int_n <= '1';    
         reg_pme_n <= '1';    
      END IF;
   END PROCESS unused_signals_s;
   S_TABORT <= reg_s_tabort ;
   S_RETRY_temp <= DDR_NOT_READY ;
   S_DISCON <= reg_s_discon ;
   S_NXTADB <= reg_s_nxtadb ;
   S_WAIT <= s_wait_w AND NOT S_RETRY_temp ;
   S_SPLIT <= reg_s_split ;
   INT_N <= reg_int_n ;
   PME_N <= reg_pme_n ;
   S_DATA_IN <= RAMDOUT(31 DOWNTO 0) & RAMDOUT(63 DOWNTO 32) ;
   --******************************************************************//
   -- This section contains unused signals.                            //
   --******************************************************************//
   unused_signals_m : PROCESS (CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         reg_m_finish <= '1';    
      ELSIF (CLK'EVENT AND CLK = '1') THEN
         reg_m_finish <= '0';    
      END IF;
   END PROCESS unused_signals_m;
   M_CBEA_IN <= "00000000" ;
   M_DATA_IN <= X"0000000000000000" ;
   M_FINISH <= reg_m_finish ;
   M_REQ <= '0' ;
   CSR_SYSTEMERR_N <= '1' ;
   CSRX_UNEXPECTED <= '0' ;
   CSRX_DISCARDED <= '0' ;


  --******************************************************************
  --  This section contains DDR-1 DIMM controller - Optimized for PCI 
  --  and PCI-X                                                       
  --  Dual Rank, 4 word burst, 133 MHz, 128 MB(16MEG x 64)            
  --******************************************************************

  --*******************************************************************
  -- In order to meet PCI-X 16 cycle latency spec we need to predecode
  -- the address bus for read operations.  This will allow the DDR 
  -- controller to activate row/bank at the earliest possible time
  --*******************************************************************

   PROCESS (CLK)
   BEGIN
      IF (CLK'EVENT AND CLK = '1') THEN
         IF (RST = '1') THEN
            s_data_keep(7 DOWNTO 0) <= "00000000";    
         ELSE
            IF (S_ADDL_VLD = '1') THEN
               s_data_keep(7 DOWNTO 0) <= S_DATA_OUT(7 DOWNTO 0);    
            ELSE
               s_data_keep <= s_data_keep;    
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         bar2_state <= "01";    
         upper_addr <= "00000";    
      ELSIF (CLK'EVENT AND CLK = '1') THEN
         CASE bar2_state IS	--S_HIT[6] is same as IDSEL delayed by 2 
            WHEN "01" => 				                   
                     IF ((S_WRITE = '1') AND (S_HIT(6)= '1') AND 
                     (s_data_keep(7 DOWNTO 0) = "00011000")) THEN
                        bar2_state <= "10";    
                     ELSE
                        bar2_state <= "01";    
                     END IF;
                     upper_addr(4 DOWNTO 0) <= upper_addr(4 DOWNTO 0);   
            WHEN "10" =>
                     IF (S_DATL_VLD = '1') THEN
                        bar2_state <= "01";    
                        upper_addr(4 DOWNTO 0) <= S_DATA_OUT(31 DOWNTO 27);    
                     ELSE
                        bar2_state <= "10";    
                        upper_addr(4 DOWNTO 0) <= upper_addr(4 DOWNTO 0);    
                     END IF;
            WHEN OTHERS  =>
                     bar2_state <= "01";    
                     upper_addr(4 DOWNTO 0) <= upper_addr(4 DOWNTO 0);               
         END CASE;
      END IF;
   END PROCESS;
   
   -- do the decode combinatorially for quickest timing
 
   pci_hit_early <= '1' WHEN ((S_DATA_OUT(31 downto 27) = upper_addr(4 downto 0)) AND (S_ADDL_VLD = '1')) ELSE '0';

   ddr_top_inst : ddr_top 
      PORT MAP (
         DDR_CLK90 => DDR_CLK90,
         DDR_CLKDIV => DDR_CLKDIV,
         CLK200_N => CLK200_N,
         CLK200_P => CLK200_P,
         USER_CLK => USER_CLK,
         SYS_RESET_IN => SYS_RESET_IN,
         DDR_DIMM_RAS_L => DDR_DIMM_RAS_L,
         DDR_DIMM_CAS_L => DDR_DIMM_CAS_L,
         DDR_DIMM_WE_L => DDR_DIMM_WE_L,
         DDR_DIMM_CKE0 => DDR_DIMM_CKE0,
         DDR_DIMM_CKE1 => DDR_DIMM_CKE1,
         DDR_DIMM_CS_N0_7 => DDR_DIMM_CS_N0_7,
         DDR_DIMM_CS_N8_15 => DDR_DIMM_CS_N8_15,
         DDR_DIMM_DQ => DDR_DIMM_DQ,
         DDR_DIMM_DQS => DDR_DIMM_DQS,
         DDR_DIMM_CLK0_P => DDR_DIMM_CLK0_P,
         DDR_DIMM_CLK0_N => DDR_DIMM_CLK0_N,
         DDR_DIMM_CLK1_P => DDR_DIMM_CLK1_P,
         DDR_DIMM_CLK1_N => DDR_DIMM_CLK1_N,
         DDR_DIMM_CLK2_P => DDR_DIMM_CLK2_P,
         DDR_DIMM_CLK2_N => DDR_DIMM_CLK2_N,
         DDR_DIMM_BA => DDR_DIMM_BA,
         DDR_DIMM_ADDRESS => DDR_DIMM_ADDRESS,
         DDR_DIMM_SA => DDR_DIMM_SA,
         DDR_DIMM_SDA => DDR_DIMM_SDA,
         DDR_DIMM_SCL => DDR_DIMM_SCL,
         DDR_DIMM_DM => DDR_DIMM_DM,
         PCI_ADDR => S_DATA_OUT(26 DOWNTO 0),
         PCI_DATA_IN => S_DATA_OUT(63 DOWNTO 0),
         PCI_CBEA => S_CBEA_OUT(7 DOWNTO 0),
         PCI_WRITE => S_WRITE,
         PCI_WRITE_VALID => S_DATH_VLD,
         PCI_HIT => S_HIT(2),
         PCI_HIT_EARLY => pci_hit_early,
         READ_DATA_OUT => RAMDOUT,
         READ_DATA_VALID => open,
         DDR_NOT_READY => DDR_NOT_READY,
         DCM_LOCK => DCM_LOCK,
         S_WAIT => s_wait_w,
         S_DONE => S_DONE);   
   

END ARCHITECTURE rtl;
