--*******************************************************************************
--
--  File name :       v4_dm_iob.v
--
--  Description :     This module instantiates the IDELAY primitive and IOB  
--                    flip-flops for the data mask I/Os.
-- 
--                    
--  Date - revision : 1/23/2004
--
--  Author :          Kraig Lund
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 - 2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY v4_dm_iob IS
   PORT (
      CLK90                   : IN std_logic;   
      DM_RISE                 : IN std_logic;   
      DM_FALL                 : IN std_logic;   
      DDR_DM                  : OUT std_logic);   
END v4_dm_iob;

ARCHITECTURE rtl OF v4_dm_iob IS

   SIGNAL dm_out                   :  std_logic;   
   SIGNAL vcc                      :  std_logic;   
   SIGNAL gnd                      :  std_logic;   
   SIGNAL dm_rise_r                :  std_logic;   
   SIGNAL dm_fall_r                :  std_logic;    

BEGIN


   wrdata_rise_reg_dimm : FD 
      PORT MAP (
         Q => dm_rise_r,
         C => CLK90,
         D => DM_RISE);   
   
   wrdata_fall_reg_dimm : FD 
      PORT MAP (
         Q => dm_fall_r,
         C => CLK90,
         D => DM_FALL);   
   
   oddr_dm : ODDR 
      GENERIC MAP (
         DDR_CLK_EDGE => "SAME_EDGE",
         SRTYPE => "SYNC")
      PORT MAP (
         Q => dm_out,
         C => CLK90,
         CE => '1',
         D1 => dm_rise_r,
         D2 => dm_fall_r,
         R => '0',
         S => '0');   
   
   obuf_dm : OBUF 
      PORT MAP (
         I => dm_out,
         O => DDR_DM);   
   

END rtl;
