-------------------------------------------------------------------------------
--
--  File name :       v4_dq_iob.vhd
--
--  Description :     This module instantiates the IDELAY primitive and IOB  
--                    flip-flops for the bi-directional data.
-- 
--                    
--  Date - revision : 1/23/2004
--                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
--
--  Author :          Maria George
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2006 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY v4_dq_iob IS
   PORT (
      CLK                     : IN std_logic;   
      CLK90                   : IN std_logic;   
      RESET                   : IN std_logic;     
      DATA_DLYINC             : IN std_logic;   
      DATA_DLYCE              : IN std_logic;   
      DATA_DLYRST             : IN std_logic;   
      WRITE_DATA_RISE         : IN std_logic;   
      WRITE_DATA_FALL         : IN std_logic;   
      CTRL_WREN               : IN std_logic;    
      DDR_DQ                  : INOUT std_logic;   
      READ_DATA_RISE          : OUT std_logic;   
      READ_DATA_FALL          : OUT std_logic);   
END v4_dq_iob;

ARCHITECTURE rtl OF v4_dq_iob IS

	attribute IOB : string;
	attribute IOB of tri_state_dq: label is "TRUE"; 
		  
   SIGNAL dq_in                    :  std_logic;   
   SIGNAL dq_out                   :  std_logic;   
   SIGNAL dq_delayed               :  std_logic;   
   SIGNAL write_en_L               :  std_logic;   
   SIGNAL write_en_L_r1            :  std_logic;   
   SIGNAL vcc                      :  std_logic;   
   SIGNAL gnd                      :  std_logic;   
   SIGNAL rd_data_rise             :  std_logic;   
   SIGNAL rd_data_fall             :  std_logic;   
   SIGNAL wr_data_rise             :  std_logic;   
   SIGNAL wr_data_fall             :  std_logic;   
   SIGNAL write_en_L_r             :  std_logic;   

BEGIN
   vcc <= '1' ;
   gnd <= '0' ;
   write_en_L <= NOT CTRL_WREN ;

   wrdata_rise_reg_dimm : FD 
      PORT MAP (
         Q => wr_data_rise,
         C => CLK90,
         D => WRITE_DATA_RISE);   
   
  
   wrdata_fall_reg_dimm : FD 
      PORT MAP (
         Q => wr_data_fall,
         C => CLK90,
         D => WRITE_DATA_FALL);   
   

   oddr_dq : ODDR 
      GENERIC MAP (
         DDR_CLK_EDGE => "SAME_EDGE",  
         SRTYPE => "SYNC")
      PORT MAP (
         Q => dq_out,
         C => CLK90,
         CE => vcc,
         D1 => wr_data_rise,
         D2 => wr_data_fall,
         R => gnd,
         S => gnd);   
   
   ctrl_wren_reg_dimm : FD 
      PORT MAP (
         Q => write_en_L_r,
         C => CLK90,
         D => write_en_L);   
   
   tri_state_dq : FDCE 
      PORT MAP (
         D => write_en_L_r,
         CLR => RESET,
         C => CLK90,
         Q => write_en_L_r1,
         CE => vcc);   
	   
   iobuf_dq : IOBUF 
      PORT MAP (
         I => dq_out,
         T => write_en_L_r1,
         IO => DDR_DQ,
         O => dq_in);   
   
 
   idelay_dq : IDELAY 
      GENERIC MAP (
         IOBDELAY_TYPE => "VARIABLE",  
         IOBDELAY_VALUE => 0)
      PORT MAP (
         O => dq_delayed,
         I => dq_in,
         C => CLK,
         CE => DATA_DLYCE,
         INC => DATA_DLYINC,
         RST => DATA_DLYRST);   
   

   iddr_dq : IDDR 
      GENERIC MAP (
         DDR_CLK_EDGE => "SAME_EDGE",
         SRTYPE => "SYNC")
      PORT MAP (
         Q1 => READ_DATA_RISE,
         Q2 => READ_DATA_FALL,
         C => CLK,
         CE => vcc,
         D => dq_delayed,
         R => gnd,
         S => gnd);   
   

END rtl;
