--*******************************************************************************
--
--  File name :       v4_dqs_iob.vhd
--
--  Description :     This module instantiates the IDELAY primitive and IOB  
--                    flip-flops for the bi-directional strobe.
-- 
--                    
--  Date - revision : 1/23/2004
--                  : 11/10/2005 modified for use in XAPP708 - Kraig Lund
--
--  Author :          Maria George
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778 
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are 
--              provided to you "as is". Xilinx and its licensors make and you 
--              receive no warranties or conditions, express, implied, 
--              statutory or otherwise, and Xilinx specifically disclaims any 
--              implied warranties of merchantability, non-infringement, or 
--              fitness for a particular purpose. Xilinx does not warrant that 
--              the functions contained in these designs will meet your 
--              requirements, or that the operation of these designs will be 
--              uninterrupted or error free, or that defects in the Designs 
--              will be corrected. Furthermore, Xilinx does not warrant or 
--              make any representations regarding use or the results of the 
--              use of the designs in terms of correctness, accuracy, 
--              reliability, or otherwise. 
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its 
--              licensors be liable for any loss of data, lost profits, cost 
--              or procurement of substitute goods or services, or for any 
--              special, incidental, consequential, or indirect damages 
--              arising from the use or operation of the designs or 
--              accompanying documentation, however caused and on any theory 
--              of liability. This limitation will apply even if Xilinx 
--              has been advised of the possibility of such damage. This 
--              limitation shall apply not-withstanding the failure of the 
--              essential purpose of any limited remedies herein. 
--
--  Copyright  2002 - 2005 Xilinx, Inc.
--  All rights reserved 
-- 
--*****************************************************************************

  LIBRARY IEEE;
  USE IEEE.std_logic_1164.ALL;
  LIBRARY UNISIM;
  USE UNISIM.VCOMPONENTS.ALL;

ENTITY v4_dqs_iob IS
   PORT (
      CLK                     : IN std_logic;   
      DLYINC                  : IN std_logic;   
      DLYCE                   : IN std_logic;     
      DLYRST                  : IN std_logic;   
      CTRL_DQS_RST            : IN std_logic;   
      CTRL_DQS_EN             : IN std_logic;    
      DDR_DQS                 : INOUT std_logic;   
      DQS                     : OUT std_logic);   
END v4_dqs_iob;

ARCHITECTURE rtl OF v4_dqs_iob IS

   attribute IOB : string;
	attribute IOB of tri_state_dqs: label is "TRUE";    

   SIGNAL dqs_in                   :  std_logic;   
   SIGNAL dqs_out                  :  std_logic;   
   SIGNAL ctrl_dqs_rst_r           :  std_logic;   
   SIGNAL ctrl_dqs_en_r            :  std_logic;   
   SIGNAL ctrl_dqs_en_r1           :  std_logic;   
   SIGNAL ctrl_dqsl_en_r1          :  std_logic;   
   SIGNAL vcc                      :  std_logic;   
   SIGNAL gnd                      :  std_logic;   
   SIGNAL clk180                   :  std_logic;   
   SIGNAL data1                    :  std_logic;   
   SIGNAL data2                    :  std_logic; 
   SIGNAL ctrl_dqs_rst_i           :  std_logic;  
   SIGNAL ctrl_dqs_en_i           :  std_logic;  

BEGIN

   vcc <= '1' ;
   gnd <= '0' ;
   clk180 <= NOT CLK ;
   ctrl_dqs_rst_i <= CTRL_DQS_RST;
   ctrl_dqs_en_i <= CTRL_DQS_EN;

   PROCESS (clk180)
   BEGIN
      IF (clk180'EVENT AND clk180 = '1') THEN
         IF (ctrl_dqs_rst_r = '1') THEN
            data1 <= '0';    
         ELSE
            data1 <= '1';    
         END IF;
      END IF;
   END PROCESS;

   PROCESS (clk180)
   BEGIN
      IF (clk180'EVENT AND clk180 = '1') THEN
         IF (ctrl_dqs_rst_r = '1') THEN
            data2 <= '1';    
         ELSE
            data2 <= '0';    
         END IF;
      END IF;
   END PROCESS;

   dqs_rst_reg_dimm : FD 
      PORT MAP (
         D => ctrl_dqs_rst_i,
         Q => ctrl_dqs_rst_r,
         C => clk180);   
   
   dqs_en_reg_dimm : FD 
      PORT MAP (
         D => ctrl_dqs_en_i,
         Q => ctrl_dqs_en_r,
         C => clk180);   
   
 
   idelay_dqs : IDELAY 
      GENERIC MAP (
         IOBDELAY_TYPE => "VARIABLE",     
         IOBDELAY_VALUE => 0)
      PORT MAP (
         O => DQS,
         I => dqs_in,
         C => CLK,
         CE => DLYCE,
         INC => DLYINC,
         RST => DLYRST);   
   

   oddr_dqs : ODDR 
      GENERIC MAP (
         DDR_CLK_EDGE => "OPPOSITE_EDGE",
         SRTYPE => "SYNC")
      PORT MAP (
         Q => dqs_out,
         C => clk180,
         CE => vcc,
         D1 => data1,
         D2 => gnd,
         R => gnd,
         S => gnd);   
   
           
   tri_state_dqs : FD 
      PORT MAP (
         D => ctrl_dqs_en_r,
         Q => ctrl_dqs_en_r1,
         C => clk180);   
   
   iobuf_dqs : IOBUF 
      PORT MAP (
         I => dqs_out,
         T => ctrl_dqs_en_r1,
         IO => DDR_DQS,
         O => dqs_in);   
   

END rtl;
